@@ -1,22 +1,22 @@
 #!/usr/bin/perl
 				
 # Purpose: Perform NCO distributions
-# Script relies heavily on SSH connectivity between $HOST, $CVSROOT, and $ftp_mch
+# Script relies heavily on SSH connectivity between ${HOST}, ${CVSROOT}, and $www_mch
 
 # Usage:
 # Export tagged, public versions
 
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln nco-2_7_1 # Build, do not install
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --nst_all nco-2_7_1 # Install, do not build
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln --nst_all nco-2_7_1 # Build and install
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_cnt nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_prs nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cgd_cnt nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cray_prs nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --bbl_cnt nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --blk_cnt nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --dat_cnt nco-2_7_1
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --ute_prs nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln nco-3_9_0 # Build, do not install
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --nst_all nco-3_9_0 # Install, do not build
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln --nst_all nco-3_9_0 # Build and install
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_cnt nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_prs nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cgd_cnt nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cray_prs nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --bbl_cnt nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --blk_cnt nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --dat_cnt nco-3_9_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --ute_prs nco-3_9_0
 
 # Export daily snapshot
 # ${HOME}/nco/bld/nco_dst.pl --dbg=2 
@@ -24,13 +24,13 @@
 
 # Machines requiring interactive builds
 # cd ${HOME}/nco;cvs update;cd bld;make;make tst
-# scp ${HOME}/nco/bld/nco_dst.pl dust.ps.uci.edu:/home/zender/nco/bld/nco_dst.pl
+# scp ${HOME}/nco/bld/nco_dst.pl dust.ess.uci.edu:/home/zender/nco/bld/nco_dst.pl
 
 BEGIN{
     unshift @INC,$ENV{'HOME'}.'/perl'; # Location of csz.pl and DBG.pm HaS98 p. 170
 } # end BEGIN
 
-my $CVS_Header='$Header: /cvsroot/nco/nco/bld/nco_dst.pl,v 1.97 2003/01/09 01:17:57 zender Exp $';
+my $CVS_Header='$Header: /cvsroot/nco/nco/bld/nco_dst.pl,v 1.158 2007/05/23 02:06:52 zender Exp $';
 
 # Specify modules
 use strict; # Protect all namespaces
@@ -57,6 +57,7 @@ my ($prg_nm,$prg_dsc,$prg_vrs,$prg_date);
 my ($pth_in,$fl_sfx);
 
 my ($dst_vrs,$dst_fl);
+my ($dst_fl_chg,$dst_fl_deb,$dst_fl_doc,$dst_fl_dsc,$dst_fl_tgz);
 my ($nco_vrs,$nco_vrs_mjr,$nco_vrs_mnr,$nco_vrs_pch);
 my ($dly_snp);
 my ($mk_cmd,$tar_cmd,$rmt_mch);
@@ -66,10 +67,11 @@ my ($rsh_cmd,$rcp_cmd,$cp_cmd,$rm_cmd,$mkdir_cmd,$cvs_cmd);
 my $False=0;
 my $True=1;
 
-my $CVS_Date='$Date: 2003/01/09 01:17:57 $';
-my $CVS_Id='$Id: nco_dst.pl,v 1.97 2003/01/09 01:17:57 zender Exp $';
-my $CVS_Revision='$Revision: 1.97 $';
-my $CVSROOT='zender@cvs.nco.sourceforge.net:/cvsroot/nco'; # CVS repository
+my $CVS_Date='$Date: 2007/05/23 02:06:52 $';
+my $CVS_Id='$Id: nco_dst.pl,v 1.158 2007/05/23 02:06:52 zender Exp $';
+my $CVS_Revision='$Revision: 1.158 $';
+my $CVSROOT='zender@nco.cvs.sf.net:/cvsroot/nco'; # CVS repository
+my $DATA=$ENV{'DATA'};
 my $HOME=$ENV{'HOME'};
 my $HOST=$ENV{'HOST'};
 my $PVM_ARCH=$ENV{'PVM_ARCH'};
@@ -77,10 +79,6 @@ my $bld=$False; # Option bld; Whether to rebuild netCDF distribution
 my $cp_cmd='cp -p -f'; # Command that behaves like cp
 my $cvs_cmd='cvs'; # Command that behaves like cvs (Use cvs -t for verbose output)
 my $data_nm=$ENV{'DATA'};
-my $ftp_drc='/var/ftp/pub/zender/nco'; # Directory on FTP machine where repository resides
-my $ftp_drc_mrr='/home/groups/ftp/pub/nco'; # Directory on FTP mirror machine where repository resides
-my $ftp_mch='dust.ps.uci.edu'; # Machine where FTP repository resides
-my $ftp_mch_mrr='nco.sourceforge.net'; # Machine where FTP repository mirror resides
 my $main_trunk_tag='nco';
 my $mkdir_cmd='mkdir -p'; # Command that behaves like mkdir
 my $mdl_sng='nco'; # Base of module name
@@ -90,9 +88,9 @@ my $rcp_cmd_no_prs_prm='scp'; # Command that behaves like rcp and does not try t
 my $rsh_cmd='ssh'; # Command that behaves like rsh
 my $usr_nm=$ENV{'USER'};
 my $vrs_tag='';
-my $www_mch='dust.ps.uci.edu'; # WWW machine for package
+my $www_mch='dust.ess.uci.edu'; # WWW machine for package
 my $www_drc='/var/www/html/nco'; # WWW directory for package
-my $www_mch_mrr='nco.sourceforge.net'; # WWW machine for package mirror
+my $www_mch_mrr='nco.sf.net'; # WWW machine for package mirror
 my $www_drc_mrr='/home/groups/n/nc/nco/htdocs'; # WWW directory for package mirror
 
 # Set defaults for command line arguments
@@ -126,7 +124,7 @@ if($data_nm eq ''){$data_nm='/data/'.$usr_nm;}
 my $dst_pth_pfx=$data_nm; # Parent of build directory
 if($dst_pth_pfx eq $HOME){die "$prg_nm: ERROR \$dst_pth_pfx eq $dst_pth_pfx";} # This could be disastrous
 if($rm_cmd =~ m/( -r)|( -R)|( --recursive)/){die "$prg_nm: ERROR Dangerous setting \$rm_cmd eq $rm_cmd";} # This would be disastrous
-#      $CVSROOT=':pserver:anonymous@cvs.nco.sourceforge.net:/cvsroot/nco'; # CVS repository
+#      $CVSROOT=':pserver:anonymous@nco.cvs.sf.net:/cvsroot/nco'; # CVS repository
 
 $prg_dsc='NCO distribution maker'; # Program description
 ($prg_nm,$prg_vrs)=$CVS_Id =~ /: (.+).pl,v ([\d.]+)/; # Program name and version
@@ -206,9 +204,15 @@ if($dly_snp){
     $nco_vrs=substr($tag_sng,length($mdl_sng)+1,length($tag_sng)-length($mdl_sng)-1); # The +/- 1 accounts for the dash '-'
     ($nco_vrs_mjr,$nco_vrs_mnr,$nco_vrs_pch)=split(/\./,$nco_vrs);
     $dst_vrs=$mdl_sng.'-'.$nco_vrs;
-    if($nco_vrs_mjr < 1 || $nco_vrs_mjr > 2){die "$prg_nm: ERROR $nco_vrs_mjr < 1 || $nco_vrs_mjr > 2"};
+    if($nco_vrs_mjr < 1 || $nco_vrs_mjr > 3){die "$prg_nm: ERROR $nco_vrs_mjr < 1 || $nco_vrs_mjr > 3"};
 } # endelse
-$dst_fl=$dst_vrs.'.tar.gz';
+$dst_fl=$dst_vrs.'.tar.gz'; # Standard tarball distribution
+$dst_fl_chg=$mdl_sng.'_'.$nco_vrs.'-1_i386.changes'; # Debian changes
+$dst_fl_deb=$mdl_sng.'_'.$nco_vrs.'-1_i386.deb'; # Debian executables and libraries
+$dst_fl_doc=$mdl_sng.'-doc_'.$nco_vrs.'-1_all.deb'; # Debian documentation (deprecated)
+$dst_fl_dsc=$mdl_sng.'_'.$nco_vrs.'-1.dsc'; # Debian description
+$dst_fl_tgz=$mdl_sng.'_'.$nco_vrs.'-1.tar.gz'; # Debian tarball
+
 my $dst_pth_bld=$dst_pth_pfx.'/'.$dst_vrs; # Build directory
 
 if($dbg_lvl >= 1){		 
@@ -224,8 +228,8 @@ if($dbg_lvl >= 1){
 # Build distribution from scratch
 if($bld){
     cmd_prc("$rm_cmd -r $dst_pth_bld"); # Remove contents of current directory, if any
-    cmd_prc("$mkdir_cmd $dst_pth_bld"); # Create directory
-    chdir $dst_pth_bld.'/..' or die "$prg_nm: ERROR unable to chdir to $dst_pth_bld: $!\n"; # $! is system error string
+#    cmd_prc("$mkdir_cmd $dst_pth_bld"); # Create directory
+    chdir $dst_pth_pfx or die "$prg_nm: ERROR unable to chdir to $dst_pth_pfx: $!\n"; # $! is system error string
     if($CVSROOT =~ m/pserver/){cmd_prc("$cvs_cmd -d $CVSROOT login");} # Login first
     if($dly_snp){
 	cmd_prc("$cvs_cmd -d $CVSROOT export -kkv -D \"1 second ago\" -d $dst_vrs nco"); # Export
@@ -239,22 +243,23 @@ if($bld){
 # Make sure documentation files are up to date
     my $bld_pth=$dst_pth_pfx.'/'."$dst_vrs".'/bld';
     chdir $bld_pth or die "$prg_nm: ERROR unable to chdir to $bld_pth: $!\n"; # $! is system error string
-# fxm: 20020203 do not make doc
-#    cmd_prc("$mk_cmd doc"); 
+    cmd_prc("$mk_cmd doc"); 
     cmd_prc("$mk_cmd clean"); 
-    
+
 # Set up FTP server
     chdir $dst_pth_pfx or die "$prg_nm: ERROR unable to chdir to $dst_pth_pfx: $!\n"; # $! is system error string
-    cmd_prc("$tar_cmd -cvzf $dst_fl ./$dst_vrs"); # Create gzipped tarfile
-    cmd_prc("$rsh_cmd $ftp_mch $rm_cmd $ftp_drc/$dst_fl"); # Remove any distribution with same name
-    if($dly_snp){cmd_prc("$rsh_cmd $ftp_mch $rm_cmd -r $ftp_drc/nco-????????.tar.gz");} # Remove previous daily snapshots from FTP server
-    cmd_prc("$rcp_cmd $dst_fl $ftp_mch:$ftp_drc"); # Copy local tarfile to FTP server
-    
+    cmd_prc("$tar_cmd cvzf $dst_fl --exclude='nco-3.9.0/debian*' --exclude='.cvsignore' --exclude=ncap_lex.c --exclude=ncap_yacc.[ch] ./$dst_vrs"); # Create gzipped tarfile
+    cmd_prc("$rsh_cmd $www_mch $rm_cmd $www_drc/src/$dst_fl"); # Remove any distribution with same name
+    if($dly_snp){cmd_prc("$rsh_cmd $www_mch $rm_cmd -r $www_drc/src/nco-????????.tar.gz");} # Remove previous daily snapshots from WWW server
+    cmd_prc("$rcp_cmd $dst_fl $www_mch:$www_drc/src"); # Copy local tarfile to WWW server
+    cmd_prc("$rcp_cmd $DATA/$dst_fl_chg $DATA/$dst_fl_deb $DATA/$dst_fl_dsc $DATA/$dst_fl_tgz $www_mch:$www_drc/src"); # Copy Debian files to WWW server
+ 
 # Full release procedure (public releases only) includes update Web pages
     if(!$dly_snp){
-	cmd_prc("$rsh_cmd $ftp_mch $rm_cmd $ftp_drc/nco.tar.gz");
-	cmd_prc("$rsh_cmd $ftp_mch \"cd $ftp_drc; ln -s $dst_fl nco.tar.gz\"");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/index.shtml $www_mch:$www_drc");
+	cmd_prc("$rsh_cmd $www_mch $rm_cmd $www_drc/nco.tar.gz");
+	cmd_prc("$rsh_cmd $www_mch \"cd $www_drc; ln -s -f ./src/$dst_fl nco.tar.gz\"");
+ 	cmd_prc("$rcp_cmd $dst_pth_bld/doc/index.shtml $www_mch:$www_drc");
+ 	cmd_prc("$rcp_cmd $dst_pth_bld/doc/nco.png $www_mch:$www_drc");
 	cmd_prc("$rcp_cmd $dst_pth_bld/doc/nco_news.shtml $www_mch:$www_drc");
 	cmd_prc("$rcp_cmd $dst_pth_bld/doc/nco.html $dst_pth_bld/doc/nco.info* $dst_pth_bld/doc/nco.dvi $dst_pth_bld/doc/nco.pdf $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.texi $www_mch:$www_drc");
 	cmd_prc("$rcp_cmd $dst_pth_bld/doc/README $www_mch:$www_drc");
@@ -268,11 +273,10 @@ if($bld){
 # Usually scp_cmd includes -p switch to preserve permissions and times, but sourceforge server does not allow this
 # Hence it generates warnings when used with -p, and is not properly updated
     if($True){
-	cmd_prc("$rcp_cmd_no_prs_prm $dst_fl $ftp_mch_mrr:$ftp_drc_mrr"); # Copy local tarfile to FTP server
-	cmd_prc("$rsh_cmd $ftp_mch_mrr $rm_cmd $ftp_drc_mrr/nco.tar.gz");
-	cmd_prc("$rsh_cmd $ftp_mch_mrr \"cd $ftp_drc_mrr; ln -s $dst_fl nco.tar.gz\"");
-	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_bld/doc/index.shtml $dst_pth_bld/doc/nco_news.shtml $dst_pth_bld/doc/README $dst_pth_bld/doc/TODO $dst_pth_bld/doc/VERSION $dst_pth_bld/doc/ChangeLog $www_mch_mrr:$www_drc_mrr");
+	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_bld/doc/index.shtml $dst_pth_bld/doc/nco.png $dst_pth_bld/doc/nco_news.shtml $dst_pth_bld/doc/README $dst_pth_bld/doc/TODO $dst_pth_bld/doc/VERSION $dst_pth_bld/doc/ChangeLog $www_mch_mrr:$www_drc_mrr");
 	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_bld/doc/nco.html $dst_pth_bld/doc/nco.info* $dst_pth_bld/doc/nco.dvi $dst_pth_bld/doc/nco.pdf $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.texi $www_mch_mrr:$www_drc_mrr");
+	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_pfx/$dst_fl $DATA/$dst_fl_chg $DATA/$dst_fl_deb $DATA/$dst_fl_dsc $DATA/$dst_fl_tgz $www_mch_mrr:$www_drc_mrr/src"); # Copy Debian files to WWW server
+	cmd_prc("$rsh_cmd $www_mch_mrr \"cd $www_drc_mrr; ln -s -f ./src/$dst_fl nco.tar.gz\"");
     } # endif SourceForge
 
 # Housekeeping
@@ -280,7 +284,7 @@ if($bld){
     if($dst_cln){cmd_prc("$rm_cmd -r $dst_pth_bld");} # Remove local distribution
 
 # Sanity check
-    cmd_prc("$rsh_cmd $ftp_mch ls -l $ftp_drc");
+    cmd_prc("$rsh_cmd $www_mch ls -l $www_drc");
 } # endif bld
 
 if($acd_prs){
@@ -309,7 +313,6 @@ if($cgd_cnt){
     print STDOUT "\n$prg_nm: Updating contrib NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("$rcp_cmd $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;gtar -xvzf nco.tar.gz;rm nco.tar.gz\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /contrib/nco-1.1/bin; setenv MY_LIB_DIR /contrib/nco-1.1/lib; setenv MY_OBJ_DIR /usr/tmp/$usr_nm/$dst_vrs/obj; gmake cln all test\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
@@ -321,7 +324,6 @@ if($dat_cnt){
     print STDOUT "\n$prg_nm: Updating contrib NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("$rcp_cmd $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;tar -xvzf nco.tar.gz;rm nco.tar.gz\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /contrib/nco-1.1/bin; setenv MY_LIB_DIR /contrib/nco-1.1/lib; setenv MY_OBJ_DIR /usr/tmp/$usr_nm/$dst_vrs/obj; gmake cln all tst\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
@@ -333,7 +335,6 @@ if($bbl_cnt){
     print STDOUT "\n$prg_nm: Updating contrib NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("$rcp_cmd $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;gunzip nco.tar.gz;tar -xvf nco.tar;rm nco.tar\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /home/blackforest/$usr_nm/bin/AIX; setenv MY_LIB_DIR /home/blackforest/$usr_nm/lib/AIX; setenv MY_OBJ_DIR /home/blackforest/$usr_nm/obj/AIX; setenv NETCDF_INC /usr/local/include; setenv NETCDF_LIB /usr/local/lib32/r4i4; gmake cln all tst\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
@@ -345,7 +346,6 @@ if($blk_cnt){
     print STDOUT "\n$prg_nm: Updating contrib NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("$rcp_cmd $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;gunzip nco.tar.gz;tar -xvf nco.tar;rm nco.tar\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /home/blackforest/$usr_nm/bin/AIX; setenv MY_LIB_DIR /home/blackforest/$usr_nm/lib/AIX; setenv MY_OBJ_DIR /home/blackforest/$usr_nm/obj/AIX; setenv NETCDF_INC /usr/local/include; setenv NETCDF_LIB /usr/local/lib32/r4i4; gmake cln all tst\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
@@ -357,7 +357,6 @@ if($ute_prs){
     print STDOUT "\n$prg_nm: Updating personal NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("$rcp_cmd $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;tar -xvzf nco.tar.gz;rm nco.tar.gz\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /home/ute/$usr_nm/bin/SGIMP64/bin; setenv MY_LIB_DIR /home/ute/$usr_nm/bin/SGIMP64/lib; setenv MY_OBJ_DIR /usr/tmp/$usr_nm/$dst_vrs/obj; gmake cln all tst\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
@@ -368,7 +367,6 @@ if($cray_prs){
     print STDOUT "\n$prg_nm: Updating private NCO on $rmt_mch...\n";
     cmd_prc("$rsh_cmd $rmt_mch \"$rm_cmd -r /usr/tmp/$usr_nm/nco*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"$mkdir_cmd /usr/tmp/$usr_nm/$dst_vrs/obj\"");
-    cmd_prc("rcp -p $ftp_mch:$ftp_drc/nco.tar.gz $rmt_mch:/usr/tmp/$usr_nm");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm;gunzip nco.tar.gz;tar -xvf nco.tar;rm nco.tar*\"");
     cmd_prc("$rsh_cmd $rmt_mch \"cd /usr/tmp/$usr_nm/$dst_vrs/bld; setenv MY_BIN_DIR /home/ouray0/$usr_nm/bin/CRAY; setenv MY_LIB_DIR /usr/tmp/$usr_nm/$dst_vrs/lib; setenv MY_OBJ_DIR /usr/tmp/$usr_nm/$dst_vrs/obj; gnumake cln all tst\"");
     print STDOUT "$prg_nm: Done updating contrib NCO on $rmt_mch\n\n";
