@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.31 2004/02/09 07:54:41 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/libnco.h,v 1.37 2007/02/23 21:59:27 zender Exp $ */
 
-/* Purpose: netCDF Operator (NCO) library */
+/* Purpose: netCDxF Operator (NCO) library */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* libnco.h headers depend on netcdf.h and nco_netcdf.h headers */
 
@@ -25,7 +25,7 @@
 #include "nco_cnf_dmn.h" /* Conform dimensions */
 #include "nco_cnf_typ.h" /* Conform variable types */
 #include "nco_cnv_arm.h" /* ARM conventions */
-#include "nco_cnv_csm.h" /* CCSM conventions */
+#include "nco_cnv_csm.h" /* CCM/CCSM/CF conventions */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_dbg.h" /* Debugging */
 #include "nco_dmn_utl.h" /* Dimension utilities */
