@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncap.h,v 1.67 2004/07/24 01:00:50 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncap.h,v 1.79 2007/02/23 21:59:27 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor definitions and function prototypes for ncap.c, ncap_utl.c, ncap_lex.l, and ncap_yacc.y */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "ncap.h" *//* netCDF arithmetic processor-specific definitions (symbol table, ...) */
@@ -90,8 +90,10 @@ typedef struct{ /* prs_sct */
   int *nbr_dmn_out;   /* number of dimensions in above list */
   sym_sct **sym_tbl; /* [fnc] Symbol table for functions */
   int sym_tbl_nbr; /* [nbr] Number of functions in table */
-  bool ntl_scn; /* [flg] Initial scan of script */
+  nco_bool ntl_scn; /* [flg] Initial scan of script */
   var_sct *var_LHS; /* [var] LHS cast variable */
+  var_sct ***var_lst; /* list of variables to be defined in O */
+  int *nbr_var;        /* [nbr] of variables in above list */ 
   int nco_op_typ; /* [enm] Operation type */
 } prs_sct;
 
@@ -129,10 +131,19 @@ ncap_aed_lookup /* [fnc] Find location of existing attribute or add new attribut
 (const char * const var_nm, /* I [sng] Variable name */
  const char * const att_nm, /* I [sng] Attribute name */
  prs_sct * const prs_arg,  /* I/O [sct] contains attribute list  */
- const bool update); /* I [flg] Delete existing value or add new attribute to list */
+ const nco_bool update); /* I [flg] Delete existing value or add new attribute to list */
+
+
+var_sct *                      /*I [sct] varibale in list */
+ncap_var_lookup
+(var_sct *var,   /* I  [sct] variable  */
+ prs_sct *prs_arg,             /* I/O [sct] contains var list */
+ const nco_bool add);              /* I if not in list then add to list */          
+
 
 void nco_var_free_wrp(var_sct **);
 
+
 nodeType * /* O [unn] Syntax tree node */
 opr_ctl /* [fnc] Operation controller function Nie02 opr() */
 (int opr_tkn, /* I [enm] Operator token */
@@ -140,7 +151,7 @@ opr_ctl /* [fnc] Operation controller function Nie02 opr() */
  ... /* I [llp] Ellipsis defined in stdarg.h */
  ); /* end opr_ctl() */
 
-int yyerror(const char * const sng);
+int yyerror(prs_sct* prs_arg, const char * const sng);
 /* End funtions in ncap_yacc.y */
 
 /* Begin funtions in ncap_utl.c */
@@ -175,17 +186,16 @@ ncap_var_var_sub /* [fnc] Subtract two variables (var_2-var_1) */
  var_sct *var_1); /* I [sct] Variable structure containing first operand */
 
   /* fxm: Following functions need editing, const'ifying, etc. */
-bool ncap_var_stretch(var_sct **,var_sct **);
+nco_bool ncap_var_stretch(var_sct **,var_sct **);
 dmn_sct **nco_dmn_out_grow(prs_sct *);
 int ncap_scv_minus(scv_sct *);
 int ncap_var_write(var_sct *,prs_sct*);
-nm_id_sct *nco_var_lst_crd_make(int,nm_id_sct *,int *);
+nm_id_sct *nco_nm_id_lst_crd_make(int,nm_id_sct *,int *);
 nm_id_sct *nco_var_lst_add(nm_id_sct *,int *,nm_id_sct *,int);
 nm_id_sct *nco_var_lst_copy(nm_id_sct *,int);
-nm_id_sct *nco_var_lst_free(nm_id_sct *,int);
 nm_id_sct *nco_var_lst_sub(nm_id_sct *,int *,nm_id_sct *,int);
 nm_id_sct *nco_dmn_lst(const int, int * const);
-nm_id_sct *nco_att_lst_mk(int, int,aed_sct** ,int ,int *);
+nm_id_sct *nco_att_lst_mk(int,int,aed_sct **,int,int *);
 ptr_unn ncap_scv_2_ptr_unn(scv_sct); 
 scv_sct ncap_scv_abs(scv_sct);
 scv_sct ncap_scv_clc(scv_sct,const char,scv_sct);
@@ -194,7 +204,8 @@ sym_sct *ncap_scl_mk_sym(double val);
 sym_sct *ncap_sym_look(char *sym_nm);
 var_sct *ncap_var_abs(var_sct *);
 var_sct *ncap_var_fnc(var_sct *,sym_sct *);
-var_sct *ncap_var_init(const char * const,prs_sct *);
+/* var_sct *ncap_var_init(const char * const,prs_sct *);*/
+var_sct *ncap_var_init(char * ,prs_sct *);
 var_sct *ncap_var_scv_add(var_sct *,scv_sct);
 var_sct *ncap_var_scv_mlt(var_sct *,scv_sct);
 var_sct *ncap_var_scv_sub(var_sct *,scv_sct);
@@ -205,7 +216,7 @@ var_sct *ncap_scv_var_dvd(scv_sct scv,var_sct *var);
 var_sct *ncap_scv_var_mod(scv_sct scv,var_sct *var);
 var_sct *ncap_scv_var_pwr(scv_sct scv,var_sct *var);
 void ncap_fnc_add(char *nm, double (*fnc_dbl)());
-void nco_lbr_vrs_prn();
+void nco_lbr_vrs_prn(void);
 /* End funtions in ncap_utl.c */
 
 /* Source code for following functions is generated by parser but parser
