@@ -1,15 +1,19 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncap_utl.c,v 1.103 2004/09/03 23:06:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncap_utl.c,v 1.135 2007/05/20 19:44:44 zender Exp $ */
 
 /* Purpose: netCDF arithmetic processor */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
+#include <assert.h>
 #include "ncap.h" /* netCDF arithmetic processor */
+extern char ncap_err_sng[200]; /* [sng] Buffer for error string (declared in ncap_lex.l) */
 
 var_sct *
-ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
+/* fxm: which prototype to use? */
+/* ncap_var_init(const char * const var_nm,prs_sct *prs_arg) */
+ncap_var_init(char *var_nm,prs_sct *prs_arg)
 {
   /* Purpose: Initialize variable structure, retrieve variable values from disk
      Parser calls ncap_var_init() when it encounters a new RHS variable */
@@ -17,20 +21,57 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
 
   int idx;
   int jdx;
-  int nbr_dmn_var;
+  int dmn_var_nbr;
   int *dim_id=NULL;
   int var_id;
   int rcd;
   int fl_id;
 
+  nco_bool DEF_VAR=False;
+
   dmn_sct *dmn_in; 
   dmn_sct **dim_new=NULL_CEWI;
   
+  var_sct var_lkp;
   var_sct *var;
+  var_sct *var_lst;  
+
+  /* Several possiblilties here:
+     var NOT in I or O
+     var in I
+     var in O (defined and filled)
+     var in 0 (defined but empty)
+
+     Must handle case var in I, var in O (defined & empty) with care
+     This occurs when var is on LHS and RHS, e.g., two=two^6
+
+     INITIAL SCAN
+     check var list for var -- 
+     if present fill with nulls and return
+     Not present check output then input
+     
+     FINAL SCAN
+     check var list for var -- 
+     if present & defined + filled return var from output
+     if present & defined + unfilled reurn var from input if defined
+     Not present check output then input */     
+     
+  var_lkp.nm=strdup(var_nm);
+  var_lst=ncap_var_lookup(&var_lkp,((prs_sct*)prs_arg),False);
+  (void)nco_free(var_lkp.nm);
+
+  if(prs_arg->ntl_scn && var_lst){
+    var=nco_var_dpl(var_lst);
+    var->val.vp=NULL;
+    return var;	
+  } /* endif */
+
+  /* Check if var in list has been defined but NOT filled */
+  if(!prs_arg->ntl_scn && var_lst && var_lst->sz >0 ) DEF_VAR=True; 
 
   /* Check output file for var */  
   rcd=nco_inq_varid_flg(prs_arg->out_id,var_nm,&var_id);
-  if(rcd == NC_NOERR){
+  if(rcd == NC_NOERR && !DEF_VAR){
     fl_id=prs_arg->out_id;
   }else{
     /* Check input file for ID */
@@ -47,12 +88,12 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
     (void)nco_redef(prs_arg->out_id);
     fl_id=prs_arg->in_id;
     
-    (void)nco_inq_varndims(fl_id,var_id,&nbr_dmn_var);
-    if(nbr_dmn_var>0){
-      dim_id=(int *)nco_malloc(nbr_dmn_var*sizeof(int));
+    (void)nco_inq_varndims(fl_id,var_id,&dmn_var_nbr);
+    if(dmn_var_nbr>0){
+      dim_id=(int *)nco_malloc(dmn_var_nbr*sizeof(int));
       
       (void)nco_inq_vardimid(fl_id,var_id,dim_id);
-      for(idx=0;idx<nbr_dmn_var;idx++) 
+      for(idx=0;idx<dmn_var_nbr;idx++) 
 	for(jdx=0;jdx<prs_arg->nbr_dmn_in;jdx++){
 	  
 	  /* De-reference */
@@ -85,7 +126,11 @@ ncap_var_init(const char * const var_nm,prs_sct *prs_arg)
   var->tally=(long *)NULL;
 
   /* Retrieve variable values from disk into memory */
-  (void)nco_var_get(fl_id,var);
+  if(prs_arg->ntl_scn) {
+    var->val.vp=(void*)NULL;
+  }else{
+    (void)nco_var_get(fl_id,var);
+  } /* end else */
   /* (void)nco_var_free(var_nm);*/
   /* (void)nco_free(var_nm->nm);*/
   /* var=nco_var_upk(var); */
@@ -96,13 +141,14 @@ dmn_sct **
 nco_dmn_out_grow
 (prs_sct * prs_arg)
 {
-  /* Purpose: Expand dimension list by 1 and return pointer to newly created member */
-  int *size;
-  size=prs_arg->nbr_dmn_out;
+  /* Purpose: Expand dimension list by one and return pointer to newly created member */
+
+  int *sz;
+  sz=prs_arg->nbr_dmn_out;
   
-  *(prs_arg->dmn_out)=(dmn_sct **)nco_realloc(*(prs_arg->dmn_out),(++*size)*sizeof(dmn_sct *));
+  *(prs_arg->dmn_out)=(dmn_sct **)nco_realloc(*(prs_arg->dmn_out),(++*sz)*sizeof(dmn_sct *));
   
-  return (*(prs_arg->dmn_out)+(*size-1));
+  return (*(prs_arg->dmn_out)+(*sz-1));
 } /* end nco_dmn_out_grow() */
 
 int 
@@ -111,24 +157,85 @@ ncap_var_write
  prs_sct *prs_arg)
 {
   /* Purpose: Define variable in output file and write variable */
-  const char mss_val_sng[]="missing_value"; /* [sng] Unidata standard string for missing value */
+
+  /*  const char mss_val_sng[]="missing_value"; *//* [sng] Unidata standard string for missing value */
   const char add_fst_sng[]="add_offset"; /* [sng] Unidata standard string for add offset */
   const char scl_fct_sng[]="scale_factor"; /* [sng] Unidata standard string for scale factor */
+
+  int rcd; /* [rcd] Return code */
   int var_out_id;
+
+  var_sct *ptr_var;
+  var_sct *var_dpl;
+  nco_bool DEF_VAR; /* True if var has been defined in O in initial scan */
   
-  (void)nco_redef(prs_arg->out_id);
+#ifdef NCO_RUSAGE_DBG
+  long maxrss; /* [B] Maximum resident set size */
+#endif /* !NCO_RUSAGE_DBG */
+
+  /* If inital scan duplicate then save in list, free val.vp */
+  if(prs_arg->ntl_scn){
+    var->val.vp=nco_free(var->val.vp);
+    var_dpl=nco_var_dpl(var);
+    assert(var_dpl->nm);
+    if(ncap_var_lookup(var_dpl,((prs_sct*)prs_arg),True) != NULL)
+      (void)fprintf(stdout,"%s: variable %s defined\n",prg_nm_get(),var->nm);
+    (void)nco_var_free(var);
+    return True;
+  } /* endif ntl_scn */
+
+  /* Check if var is in table AND has been defined
+     NB: var and ptr_var are different */
+  ptr_var=ncap_var_lookup(var,((prs_sct*)prs_arg),False);
+  DEF_VAR=(ptr_var && ptr_var->sz > 0 ? True : False);
+
+  rcd=nco_inq_varid_flg(((prs_sct *)prs_arg)->out_id,var->nm,&var_out_id);
+
+  if(!DEF_VAR){ 
+    /* Check to see if variable has already been defined and written */
+    if(rcd == NC_NOERR){
+    (void)sprintf(ncap_err_sng,"Warning: Variable %s has aleady been saved in %s",var->nm,((prs_sct *)prs_arg)->fl_out);
+    (void)yyerror(prs_arg,ncap_err_sng);
+    var = nco_var_free(var);
+    return False;
+    }
+  } /* DEF_VAR */
   
-  /* Define variable */   
-  (void)nco_def_var(prs_arg->out_id,var->nm,var->type,var->nbr_dim,var->dmn_id,&var_out_id);
-  /* Put missing value */  
-  if(var->has_mss_val) (void)nco_put_att(prs_arg->out_id,var_out_id,mss_val_sng,var->type,1,var->mss_val.vp);
+  if(DEF_VAR && (var->pck_ram || var->has_mss_val)){
+       /* Put file in define mode to allow metadata writing */
+       (void)nco_redef(prs_arg->out_id);
+
+       /* Put missing value */  
+	if(var->has_mss_val) (void)nco_put_att(prs_arg->out_id,var_out_id,nco_mss_val_sng_get(),var->type,1,var->mss_val.vp);
   
-  /* Write/overwrite scale_factor and add_offset attributes */
-  if(var->pck_ram){ /* Variable is packed in memory */
-    if(var->has_scl_fct) (void)nco_put_att(prs_arg->out_id,var_out_id,scl_fct_sng,var->typ_upk,1,var->scl_fct.vp);
-    if(var->has_add_fst) (void)nco_put_att(prs_arg->out_id,var_out_id,add_fst_sng,var->typ_upk,1,var->add_fst.vp);
-  } /* endif pck_ram */
-  (void)nco_enddef(prs_arg->out_id);
+       /* Write/overwrite scale_factor and add_offset attributes */
+       if(var->pck_ram){ /* Variable is packed in memory */
+	 if(var->has_scl_fct) (void)nco_put_att(prs_arg->out_id,var_out_id,scl_fct_sng,var->typ_upk,1,var->scl_fct.vp);
+	  if(var->has_add_fst) (void)nco_put_att(prs_arg->out_id,var_out_id,add_fst_sng,var->typ_upk,1,var->add_fst.vp);
+	} /* endif pck_ram */
+	
+        /* Take output file out of define mode */
+       (void)nco_enddef(prs_arg->out_id);
+  } /* endif DEF_VAR... */
+
+  if(!DEF_VAR){
+      /* Put file in define mode to allow metadata writing */
+      (void)nco_redef(prs_arg->out_id);
+  
+      /* Define variable */   
+      (void)nco_def_var(prs_arg->out_id,var->nm,var->type,var->nbr_dim,var->dmn_id,&var_out_id);
+      /* Put missing value */  
+      if(var->has_mss_val) (void)nco_put_att(prs_arg->out_id,var_out_id,nco_mss_val_sng_get(),var->type,1,var->mss_val.vp);
+  
+      /* Write/overwrite scale_factor and add_offset attributes */
+      if(var->pck_ram){ /* Variable is packed in memory */
+	if(var->has_scl_fct) (void)nco_put_att(prs_arg->out_id,var_out_id,scl_fct_sng,var->typ_upk,1,var->scl_fct.vp);
+	if(var->has_add_fst) (void)nco_put_att(prs_arg->out_id,var_out_id,add_fst_sng,var->typ_upk,1,var->add_fst.vp);
+      } /* endif pck_ram */
+
+      /* Take output file out of define mode */
+      (void)nco_enddef(prs_arg->out_id);
+    } /* end if */
   
   /* Write variable */ 
   if(var->nbr_dim == 0){
@@ -137,9 +244,22 @@ ncap_var_write
     (void)nco_put_vara(prs_arg->out_id,var_out_id,var->srt,var->cnt,var->val.vp,var->type);
   } /* end else */
   
-  return 1;
-} /* end ncap_var_write() */
+#ifdef NCO_RUSAGE_DBG
+  /* Compile: cd ~/nco/bld;make 'USR_TKN=-DNCO_RUSAGE_DBG';cd - */
+  /* Print rusage memory usage statistics */
+  if(dbg_lvl_get() >= 0) (void)fprintf(stdout,"%s: INFO ncap_var_write() writing variable %s\n",prg_nm_get(),var->nm);
+  maxrss=nco_mmr_rusage_prn((int)0);
+#endif /* !NCO_RUSAGE_DBG */
 
+  /* Free varible */
+  var=nco_var_free(var);
+
+  /* Use sz to keep track of defined and written variables */
+  if(DEF_VAR) ptr_var->sz=-1;
+  
+  return rcd;
+} /* end ncap_var_write() */
+  
 sym_sct *
 ncap_sym_init
 (const char * const sym_nm,
@@ -172,8 +292,14 @@ ncap_scv_2_ptr_unn
   case NC_DOUBLE: *val.dp=scv.val.d; break;
   case NC_INT: *val.lp=scv.val.l; break;
   case NC_SHORT: *val.sp=scv.val.s; break;
+  case NC_USHORT: *val.usp=scv.val.us; break;
+  case NC_UINT: *val.uip=scv.val.ui; break;
+  case NC_INT64: *val.i64p=scv.val.i64; break;
+  case NC_UINT64: *val.ui64p=scv.val.ui64; break;
   case NC_BYTE: *val.bp=scv.val.b; break;
+  case NC_UBYTE: *val.ubp=scv.val.ub; break;
   case NC_CHAR: break; /* do nothing */
+  case NC_STRING: break; /* do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   (void)cast_nctype_void(type,&val);
@@ -187,15 +313,37 @@ ncap_var_var_add /* [fnc] Add two variables */
 {
   /* Purpose: Add two variables */
   /* Store result in var_2 */
-  
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
   (void)ncap_var_retype(var_1,var_2);
+   
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
+  
+  //  if(var_1 != var_chk1) var_chk1 = (var_sct*)nco_var_free(var_chk1);
+  //if(var_2 != var_chk2) var_chk2 = (var_sct*)nco_var_free(var_chk2);
+
   /* fxm: bug in nco_var_add()? missing_value is not carried over to var_2 in result when var_1->has_mss_val is true */
   if(var_1->has_mss_val){
     (void)nco_var_add(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
     (void)nco_var_add(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   } /* end if */
+
+  var_1=nco_var_free(var_1);
   return var_2;
 } /* end ncap_var_var_add() */
 
@@ -205,13 +353,35 @@ ncap_var_var_dvd /* [fnc] Divide two variables (var_2/var_1) */
  var_sct *var_2) /* I [sct] Variable structure containing numerator */
 {
   /* Purpose: Divide two variables (var_2/var_1) */
+
+  /* Purpose: Divide two variables */
+  /* Store result in var_2 */
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
   (void)ncap_var_retype(var_1,var_2);
+   
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
     (void)nco_var_dvd(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
     (void)nco_var_dvd(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   } /* end else */
+
+  var_1=nco_var_free(var_1);
   return var_2;
 } /* end ncap_var_var_dvd() */
 
@@ -220,14 +390,35 @@ ncap_var_var_mlt /* [fnc] Multiply two variables */
 (var_sct *var_1, /* I [sct] Variable structure containing first operand */
  var_sct *var_2) /* I [sct] Variable structure containing second operand */
 {
-  /* Purpose: Multiply two variables variables (var_1*var_2) */
+
+  /* Purpose: Multiply two variables */
+  /* Store result in var_2 */
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
   (void)ncap_var_retype(var_1,var_2);
+   
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
     (void)nco_var_mlt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
     (void)nco_var_mlt(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   } /* end else */
+
+  var_1=nco_var_free(var_1);
    return var_2;
 } /* end ncap_var_var_mlt() */
 
@@ -237,13 +428,33 @@ ncap_var_var_mod /* [fnc] Remainder (modulo) operation of two variables */
  var_sct *var_2) /* I [sct] Variable structure containing divisor */
 {
   /* Purpose: Remainder (modulo) operation of two variables (var_1%var_2) */
+  /* Store result in var_2 */
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
   (void)ncap_var_retype(var_1,var_2);
+   
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+ 
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
     (void)nco_var_mod(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
     (void)nco_var_mod(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   } /* end else */
+   
+   var_1=nco_var_free(var_1);
    return var_2;
 } /* end ncap_var_var_mod() */
 
@@ -253,7 +464,30 @@ ncap_var_var_pwr /* [fnc] Empowerment of two variables */
  var_sct *var_2) /* I [sct] Variable structure containing exponent */
 {
   /* Purpose: Empower two variables (var_1^var_2) */
-  (void)ncap_var_retype(var_1,var_2);
+
+  /* Purpose: Add two variables */
+  /* Store result in var_2 */
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
+  /* Make sure vars are at least float precision */
+  if(nco_rth_prc_rnk(var_1->type) < nco_rth_prc_rnk_float && nco_rth_prc_rnk(var_2->type < nco_rth_prc_rnk_float)) var_1=nco_var_cnf_typ((nc_type)NC_FLOAT,var_1);
+
+  (void)ncap_var_retype(var_1,var_2);   
+
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
     (void)nco_var_pwr(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
@@ -269,14 +503,34 @@ ncap_var_var_sub /* [fnc] Subtract two variables (var_2-var_1) */
  var_sct *var_1) /* I [sct] Variable structure containing first operand */
 {
   /* Purpose: Subtract a variable from another variable */
-  
+
+  /* Purpose: Add two variables */
+  /* Store result in var_2 */
+  if(var_1->undefined) var_2->undefined=True;
+    if(var_2->undefined) {
+    var_1=nco_var_free(var_1);
+    return var_2;
+  }
   (void)ncap_var_retype(var_1,var_2);
+   
+  /* Handle initial scan */
+  if(var_1->val.vp==(void*)NULL ) {
+    if(var_1->nbr_dim > var_2->nbr_dim) {
+      var_2=nco_var_free(var_2);
+      return var_1;
+    }else{
+      var_1=nco_var_free(var_1);
+      return var_2;
+    }
+  } 
+
   (void)ncap_var_cnf_dmn(&var_1,&var_2);
   if(var_1->has_mss_val){
     (void)nco_var_sbt(var_1->type,var_1->sz,var_1->has_mss_val,var_1->mss_val,var_1->val,var_2->val);
   }else{
     (void)nco_var_sbt(var_1->type,var_1->sz,var_2->has_mss_val,var_2->mss_val,var_1->val,var_2->val);
   }/* end else */
+  var_1=nco_var_free(var_1);
   return var_2;
 } /* end ncap_var_var_sub() */
 
@@ -288,9 +542,15 @@ ncap_var_fnc(var_sct *var_in,sym_sct *app)
   long idx;
   long sz;
   ptr_unn op1;
+
+  if(var_in->undefined) return var_in;
+  
   
   /* Promote variable to NC_FLOAT */
-  if(var_in->type < NC_FLOAT) var_in=nco_var_cnf_typ((nc_type)NC_FLOAT,var_in);
+  if(nco_rth_prc_rnk(var_in->type) < nco_rth_prc_rnk_float) var_in=nco_var_cnf_typ((nc_type)NC_FLOAT,var_in);
+
+  /* Handle initial scan differently */
+  if(var_in->val.vp==NULL) return var_in; 
   
   op1=var_in->val;
   sz=var_in->sz;
@@ -331,7 +591,15 @@ var_sct *
 ncap_var_scv_add(var_sct *var,scv_sct scv)
 {
   /* Purpose: add the value in scv to each element of var */
+
+  if(var->undefined) return var;
+
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
+
   (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   return var;
 } /* end ncap_var_scv_add() */
@@ -340,7 +608,12 @@ var_sct *
 ncap_var_scv_sub(var_sct *var,scv_sct scv)
 {
   /* Purpose: Subtract value in scv from each element of var */
+  if(var->undefined) return var;
+
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)ncap_scv_minus(&scv);
   (void)var_scv_add(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   
@@ -351,7 +624,14 @@ var_sct *
 ncap_var_scv_mlt(var_sct *var,scv_sct scv)
 {
   /* Purpose: Multiply variable by value in scv */
+  if(var->undefined) return var;
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) 
+    return var; 
+
+
   (void)var_scv_mlt(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   return var;
 } /* end ncap_var_scv_mlt */
@@ -361,6 +641,10 @@ ncap_var_scv_dvd(var_sct *var,scv_sct scv)
 {
   /* Purpose: Divide each element of var by value in scv */
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)var_scv_dvd(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   return var;
 } /* end ncap_var_scv_dvd */
@@ -369,7 +653,14 @@ var_sct *
 ncap_scv_var_dvd(scv_sct scv,var_sct *var)
 {
   /* Purpose: Divide scv by value of each element in var */
+  if(var->undefined) return var;
+
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
+
   (void)scv_var_dvd(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
   return var;
 } /* end ncap_scv_var_dvd */
@@ -378,7 +669,13 @@ var_sct *
 ncap_var_scv_mod(var_sct *var,scv_sct scv)
 {
   /* Purpose: Modulus of each element of var with scv */
+   if(var->undefined) return var;
+
+
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)var_scv_mod(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   return var;
 } /* ncap_var_scv_mod */
@@ -387,7 +684,13 @@ var_sct *
 ncap_scv_var_mod(scv_sct scv,var_sct *var)
 {
   /* Purpose: Modulus of scv with each element of var */
+  if(var->undefined) return var;
+
   (void)ncap_var_scv_cnf_typ_hgh_prc(&var,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)scv_var_mod(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
   return var;
 } /* ncap_var_scv_mod */
@@ -398,8 +701,12 @@ ncap_var_scv_pwr(var_sct *var,scv_sct scv)
   /* Purpose: Empower each element in var by scv */
   /* Promote scv and var to NC_FLOAT if necessary since C has no integer empowerment 
      This reduces type conversion warnings (it is not done to avoid overflow) */
-  if(var->type < NC_FLOAT) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
+  if(nco_rth_prc_rnk(var->type) < nco_rth_prc_rnk_float) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
   (void)nco_scv_cnf_typ(var->type,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)var_scv_pwr(var->type,var->sz,var->has_mss_val,var->mss_val,var->val,&scv);
   return var;
 } /* end ncap_var_scv_pwr */
@@ -410,8 +717,14 @@ ncap_scv_var_pwr(scv_sct scv,var_sct *var)
   /* Purpose: Empower each element in scv by var */
   /* Promote scv and var to NC_FLOAT if necessary since C has no integer empowerment 
      This reduces type conversion warnings (it is not done to avoid overflow) */
-  if(var->type < NC_FLOAT) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
+  if(var->undefined) return var;
+
+  if(nco_rth_prc_rnk(var->type) < nco_rth_prc_rnk_float) var=nco_var_cnf_typ((nc_type)NC_FLOAT,var);
   (void)nco_scv_cnf_typ(var->type,&scv);
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)scv_var_pwr(var->type,var->sz,var->has_mss_val,var->mss_val,&scv,var->val);
   return var;
 } /* end ncap_var_scv_pwr */
@@ -420,6 +733,11 @@ var_sct *
 ncap_var_abs(var_sct *var)
 {
   /* Purpose: Find absolute value of each element of var */
+  if(var->undefined) return var;
+
+  /* deal with inital scan */
+  if(var->val.vp==NULL) return var; 
+
   (void)nco_var_abs(var->type,var->sz,var->has_mss_val,var->mss_val,var->val);
   return var;
 } /* end ncap_var_abs */
@@ -436,41 +754,21 @@ ncap_scv_clc
      __GNUC__ : Defined by gcc 
      __GNUG__ : Defined by g++, equivalent to (__GNUC__ && __cplusplus) */
 
-  /* #ifndef __GNUG__ */
+#ifndef __GNUG__ 
   extern float fmodf(float,float); /* Cannot insert fmodf in ncap_sym_init() because it takes two arguments TODO #20 */
   extern float fabsf(float); /* Sun math.h does not include fabsf() prototype */
-  /* #endif */ /* __GNUG__ */
+#endif   /* __GNUG__ */
   
   scv_sct scv_out;
   scv_out.type=scv_1.type;
+
+  /*
+  if(scv_1.undefined || scv_2.undefined ) {
+    scv_out.undefined=True;
+    return scv_out;
+  }
+  */
   switch(scv_out.type){ 
-  case NC_BYTE:
-    switch(op){
-    case '+': scv_out.val.b=scv_1.val.b+scv_2.val.b;break;
-    case '-': scv_out.val.b=scv_1.val.b-scv_2.val.b;break;
-    case '/': scv_out.val.b=scv_1.val.b/scv_2.val.b;break;
-    case '*': scv_out.val.b=scv_1.val.b*scv_2.val.b;break;
-    case '%': scv_out.val.b=scv_1.val.b%scv_2.val.b;break;
-    } break;
-  case NC_CHAR:
-    /* Do nothing */
-    break;
-  case NC_SHORT:
-    switch(op){
-    case '+': scv_out.val.s=scv_1.val.s+scv_2.val.s;break;
-    case '-': scv_out.val.s=scv_1.val.s-scv_2.val.s;break;
-    case '/': scv_out.val.s=scv_1.val.s/scv_2.val.s;break;
-    case '*': scv_out.val.s=scv_1.val.s*scv_2.val.s;break;
-    case '%': scv_out.val.s=scv_1.val.s%scv_2.val.s;break;
-    } break;
-  case NC_INT:
-    switch(op){
-    case '+': scv_out.val.l=scv_1.val.l+scv_2.val.l;break;
-    case '-': scv_out.val.l=scv_1.val.l-scv_2.val.l;break;
-    case '/': scv_out.val.l=scv_1.val.l/scv_2.val.l;break;
-    case '*': scv_out.val.l=scv_1.val.l*scv_2.val.l;break;
-    case '%': scv_out.val.l=scv_1.val.l%scv_2.val.l;break;
-    } break;
   case NC_FLOAT:
     switch(op){
     case '+': scv_out.val.f=scv_1.val.f+scv_2.val.f;break;
@@ -487,6 +785,72 @@ ncap_scv_clc
     case '*': scv_out.val.d=scv_1.val.d*scv_2.val.d;break;
     case '%': scv_out.val.d=fmod(scv_1.val.d,fabs(scv_2.val.d));break;
     } break;
+  case NC_INT:
+    switch(op){
+    case '+': scv_out.val.l=scv_1.val.l+scv_2.val.l;break;
+    case '-': scv_out.val.l=scv_1.val.l-scv_2.val.l;break;
+    case '/': scv_out.val.l=scv_1.val.l/scv_2.val.l;break;
+    case '*': scv_out.val.l=scv_1.val.l*scv_2.val.l;break;
+    case '%': scv_out.val.l=scv_1.val.l%scv_2.val.l;break;
+    } break;
+  case NC_SHORT:
+    switch(op){
+    case '+': scv_out.val.s=scv_1.val.s+scv_2.val.s;break;
+    case '-': scv_out.val.s=scv_1.val.s-scv_2.val.s;break;
+    case '/': scv_out.val.s=scv_1.val.s/scv_2.val.s;break;
+    case '*': scv_out.val.s=scv_1.val.s*scv_2.val.s;break;
+    case '%': scv_out.val.s=scv_1.val.s%scv_2.val.s;break;
+    } break;
+  case NC_USHORT:
+    switch(op){
+    case '+': scv_out.val.us=scv_1.val.us+scv_2.val.us;break;
+    case '-': scv_out.val.us=scv_1.val.us-scv_2.val.us;break;
+    case '/': scv_out.val.us=scv_1.val.us/scv_2.val.us;break;
+    case '*': scv_out.val.us=scv_1.val.us*scv_2.val.us;break;
+    case '%': scv_out.val.us=scv_1.val.us%scv_2.val.us;break;
+    } break;
+  case NC_UINT:
+    switch(op){
+    case '+': scv_out.val.ui=scv_1.val.ui+scv_2.val.ui;break;
+    case '-': scv_out.val.ui=scv_1.val.ui-scv_2.val.ui;break;
+    case '/': scv_out.val.ui=scv_1.val.ui/scv_2.val.ui;break;
+    case '*': scv_out.val.ui=scv_1.val.ui*scv_2.val.ui;break;
+    case '%': scv_out.val.ui=scv_1.val.ui%scv_2.val.ui;break;
+    } break;
+  case NC_INT64:
+    switch(op){
+    case '+': scv_out.val.i64=scv_1.val.i64+scv_2.val.i64;break;
+    case '-': scv_out.val.i64=scv_1.val.i64-scv_2.val.i64;break;
+    case '/': scv_out.val.i64=scv_1.val.i64/scv_2.val.i64;break;
+    case '*': scv_out.val.i64=scv_1.val.i64*scv_2.val.i64;break;
+    case '%': scv_out.val.i64=scv_1.val.i64%scv_2.val.i64;break;
+    } break;
+  case NC_UINT64:
+    switch(op){
+    case '+': scv_out.val.ui64=scv_1.val.ui64+scv_2.val.ui64;break;
+    case '-': scv_out.val.ui64=scv_1.val.ui64-scv_2.val.ui64;break;
+    case '/': scv_out.val.ui64=scv_1.val.ui64/scv_2.val.ui64;break;
+    case '*': scv_out.val.ui64=scv_1.val.ui64*scv_2.val.ui64;break;
+    case '%': scv_out.val.ui64=scv_1.val.ui64%scv_2.val.ui64;break;
+    } break;
+  case NC_BYTE:
+    switch(op){
+    case '+': scv_out.val.b=scv_1.val.b+scv_2.val.b;break;
+    case '-': scv_out.val.b=scv_1.val.b-scv_2.val.b;break;
+    case '/': scv_out.val.b=scv_1.val.b/scv_2.val.b;break;
+    case '*': scv_out.val.b=scv_1.val.b*scv_2.val.b;break;
+    case '%': scv_out.val.b=scv_1.val.b%scv_2.val.b;break;
+    } break;
+  case NC_UBYTE:
+    switch(op){
+    case '+': scv_out.val.ub=scv_1.val.ub+scv_2.val.ub;break;
+    case '-': scv_out.val.ub=scv_1.val.ub-scv_2.val.ub;break;
+    case '/': scv_out.val.ub=scv_1.val.ub/scv_2.val.ub;break;
+    case '*': scv_out.val.ub=scv_1.val.ub*scv_2.val.ub;break;
+    case '%': scv_out.val.ub=scv_1.val.ub%scv_2.val.ub;break;
+    } break;
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   }/* end switch */    
   
@@ -497,29 +861,27 @@ scv_sct
 ncap_scv_abs(scv_sct scv)
 {
   /* Purpose: Find the absolute value of a scalar value */
+
+#ifndef __GNUG__ 
   extern float fabsf(float); /* Sun math.h does not include fabsf() prototype */
+#endif   
   
   scv_sct scv_out;
   scv_out.type=scv.type;
+  
   switch(scv.type){ 
-  case NC_BYTE:
-    scv_out.val.b=((scv.val.b >= 0) ? scv.val.b : -scv.val.b);
-    break;
-  case NC_CHAR:
-    /* Do nothing */
-    break;
-  case NC_SHORT:
-    scv_out.val.s=((scv.val.s >= 0) ? scv.val.s : -scv.val.s);
-    break;
-  case NC_INT:
-    scv_out.val.l=labs(scv.val.l); /* int abs(int), long labs(long) */
-    break;            
-  case NC_FLOAT:
-    scv_out.val.f=fabsf(scv.val.f);
-    break;
-  case NC_DOUBLE:
-    scv_out.val.d=fabs(scv.val.d);
-    break;
+  case NC_FLOAT: scv_out.val.f=fabsf(scv.val.f); break;
+  case NC_DOUBLE: scv_out.val.d=fabs(scv.val.d); break;
+  case NC_INT: scv_out.val.l=labs(scv.val.l); break; /* int abs(int), long labs(long) */ break;            
+  case NC_SHORT: scv_out.val.s=((scv.val.s >= 0) ? scv.val.s : -scv.val.s); break;
+  case NC_USHORT: scv_out.val.us=scv.val.us; break;
+  case NC_UINT: scv_out.val.ui=scv.val.ui; break;
+  case NC_INT64: scv_out.val.i64=llabs(scv.val.i64); break;
+  case NC_UINT64: scv_out.val.ui64=scv.val.ui64; break;
+  case NC_BYTE: scv_out.val.b=((scv.val.b >= 0) ? scv.val.b : -scv.val.b); break;
+  case NC_UBYTE: scv_out.val.ub=scv.val.ub; break;
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;    
   } /* end switch */
   return scv_out;
@@ -528,25 +890,23 @@ ncap_scv_abs(scv_sct scv)
 int 
 ncap_scv_minus(scv_sct *scv)
 {
+  /* Purpose: Return negative of input */
   switch(scv->type){ 
-  case NC_BYTE:
-    scv->val.b=-scv->val.b; 
-    break;
-  case NC_CHAR:
-    /* Do nothing */
-    break;
-  case NC_SHORT:
-    scv->val.s=-scv->val.s;
-    break;
-  case NC_INT:
-    scv->val.l=-scv->val.l;
-    break;            
-  case NC_FLOAT:
-    scv->val.f=-scv->val.f;
-    break;
-  case NC_DOUBLE:
-    scv->val.d=-scv->val.d;
+  case NC_FLOAT: scv->val.f=-scv->val.f; break;
+  case NC_DOUBLE: scv->val.d=-scv->val.d; break;
+  case NC_INT: scv->val.l=-scv->val.l; break;            
+  case NC_SHORT: scv->val.s=-scv->val.s; break;
+  case NC_INT64: scv->val.i64=-scv->val.i64; break;
+  case NC_BYTE: scv->val.b=-scv->val.b; break;
+  case NC_USHORT: /* NB: Unsigned */
+  case NC_UINT: /* NB: Unsigned */
+  case NC_UINT64: /* NB: Unsigned */
+  case NC_UBYTE: /* NB: Unsigned */
+    (void)fprintf(stdout,"%s: ERROR ncap_scr_minus() reports attempt to convert unsigned integer type to a negative number\n",prg_nm_get());
+    nco_exit(EXIT_FAILURE);
     break;
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;  
   } /* end switch */    
   return scv->type;
@@ -568,18 +928,6 @@ nco_var_lst_copy(nm_id_sct *xtr_lst,int lst_nbr)
   return xtr_new_lst;           
 } /* end nco_var_lst_copy() */
 
-nm_id_sct *
-nco_var_lst_free(nm_id_sct *xtr_lst,int lst_nbr)
-{
-  /* Purpose: free xtr_lst and return null pointer */
-  int idx;
-  
-  for(idx=0;idx<lst_nbr;idx++) nco_free(xtr_lst[idx].nm);
-  nco_free(xtr_lst);
-  
-  return NULL;
-} /* end nco_var_lst_free() */
-
 nm_id_sct *
 nco_var_lst_sub(nm_id_sct *xtr_lst,int *nbr_xtr,nm_id_sct *xtr_lst_b,int nbr_lst_b)
 {
@@ -588,7 +936,7 @@ nco_var_lst_sub(nm_id_sct *xtr_lst,int *nbr_xtr,nm_id_sct *xtr_lst_b,int nbr_lst
   int xtr_idx;
   int xtr_nbr_new=0;
   
-  bool match;
+  nco_bool match;
   
   nm_id_sct *xtr_new_lst=NULL;
   
@@ -603,6 +951,11 @@ nco_var_lst_sub(nm_id_sct *xtr_lst,int *nbr_xtr,nm_id_sct *xtr_lst_b,int nbr_lst
     xtr_new_lst[xtr_nbr_new].nm=(char *)strdup(xtr_lst[idx].nm);
     xtr_new_lst[xtr_nbr_new++].id=xtr_lst[idx].id;
   } /* end loop over idx */
+  /* realloc to actual size */
+  xtr_new_lst=(nm_id_sct*)nco_realloc(xtr_new_lst,xtr_nbr_new*sizeof(nm_id_sct)); 
+  /* free old list */
+  xtr_lst=nco_nm_id_lst_free(xtr_lst,*nbr_xtr);
+
   *nbr_xtr=xtr_nbr_new;
   return xtr_new_lst;     
 }/* end nco_var_lst_sub */
@@ -617,7 +970,7 @@ nco_var_lst_add(nm_id_sct *xtr_lst,int *nbr_xtr,nm_id_sct *xtr_lst_a,int nbr_lst
   
   nm_id_sct *xtr_new_lst;
   
-  bool match;
+  nco_bool match;
   
   nbr_xtr_crr=*nbr_xtr;
   if(nbr_xtr_crr > 0){
@@ -645,7 +998,7 @@ nco_var_lst_add(nm_id_sct *xtr_lst,int *nbr_xtr,nm_id_sct *xtr_lst_a,int nbr_lst
 } /* nco_var_lst_add */
 
 nm_id_sct *
-nco_var_lst_crd_make
+nco_nm_id_lst_crd_make
 (int nc_id,
  nm_id_sct *xtr_lst,
  int *nbr_xtr)
@@ -653,7 +1006,7 @@ nco_var_lst_crd_make
 	int nc_id: I netCDF file ID
 	nm_id_sct *xtr_lst: I/O current extraction list 
 	int *nbr_xtr: I/O number of variables in current extraction list: Overwritten by new list 
-	nm_id_sct nco_var_lst_crd_make: list of coordinate dimensions 
+	nm_id_sct nco_nm_id_lst_crd_make: list of coordinate dimensions 
      */
 {
   /* Purpose: Make list co-ordinate dimensions from list of ordinary and co-ordinate variables */
@@ -691,7 +1044,7 @@ nco_var_lst_crd_make
   
   *nbr_xtr=nbr_new_lst;
   return new_lst;
-} /* end nco_var_lst_crd_make() */
+} /* end nco_nm_id_lst_crd_make() */
 
 nm_id_sct * /* O [sct] List of dimensions associated with input variable list */
 nco_dmn_lst /* [fnc] Create list of all dimensions in file  */
@@ -701,7 +1054,7 @@ nco_dmn_lst /* [fnc] Create list of all dimensions in file  */
   int idx;
   int nbr_dmn_in;
   char dmn_nm[NC_MAX_NAME];
-  nm_id_sct  *dmn;
+  nm_id_sct *dmn;
   /* Get number of dimensions */
   (void)nco_inq(nc_id,&nbr_dmn_in,(int *)NULL,(int *)NULL,(int *)NULL);
   
@@ -740,6 +1093,7 @@ nco_att_lst_mk
       for(jdx=0;jdx<size;jdx++)
 	if(!strcmp(xtr_lst[jdx].nm,att_lst[idx]->var_nm)) break;
       if(jdx!=size) continue;
+      /* fxm mmr TODO 491: memory leak xtr_lst */
       xtr_lst=(nm_id_sct *)nco_realloc(xtr_lst,(size+1)*sizeof(nm_id_sct));
       xtr_lst[size].id=var_id;
       xtr_lst[size++].nm=(char *)strdup(att_lst[idx]->var_nm);
@@ -751,7 +1105,7 @@ nco_att_lst_mk
   return xtr_lst;
 } /* end nco_att_lst_mk() */
 
-bool /* O [flg] Variables now conform */
+nco_bool /* O [flg] Variables now conform */
 ncap_var_stretch /* [fnc] Stretch variables */
 (var_sct **var_1, /* I/O [ptr] First variable */
  var_sct **var_2) /* I/O [ptr] Second variable */
@@ -766,7 +1120,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
      Terminology--- 
      Broadcast: Inflate smaller conforming variable to larger variable
      Conform: Dimensions of one variable are subset of other variable
-     Convolve: Construct array which is product of ranks of two variables
+     Convolve: Construct array whose rank is sum of non-duplicated dimensions
      Stretch: Union of broadcast and convolve
      
      Logic is pared down version of nco_var_cnf_dmn()
@@ -788,10 +1142,10 @@ ncap_var_stretch /* [fnc] Stretch variables */
      var_gtr_out is required since both variables may change
      var_gtr_out=var_gtr unless convolution is required */
   
-  bool CONFORMABLE=False; /* [flg] Whether var_lsr can be made to conform to var_gtr */
-  bool CONVOLVE=False; /* [flg] var_1 and var_2 had to be convolved */
-  bool DO_CONFORM; /* [flg] Did var_1 and var_2 conform? */
-  bool MUST_CONFORM=False; /* [flg] Must var_1 and var_2 conform? */
+  nco_bool CONFORMABLE=False; /* [flg] Whether var_lsr can be made to conform to var_gtr */
+  nco_bool CONVOLVE=False; /* [flg] var_1 and var_2 had to be convolved */
+  nco_bool DO_CONFORM; /* [flg] Did var_1 and var_2 conform? */
+  nco_bool MUST_CONFORM=False; /* [flg] Must var_1 and var_2 conform? */
   
   int idx;
   int idx_dmn;
@@ -924,9 +1278,14 @@ ncap_var_stretch /* [fnc] Stretch variables */
     (void)nco_xrf_var(var_lsr,var_lsr_out);
     
     /* Modify elements of lesser variable array */
-    var_lsr_out->nm=var_lsr->nm;
+    var_lsr_out->nm=(char *)nco_free(var_lsr_out->nm);
+    var_lsr_out->nm=(char *)strdup(var_lsr->nm);
     var_lsr_out->id=var_lsr->id;
     var_lsr_out->type=var_lsr->type;
+    /* Added 20050323: 
+       Not quite sure why, but var->val.vp may already have values here when LHS-casting
+       Perform safety free to guard against memory leaks */
+    var_lsr_out->val.vp=nco_free(var_lsr_out->val.vp);
     var_lsr_out->val.vp=(void *)nco_malloc_dbg(var_lsr_out->sz*nco_typ_lng(var_lsr_out->type),"Unable to malloc() value buffer in variable stretching",fnc_nm);
     var_lsr_cp=(char *)var_lsr->val.vp;
     var_lsr_out_cp=(char *)var_lsr_out->val.vp;
@@ -935,6 +1294,15 @@ ncap_var_stretch /* [fnc] Stretch variables */
     if(var_lsr_out->nbr_dim == 0){
       /* Variables are scalars, not arrays */
       (void)memcpy(var_lsr_out_cp,var_lsr_cp,var_lsr_typ_sz);
+    }else if(var_lsr->nbr_dim == 0){
+      /* Lesser-ranked input variable is scalar 
+	 Expansion in this degenerate case needs no index juggling (reverse-mapping)
+	 Code as special case to speed-up important applications of ncap
+	 for synthetic file creation */
+      var_gtr_sz=var_gtr->sz;
+      for(var_gtr_lmn=0;var_gtr_lmn<var_gtr_sz;var_gtr_lmn++){
+	(void)memcpy(var_lsr_out_cp+var_gtr_lmn*var_lsr_typ_sz,var_lsr_cp,var_lsr_typ_sz);
+      } /* end loop over var_gtr_lmn */
     }else{
       /* Variables are arrays, not scalars */
       
@@ -956,9 +1324,7 @@ ncap_var_stretch /* [fnc] Stretch variables */
       Since mapping arrays (dmn_var_gtr_map and dmn_var_lsr_map) are ultimately used for a
       memcpy() operation, they could (read: should) be computed as byte offsets, not type offsets.
       This is why netCDF generic hyperslab routines (ncvarputg(), ncvargetg())
-      request imap vector to specify offset (imap) vector in bytes.
-      */
-      
+      request imap vector to specify offset (imap) vector in bytes. */
       for(idx=0;idx<var_lsr->nbr_dim;idx++){
 	for(idx_dmn=0;idx_dmn<var_gtr->nbr_dim;idx_dmn++){
 	  /* Compare names, not dimension IDs */
