@@ -1,20 +1,20 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.72 2004/08/16 04:13:33 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.107 2007/05/25 17:36:46 zender Exp $ */
 
 /* ncatted -- netCDF attribute editor */
 
 /* Purpose: Add, create, delete, or overwrite attributes in a netCDF file */
 
-/* Copyright (C) 1995--2004 Charlie Zender
+/* Copyright (C) 1995--2007 Charlie Zender
 
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   The full license text is at http://www.gnu.ai.mit.edu/copyleft/gpl.html 
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   The full license text is at http://www.gnu.org/copyleft/gpl.html 
    and in the file nco/doc/LICENSE in the NCO source distribution.
    
    As a special exception to the terms of the GPL, you are permitted 
-   to link the NCO source code with the DODS, HDF, netCDF, and UDUnits
+   to link the NCO source code with the HDF, netCDF, OPeNDAP, and UDUnits
    libraries and to distribute the resulting executables under the terms 
    of the GPL, but in addition obeying the extra stipulations of the 
-   DODS, HDF, netCDF, and UDUnits licenses.
+   HDF, netCDF, OPeNDAP, and UDUnits licenses.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,73 +23,73 @@
    
    The original author of this software, Charlie Zender, wants to improve it
    with the help of your suggestions, improvements, bug-reports, and patches.
-   Please contact the NCO project at http://nco.sf.net or by writing
+   Please contact the NCO project at http://nco.sf.net or write to
    Charlie Zender
    Department of Earth System Science
-   University of California at Irvine
+   University of California, Irvine
    Irvine, CA 92697-3100 */
 
 /* Usage:
 
    Use C language escape sequences:
-   ncatted -D 3 -h -a history,global,o,c,"String\tformatting\tfor\nDennis" in.nc ; ncks -M in.nc
-   ncatted -D 3 -h -a history,global,o,c,'\a\b\f\n\r\t\v\\\?\0' in.nc ; ncks -M in.nc
-   ncatted -D 3 -h -a history,global,o,c,'Characters which require protection by backslash:\nDouble quote: \"\nTwo consecutive double quotes: \"\"\nSingle quote: Beyond my shell abilities!\nBackslash: \\\nTwo consecutive backslashes: \\\\\nQuestion mark: \?\n' in.nc ; ncks -M in.nc
-   ncatted -D 3 -h -a history,global,o,c,'Characters which do not require protection by backslash:\nSingle backquote: `\nDollarsign: $\nLeft brace: {\nRight brace: }\nPipe: |\nAmpersand: &\nAt sign: @\nPercent: %\n\n' in.nc ; ncks -M in.nc
+   ncatted -D 3 -h -a history,global,o,c,"String\tformatting\tfor\nDennis" ~/nco/data/in.nc ~/foo.nc
+   ncatted -D 3 -h -a history,global,o,c,'\a\b\f\n\r\t\v\\\?\0' ~/nco/data/in.nc ~/foo.nc
+   ncatted -D 3 -h -a history,global,o,c,'Characters which require protection by backslash:\nDouble quote: \"\nTwo consecutive double quotes: \"\"\nSingle quote: Beyond my shell abilities!\nBackslash: \\\nTwo consecutive backslashes: \\\\\nQuestion mark: \?\n' ~/nco/data/in.nc ~/foo.nc
+   ncatted -D 3 -h -a history,global,o,c,'Characters which do not require protection by backslash:\nSingle backquote: `\nDollarsign: $\nLeft brace: {\nRight brace: }\nPipe: |\nAmpersand: &\nAt sign: @\nPercent: %\n\n' ~/nco/data/in.nc ~/foo.nc
 
    Append to existing string:
-   ncatted -D 5 -O -a char_att,att_var,a,c,"and appended Sentence three." in.nc foo.nc
+   ncatted -D 5 -O -a char_att,att_var,a,c,"and appended Sentence three." ~/nco/data/in.nc ~/foo.nc
 
    Append to existing string with internal delimiter characters (commas):
-   ncatted -D 5 -O -a char_att,att_var,a,c,"appended a comma, and three more commas,,," in.nc foo.nc
+   ncatted -D 5 -O -a char_att,att_var,a,c,"appended a comma, and three more commas,,," ~/nco/data/in.nc ~/foo.nc
 
    Append to existing float:
-   ncatted -D 5 -O -a float_att,att_var,a,f,74 in.nc foo.nc
-   ncatted -D 5 -O -a float_att,att_var,a,f,74,75,76 in.nc foo.nc
+   ncatted -D 5 -O -a float_att,att_var,a,f,74 ~/nco/data/in.nc ~/foo.nc
+   ncatted -D 5 -O -a float_att,att_var,a,f,74,75,76 ~/nco/data/in.nc ~/foo.nc
 
    Create new float:
-   ncatted -D 5 -O -a new_float_att,att_var,c,f,74 in.nc foo.nc
+   ncatted -D 5 -O -a new_float_att,att_var,c,f,74 ~/nco/data/in.nc ~/foo.nc
+
+   Create new netCDF4 atomic type attributes:
+   ncatted -D 5 -O -a new_ubyte_att,att_var,c,ub,0 -a new_ushort_att,att_var,c,us,74 -a new_uint_att,att_var,c,u,71,72,73,74 -a 'new_int64_att,att_var,c,ll,-74' -a new_uint64_att,att_var,c,ull,74,75 ~/nco/data/in_4.nc ~/foo.nc
 
    Delete attribute:
-   ncatted -D 5 -O -a float_att,att_var,d,,, in.nc foo.nc
+   ncatted -D 5 -O -a float_att,att_var,d,,, ~/nco/data/in.nc ~/foo.nc
 
    Delete all attributes for given var:
-   ncatted -D 5 -O -a ,att_var,d,,, in.nc foo.nc
+   ncatted -D 5 -O -a ,att_var,d,,, ~/nco/data/in.nc ~/foo.nc
 
    Modify existing float:
-   ncatted -D 5 -O -a float_att,att_var,m,f,74 in.nc foo.nc
+   ncatted -D 5 -O -a float_att,att_var,m,f,74 ~/nco/data/in.nc ~/foo.nc
 
    Modify existing missing value attribute:
-   ncatted -D 5 -O -a missing_value,mss_val,m,f,74 in.nc foo.nc
+   ncatted -D 5 -O -a missing_value,mss_val,m,f,74 ~/nco/data/in.nc ~/foo.nc
    
    Multiple attribute edits:
-   ncatted -D 5 -O -a char_att,att_var,a,c,"and appended Sentence three." -a short_att,att_var,c,s,37,38,39 -a float_att,att_var,d,,, -a long_att,att_var,o,l,37 -a new_att,att_var,o,d,73,74,75 in.nc foo.nc
+   ncatted -D 5 -O -a char_att,att_var,a,c,"and appended Sentence three." -a short_att,att_var,c,s,37,38,39 -a float_att,att_var,d,,, -a long_att,att_var,o,l,37 -a new_att,att_var,o,d,73,74,75 ~/nco/data/in.nc ~/foo.nc
 
    Create global attribute:
-   ncatted -D 5 -O -a float_att,global,c,f,74 in.nc foo.nc
-
-   Verify results:
-   ncks -C -H -v att_var foo.nc
+   ncatted -D 5 -O -a float_att,global,c,f,74 ~/nco/data/in.nc ~/foo.nc
 
    Test algorithm for all variables:
    Append to existing string for all variables:
-   ncatted -D 5 -O -a char_att,,a,c,"and appended Sentence three." in.nc foo.nc
+   ncatted -D 5 -O -a char_att,,a,c,"and appended Sentence three." ~/nco/data/in.nc ~/foo.nc
 
    Append to existing float for all variables:
-   ncatted -D 5 -O -a float_att,,a,f,74 in.nc foo.nc
-   ncatted -D 5 -O -a float_att,,a,f,74,75,76 in.nc foo.nc
+   ncatted -D 5 -O -a float_att,,a,f,74 ~/nco/data/in.nc ~/foo.nc
+   ncatted -D 5 -O -a float_att,,a,f,74,75,76 ~/nco/data/in.nc ~/foo.nc
 
    Create new float for all variables:
-   ncatted -D 5 -O -a float_att,,c,f,74 in.nc foo.nc
+   ncatted -D 5 -O -a float_att,,c,f,74 ~/nco/data/in.nc ~/foo.nc
 
    Delete attribute for all variables:
-   ncatted -D 5 -O -a float_att,,d,,, in.nc foo.nc
+   ncatted -D 5 -O -a float_att,,d,,, ~/nco/data/in.nc ~/foo.nc
 
    Modify existing float for all variables:
-   ncatted -D 5 -O -a float_att,,m,f,74 in.nc foo.nc
+   ncatted -D 5 -O -a float_att,,m,f,74 ~/nco/data/in.nc ~/foo.nc
 
    Verify results:
-   ncks -C -h foo.nc | m */
+   ncks -C -h ~/foo.nc | m */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h> /* Autotools tokens */
@@ -125,45 +125,64 @@ main(int argc,char **argv)
 {
   aed_sct *aed_lst=NULL_CEWI;
 
-  bool OUTPUT_TO_NEW_NETCDF_FILE=False;
-  bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
-  bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
-  bool FORCE_APPEND=False; /* Option A */
-  bool FORCE_OVERWRITE=False; /* Option O */
-  bool HISTORY_APPEND=True; /* Option h */
-  bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
+  nco_bool FILE_RETRIEVED_FROM_REMOTE_LOCATION;
+  nco_bool FL_LST_IN_FROM_STDIN=False; /* [flg] fl_lst_in comes from stdin */
+  nco_bool FORCE_APPEND=False; /* Option A */
+  nco_bool FORCE_OVERWRITE=False; /* Option O */
+  nco_bool HISTORY_APPEND=True; /* Option h */
+  nco_bool OUTPUT_TO_NEW_NETCDF_FILE=False;
+  nco_bool REMOVE_REMOTE_FILES_AFTER_PROCESSING=True; /* Option R */
+  nco_bool flg_cln=False; /* [flg] Clean memory prior to exit */
   
   char **fl_lst_abb=NULL; /* Option n */
   char **fl_lst_in;
-  char *fl_in=NULL;
   char *aed_arg[NC_MAX_ATTRS];
-  char *fl_out=NULL; /* Option o */
-  char *fl_pth_lcl=NULL; /* Option l */
-  char *fl_pth=NULL; /* Option p */
-  char *time_bfr_srt;
   char *cmd_ln;
+  char *fl_in=NULL;
+  char *fl_out=NULL; /* Option o */
+  char *fl_pth=NULL; /* Option p */
+  char *fl_pth_lcl=NULL; /* Option l */
+  char *opt_crr=NULL; /* [sng] String representation of current long-option name */
 
-  const char * const CVS_Id="$Id: ncatted.c,v 1.72 2004/08/16 04:13:33 zender Exp $"; 
-  const char * const CVS_Revision="$Revision: 1.72 $";
-  const char * const opt_sng="Aa:D:hl:Oo:p:Rr-:";
+  const char * const CVS_Id="$Id: ncatted.c,v 1.107 2007/05/25 17:36:46 zender Exp $"; 
+  const char * const CVS_Revision="$Revision: 1.107 $";
+  const char * const opt_sht_lst="Aa:D:hl:Oo:p:Rr-:";
   
+#if defined(__cplusplus) || defined(PGI_CC)
+  ddra_info_sct ddra_info;
+#else /* !__cplusplus */
+  ddra_info_sct ddra_info={.flg_ddra=False};
+#endif /* !__cplusplus */
+
   extern char *optarg;
   extern int optind;
   
+  int abb_arg_nbr=0;
+  int fl_nbr=0;
   int idx;
   int idx_var;
-  int nc_id;  
-  int abb_arg_nbr=0;
-  int nbr_var_fl;
   int nbr_aed=0; /* Option a. NB: nbr_var_aed gets incremented */
-  int fl_nbr=0;
+  int nbr_var_fl;
+  int nc_id;  
+  int ncopen_mode=NC_WRITE; /* [enm] Mode flag for nco_open() call */
   int opt;
   int rcd=NC_NOERR; /* [rcd] Return code */
 
-  time_t time_crr_time_t;
+  size_t hdr_pad=0UL; /* [B] Pad at end of header section */
 
   static struct option opt_lng[]=
     { /* Structure ordered by short option key if possible */
+      /* Long options with no argument, no short option counterpart */
+      {"cln",no_argument,0,0}, /* [flg] Clean memory prior to exit */
+      {"clean",no_argument,0,0}, /* [flg] Clean memory prior to exit */
+      {"mmr_cln",no_argument,0,0}, /* [flg] Clean memory prior to exit */
+      {"drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"dirty",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      {"mmr_drt",no_argument,0,0}, /* [flg] Allow dirty memory on exit */
+      /* Long options with argument, no short option counterpart */
+      {"hdr_pad",required_argument,0,0},
+      {"header_pad",required_argument,0,0},
+      /* Long options with short counterparts */
       {"append",no_argument,0,'A'},
       {"attribute",required_argument,0,'a'},
       {"debug",required_argument,0,'D'},
@@ -186,22 +205,38 @@ main(int argc,char **argv)
     }; /* end opt_lng */
   int opt_idx=0; /* Index of current long option into opt_lng array */
 
-  /* Start clock and save command line */ 
+  /* Start timer and save command line */ 
+  ddra_info.tmr_flg=nco_tmr_srt;
+  rcd+=nco_ddra((char *)NULL,(char *)NULL,&ddra_info);
+  ddra_info.tmr_flg=nco_tmr_mtd;
   cmd_ln=nco_cmd_ln_sng(argc,argv);
-  time_crr_time_t=time((time_t *)NULL);
-  time_bfr_srt=ctime(&time_crr_time_t); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   
   /* Get program name and set program enum (e.g., prg=ncra) */
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
+  while(1){
+    /* getopt_long_only() allows one dash to prefix long options */
+    opt=getopt_long(argc,argv,opt_sht_lst,opt_lng,&opt_idx);
+    /* NB: access to opt_crr is only valid when long_opt is detected */
+    if(opt == EOF) break; /* Parse positional arguments once getopt_long() returns EOF */
+    opt_crr=(char *)strdup(opt_lng[opt_idx].name);
+
+    /* Process long options without short option counterparts */
+    if(opt == 0){
+      if(!strcmp(opt_crr,"cln") || !strcmp(opt_crr,"mmr_cln") || !strcmp(opt_crr,"clean")) flg_cln=True; /* [flg] Clean memory prior to exit */
+      if(!strcmp(opt_crr,"drt") || !strcmp(opt_crr,"mmr_drt") || !strcmp(opt_crr,"dirty")) flg_cln=False; /* [flg] Clean memory prior to exit */
+      if(!strcmp(opt_crr,"hdr_pad") || !strcmp(opt_crr,"header_pad")) hdr_pad=strtoul(optarg,(char **)NULL,10);
+    } /* opt != 0 */
+    /* Process short options */
     switch(opt){
+    case 0: /* Long options have already been processed, return */
+      break;
     case 'A': /* Toggle FORCE_APPEND */
       FORCE_APPEND=!FORCE_APPEND;
       break;
     case 'a': /* Copy argument for later processing */
-      aed_arg[nbr_aed]=optarg;
+      aed_arg[nbr_aed]=(char *)strdup(optarg);
       nbr_aed++;
       break;
     case 'D': /* Debugging level. Default is 0. */
@@ -211,7 +246,7 @@ main(int argc,char **argv)
       HISTORY_APPEND=!HISTORY_APPEND;
       break;
     case 'l': /* Local path prefix for files retrieved from remote file system */
-      fl_pth_lcl=optarg;
+      fl_pth_lcl=(char *)strdup(optarg);
       break;
     case 'O': /* Toggle FORCE_OVERWRITE */
       FORCE_OVERWRITE=!FORCE_OVERWRITE;
@@ -220,7 +255,7 @@ main(int argc,char **argv)
       fl_out=(char *)strdup(optarg);
       break;
     case 'p': /* Common file path */
-      fl_pth=optarg;
+      fl_pth=(char *)strdup(optarg);
       break;
     case 'R': /* Toggle removal of remotely-retrieved-files. Default is True. */
       REMOVE_REMOTE_FILES_AFTER_PROCESSING=!REMOVE_REMOTE_FILES_AFTER_PROCESSING;
@@ -242,11 +277,12 @@ main(int argc,char **argv)
       (void)nco_usg_prn();
       nco_exit(EXIT_FAILURE);
     } /* end switch */
+    if(opt_crr != NULL) opt_crr=(char *)nco_free(opt_crr);
   } /* end while loop */
   
   /* Process positional arguments and fill in filenames */
   fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&fl_nbr,&fl_out,&FL_LST_IN_FROM_STDIN);
-  if(fl_out != NULL) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=fl_lst_in[0];
+  if(fl_out != NULL) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=(char *)strdup(fl_lst_in[0]);
 
   if(nbr_aed == 0){
     (void)fprintf(stdout,"%s: ERROR must specify an attribute to edit\n",prg_nm);
@@ -278,7 +314,7 @@ main(int argc,char **argv)
         
         usr_reply='z';
         while(usr_reply != 'n' && usr_reply != 'y'){
-          (void)fprintf(stdout,"ncrename: overwrite %s (y/n)? ",fl_out);
+          (void)fprintf(stdout,"%s: overwrite %s (y/n)? ",prg_nm_get(),fl_out);
 	  (void)fflush(stdout);
           usr_reply=(char)fgetc(stdin);
         } /* end while */
@@ -295,12 +331,17 @@ main(int argc,char **argv)
   } /* end if */
 
   /* Open file. Writing must be enabled and file should be in define mode for renaming */
-  rcd=nco_open(fl_out,NC_WRITE,&nc_id);
+  /*  if(dbg_lvl == 8) ncopen_mode|=NC_SHARE;*/
+  rcd=nco_open(fl_out,ncopen_mode,&nc_id);
   (void)nco_redef(nc_id);
 
   /* Get number of variables in file */
   (void)nco_inq(nc_id,(int *)NULL,&nbr_var_fl,(int *)NULL,(int *)NULL);
 
+  /* Timestamp end of metadata setup and disk layout */
+  rcd+=nco_ddra((char *)NULL,(char *)NULL,&ddra_info);
+  ddra_info.tmr_flg=nco_tmr_rgl;
+
   for(idx=0;idx<nbr_aed;idx++){
 
     if(aed_lst[idx].var_nm != NULL){
@@ -325,8 +366,13 @@ main(int argc,char **argv)
   /* Catenate the timestamped command line to the "history" global attribute */
   if(HISTORY_APPEND) (void)nco_hst_att_cat(nc_id,cmd_ln);
   
-  /* Take file out of define mode */
-  (void)nco_enddef(nc_id);
+  /* Take output file out of define mode */
+  if(hdr_pad == 0UL){
+    (void)nco_enddef(nc_id);
+  }else{
+    (void)nco__enddef(nc_id,hdr_pad);
+    if(dbg_lvl >= nco_dbg_scl) (void)fprintf(stderr,"%s: INFO Padding header with %lu extra bytes \n",prg_nm_get(),(unsigned long)hdr_pad);
+  } /* hdr_pad */
     
   /* Close the open netCDF file */
   nco_close(nc_id);
@@ -334,6 +380,34 @@ main(int argc,char **argv)
   /* Remove local copy of file */
   if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
 
+  /* Clean memory unless dirty memory allowed */
+  if(flg_cln){
+    /* ncatted-specific memory */
+    for(idx=0;idx<nbr_aed;idx++) aed_arg[idx]=(char *)nco_free(aed_arg[idx]);
+    for(idx=0;idx<nbr_aed;idx++){
+      if(aed_lst[idx].att_nm != NULL) aed_lst[idx].att_nm=(char *)nco_free(aed_lst[idx].att_nm);
+      if(aed_lst[idx].var_nm != NULL) aed_lst[idx].var_nm=(char *)nco_free(aed_lst[idx].var_nm);
+      aed_lst[idx].val.vp=(void *)nco_free(aed_lst[idx].val.vp);
+    } /* end for */
+    if(nbr_aed > 0) aed_lst=(aed_sct *)nco_free(aed_lst);
+    
+    /* NCO-generic clean-up */
+    /* Free individual strings/arrays */
+    if(cmd_ln != NULL) cmd_ln=(char *)nco_free(cmd_ln);
+    if(fl_in != NULL) fl_in=(char *)nco_free(fl_in);
+    if(fl_out != NULL) fl_out=(char *)nco_free(fl_out);
+    if(fl_pth != NULL) fl_pth=(char *)nco_free(fl_pth);
+    if(fl_pth_lcl != NULL) fl_pth_lcl=(char *)nco_free(fl_pth_lcl);
+    /* Free lists of strings */
+    if(fl_lst_in != NULL && fl_lst_abb == NULL) fl_lst_in=nco_sng_lst_free(fl_lst_in,fl_nbr); 
+    if(fl_lst_in != NULL && fl_lst_abb != NULL) fl_lst_in=nco_sng_lst_free(fl_lst_in,1);
+    if(fl_lst_abb != NULL) fl_lst_abb=nco_sng_lst_free(fl_lst_abb,abb_arg_nbr);
+  } /* !flg_cln */
+  
+  /* End timer */ 
+  ddra_info.tmr_flg=nco_tmr_end; /* [enm] Timer flag */
+  rcd+=nco_ddra((char *)NULL,(char *)NULL,&ddra_info);
+
   if(rcd != NC_NOERR) nco_err_exit(rcd,"main");
   nco_exit_gracefully();
   return EXIT_SUCCESS;
