@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.67 2004/08/06 20:56:39 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco.h,v 1.116 2007/05/22 18:26:54 zender Exp $ */
 
 /* Purpose: netCDF Operator (NCO) definitions */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco.h" *//* netCDF Operator (NCO) definitions */
@@ -19,64 +19,91 @@
 #include <netcdf.h> /* netCDF definitions and C library */
 
 /* Personal headers */
+#include "nco_typ.h" /* Type definitions, opaque types */
+
+  /* C pre-processor macros for instantiating variable values with string tokens
+     Macros for token pasting described at http://www.parashift.com/c++-faq-lite
+     Layer of indirection is required, use public macro to call private macro */
+#define TKN2SNG_PRV(x) #x
+#define TKN2SNG(x) TKN2SNG_PRV(x)
 
 /* Encapsulate C++ const usage in C99-safe macro 
    C++ compilers will use type-safe version
    C89 and C99 compilers use less type-safe version that is standards-compliant */
-/* fxm: This is incomplete and needs to be extended and applied to all source */
 #ifdef __cplusplus
-#define CST_X_PTR_CST_PTR_CST_Y(x,y) const x * const * const y
-#define X_CST_PTR_CST_PTR_Y(x,y) x const * const * y
+# define CST_X_PTR_CST_PTR_CST_Y(x,y) const x * const * const y
+# define X_CST_PTR_CST_PTR_Y(x,y) x const * const * y
 #else /* !__cplusplus */
-#define CST_X_PTR_CST_PTR_CST_Y(x,y) x * const * const y
-#define X_CST_PTR_CST_PTR_Y(x,y) x * const * y
+# define CST_X_PTR_CST_PTR_CST_Y(x,y) x * const * const y
+# define X_CST_PTR_CST_PTR_Y(x,y) x * const * y
 #endif /* !__cplusplus */
 
 #ifdef __cplusplus
+
 /* Use C-bindings so C++-compiled and C-compiled libraries are compatible */
 extern "C" {
 #endif /* !__cplusplus */
 
-/* NCO uses native type nco_long to store variables of type NC_INT */
-  typedef long nco_long;
-  
-/* Uncomment next line to remove C99 restrict type qualifier from all source */
-/*#define restrict*/
+/* Uncomment next line to remove C99 restrict type-qualifier from all source */
+/* #define restrict */
 
 /* Replace restrict by __restrict in g++ compiler
    Substitute whitespace for __restrict in all other C++ compilers */
 #ifdef __cplusplus
-#ifdef __GNUG__
-#define restrict __restrict
-#else /* !__GNUG__ */
-#define restrict
-#endif /* !__GNUG__ */
+# ifdef __GNUG__
+#  define restrict __restrict
+# else /* !__GNUG__ */
+#  define restrict
+# endif /* !__GNUG__ */
 #endif /* !__cplusplus */
 
 /* Boolean values */
-#ifndef bool
-#define bool int
-#endif /* bool */
+/* From Wikipedia:
+   "On a recent C compiler (supporting the C99 standard), there is a _Bool type, which is used to define bool by the stdbool.h header: 
+
+   #include <stdbool.h>
+   bool b = false;
+   ...
+   b = true;
+
+   During its standardization process, the C++ programming language introduced the bool, true and false keywords, adding a native datatype to support boolean data.
+   Preprocessor macros may be used to turn bool into _Bool, false into 0 and true into 1, allowing compatibility with the aforementioned C99 use of the stdbool.h header." */
+#define nco_bool int
+#ifndef __cplusplus
+# ifndef bool
+#  define bool int
+# endif /* bool */
+#endif /* __cplusplus */
 #ifndef True
-#define True 1
+# define True 1
 #endif /* True */
 #ifndef False
-#define False 0
+# define False 0
 #endif /* False */
   
   /* Variables marked CEWI "Compiler Error Warning Initialization" are initialized
      to prevent spurious "warning: `float foo' might be used uninitialized in 
      this function" warnings when, e.g., GCC -Wuninitialized is turned on.
-     Note that these warning messages are compiler and OS dependent
-     GCC on Alpha, e.g., emits warnings which cannot be removed by this trick */
+     Note that these warning messages are compiler- and OS-dependent
+     GCC warning on Alpha, e.g., cannot be removed by this trick */
 #define NULL_CEWI NULL
-#define byte_CEWI 0
 #define char_CEWI '\0'
 #define double_CEWI 0.0
 #define float_CEWI 0.0
 #define int_CEWI 0
 #define long_CEWI 0L
-#define nco_long_CEWI 0L
+#define long_long_CEWI ((long long)0LL)
+#define nco_bool_CEWI 0
+#define nco_byte_CEWI ((signed char)0)
+#define nco_char_CEWI '\0'
+#define nco_int_CEWI (0L)
+#define nco_short_CEWI ((short)0)
+#define nco_ubyte_CEWI ((unsigned char)0)
+#define nco_ushort_CEWI ((unsigned short)0)
+#define nco_uint_CEWI (0U)
+#define nco_int64_CEWI ((long long)0LL)
+#define nco_uint64_CEWI ((unsigned long long)0ULL)
+#define nco_string_CEWI NULL
 #define short_CEWI 0
 #define size_t_CEWI 0UL
   
@@ -86,14 +113,20 @@ extern "C" {
      NB: nc_inq() family is defined to return -1 for missing record dimensions */
 #define NCO_REC_DMN_UNDEFINED -1
   
+  /* Debugging level that quiets all non-requested informational messages
+     This value is compared against user-selected dbg_lvl 
+     Running operators with --quiet automatically sets dbg_lvl=NCO_DBG_QUIET */
+#define NCO_DBG_QUIET 6
+
   /* Prototype global functions before defining them in next block */
+  char *nco_mss_val_sng_get(void); /* [sng] Missing value attribute name */
   char *prg_nm_get(void);
   int prg_get(void);
   unsigned short dbg_lvl_get(void);
 
 #ifdef MAIN_PROGRAM_FILE /* Current file contains main() */
   
-  /* Global variables and variables with scope limited to main.c allocated here */
+  /* Tokens and variables with scope limited to main.c, and global variables allocated here */
   
   int prg; /* [enm] Program ID */
   int prg_get(void){return prg;} /* [enm] Program ID */
@@ -104,6 +137,17 @@ extern "C" {
   unsigned short dbg_lvl=0; /* [enm] Debugging level */
   unsigned short dbg_lvl_get(void){return dbg_lvl;} /* [enm] Debugging level */
   
+  /* NCO_MSS_VAL_SNG contains the name of the attribute whose value is treated as
+     "missing" during arithmetic. Choose either missing_value of _FillValue. */
+#ifndef NCO_MSS_VAL_SNG
+# define NCO_MSS_VAL_SNG missing_value
+  /* # define NCO_MSS_VAL_SNG _FillValue */
+#endif /* NCO_MSS_VAL_SNG */
+
+  /* Unidata standard string for missing values */
+  char nco_mss_val_sng[]=TKN2SNG(NCO_MSS_VAL_SNG); /* [sng] Missing value attribute name */
+  char *nco_mss_val_sng_get(void){return nco_mss_val_sng;} /* [sng] Missing value attribute name */
+
 #else /* MAIN_PROGRAM_FILE is NOT defined, i.e., current file does not contain main() */
   
   /* External references to global variables are declared as extern here
@@ -111,11 +155,40 @@ extern "C" {
   
 #endif /* MAIN_PROGRAM_FILE is NOT defined, i.e., the current file does not contain main() */
   
+  /* Define compatibility tokens when user does not have netCDF4 */
+#ifndef ENABLE_NETCDF4
+/* Single compatibility token new to netCDF4 netcdf.h */
+# define NC_NETCDF4     (0x1000) /* Use netCDF-4/HDF5 format */
+/* Four compatibility tokens from netCDF3.6 netcdf.h */
+# ifndef NC_CLASSIC_MODEL
+#  define NC_CLASSIC_MODEL (0x0008) /* Enforce strict netcdf-3 rules. */
+# endif
+# ifndef NC_FORMAT_CLASSIC
+#  define NC_FORMAT_CLASSIC (1)
+# endif
+# ifndef NC_FORMAT_64BIT
+#  define NC_FORMAT_64BIT   (2)
+# endif
+# ifndef NC_FORMAT_NETCDF4
+#  define NC_FORMAT_NETCDF4 (3)
+# endif
+# ifndef NC_FORMAT_NETCDF4_CLASSIC
+#  define NC_FORMAT_NETCDF4_CLASSIC  (4) /* create netcdf-4 files, with NC_CLASSIC_MODEL. */
+# endif
+#endif /* !ENABLE_NETCDF4 */
+
+  /* Define compatibility tokens when user does not have znetCDF */
+#ifndef ENABLE_ZNETCDF
+# ifndef NC_COMPRESS
+#  define NC_COMPRESS 0x200 /* bn file is compressed */
+# endif
+#endif /* !ENABLE_ZNETCDF */
+
 #ifndef EXIT_SUCCESS /* Most likely this is a SUN4 machine */
-#define EXIT_SUCCESS 0
+# define EXIT_SUCCESS 0
 #endif /* SUN4 */
 #ifndef EXIT_FAILURE /* Most likely this is a SUN4 machine */
-#define EXIT_FAILURE 1
+# define EXIT_FAILURE 1
 #endif /* SUN4 */
 
   enum prg{ /* [enm] Key value for all netCDF operators */
@@ -139,17 +212,47 @@ extern "C" {
     aed_delete,
     aed_modify,
     aed_overwrite
-  }; /* end enum */
-  
-  enum nco_rlt_opr{ /* [enm] Arithmetic relations (comparisons) for masking */
-    nco_op_eq, /* Equality */
-    nco_op_ne, /* Inequality */
-    nco_op_lt, /* Less than */
-    nco_op_gt, /* Greater than */
-    nco_op_le, /* Less than or equal to */
-    nco_op_ge /* Greater than or equal to */
-  }; /* end enum */
+  }; /* end aed enum */
   
+  enum nco_dbg_typ_enm{ /* [enm] Debugging levels */
+    /* List in increasing levels of verbosity */
+    nco_dbg_quiet, /* Quiet all non-error messages. */
+    nco_dbg_std, /* Standard mode. Minimal, but some, messages. */
+    nco_dbg_fl, /* Filenames */
+    nco_dbg_scl, /* Scalars, other per-file information  */
+    nco_dbg_var, /* Variables, highest level per-file loop information */
+    nco_dbg_crr, /* Current task */
+    nco_dbg_sbr, /* Subroutine names on entry and exit */
+    nco_dbg_io, /* Subroutine I/O */
+    nco_dbg_vec, /* Entire vectors */
+    nco_dbg_vrb, /* Verbose, print everything possible */
+    nco_dbg_old, /* Old debugging blocks not used anymore */
+    nco_dbg_nbr /* Number of debugging types. Should be last enumerated value. */
+  }; /* end nco_dbg_typ_enm */
+
+  typedef enum { /* [enm] Memory allocation type */
+    nco_mmr_calloc, /* [enm] nco_calloc() */
+    nco_mmr_free, /* [enm] nco_free() */
+    nco_mmr_malloc, /* [enm] nco_malloc() */
+    nco_mmr_realloc /* [enm] nco_realloc() */
+  } nco_mmr_typ_enm; /* end nco_mmr_typ enum */
+
+  typedef enum { /* [enm] Arithmetic precision rank */
+    /* Ranked ordering of "arithmetic precision" from lowest to highest */
+    nco_rth_prc_rnk_string, /* [enm] */ /* Least precise */
+    nco_rth_prc_rnk_char, /* [enm] */
+    nco_rth_prc_rnk_ubyte, /* [enm] */
+    nco_rth_prc_rnk_byte, /* [enm] */
+    nco_rth_prc_rnk_ushort, /* [enm] */
+    nco_rth_prc_rnk_short, /* [enm] */
+    nco_rth_prc_rnk_uint, /* [enm] */
+    nco_rth_prc_rnk_int, /* [enm] */
+    nco_rth_prc_rnk_uint64, /* [enm] */
+    nco_rth_prc_rnk_int64, /* [enm] */
+    nco_rth_prc_rnk_float, /* [enm] */
+    nco_rth_prc_rnk_double /* [enm] */ /* Most precise */
+  } nco_rth_prc_rnk_enm; /* end nco_rth_prc_rnk enum */
+
   enum nco_op_typ{ /* [enm] Operation type */
     /* Types used in ncbo(): */
     nco_op_add, /* [enm] Add file_1 to file_2 */
@@ -169,23 +272,38 @@ extern "C" {
     nco_op_nil /* [enm] Nil or undefined operation type  */
   }; /* end nco_op_typ enum */
 
-  /* Following typedef's from Nie02 */
+  enum nco_rlt_opr{ /* [enm] Arithmetic relations (comparisons) for masking */
+    nco_op_eq, /* Equality */
+    nco_op_ne, /* Inequality */
+    nco_op_lt, /* Less than */
+    nco_op_gt, /* Greater than */
+    nco_op_le, /* Less than or equal to */
+    nco_op_ge /* Greater than or equal to */
+  }; /* end nco_rlt_opr enum */
+  
+  typedef enum { /* [enm] Timer flag */
+    nco_tmr_srt, /* [enm] Initialize timer (first timer call) */
+    nco_tmr_mtd, /* [enm] Metadata timer (second timer call) */
+    nco_tmr_rgl, /* [enm] Regular timer call (main loop timer call) */
+    nco_tmr_end /* [enm] Close timer (last timer call) */
+  } nco_tmr_flg; /* [enm] Timer flag */
+
   typedef enum { /* [enm] Node enumerator Nie02 nodeEnum */
     typ_scv, /* [enm] Scalar value */
     typ_sym, /* [enm] Symbol identifier */
     typ_opr /* [enm] Operator */
-  } nod_typ_enm;
+  } nod_typ_enm; /* end Node enumerator */
   /* end enumeration section */
-  
+
   /* Limit structure */
   typedef struct { /* lmt_sct */
-    char *nm; /* [sng] Variable name */
+    char *nm; /* [sng] Dimension name */
     int lmt_typ; /* crd_val or dmn_idx */
     /* Following four flags are used only by multi-file operators ncra and ncrcat: */
-    bool is_usr_spc_lmt; /* True if any part of limit is user-specified, else False */
-    bool is_usr_spc_min; /* True if user-specified, else False */
-    bool is_usr_spc_max; /* True if user-specified, else False */
-    bool is_rec_dmn; /* True if record dimension, else False */
+    nco_bool is_usr_spc_lmt; /* True if any part of limit is user-specified, else False */
+    nco_bool is_usr_spc_min; /* True if user-specified, else False */
+    nco_bool is_usr_spc_max; /* True if user-specified, else False */
+    nco_bool is_rec_dmn; /* True if record dimension, else False */
     long rec_skp_vld_prv; /* Records skipped at end of previous valid file (multi-file record dimension only) */
     long rec_skp_nsh_spf; /* Number of records skipped in initial superfluous files (multi-file record dimension only) */
     char *min_sng; /* User-specified string for dimension minimum */
@@ -208,8 +326,8 @@ extern "C" {
     long dmn_cnt; /* [nbr] Total number of hyperslabs to extract */
     long dmn_sz_org; /* [nbr] Size of dimension in INPUT file */
     int lmt_dmn_nbr; /* [nbr] Number of lmt args */
-    bool BASIC_DMN; /* [flg] Limit is same as dimension in input file */
-    bool WRP; /* [flg] Limit is wrapped - (only true if wrapping ,lmt_dmn_nbr=2 */ 
+    nco_bool BASIC_DMN; /* [flg] Limit is same as dimension in input file */
+    nco_bool WRP; /* [flg] Limit is wrapped (true iff wrapping, lmt_dmn_nbr==2) */ 
     lmt_sct **lmt_dmn; /* [sct] List of limit structures associated with each dimension */  
   } lmt_all_sct;
   
@@ -226,14 +344,37 @@ extern "C" {
     int id;
   } rnm_sct;
 
+  /* DDRA information structure */
+  typedef struct{ /* ddra_info_sct */
+    int nco_op_typ; /* [enm] Operation type */
+    int rnk_avg; /* [nbr] Rank of averaging space */
+    int rnk_var; /* [nbr] Variable rank (in input file) */
+    int rnk_wgt; /* [nbr] Rank of weight */
+    int var_idx; /* [enm] Index */
+    int wrd_sz; /* [B] Bytes per element */
+    long long lmn_nbr; /* [nbr] Variable size */
+    long long lmn_nbr_avg; /* [nbr] Averaging block size */
+    long long lmn_nbr_wgt; /* [nbr] Weight size */
+    nco_bool flg_ddra; /* [flg] Run DDRA diagnostics */
+    nco_bool MRV_flg; /* [flg] Avergaging dimensions are MRV dimensions */
+    nco_bool wgt_brd_flg; /* [flg] Broadcast weight for this variable */
+    nco_tmr_flg tmr_flg; /* [enm] Timer flag */
+  } ddra_info_sct;
+  
   /* Pointer union */
   typedef union{ /* ptr_unn */
     float * restrict fp;
     double * restrict dp;
-    nco_long * restrict lp; /* NC_INT is stored in native type nco_long */
+    nco_int * restrict lp;
     short * restrict sp;
-    unsigned char * restrict cp; /* Range of unsigned char is 0--255 */
-    signed char * restrict bp; /* Range of signed char is -127--127 */
+    nco_char * restrict cp;
+    nco_byte * restrict bp;
+    nco_ubyte * restrict ubp;
+    nco_ushort * restrict usp;
+    nco_uint * restrict uip;
+    nco_int64 * restrict i64p;
+    nco_uint64 * restrict ui64p;
+    nco_string * restrict sngp;
     void * restrict vp;
   } ptr_unn;
 
@@ -241,10 +382,16 @@ extern "C" {
   typedef union{ /* val_unn */
     float f;
     double d;
-    nco_long l;
+    nco_int l;
     short s;
-    unsigned char c; /* Range of unsigned char is 0--255 */
-    signed char b; /* Range of signed char is -127--127 */
+    nco_char c;
+    nco_byte b;
+    nco_ubyte ub;
+    nco_ushort us;
+    nco_uint ui;
+    nco_int64 i64;
+    nco_uint64 ui64;
+    nco_string sng;
   } val_unn;
 
   /* Scalar value structure */
@@ -289,7 +436,7 @@ extern "C" {
     long end; /* [idx] Index to end of hyperslab */
     long cnt; /* [nbr] Number of valid elements in this dimension (including effects of stride and wrapping) */
     long srd; /* [nbr] Stride of hyperslab */
-    ptr_unn val; /* [sct] Buffer to hold hyperslab */
+    ptr_unn val; /* [sct] Buffer to hold hyperslab fxm: is this ever used? */
     struct dmn_sct_tag *xrf; /* [sct] Cross-reference to associated dimension structure (usually the structure for dimension on output) */
   } dmn_sct; /* end dmn_sct_tag */
   
@@ -322,7 +469,7 @@ extern "C" {
     long *srd; /* [id] Contiguous vector of stride of hyperslab */
     ptr_unn val; /* [bfr] Buffer to hold hyperslab */
     long *tally; /* [nbr] Number of valid operations performed so far */
-    struct var_sct_tag *xrf; /* [sct] Cross-reference to associated variable structure (usually the structure for variable on output) */
+    struct var_sct_tag *xrf; /* [sct] Cross-reference to associated variable structure (usually structure for variable on output) fxm: deprecate! TODO nco226 */
     int pck_dsk; /* [flg] Variable is packed on disk (valid scale_factor, add_offset, or both attributes exist) */
     int pck_ram; /* [flg] Variable is packed in memory (valid scale_factor, add_offset, or both attributes exist) */
     int has_scl_fct; /* [flg] Valid scale_factor attribute exists */
@@ -331,6 +478,8 @@ extern "C" {
     ptr_unn add_fst; /* [frc] Value of add_offset attribute of type typ_upk */
     nc_type typ_pck; /* [enm] Type of variable when packed (on disk). typ_pck = typ_dsk except in cases where variable is packed in input file and unpacked in output file. */
     nc_type typ_upk; /* [enm] Type of variable when unpacked (expanded) (in memory) */
+    int undefined; /* [flg] Variable is still undefined (in first parser pass) */
+    int is_fix_var; /* [flg] Is this a fixed (non-processed) variable? */
   } var_sct; /* end var_sct_tag */
   
 #ifdef __cplusplus
