@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.c,v 1.41 2004/09/05 21:41:37 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.c,v 1.76 2007/05/25 17:36:46 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_att_utl.h" /* Attribute utilities */
 
@@ -51,7 +51,7 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
      architected differently from other NCO operators. */
   if(
      aed.att_nm /* Linux strcmp() dumps core if attribute name is blank */
-     && strcmp(aed.att_nm,"missing_value") == 0 /* Current attribute is "missing_value" */
+     && strcmp(aed.att_nm,nco_mss_val_sng_get()) == 0 /* Current attribute is "missing_value" */
      && var_id != NC_GLOBAL /* Current attribute is not global */
      && (aed.mode == aed_modify || aed.mode == aed_overwrite)  /* Modifying or overwriting existing value */
      && rcd == NC_NOERR /* Only when existing missing_value attribute is modified */
@@ -69,7 +69,7 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
     ptr_unn var_val;
     var_sct *var=NULL_CEWI;
 
-    (void)fprintf(stdout,"%s: WARNING Replacing missing value data in variable %s\n",prg_nm_get(),var_nm);
+    if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: INFO Replacing missing value data in variable %s\n",prg_nm_get(),var_nm);
 
     /* Take file out of define mode */
     (void)nco_enddef(nc_id);
@@ -81,15 +81,15 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
     var->sz=1L;
 
     /* Get type of variable and number of dimensions */
-    (void)nco_inq_var(var->nc_id,var->id,(char *)NULL,&var->type,&var->nbr_dim,(int *)NULL,(int *)NULL);
+    (void)nco_inq_var(nc_id,var_id,(char *)NULL,&var->type,&var->nbr_dim,(int *)NULL,(int *)NULL);
     dmn_id=(int *)nco_malloc(var->nbr_dim*sizeof(int));
     dmn_sz=(long *)nco_malloc(var->nbr_dim*sizeof(long));
     dmn_srt=(long *)nco_malloc(var->nbr_dim*sizeof(long));
-    (void)nco_inq_vardimid(var->nc_id,var->id,dmn_id);
+    (void)nco_inq_vardimid(nc_id,var_id,dmn_id);
 
     /* Get dimension sizes and construct variable size */
     for(idx=0;idx<var->nbr_dim;idx++){
-      (void)nco_inq_dimlen(var->nc_id,dmn_id[idx],dmn_sz+idx);
+      (void)nco_inq_dimlen(nc_id,dmn_id[idx],dmn_sz+idx);
       var->sz*=dmn_sz[idx];
       dmn_srt[idx]=0L;
     } /* end loop over dim */
@@ -103,18 +103,18 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
       nco_exit(EXIT_FAILURE); 
     } /* end if */
     if(var->sz > 1L){
-      (void)nco_get_vara(var->nc_id,var->id,var->srt,var->cnt,var->val.vp,var->type);
+      (void)nco_get_vara(nc_id,var_id,var->srt,var->cnt,var->val.vp,var->type);
     }else{
-      (void)nco_get_var1(var->nc_id,var->id,var->srt,var->val.vp,var->type);
+      (void)nco_get_var1(nc_id,var_id,var->srt,var->val.vp,var->type);
     } /* end else */
     
     /* Get current missing value attribute */
     var->mss_val.vp=NULL;
-    var->has_mss_val=nco_mss_val_get(var->nc_id,var);
+    var->has_mss_val=nco_mss_val_get(nc_id,var);
 
     /* Sanity check */
     if(var->has_mss_val == False){
-      (void)fprintf(stdout,"%s: ERROR \"missing_value\" attribute does not exist in nco_aed_prc()\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: ERROR \"NCO_MSS_VAL_SNG\" attribute does not exist in nco_aed_prc()\n",prg_nm_get());
       nco_exit(EXIT_FAILURE);
     } /* end if */
 
@@ -141,6 +141,12 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
     case NC_SHORT: for(idx=0L;idx<var_sz;idx++) {if(var_val.sp[idx] == *mss_val_crr.sp) var_val.sp[idx]=*mss_val_new.sp;} break;
     case NC_CHAR: for(idx=0L;idx<var_sz;idx++) {if(var_val.cp[idx] == *mss_val_crr.cp) var_val.cp[idx]=*mss_val_new.cp;} break;
     case NC_BYTE: for(idx=0L;idx<var_sz;idx++) {if(var_val.bp[idx] == *mss_val_crr.bp) var_val.bp[idx]=*mss_val_new.bp;} break;
+    case NC_UBYTE: for(idx=0L;idx<var_sz;idx++) {if(var_val.ubp[idx] == *mss_val_crr.ubp) var_val.ubp[idx]=*mss_val_new.ubp;} break;
+    case NC_USHORT: for(idx=0L;idx<var_sz;idx++) {if(var_val.usp[idx] == *mss_val_crr.usp) var_val.usp[idx]=*mss_val_new.usp;} break;
+    case NC_UINT: for(idx=0L;idx<var_sz;idx++) {if(var_val.uip[idx] == *mss_val_crr.uip) var_val.uip[idx]=*mss_val_new.uip;} break;
+    case NC_INT64: for(idx=0L;idx<var_sz;idx++) {if(var_val.i64p[idx] == *mss_val_crr.i64p) var_val.i64p[idx]=*mss_val_new.i64p;} break;
+    case NC_UINT64: for(idx=0L;idx<var_sz;idx++) {if(var_val.ui64p[idx] == *mss_val_crr.ui64p) var_val.ui64p[idx]=*mss_val_new.ui64p;} break;
+    case NC_STRING: for(idx=0L;idx<var_sz;idx++) {if(var_val.sngp[idx] == *mss_val_crr.sngp) var_val.sngp[idx]=*mss_val_new.sngp;} break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end switch */
 
@@ -151,9 +157,9 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
 
     /* Write to disk */
     if(var->nbr_dim == 0){
-      (void)nco_put_var1(nc_id,var->id,var->srt,var->val.vp,var->type);
-    }else{ /* end if variable is a scalar */
-      (void)nco_put_vara(nc_id,var->id,var->srt,var->cnt,var->val.vp,var->type);
+      (void)nco_put_var1(nc_id,var_id,var->srt,var->val.vp,var->type);
+    }else{ /* end if variable is scalar */
+      (void)nco_put_vara(nc_id,var_id,var->srt,var->cnt,var->val.vp,var->type);
     } /* end else */
 
     /* Free memory */
@@ -164,6 +170,8 @@ nco_aed_prc /* [fnc] Process single attribute edit for single variable */
     var->dmn_id=(int *)nco_free(var->dmn_id);
     var->srt=(long *)nco_free(var->srt);
     var->cnt=(long *)nco_free(var->cnt);
+    /* 20050704 try and use nco_free() to avoid valgrind error message */
+    var=(var_sct *)nco_free(var);
 
     /* Put file back in define mode */
     (void)nco_redef(nc_id);
@@ -226,7 +234,7 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
  const int out_id, /* I [id] netCDF output-file ID */
  const int var_in_id, /* I [id] netCDF input-variable ID */
  const int var_out_id, /* I [id] netCDF output-variable ID */
- const bool PCK_ATT_CPY) /* I [flg] Copy attributes "scale_factor", "add_offset" */
+ const nco_bool PCK_ATT_CPY) /* I [flg] Copy attributes "scale_factor", "add_offset" */
 {
   /* Purpose: Copy attributes from input netCDF file to output netCDF file
      If var_in_id == NC_GLOBAL, then copy global attributes
@@ -249,26 +257,29 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
   /* Jump back to here if current attribute is treated specially */
   for(idx=0;idx<nbr_att;idx++){
     (void)nco_inq_attname(in_id,var_in_id,idx,att_nm);
+    /* Look for same attribute in output variable in output file */
     rcd=nco_inq_att_flg(out_id,var_out_id,att_nm,(nc_type *)NULL,(long *)NULL);
       
     /* If instructed not to copy packing attributes... */
     if(!PCK_ATT_CPY)
-      /* ...verify attributed is "scale_factor" or "add_offset" ... */
+      /* ...and attribute is "scale_factor" or "add_offset" ... */
       if(!strcmp(att_nm,"scale_factor") || !strcmp(att_nm,"add_offset"))
 	/* ...then skip remainder of loop, thereby skipping attribute copy... */
 	continue;
     
-    /* Will copy overwrite an existing attribute? */
-    if(rcd == NC_NOERR){
-      if(var_out_id == NC_GLOBAL){
-	(void)fprintf(stderr,"%s: WARNING Overwriting global attribute %s\n",prg_nm_get(),att_nm);
-      }else{
-	(void)nco_inq_varname(out_id,var_out_id,var_nm);
-	(void)fprintf(stderr,"%s: WARNING Overwriting attribute %s for output variable %s\n",prg_nm_get(),att_nm,var_nm);
-      } /* end else */
-    } /* end if */
+    /* Inform user when copy will overwrite an existing attribute */
+    if(dbg_lvl_get() >= nco_dbg_std){
+      if(rcd == NC_NOERR){
+	if(var_out_id == NC_GLOBAL){
+	  (void)fprintf(stderr,"%s: INFO Overwriting global attribute %s\n",prg_nm_get(),att_nm);
+	}else{
+	  (void)nco_inq_varname(out_id,var_out_id,var_nm);
+	  (void)fprintf(stderr,"%s: INFO Overwriting attribute %s for output variable %s\n",prg_nm_get(),att_nm,var_nm);
+	} /* end else */
+      } /* end if */
+    } /* end if dbg */
 
-    if(PCK_ATT_CPY || strcmp(att_nm,"missing_value")){
+    if(PCK_ATT_CPY || strcmp(att_nm,nco_mss_val_sng_get())){
       /* Copy all attributes except missing_value with fast library routine */
       (void)nco_copy_att(in_id,var_in_id,att_nm,out_id,var_out_id);
     }else{
@@ -332,7 +343,7 @@ nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
   char att_nm_nbr[]="nco_input_file_number";
   char spc_sng[]=" "; /* [sng] Intervening space */
   char *fl_in_lst_sng;
-  nco_long fl_nbr_lng; /* [nbr] Number of files in input file list */
+  nco_int fl_nbr_lng; /* [nbr] Number of files in input file list */
   int fl_idx;
   size_t fl_in_lst_sng_lng; /* [nbr] Filename list string length */
   ptr_unn att_val;
@@ -367,7 +378,7 @@ nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
   (void)nco_aed_prc(out_id,NC_GLOBAL,fl_in_nbr_aed);
 
   /* Insert file list into value */
-  att_val.cp=(unsigned char *)fl_in_lst_sng;
+  att_val.cp=(nco_char *)fl_in_lst_sng;
   /* Initialize nco_input_file_list attribute edit structure */
   fl_in_lst_aed.att_nm=att_nm_lst;
   fl_in_lst_aed.var_nm=NULL;
@@ -384,10 +395,10 @@ nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
   fl_in_lst_sng=(char *)nco_free(fl_in_lst_sng);
 } /* end nco_fl_lst_att_cat() */
  
- void 
- nco_hst_att_cat /* [fnc] Add command line, date stamp to history attribute */
- (const int out_id, /* I [id] netCDF output-file ID */
-  const char * const hst_sng) /* I [sng] String to add to history attribute */
+void 
+nco_hst_att_cat /* [fnc] Add command line, date stamp to history attribute */
+(const int out_id, /* I [id] netCDF output-file ID */
+ const char * const hst_sng) /* I [sng] String to add to history attribute */
 {
   /* Purpose: Add command line and date stamp to existing history attribute, if any,
      and write them to specified output file */
@@ -444,7 +455,7 @@ nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
     /* NB: ncattinq(), unlike strlen(), counts terminating NUL for stored NC_CHAR arrays */
     (void)nco_inq_att(out_id,NC_GLOBAL,att_nm,&att_typ,&att_sz);
     if(att_typ != NC_CHAR){
-      (void)fprintf(stderr,"%s: WARNING the \"%s\" global attribute is type %s, not %s. Therefore current command line will not be appended to %s in output file.\n",prg_nm_get(),att_nm,nco_typ_sng(att_typ),nco_typ_sng(NC_CHAR),att_nm);
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stderr,"%s: WARNING the \"%s\" global attribute is type %s, not %s. Therefore current command line will not be appended to %s in output file.\n",prg_nm_get(),att_nm,nco_typ_sng(att_typ),nco_typ_sng(NC_CHAR),att_nm);
       return;
     } /* end if */
 
@@ -464,6 +475,7 @@ nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
   history_crr=(char *)nco_free(history_crr);
   history_new=(char *)nco_free(history_new);
 
+  return; /* 20050109: fxm added return to void function to squelch erroneous gcc-3.4.2 warning */ 
 } /* end nco_hst_att_cat() */
 
 aed_sct * /* O [sct] List of attribute edit structures */
@@ -508,8 +520,6 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
      This is default mode.
    */
 
-  void nco_usg_prn(void);
-
   char **arg_lst;
 
   const char * const dlm_sng=",";
@@ -525,8 +535,8 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
 
   for(idx=0;idx<nbr_aed;idx++){
 
-    /* Attribute edit specifications are processed as normal text list */
-    arg_lst=lst_prs(aed_arg[idx],dlm_sng,&arg_nbr);
+    /* Process attribute edit specifications as normal text list */
+    arg_lst=nco_lst_prs_2D(aed_arg[idx],dlm_sng,&arg_nbr);
 
     /* Check syntax */
     if(
@@ -554,8 +564,8 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
     aed_lst[idx].id=-1;
 
     /* Fill in structure */
-    aed_lst[idx].att_nm=arg_lst[0];
-    aed_lst[idx].var_nm=arg_lst[1];
+    if(arg_lst[0] != NULL) aed_lst[idx].att_nm=strdup(arg_lst[0]);
+    if(arg_lst[1] != NULL) aed_lst[idx].var_nm=strdup(arg_lst[1]);
 
     /* fxm: Change these switches to string comparisons someday */
     /* Set mode of current aed structure */
@@ -592,10 +602,17 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
       case 'c':	aed_lst[idx].type=NC_CHAR; break;
       case 'b':	aed_lst[idx].type=NC_BYTE; break;
       default: 
-	(void)fprintf(stderr,"%s: ERROR `%s' is not a supported netCDF data type\n",prg_nm_get(),arg_lst[3]);
-	(void)fprintf(stderr,"%s: HINT: Valid data types are `c' = char, `f' = float, `d' = double,`s' = short, `l' = long, `b' = byte",prg_nm_get());
-	nco_exit(EXIT_FAILURE);
-	break;
+	if(!strcasecmp(arg_lst[3],"ub")) aed_lst[idx].type=NC_UBYTE; 
+	else if(!strcasecmp(arg_lst[3],"us")) aed_lst[idx].type=NC_USHORT; 
+	else if(!strcasecmp(arg_lst[3],"u") || !strcasecmp(arg_lst[3],"ui") || !strcasecmp(arg_lst[3],"ul")) aed_lst[idx].type=NC_UINT; 
+	else if(!strcasecmp(arg_lst[3],"ll") || !strcasecmp(arg_lst[3],"int64")) aed_lst[idx].type=NC_INT64; 
+	else if(!strcasecmp(arg_lst[3],"ull") || !strcasecmp(arg_lst[3],"uint64")) aed_lst[idx].type=NC_UINT64; 
+	else if(!strcasecmp(arg_lst[3],"sng")) aed_lst[idx].type=NC_STRING; 
+	else{
+	  (void)fprintf(stderr,"%s: ERROR `%s' is not a supported netCDF data type\n",prg_nm_get(),arg_lst[3]);
+	  (void)fprintf(stderr,"%s: HINT: Valid data types are `c' = char, `f' = float, `d' = double,`s' = short, 'l' = `i' = integer, `b' = byte",prg_nm_get());
+	  nco_exit(EXIT_FAILURE);} /*  end if error */
+      break;
       } /* end switch */
       
       /* Re-assemble string list values which inadvertently contain delimiters */
@@ -603,9 +620,10 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
 	/* Number of elements which must be concatenated into single string value */
 	long lmn_nbr;
 	lmn_nbr=arg_nbr-idx_att_val_arg; 
-	if(dbg_lvl_get() >= 2) (void)fprintf(stdout,"%s: WARNING NC_CHAR (string) attribute is embedded with %li literal element delimiters (\"%s\"), re-assembling...\n",prg_nm_get(),lmn_nbr-1L,dlm_sng);
+	if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: WARNING NC_CHAR (string) attribute is embedded with %li literal element delimiters (\"%s\"), re-assembling...\n",prg_nm_get(),lmn_nbr-1L,dlm_sng);
 	/* Rewrite list, splicing in original delimiter string */
-	arg_lst[idx_att_val_arg]=sng_lst_prs(arg_lst+idx_att_val_arg,lmn_nbr,dlm_sng);
+	/* fxm: TODO nco527 this is probably where ncatted memory is lost */
+	arg_lst[idx_att_val_arg]=sng_lst_cat(arg_lst+idx_att_val_arg,lmn_nbr,dlm_sng);
 	/* Keep bookkeeping straight, just in case */
 	arg_nbr=idx_att_val_arg+1L;
 	lmn_nbr=1L;
@@ -617,7 +635,7 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
       /* Set size of current aed structure */
       if(aed_lst[idx].type == NC_CHAR){
 	/* Include NUL-terminator in string length */
-	aed_lst[idx].sz=(arg_lst[idx_att_val_arg] == NULL) ? 0 : strlen(arg_lst[idx_att_val_arg])+1;
+	aed_lst[idx].sz=(arg_lst[idx_att_val_arg] == NULL) ? 0L : strlen(arg_lst[idx_att_val_arg])+1L;
       }else{
 	/* Number of elements of numeric types is determined by number of delimiters */
 	aed_lst[idx].sz=arg_nbr-idx_att_val_arg;
@@ -625,39 +643,72 @@ nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure lis
       
       /* Set value of current aed structure */
       if(aed_lst[idx].type == NC_CHAR){
-	aed_lst[idx].val.cp=(unsigned char *)arg_lst[idx_att_val_arg];
+	aed_lst[idx].val.cp=(nco_char *)strdup(arg_lst[idx_att_val_arg]);
       }else{
 	double *val_arg_dbl=NULL_CEWI;
+	long long *val_arg_lng_lng=NULL_CEWI;
+	unsigned long long *val_arg_ulng_lng=NULL_CEWI;
 	
 	long lmn;
 	
-	val_arg_dbl=(double *)nco_malloc(aed_lst[idx].sz*sizeof(double));
 	aed_lst[idx].val.vp=(void *)nco_malloc(aed_lst[idx].sz*nco_typ_lng(aed_lst[idx].type));
 	
-	for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) val_arg_dbl[lmn]=strtod(arg_lst[idx_att_val_arg+lmn],(char **)NULL); 
+	/* Use type-appropriate conversion */
+	switch(aed_lst[idx].type){
+	case NC_FLOAT: 
+	case NC_DOUBLE: 
+	  val_arg_dbl=(double *)nco_malloc(aed_lst[idx].sz*sizeof(double));
+	  for(lmn=0L;lmn<aed_lst[idx].sz;lmn++){val_arg_dbl[lmn]=strtod(arg_lst[idx_att_val_arg+lmn],(char **)NULL);} break; 
+	case NC_BYTE:
+	case NC_INT: 
+	case NC_SHORT: 
+	case NC_INT64: 
+	  val_arg_lng_lng=(long long *)nco_malloc(aed_lst[idx].sz*sizeof(long long));
+	  for(lmn=0L;lmn<aed_lst[idx].sz;lmn++){val_arg_lng_lng[lmn]=strtoll(arg_lst[idx_att_val_arg+lmn],(char **)NULL,10);} break;
+	case NC_CHAR:
+	case NC_UBYTE: 
+	case NC_USHORT: 
+	case NC_UINT: 
+	case NC_UINT64: 
+	  val_arg_ulng_lng=(unsigned long long *)nco_malloc(aed_lst[idx].sz*sizeof(unsigned long long));
+	  for(lmn=0L;lmn<aed_lst[idx].sz;lmn++){val_arg_ulng_lng[lmn]=strtoull(arg_lst[idx_att_val_arg+lmn],(char **)NULL,10);} break;
+	case NC_STRING: break;
+	default: nco_dfl_case_nc_type_err(); break;
+	} /* end switch */
 	
 	/* Copy and typecast entire array of values, using implicit coercion rules of C */
 	/* 20011001: Use explicit coercion rules to quiet C++ compiler warnings */
 	switch(aed_lst[idx].type){
 	case NC_FLOAT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.fp[lmn]=(float)val_arg_dbl[lmn];} break; 
 	case NC_DOUBLE: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.dp[lmn]=(double)val_arg_dbl[lmn];} break; 
-	case NC_INT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.lp[lmn]=(long)val_arg_dbl[lmn];} break; 
-	case NC_SHORT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.sp[lmn]=(short)val_arg_dbl[lmn];} break; 
-	case NC_CHAR: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.cp[lmn]=(unsigned char)val_arg_dbl[lmn];} break; 
-	case NC_BYTE: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.bp[lmn]=(signed char)val_arg_dbl[lmn];} break; 
+	case NC_INT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.lp[lmn]=(nco_int)val_arg_lng_lng[lmn];} break; 
+	case NC_SHORT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.sp[lmn]=(nco_short)val_arg_lng_lng[lmn];} break; 
+	case NC_CHAR: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.cp[lmn]=(nco_char)val_arg_ulng_lng[lmn];} break; 
+	case NC_BYTE: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.bp[lmn]=(nco_byte)val_arg_lng_lng[lmn];} break; 
+	case NC_UBYTE: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.ubp[lmn]=(nco_ubyte)val_arg_ulng_lng[lmn];} break; 
+	case NC_USHORT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.usp[lmn]=(nco_ushort)val_arg_ulng_lng[lmn];} break; 
+	case NC_UINT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.uip[lmn]=(nco_uint)val_arg_ulng_lng[lmn];} break; 
+	case NC_INT64: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.i64p[lmn]=(nco_int64)val_arg_lng_lng[lmn];} break; 
+	case NC_UINT64: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.ui64p[lmn]=(nco_uint64)val_arg_ulng_lng[lmn];} break; 
+	case NC_STRING: break;
 	default: nco_dfl_case_nc_type_err(); break;
 	} /* end switch */
 	
-	/* Free array used to hold double values */
-	val_arg_dbl=(double *)nco_free(val_arg_dbl);
+	/* Free array used to hold input values */
+	if(val_arg_dbl) val_arg_dbl=(double *)nco_free(val_arg_dbl);
+	if(val_arg_lng_lng) val_arg_lng_lng=(long long *)nco_free(val_arg_lng_lng);
+	if(val_arg_ulng_lng) val_arg_ulng_lng=(unsigned long long *)nco_free(val_arg_ulng_lng);
       } /* end else */
       /* Un-typecast pointer to values after access */
       (void)cast_nctype_void(aed_lst[idx].type,&aed_lst[idx].val);
       
     } /* end if mode is not delete */
+
+    arg_lst=nco_sng_lst_free(arg_lst,arg_nbr);
+
   } /* end loop over aed */
   
-  if(dbg_lvl_get() == 5){
+  if(dbg_lvl_get() >= nco_dbg_io){
     for(idx=0;idx<nbr_aed;idx++){
       (void)fprintf(stderr,"aed_lst[%d].att_nm = %s\n",idx,aed_lst[idx].att_nm);
       (void)fprintf(stderr,"aed_lst[%d].var_nm = %s\n",idx,aed_lst[idx].var_nm == NULL ? "NULL" : aed_lst[idx].var_nm);
@@ -718,7 +769,7 @@ nco_prs_att /* [fnc] Parse conjoined variable and attribute names */
 rnm_sct * /* O [sng] Structured list of old, new names */
 nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
 (const int nbr_rnm, /* I [nbr] Number of elements in rename list */
- CST_X_PTR_CST_PTR_CST_Y(char,rnm_arg)) /* I [sng] Unstructured list of old, new names */
+ char * const * const rnm_arg) /* I [sng] Unstructured list of old, new names */
 {
   /* Purpose: Set old_nm, new_nm elements of rename structure
      Routine merely fills rename structure and does not attempt to validate 
@@ -764,7 +815,7 @@ nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
     
   } /* end loop over rnm_lst */
 
-  if(dbg_lvl_get() == 5){
+  if(dbg_lvl_get() >= nco_dbg_io){
     for(idx=0;idx<nbr_rnm;idx++){
       (void)fprintf(stderr,"%s\n",rnm_lst[idx].old_nm);
       (void)fprintf(stderr,"%s\n",rnm_lst[idx].new_nm);
@@ -782,7 +833,7 @@ nco_thr_att_cat /* [fnc] Add threading global attribute */
   /* Purpose: Write thread information to global metadata */
   aed_sct thr_nbr_aed;
   char att_nm_nbr[]="nco_openmp_thread_number";
-  nco_long thr_nbr_lng; /* [nbr] Thread number copy */
+  nco_int thr_nbr_lng; /* [nbr] Thread number copy */
   ptr_unn att_val;
   
   /* Copy thr_nbr so can take address without endangering number */
@@ -803,3 +854,31 @@ nco_thr_att_cat /* [fnc] Add threading global attribute */
 
 } /* end nco_thr_att_cat() */
  
+void 
+nco_mpi_att_cat /* [fnc] Add MPI tasks global attribute */
+(const int out_id, /* I [id] netCDF output-file ID */
+ const int mpi_nbr) /* I [nbr] MPI nodes/tasks number */
+{
+  /* Purpose: Write number of tasks information to global metadata */
+  aed_sct mpi_nbr_aed;
+  char att_nm_nbr[]="nco_mpi_task_number";
+  nco_int mpi_nbr_lng; /* [nbr] MPI tasks number copy */
+  ptr_unn att_val;
+  
+  /* Copy mpi_nbr so can take address without endangering number */
+  mpi_nbr_lng=mpi_nbr;
+  /* Insert tasks number into value */
+  att_val.lp=&mpi_nbr_lng;
+  /* Initialize nco_mpi_task_number attribute edit structure */
+  mpi_nbr_aed.att_nm=att_nm_nbr;
+  mpi_nbr_aed.var_nm=NULL;
+  mpi_nbr_aed.id=NC_GLOBAL;
+  mpi_nbr_aed.sz=1L;
+  mpi_nbr_aed.type=NC_INT;
+  /* Insert value into attribute structure */
+  mpi_nbr_aed.val=att_val;
+  mpi_nbr_aed.mode=aed_overwrite;
+  /* Write nco_mpi_tasks_number attribute to disk */
+  (void)nco_aed_prc(out_id,NC_GLOBAL,mpi_nbr_aed);
+
+} /* end nco_mpi_att_cat() */
