@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.18 2004/08/16 04:13:33 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_att_utl.h,v 1.25 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Attribute utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_att_utl.h" *//* Attribute utilities */
@@ -50,7 +50,7 @@ nco_att_cpy  /* [fnc] Copy attributes from input netCDF file to output netCDF fi
  const int out_id, /* I [id] netCDF output-file ID */
  const int var_in_id, /* I [id] netCDF input-variable ID */
  const int var_out_id, /* I [id] netCDF output-variable ID */
- const bool PCK_ATT_CPY); /* I [flg] Copy attributes "scale_factor", "add_offset" */
+ const nco_bool PCK_ATT_CPY); /* I [flg] Copy attributes "scale_factor", "add_offset" */
 
 void 
 nco_fl_lst_att_cat /* [fnc] Add input file list global attribute */
@@ -63,6 +63,11 @@ nco_hst_att_cat /* [fnc] Add command line, date stamp to history attribute */
 (const int out_id, /* I [id] netCDF output-file ID */
  const char * const hst_sng); /* I [sng] String to add to history attribute */
 
+void 
+nco_mpi_att_cat /* [fnc] Add MPI tasks global attribute */
+(const int out_id, /* I [id] netCDF output-file ID */
+ const int mpi_nbr); /* I [nbr] MPI nodes/tasks number */
+
 aed_sct * /* O [sct] List of attribute edit structures */
 nco_prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure list */
 (const int nbr_aed, /* I [nbr] Number of attributes in list */
@@ -76,7 +81,7 @@ nco_prs_att /* [fnc] Parse conjoined variable and attribute names */
 rnm_sct * /* O [sng] Structured list of old, new names */
 nco_prs_rnm_lst /* [fnc] Set old_nm, new_nm elements of rename structure */
 (const int nbr_rnm, /* I [nbr] Number of elements in rename list */
- CST_X_PTR_CST_PTR_CST_Y(char,rnm_arg)); /* I [sng] Unstructured list of old, new names */
+ char * const * const rnm_arg); /* I [sng] Unstructured list of old, new names */
 
 void 
 nco_thr_att_cat /* [fnc] Add threading global attribute */
