@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cln_utl.c,v 1.8 2004/01/05 17:29:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cln_utl.c,v 1.13 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Calendar utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_cln_utl.h" /* Calendar utilities */
 
@@ -30,9 +30,9 @@ nco_nd2endm /* [fnc] Compute number of days to end of month */
   return nbr_day_2_mth_end;
 } /* nco_nd2endm */
 
-nco_long /* O [YYMMDD] Date a specified number of days from input date */
+nco_int /* O [YYMMDD] Date a specified number of days from input date */
 nco_newdate /* [fnc] Compute date a specified number of days from input date */
-(const nco_long date, /* I [YYMMDD] Date */
+(const nco_int date, /* I [YYMMDD] Date */
  const long day_srt) /* I [day] Days ahead of input date */
 {
   /* Purpose: Find date a specified number of days (possibly negative) from given date 
@@ -53,8 +53,8 @@ nco_newdate /* [fnc] Compute date a specified number of days from input date */
   long mth_tmp; /* Current month as we increment date */
   long yr_crr; /* Year of date */
 
-  nco_long date_srt; /* Initial value of date (may change sign) */
-  nco_long newdate_YYMMDD; /* New date in YYMMDD format */
+  nco_int date_srt; /* Initial value of date (may change sign) */
+  nco_int newdate_YYMMDD; /* New date in YYMMDD format */
 
   if(day_srt == 0L) return date;
 
