@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cln_utl.h,v 1.9 2004/01/12 18:11:07 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cln_utl.h,v 1.14 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Calendar utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_cln_utl.h" *//* Calendar utilities */
@@ -31,9 +31,9 @@ nco_nd2endm /* [fnc] Compute number of days to end of month */
 (const int mth, /* I [mth] Month */
  const int day); /* I [day] Current day */
 
-nco_long /* O [YYMMDD] Date a specified number of days from input date */
+nco_int /* O [YYMMDD] Date a specified number of days from input date */
 nco_newdate /* [fnc] Compute date a specified number of days from input date */
-(const nco_long date, /* I [YYMMDD] Date */
+(const nco_int date, /* I [YYMMDD] Date */
  const long day_srt); /* I [day] Days ahead of input date */
 
 #ifdef __cplusplus
