@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_dmn.c,v 1.39 2004/08/07 21:16:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_dmn.c,v 1.62 2007/05/09 23:29:16 zender Exp $ */
 
 /* Purpose: Conform dimensions between variables */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_cnf_dmn.h" /* Conform dimensions */
 
@@ -13,8 +13,8 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 (const var_sct * const var, /* I [ptr] Pointer to variable structure to serve as template */
  var_sct * const wgt, /* I [ptr] Pointer to variable structure to make conform to var */
  var_sct *wgt_crr, /* I/O [ptr] Pointer to existing conforming variable structure, if any (destroyed when does not conform to var) */
- const bool MUST_CONFORM, /* I [flg] Must wgt and var conform? */
- bool *DO_CONFORM) /* O [flg] Do wgt and var conform? */
+ const nco_bool MUST_CONFORM, /* I [flg] Must wgt and var conform? */
+ nco_bool *DO_CONFORM) /* O [flg] Do wgt and var conform? */
 {
   /* Threads: Routine is thread safe and calls no unsafe routines */
   /* fxm: TODO 226. Is xrf in nco_var_cnf_dmn() really necessary? If not, remove it and make wgt arg const var_sct * const */
@@ -23,34 +23,34 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
      Dimensions in var which are not in wgt will be present in wgt_out, with values
      replicated from existing dimensions in wgt.
      By default, wgt's dimensions must be subset of var's dimensions (MUST_CONFORM=true)
-     If it is permissible for wgt not to conform to var then set MUST_CONFORM=false before calling this routine
-     In this case when wgt and var do not conform then then nco_var_cnf_dmn sets *DO_CONFORM=False and returns a copy of var with all values set to 1.0
-     The calling procedure can then decide what to do with the output
-     MUST_CONFORM is True for ncbo: Variables of like name to be, e.g., differenced, must be same rank
-     MUST_CONFORM is False false for ncap, ncflint, ncwa: Some variables to be averaged may not conform to the specified weight, e.g., lon will not conform to gw. This is fine and the returned wgt_out may not be used. */
+     Calling routine should set MUST_CONFORM=false if wgt and var need not conform
+     When wgt and var do not conform then then nco_var_cnf_dmn sets *DO_CONFORM=False and returns copy of var with all values set to 1.0
+     Calling procedure then decides what to do with unity output
+     MUST_CONFORM is True for ncbo: Variables of like name to be, e.g., differenced, must conform
+     MUST_CONFORM is False false for ncap, ncflint, ncwa: Some variables to be averaged may not conform to specified weight, e.g., lon will not conform to gw. This is fine and returned wgt_out may be discarded. */
 
   /* There are many inelegant ways to accomplish this (without using C++): */  
 
-  /* Perhaps most efficient method to accomplish this in general case is to expand 
-     weight array until it is same size as variable array, and then multiply these
-     together element-by-element in highly vectorized loop, preferably in Fortran. 
-     To enhance speed, (enlarged) weight-values array could be static, only remade
+  /* Perhaps most efficient method in general case is to expand weight array until
+     it is same size as variable array, and then multiply these arrays together 
+     element-by-element in highly vectorized loop (possibly in Fortran or BLAS). 
+     To enhance speed, (enlarged) weight-values array could be static, only re-made
      when dimensions of incoming variables change. */
 
-  /* Another method for the general case, though an expensive one, is to use C to 
-     figure out the multidimensional indices into the one dimensional hyperslab, 
-     a la ncks. Knowing these indices, one can loop over the one-dimensional array
+  /* Another general method, though more expensive, is to use C to figure out the 
+     multidimensional indices into the one dimensional hyperslab, a la ncks. 
+     Knowing these indices, routine could loop over the one-dimensional array
      element by element, choosing the appropriate index into the weight array from 
-     those same multidimensional indices. This method can also create a static weight-value
-     array that is only destroyed when an incoming variable changes dimensions from the
-     previous variable. */
+     those same multidimensional indices. 
+     This method can also create a static weight-value array that is only destroyed 
+     when an incoming variable changes dimensions from the previous variable. */
 
-  /* Yet another method, which is not completely general, but which may be good enough for
-     governement work, is to create fortran subroutines which expect variables of a given
-     number of dimensions as input. Creating these functions for up to five dimensions would
-     satisfy all foreseeable situations. The branch as to which function to call would be
-     done based on number of dimensions, here in the C code. C++ or Fortran9x overloading
-     could accomplish some of this interface more elegantly. */
+  /* Another method, which is not completely general, but which may be good enough for
+     governement work, is to create Fortran subroutines which expect variables of 
+     a given number of dimensions as input. 
+     Creating these functions for up to five dimensions would satisfy most situations
+     C code would determine which branch to call based on number of dimensions
+     C++ or Fortran9x overloading could construct this interface more elegantly */
 
   /* An (untested) simplification to some of these methods is to copy the 1-D array
      value pointer of variable and cast it to an N-D array pointer
@@ -62,8 +62,8 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
      (otherwise which hyperslab of weight to use would be ill-defined). 
      However, weight may (and often will) have fewer dimensions than variable */
 
-  bool CONFORMABLE=False; /* [flg] wgt can be made to conform to var */
-  bool USE_DUMMY_WGT=False; /* [flg] Fool NCO into thinking wgt conforms to var */
+  nco_bool CONFORMABLE=False; /* [flg] wgt can be made to conform to var */
+  nco_bool USE_DUMMY_WGT=False; /* [flg] Fool NCO into thinking wgt conforms to var */
 
   int idx; /* [idx] Counting index */
   int idx_dmn; /* [idx] Dimension index */
@@ -83,14 +83,25 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 	if(strcmp(wgt_crr->dim[idx]->nm,var->dim[idx]->nm)) break;
       } /* end loop over dimensions */
       if(idx == var->nbr_dim) *DO_CONFORM=True;
-    } /* end if */
+    } /* end if ranks are equal */
+
+    /* 20060425: Weight re-use will not occur if wgt_crr is free()'d here
+       Some DDRA benchmarks need to know cost of broadcasting weights
+       To turn off weight re-use and cause broadcasting, execute "else" block below
+       by (temporarily) using in following condition
+
+    if(*DO_CONFORM && False){
+
+    instead of
+
+    if(*DO_CONFORM){ */
     if(*DO_CONFORM){
       wgt_out=wgt_crr;
     }else{
       wgt_crr=nco_var_free(wgt_crr);
       wgt_out=NULL;
-    } /* end if */
-  } /* end if */
+    } /* !*DO_CONFORM */
+  } /* wgt_crr == NULL */
 
   /* Does original weight (wgt) conform to variable's dimensions? */
   if(wgt_out == NULL){
@@ -172,6 +183,9 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
     } /* end if */
   } /* end if */
 
+  /* Set diagnostic DDRA information DDRA */
+  /* ddra_info->wgt_brd_flg=(wgt_out == NULL) ? True : False; *//* [flg] Broadcast weight for this variable */
+
   if(wgt_out == NULL){
     /* Expand original weight (wgt) to match size of current variable */
     char * restrict wgt_cp;
@@ -192,14 +206,19 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 
     size_t wgt_typ_sz;
 
-    /* Copy main attributes of variable into current weight */
+    /* Copy main attributes of variable into output weight */
     wgt_out=nco_var_dpl(var);
     (void)nco_xrf_var(wgt,wgt_out);
 
-    /* Modify a few elements of weight array */
-    wgt_out->nm=wgt->nm;
+    /* wgt_out variable was copied from template var
+       Modify key fields so its name and type are based on wgt, not var
+       wgt_out will then be hybrid between wgt and var 
+       Remainder of routine fills wgt_out's var-dimensionality with wgt-values */
+    wgt_out->nm=(char *)nco_free(wgt_out->nm);
+    wgt_out->nm=(char *)strdup(wgt->nm);
     wgt_out->id=wgt->id;
     wgt_out->type=wgt->type;
+    wgt_out->val.vp=(void *)nco_free(wgt_out->val.vp);
     wgt_out->val.vp=(void *)nco_malloc(wgt_out->sz*nco_typ_lng(wgt_out->type));
     wgt_cp=(char *)wgt->val.vp;
     wgt_out_cp=(char *)wgt_out->val.vp;
@@ -207,11 +226,18 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 
     if(wgt_out->nbr_dim == 0){
       /* Variable (and weight) are scalars, not arrays */
-
       (void)memcpy(wgt_out_cp,wgt_cp,wgt_typ_sz);
-
+    }else if(wgt->nbr_dim == 0){
+      /* Lesser-ranked input variable is scalar 
+	 Expansion in this degenerate case needs no index juggling (reverse-mapping)
+	 Code as special case to speed-up important applications of ncap
+	 for synthetic file creation */
+      var_sz=var->sz;
+      for(var_lmn=0;var_lmn<var_sz;var_lmn++){
+	(void)memcpy(wgt_out_cp+var_lmn*wgt_typ_sz,wgt_cp,wgt_typ_sz);      
+      } /* end loop over var_lmn */
     }else{
-      /* Variable (and weight) are arrays, not scalars */
+      /* Variable (and therefore wgt_out) are arrays, not scalars */
       
       /* Create forward and reverse mappings from variable's dimensions to weight's dimensions:
 
@@ -231,8 +257,7 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 	 Since mapping arrays (dmn_var_map and dmn_wgt_map) are ultimately used for a
 	 memcpy() operation, they could (read: should) be computed as byte offsets, not type offsets. 
 	 This is why netCDF generic hyperslab routines (ncvarputg(), ncvargetg())
-	 request imap vector to specify offset (imap) vector in bytes.
-      */
+	 request imap vector to specify offset (imap) vector in bytes. */
 
       for(idx=0;idx<wgt->nbr_dim;idx++){
 	for(idx_dmn=0;idx_dmn<var->nbr_dim;idx_dmn++){
@@ -270,16 +295,32 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 
       /* var_lmn is offset into 1-D array corresponding to N-D indices dmn_ss */
       for(var_lmn=0;var_lmn<var_sz;var_lmn++){
+	/* dmn_ss are corresponding indices (subscripts) into N-D array */
+	/* Operations: 1 modulo, 1 pointer offset, 1 user memory fetch
+	   Repetitions: \lmnnbr
+	   Total Counts: \rthnbr=2\lmnnbr, \mmrusrnbr=\lmnnbr
+	   NB: LHS assumed compact and cached, counted RHS offsets and fetches only */
 	dmn_ss[var_nbr_dmn_m1]=var_lmn%var_cnt[var_nbr_dmn_m1];
 	for(idx=0;idx<var_nbr_dmn_m1;idx++){
+	  /* Operations: 1 divide, 1 modulo, 2 pointer offset, 2 user memory fetch
+	     Repetitions: \lmnnbr(\dmnnbr-1)
+	     Counts: \rthnbr=4\lmnnbr(\dmnnbr-1), \mmrusrnbr=2\lmnnbr(\dmnnbr-1)
+	     NB: LHS assumed compact and cached, counted RHS offsets and fetches only
+	     NB: Neglected loop arithmetic/compare */
 	  dmn_ss[idx]=(long)(var_lmn/dmn_var_map[idx]);
 	  dmn_ss[idx]%=var_cnt[idx];
 	} /* end loop over dimensions */
 	
 	/* Map (shared) N-D array indices into 1-D index into original weight data */
 	wgt_lmn=0L;
+	/* Operations: 1 add, 1 multiply, 3 pointer offset, 3 user memory fetch
+	   Repetitions: \lmnnbr\rnkwgt
+	   Counts: \rthnbr=5\lmnnbr\rnkwgt, \mmrusrnbr=3\lmnnbr\rnkwgt */
 	for(idx=0;idx<wgt_nbr_dim;idx++) wgt_lmn+=dmn_ss[idx_wgt_var[idx]]*dmn_wgt_map[idx];
 	
+	/* Operations: 2 add, 2 multiply, 0 pointer offset, 1 system memory copy
+	   Repetitions: \lmnnbr
+	   Counts: \rthnbr=4\lmnnbr, \mmrusrnbr=0, \mmrsysnbr=1 */
 	(void)memcpy(wgt_out_cp+var_lmn*wgt_typ_sz,wgt_cp+wgt_lmn*wgt_typ_sz,wgt_typ_sz);
 	
       } /* end loop over var_lmn */
@@ -289,8 +330,8 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
     *DO_CONFORM=True;
   } /* end if we had to stretch weight to fit variable */
   
-  if(*DO_CONFORM == -1){
-    (void)fprintf(stdout,"%s: ERROR *DO_CONFORM == -1 on exit from nco_var_cnf_dmn()\n",prg_nm_get());
+  if(MUST_CONFORM && !(*DO_CONFORM)){
+    (void)fprintf(stdout,"%s: ERROR Variables which MUST_CONFORM do not on exit from nco_var_cnf_dmn()\n",prg_nm_get());
     nco_exit(EXIT_FAILURE);
   } /* endif */
   
@@ -299,36 +340,43 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
   
 } /* end nco_var_cnf_dmn() */
 
-bool /* [flg] var_1 and var_2 conform after processing */
+nco_bool /* [flg] var_1 and var_2 conform after processing */
 ncap_var_cnf_dmn /* [fnc] Broadcast smaller variable into larger */
 (var_sct **var_1, /* I/O [ptr] First variable */
  var_sct **var_2) /* I/O [ptr] Second variable */
 {
-  /* Purpose: Return conforming variables. If this is not possible then die. 
-     Routine is a wrapper for nco_var_cnf_dmn() which does the hard work */
+  /* Purpose: Conform lesser ranked to greater ranked variable, so that
+     both variables are equal size on return.
+     If this is possible then return true, otherwise die.
+     Routine wraps nco_var_cnf_dmn(), which does the hard work
+     If an input variable is replaced by a broadcast version of itself,
+     then calling routine must free original version or it will leak. */
 
-  bool DO_CONFORM; /* [flg] Do var_1 and var_2 conform after processing? */
-  bool MUST_CONFORM=True; /* [flg] Must var_1 and var_2 conform? */
+  nco_bool DO_CONFORM; /* [flg] Do var_1 and var_2 conform after processing? */
+  nco_bool MUST_CONFORM=True; /* [flg] Must var_1 and var_2 conform? */
   var_sct *var_1_org; /* [ptr] Original location of var_1 */
   var_sct *var_2_org; /* [ptr] Original location of var_2 */
+  var_sct *var_tmp=NULL;
 
   var_1_org=*var_1; /* [ptr] Original location of var_1 */
   var_2_org=*var_2; /* [ptr] Original location of var_2 */
-
-  if(var_1_org->nbr_dim > var_2_org->nbr_dim) *var_2=nco_var_cnf_dmn(var_1_org,var_2_org,NULL,MUST_CONFORM,&DO_CONFORM); else *var_1=nco_var_cnf_dmn(var_2_org,var_1_org,NULL,MUST_CONFORM,&DO_CONFORM);
   
-  /* fxm: Memory leak?
-     nco_var_cnf_dmn() does not do its own memory handling
-     If original var_1 or var_2 was overwritten (replaced by conforming variable),
-     then original must be free()'d now before its location is lost.
-     Test for equality between pointers on entry and exit, and
-     nco_var_free() calling variable if it changed 
-     20020114: Freeing variables here causes core dump, not sure why */
-  /*  if(*var_1 != var_1_org) var_1_org=nco_var_free(var_1_org);*/
-  /*  if(*var_2 != var_2_org) var_2_org=nco_var_free(var_2_org);*/
+  if(var_1_org->nbr_dim > var_2_org->nbr_dim){
+    var_tmp=nco_var_cnf_dmn(var_1_org,var_2_org,var_tmp,MUST_CONFORM,&DO_CONFORM);
+    if(var_2_org != var_tmp){
+      var_2_org=nco_var_free(var_2_org);
+      *var_2=var_tmp;
+    } /* endif replace var_2 */
+  }else{ 
+    var_tmp=nco_var_cnf_dmn(var_2_org,var_1_org,var_tmp,MUST_CONFORM,&DO_CONFORM);
+    if(var_1_org != var_tmp){
+      var_1_org=nco_var_free(var_1_org);
+      *var_1=var_tmp;
+    } /* endif replace var_1 */
+  } /* endif var_1 > var_2 */
 
   if(!DO_CONFORM){
-    (void)fprintf(stderr,"%s: Variables do not have have conforming dimensions. Cannot proceed with operation\n",prg_nm_get());
+    (void)fprintf(stderr,"%s: ncap_var_cnf_dmn() reports that variables %s and %s do not have have conforming dimensions. Cannot proceed with operation\n",prg_nm_get(),(*var_1)->nm,(*var_2)->nm);
     nco_exit(EXIT_FAILURE);
   } /* endif */
 
@@ -342,8 +390,8 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_rdr), /* I [sct] List of dimension structures in new order */
  const int dmn_rdr_nbr, /* I [nbr] Number of dimension structures in structure list */
  int * const dmn_idx_out_in, /* O [idx] Dimension correspondence, output->input */
- const bool * const dmn_rvr_rdr, /* I [idx] Reverse dimension */
- bool * const dmn_rvr_in) /* O [idx] Reverse dimension */
+ const nco_bool * const dmn_rvr_rdr, /* I [idx] Reverse dimension */
+ nco_bool * const dmn_rvr_in) /* O [idx] Reverse dimension */
 {
   /* Purpose: Re-order dimensions in a given variable
      dmn_rdr contains new dimension order for dimensions
@@ -404,6 +452,8 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
      Host routine passes dimension reversing flags in dmn_rvr_rdr
      Dimensions may be re-ordered, reversed, or both */
   
+  /* 20070509 CEWI RUVICFFU: dmn_idx_rdr_in */
+
   const char fnc_nm[]="nco_var_dmn_rdr_mtd()"; /* [sng] Function name */
   
   char *rec_dmn_nm_out=NULL; /* [sng] Name of record dimension, if any, required by re-order */
@@ -411,10 +461,10 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
   dmn_sct **dmn_in=NULL; /* [sct] List of dimension structures in input order */
   dmn_sct **dmn_out; /* [sct] List of dimension structures in output order */
   
-  int dmn_idx_in_shr[NC_MAX_DIMS]; /* [idx] Dimension correspondence, input->share  Purely diagnostic */
+  int dmn_idx_in_shr[NC_MAX_DIMS]; /* [idx] Dimension correspondence, input->share Purely diagnostic */
   int dmn_idx_in_out[NC_MAX_DIMS]; /* [idx] Dimension correspondence, input->output */
   int dmn_idx_in_rdr[NC_MAX_DIMS]; /* [idx] Dimension correspondence, input->re-order NB: Purely diagnostic */
-  int dmn_idx_rdr_in[NC_MAX_DIMS]; /* [idx] Dimension correspondence, re-order->input NB: Purely diagnostic */
+  /* int dmn_idx_rdr_in[NC_MAX_DIMS]; */ /* [idx] Dimension correspondence, re-order->input NB: Purely diagnostic */
   int dmn_idx_shr_rdr[NC_MAX_DIMS]; /* [idx] Dimension correspondence, share->re-order */	  
   int dmn_idx_shr_in[NC_MAX_DIMS]; /* [idx] Dimension correspondence, share->input */	  
   int dmn_idx_shr_out[NC_MAX_DIMS]; /* [idx] Dimension correspondence, share->output */	  
@@ -435,11 +485,11 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
   dmn_out_nbr=var_out->nbr_dim;
 
   /* Initialize dimension maps to missing_value to aid debugging */
-  if(dbg_lvl_get() > 3){
+  if(dbg_lvl_get() == nco_dbg_old){
     for(dmn_out_idx=0;dmn_out_idx<dmn_out_nbr;dmn_out_idx++)
       dmn_idx_out_in[dmn_out_idx]=idx_err;
-    for(dmn_rdr_idx=0;dmn_rdr_idx<dmn_rdr_nbr;dmn_rdr_idx++)
-      dmn_idx_rdr_in[dmn_rdr_idx]=idx_err;
+    /* for(dmn_rdr_idx=0;dmn_rdr_idx<dmn_rdr_nbr;dmn_rdr_idx++)
+       dmn_idx_rdr_in[dmn_rdr_idx]=idx_err; */
     for(dmn_in_idx=0;dmn_in_idx<dmn_in_nbr;dmn_in_idx++){
       dmn_idx_in_shr[dmn_in_idx]=idx_err;
       dmn_idx_in_rdr[dmn_in_idx]=idx_err;
@@ -470,7 +520,7 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
       /* ...by comparing names, not dimension IDs... */
       if(!strcmp(var_in->dim[dmn_in_idx]->nm,dmn_rdr[dmn_rdr_idx]->nm)){
 	dmn_idx_in_rdr[dmn_in_idx]=dmn_rdr_idx;
-	dmn_idx_rdr_in[dmn_rdr_idx]=dmn_in_idx;
+	/* dmn_idx_rdr_in[dmn_rdr_idx]=dmn_in_idx; */
 	dmn_idx_shr_rdr[dmn_shr_nbr]=dmn_rdr_idx;
 	dmn_idx_shr_in[dmn_shr_nbr]=dmn_in_idx;
 	dmn_idx_in_shr[dmn_in_idx]=dmn_shr_nbr;
@@ -591,23 +641,25 @@ nco_var_dmn_rdr_val /* [fnc] Change dimension ordering of variable values */
 (const var_sct * const var_in, /* I [ptr] Variable with metadata and data in original order */
  var_sct * const var_out, /* I/O [ptr] Variable whose data will be re-ordered */
  const int * const dmn_idx_out_in, /* I [idx] Dimension correspondence, output->input */
- const bool * const dmn_rvr_in) /* I [idx] Reverse dimension */
+ const nco_bool * const dmn_rvr_in) /* I [idx] Reverse dimension */
 {
   /* Purpose: Re-order values in given variable according to supplied dimension map
      Description of re-ordering concepts is in nco_var_dmn_rdr_mtd()
      Description of actual re-ordering algorithm is in nco_var_dmn_rdr_val() */
 
-  bool IDENTITY_REORDER=False; /* [flg] User requested identity re-ordering */
+  /* 20070509 CEWI RUVICFFU: dmn_in, dmn_id_out */
+
+  nco_bool IDENTITY_REORDER=False; /* [flg] User requested identity re-ordering */
 
   char *val_in_cp; /* [ptr] Input data location as char pointer */
   char *val_out_cp; /* [ptr] Output data location as char pointer */
   
   const char fnc_nm[]="nco_var_dmn_rdr_val()"; /* [sng] Function name */
 
-  dmn_sct **dmn_in=NULL; /* [sct] List of dimension structures in input order */
+  /* dmn_sct **dmn_in=NULL; */ /* [sct] List of dimension structures in input order */
   dmn_sct **dmn_out; /* [sct] List of dimension structures in output order */
-  
-  int *dmn_id_out; /* [id] Contiguous vector of dimension IDs */
+
+  /* int *dmn_id_out; */ /* [id] Contiguous vector of dimension IDs */
   int dmn_idx; /* [idx] Index over dimensions */
   int dmn_in_idx; /* [idx] Counting index for dmn_in */
   int dmn_in_nbr; /* [nbr] Number of dimensions in input variable */
@@ -636,10 +688,10 @@ nco_var_dmn_rdr_val /* [fnc] Change dimension ordering of variable values */
      2. var_out->val buffer has been allocated (calling routine must do this) */
 
   /* Get ready to re-order */
-  dmn_id_out=var_out->dmn_id;
+  /* dmn_id_out=var_out->dmn_id; */
+  /* dmn_in=var_in->dim; */
   dmn_in_nbr_m1=dmn_in_nbr-1;
   dmn_out=var_out->dim;
-  dmn_in=var_in->dim;
   typ_sz=nco_typ_lng(var_out->type);
   val_in_cp=(char *)var_in->val.vp;
   val_out_cp=(char *)var_out->val.vp;
