@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_dmn.h,v 1.22 2004/08/05 04:27:24 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_dmn.h,v 1.29 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Conform dimensions */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_cnf_dmn.h" *//* Conform dimensions */
@@ -38,10 +38,10 @@ nco_var_cnf_dmn /* [fnc] Stretch second variable to match dimensions of first va
 (const var_sct * const var, /* I [ptr] Pointer to variable structure to serve as template */
  var_sct * const wgt, /* I [ptr] Pointer to variable structure to make conform to var */
  var_sct *wgt_crr, /* I/O [ptr] pointer to existing conforming variable structure, if any (destroyed when does not conform to var) */
- const bool MUST_CONFORM, /* I [flg] Must wgt and var conform? */
- bool *DO_CONFORM); /* O [flg] Do wgt and var conform? */
+ const nco_bool MUST_CONFORM, /* I [flg] Must wgt and var conform? */
+ nco_bool *DO_CONFORM); /* O [flg] Do wgt and var conform? */
 
-bool /* [flg] var_1 and var_2 conform after processing */
+nco_bool /* [flg] var_1 and var_2 conform after processing */
 ncap_var_cnf_dmn /* [fnc] Broadcast smaller variable into larger */
 (var_sct **var_1, /* I/O [ptr] First variable */
  var_sct **var_2); /* I/O [ptr] Second variable */
@@ -59,15 +59,15 @@ nco_var_dmn_rdr_mtd /* [fnc] Change dimension ordering of variable metadata */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_rdr), /* I [sct] List of dimension structures in new order */
  const int dmn_rdr_nbr, /* I [nbr] Number of dimension structures in structure list */
  int * const dmn_idx_out_in, /* O [idx] Dimension correspondence, output->input */
- const bool * const dmn_rvr_rdr, /* I [idx] Reverse dimension */
- bool * const dmn_rvr_in); /* O [idx] Reverse dimension */
+ const nco_bool * const dmn_rvr_rdr, /* I [idx] Reverse dimension */
+ nco_bool * const dmn_rvr_in); /* O [idx] Reverse dimension */
 
 int /* O [enm] Return success code */
 nco_var_dmn_rdr_val /* [fnc] Change dimension ordering of variable values */
 (const var_sct * const var_in, /* I [ptr] Variable with metadata and data in original order */
  var_sct * const var_out, /* I/O [ptr] Variable whose data will be re-ordered */
  const int * const dmn_idx_out_in, /* I [idx] Dimension correspondence, output->input */
- const bool * const dmn_rvr_in); /* I [idx] Reverse dimension */
+ const nco_bool * const dmn_rvr_in); /* I [idx] Reverse dimension */
 
 #ifdef __cplusplus
 } /* end extern "C" */
