@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.19 2004/09/06 22:48:48 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnf_typ.h,v 1.24 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Conform variable types */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_cnf_typ.h" *//* Conform variable types */
@@ -51,7 +51,7 @@ nco_cnv_var_typ_dsk  /* [fnc] Revert variable to on-disk type */
 var_sct * /* O [sct] Pointer to variable structure of type var_out_typ */
 nco_var_cnf_typ /* [fnc] Return copy of input variable typecast to desired type */
 (const nc_type var_out_typ, /* I [enm] Type to convert variable structure to */
- var_sct *var_in); /* I/O [enm] Pointer to variable structure (may be destroyed) */
+ var_sct * const var_in); /* I/O [enm] Pointer to variable structure (may be destroyed) */
 
 var_sct * /* O [sct] Variable with mss_val converted to typ_upk */
 nco_cnv_mss_val_typ  /* [fnc] Convert missing_value, if any, to mss_val_out_typ */
