@@ -1,19 +1,20 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.c,v 1.6 2004/01/05 17:29:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.c,v 1.18 2007/02/23 21:59:28 zender Exp $ */
 
-/* Purpose: ARM conventions */
+/* Purpose: ARM conventions, e.g., http://www.arm.gov/data/time.stm */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_cnv_arm.h" /* ARM conventions */
 
-bool /* O [flg] File obeys ARM conventions */
-arm_inq /* O [fnc] Check if file obeys ARM conventions */
+nco_bool /* O [flg] File obeys ARM conventions */
+nco_cnv_arm_inq /* O [fnc] Check if file obeys ARM conventions */
 (const int nc_id) /* I [id] netCDF file ID */
 {
-  /* Purpose: Check whether file adheres to ARM time format */
-  bool ARM_FORMAT;
+  /* Purpose: Check whether file adheres to ARM time format defined
+     http://www.arm.gov/data/time.stm */
+  nco_bool CNV_ARM;
 
   const char time_sng[]="time"; /* CEWI */
   const char base_time_sng[]="base_time"; /* CEWI */
@@ -31,45 +32,51 @@ arm_inq /* O [fnc] Check if file obeys ARM conventions */
   
   /* All three IDs must be valid to handle ARM format */
   if(rcd != NC_NOERR){
-    ARM_FORMAT=False;
+    CNV_ARM=False;
   }else{
     if(dbg_lvl_get() > 0) (void)fprintf(stderr,"%s: CONVENTION File convention is DOE ARM\n",prg_nm_get()); 
-    ARM_FORMAT=True;
+    CNV_ARM=True;
   } /* end else */
 
-  return ARM_FORMAT;
-} /* end arm_inq */
+  return CNV_ARM;
+} /* end nco_cnv_arm_inq */
 
 double /* O [s] base_time + current time_offset */ 
-arm_time_mk /* [fnc] Return time corresponding to current time offset */
+nco_cnv_arm_time_mk /* [fnc] Return time corresponding to current time offset */
 (const int nc_id, /* I [id] netCDF file ID */
  const double time_offset) /* I [s] Current time offset */
 {
+  /* NB: nco_cnv_arm_time_mk() with same nc_id contains OpenMP critical region */
   /* Purpose: Return time corresponding to current time offset */
   double arm_time;
 
   int base_time_id;
   int rcd=NC_NOERR; /* [rcd] Return code */
 
-  nco_long base_time;
+  nco_int base_time;
 
   /* Find base_time variable (NC_INT: base UNIX time of file) */
   rcd=nco_inq_varid_flg(nc_id,"base_time",&base_time_id);
   if(rcd != NC_NOERR){
-    (void)fprintf(stderr,"%s: WARNING ARM file does not have variable \"base_time\", exiting arm_time_mk()...\n",prg_nm_get());
+    (void)fprintf(stderr,"%s: WARNING ARM file does not have variable \"base_time\", exiting nco_cnv_arm_time_mk()...\n",prg_nm_get());
     return -1;
   } /* end if */
-  (void)nco_get_var1(nc_id,base_time_id,0L,&base_time,NC_INT);
+
+  { /* begin potential OpenMP critical */
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+    (void)nco_get_var1(nc_id,base_time_id,0L,&base_time,NC_INT);
+  } /* end potential OpenMP critical */
   arm_time=base_time+time_offset;
 
   return arm_time;
-} /* end arm_time_mk() */
+} /* end nco_cnv_arm_time_mk() */
 
 void
-nco_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
+nco_cnv_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
 (const int nc_id, /* I [id] netCDF file ID */
- const nco_long base_time_srt) /* I [s] base_time of first input file */
+ const nco_int base_time_srt) /* I [s] base_time of first input file */
 {
+  /* NB: nco_cnv_arm_time_install() contains OpenMP critical region */
   /* Purpose: Add time variable to concatenated ARM files */
 
   const char att_long_name[]="UNIX time";
@@ -95,7 +102,7 @@ nco_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
   /* Find time_offset variable */
   rcd=nco_inq_varid_flg(nc_id,"time_offset",&time_offset_id);
   if(rcd != NC_NOERR){
-    (void)fprintf(stderr,"%s: WARNING ARM file does not have variable \"time_offset\", exiting nco_arm_time_install()...\n",prg_nm_get());
+    (void)fprintf(stderr,"%s: WARNING ARM file does not have variable \"time_offset\", exiting nco_cnv_arm_time_install()...\n",prg_nm_get());
     return;
   } /* endif */
 
@@ -118,7 +125,11 @@ nco_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
   /* If the time coordinate does not already exist, create it */
   time_offset=(double *)nco_malloc(cnt*nco_typ_lng(NC_DOUBLE));
 
-  (void)nco_get_vara(nc_id,time_offset_id,&srt,&cnt,(void *)time_offset,NC_DOUBLE);
+  { /* begin potential OpenMP critical */
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+    (void)nco_get_vara(nc_id,time_offset_id,&srt,&cnt,(void *)time_offset,NC_DOUBLE);
+  } /* end potential OpenMP critical */
+
   for(idx=0L;idx<cnt;idx++) time_offset[idx]+=base_time_srt;
 
   /* File must be in define mode */
@@ -135,25 +146,35 @@ nco_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
   /* Take file out of define mode */
   (void)nco_enddef(nc_id);
 
-  /* Write time variable */
-  (void)nco_put_vara(nc_id,time_id,&srt,&cnt,(void *)time_offset,NC_DOUBLE);
+  /* Block is always critical */
+  { /* begin OpenMP critical */
+    /* Write time variable */
+    (void)nco_put_vara(nc_id,time_id,&srt,&cnt,(void *)time_offset,NC_DOUBLE);
+  } /* end OpenMP critical */
 
   /* Free time_offset buffer */
   time_offset=(double *)nco_free(time_offset);
-} /* end nco_arm_time_install */
 
-nco_long /* O [s] Value of base_time variable */
-arm_base_time_get /* [fnc] Get base_time variable from ARM file */
+  return; /* 20050109: fxm added return to void function to squelch erroneous gcc-3.4.2 warning */ 
+} /* end nco_cnv_arm_time_install */
+
+nco_int /* O [s] Value of base_time variable */
+nco_cnv_arm_base_time_get /* [fnc] Get base_time variable from ARM file */
 (const int nc_id) /* I [id] netCDF file ID */
 {
+  /* NB: nco_cnv_arm_base_time_get() with same nc_id contains OpenMP critical region */
   /* Purpose: Get base_time variable from ARM file */
   int base_time_id;
 
-  nco_long base_time;
+  nco_int base_time;
 
   (void)nco_inq_varid(nc_id,"base_time",&base_time_id);
-  (void)nco_get_var1(nc_id,base_time_id,0L,&base_time,NC_INT);
+
+  /* Block is critical/thread-safe for identical/distinct in_id's */
+  { /* begin potential OpenMP critical */
+    (void)nco_get_var1(nc_id,base_time_id,0L,&base_time,NC_INT);
+  } /* end potential OpenMP critical */
 
   return base_time;
-} /* end arm_base_time_get */
+} /* end nco_cnv_arm_base_time_get */
 
