@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.h,v 1.8 2004/01/12 18:11:07 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_cnv_arm.h,v 1.15 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: ARM conventions */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_cnv_arm.h" *//* ARM conventions */
@@ -29,22 +29,22 @@
 extern "C" {
 #endif /* __cplusplus */
 
-bool /* O [flg] File obeys ARM conventions */
-arm_inq /* O [fnc] Check if file obeys ARM conventions */
+nco_bool /* O [flg] File obeys ARM conventions */
+nco_cnv_arm_inq /* O [fnc] Check if file obeys ARM conventions */
 (const int nc_id); /* I [id] netCDF file ID */
 
 double /* O [s] base_time + current time_offset */ 
-arm_time_mk /* [fnc] Return time corresponding to current time offset */
+nco_cnv_arm_time_mk /* [fnc] Return time corresponding to current time offset */
 (const int nc_id, /* I [id] netCDF file ID */
  const double time_offset); /* I [s] Current time offset */
 
 void
-nco_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
+nco_cnv_arm_time_install /* [fnc] Add time variable to concatenated ARM files */
 (const int nc_id, /* I [id] netCDF file ID */
- const nco_long base_time_srt); /* I [s] base_time of first input file */
+ const nco_int base_time_srt); /* I [s] base_time of first input file */
 
-nco_long /* O [s] Value of base_time variable */
-arm_base_time_get /* [fnc] Get base_time variable from ARM file */
+nco_int /* O [s] Value of base_time variable */
+nco_cnv_arm_base_time_get /* [fnc] Get base_time variable from ARM file */
 (const int nc_id); /* I [id] netCDF file ID */
 
 #ifdef __cplusplus
