@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.15 2004/06/18 01:21:05 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_ctl.h,v 1.26 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Program flow control functions */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_ctl.h" *//* Program flow control functions */
@@ -24,6 +24,9 @@
 #include <unistd.h> /* POSIX stuff */
 
 /* 3rd party vendors */
+#ifdef ENABLE_MPI
+#include <mpi.h> /* MPI definitions */
+#endif /* !ENABLE_MPI */
 #include <netcdf.h> /* netCDF definitions and C library */
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 
@@ -35,6 +38,9 @@
 extern "C" {
 #endif /* __cplusplus */
 
+const char * /* O [sng] Compiler and version */
+nco_cmp_get(void); /* [fnc] Return compiler and version */
+
 void
 nco_exit /* [fnc] Wrapper for exit() */
 (int rcd); /* I [enm] Return code */
@@ -42,17 +48,29 @@ nco_exit /* [fnc] Wrapper for exit() */
 void 
 nco_exit_gracefully(void); /* [fnc] Clean up timers, file descriptors, memory, then exit */
 
-bool /* [flg] Program does arithmetic */
+nco_bool /* [flg] Program does arithmetic */
 nco_is_rth_opr /* [fnc] Query whether program does arithmetic */
 (const int prg_id); /* [enm] Program ID */
 
-bool /* [flg] Program is multi-file operator */
+nco_bool /* [flg] Program is multi-file operator */
 nco_is_mlt_fl_opr /* [fnc] Query whether program is multi-file operator */
 (const int prg_id); /* [enm] Program ID */
 
+int /* O [rcd] Return code */
+nco_ddra /* [fnc] Count operations */
+(const char * const var_nm, /* I [sng] Variable name */
+ const char * const wgt_nm, /* I [sng] Weight name */
+ const ddra_info_sct * const ddra_info); /* I [sct] DDRA information */
+
+void 
+nco_dfl_case_tmr_typ_err(void); /* [fnc] Print error and exit for illegal switch(tmr_typ) case */
+
 void
 nco_lbr_vrs_prn(void); /* [fnc] Print netCDF library version */
 
+const char * /* O [sng] MPI implementation */
+nco_mpi_get(void); /* [fnc] Return MPI implementation */
+
 char * /* O [sng] nm_in stripped of any path (i.e., program name stub) */ 
 prg_prs /* [fnc] Strip program name to stub and return program ID */
 (const char * const nm_in, /* I [sng] Name of program, i.e., argv[0] (may include path prefix) */
