@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.15 2004/08/14 21:00:00 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.c,v 1.27 2007/02/23 21:59:28 zender Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_dmn_utl.h" /* Dimension utilities */
 
@@ -53,10 +53,10 @@ nco_dmn_dpl /* [fnc] Duplicate input dimension structure */
   /* Shallow copy structure */
   (void)memcpy((void *)dmn_cpy,dmn_vp,sizeof(dmn_sct));
 
-  /* fxm: Should copy name as well */
+  /* Make sure dmn_free() frees names when dimension is destructed */
+  if(dmn->nm != NULL) dmn_cpy->nm=(char *)strdup(dmn->nm);
 
   return dmn_cpy;
-
 } /* end nco_dmn_dpl() */
 
 dmn_sct * /* O [sct] Output dimension structure */
@@ -106,11 +106,24 @@ nco_dmn_fll /* [fnc] Create and return completed dmn_sct */
   return dmn;
 } /* end nco_dmn_fll() */
 
+dmn_sct * /* O [sct] Pointer to free'd dimension */
+nco_dmn_free /* [fnc] Free all memory associated with dimension structure */
+(dmn_sct *dmn) /* I/O [sct] Dimension to free */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with a dynamically allocated dimension structure */
+  dmn->nm=(char *)nco_free(dmn->nm);
+  /* Free structure pointer last */
+  dmn=(dmn_sct *)nco_free(dmn);
+
+  return NULL;
+} /* end nco_dmn_free() */
+
 void
 nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension structures */
 (dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
  const int nbr_dmn, /* I [nbr] Number of dimension structures in structure list */
- const lmt_sct * const lmt, /* I [sct] Dimension limit information */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_sct,lmt), /* I [sct] Dimension limit information */
  const int lmt_nbr) /* I [nbr] Number of dimensions with user-specified limits */
 {
   /* Purpose: Merge limit structure information into dimension structures */
@@ -119,14 +132,15 @@ nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension struct
   int lmt_idx;
 
   for(idx=0;idx<nbr_dmn;idx++){
-
-    /* Decide whether this dimension has any user-specified limits */
+    /* Does this dimension have user-specified limits? */
     for(lmt_idx=0;lmt_idx<lmt_nbr;lmt_idx++){
-      if(lmt[lmt_idx].id == dmn[idx]->id){
-	dmn[idx]->cnt=lmt[lmt_idx].cnt;
-	dmn[idx]->srt=lmt[lmt_idx].srt;
-	dmn[idx]->end=lmt[lmt_idx].end;
-	dmn[idx]->srd=lmt[lmt_idx].srd;
+      /* 20050707: Match on name not ID so nco_dmn_lmt_mrg() works with single
+	 limit list applied to any input file */
+      if(!strcmp(lmt[lmt_idx]->nm,dmn[idx]->nm)){
+	dmn[idx]->cnt=lmt[lmt_idx]->cnt;
+	dmn[idx]->srt=lmt[lmt_idx]->srt;
+	dmn[idx]->end=lmt[lmt_idx]->end;
+	dmn[idx]->srd=lmt[lmt_idx]->srd;
 	break;
       } /* end if */
     } /* end loop over lmt_idx */
@@ -142,7 +156,7 @@ nco_dmn_lst_ass_var /* [fnc] Create list of all dimensions associated with input
 {
   /* Purpose: Create list of all dimensions associated with input variable list */
 
-  bool dmn_has_been_placed_on_list;
+  nco_bool dmn_has_been_placed_on_list;
 
   char dmn_nm[NC_MAX_NAME];
 
@@ -200,31 +214,44 @@ nco_dmn_lst_ass_var /* [fnc] Create list of all dimensions associated with input
   /* We now have final list of dimensions to extract. Phew. */
   
   /* Free unused space in output dimension list */
-  /* valgrind fxm: 20040814 valgrind reports: Are obsolete names free()'d before realloc()?
-     ==31046== 112 bytes in 1 blocks are definitely lost in loss record 2 of 6
-     ==31046==    at 0x1B9079FF: realloc (vg_replace_malloc.c:197)
-     ==31046==    by 0x8056084: nco_realloc (nco_mmr.c:235)
-     ==31046==    by 0x8051490: nco_dmn_lst_ass_var (nco_dmn_utl.c:203)
-     ==31046==    by 0x804A057: main (ncpdq.c:327) */
   dmn=(nm_id_sct *)nco_realloc((void *)dmn,*nbr_dmn*sizeof(nm_id_sct));
   
   return dmn;
 } /* end nco_dmn_lst_ass_var() */
 
+dmn_sct ** /* O [sct] Pointer to free'd structure list */
+nco_dmn_lst_free /* [fnc] Free memory associated with dimension structure list */
+(dmn_sct **dmn_lst, /* I/O [sct] Dimension structure list to free */
+ const int dmn_nbr) /* I [nbr] Number of dimension structures in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated dimension structure list */
+  int idx;
+
+  for(idx=0;idx<dmn_nbr;idx++){
+    dmn_lst[idx]=nco_dmn_free(dmn_lst[idx]);
+  } /* end loop over idx */
+
+  /* Free structure pointer last */
+  dmn_lst=(dmn_sct **)nco_free(dmn_lst);
+
+  return dmn_lst;
+} /* end nco_dmn_lst_free() */
+
 nm_id_sct * /* O [sct] Dimension list */
 nco_dmn_lst_mk /* [fnc] Attach dimension IDs to dimension list */
 (const int nc_id, /* I [id] netCDF file ID */
  CST_X_PTR_CST_PTR_CST_Y(char,dmn_lst_in), /* I [sng] User-specified list of dimension names */
  const int nbr_dmn) /* I [nbr] Total number of dimensions in list */
 {
-  /* Purpose: Create list of dimension name-id structures from list of dimension name strings */
+  /* Purpose: Create list of dimension name-ID structures from list of dimension name strings */
   int idx;
 
   nm_id_sct *dmn_lst;
   
   dmn_lst=(nm_id_sct *)nco_malloc(nbr_dmn*sizeof(nm_id_sct));
   for(idx=0;idx<nbr_dmn;idx++){
-    /* See if requested dimension is in input file */
+    /* Copy name and then get requested dimension ID from input file */
     dmn_lst[idx].nm=(char *)strdup(dmn_lst_in[idx]);
     (void)nco_inq_dimid(nc_id,dmn_lst[idx].nm,&dmn_lst[idx].id);
   } /* end loop over idx */
