@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.13 2004/07/26 17:45:49 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_dmn_utl.h,v 1.21 2007/02/23 21:59:29 zender Exp $ */
 
 /* Purpose: Dimension utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_dmn_utl.h" *//* Dimension utilities */
@@ -49,13 +49,22 @@ nco_dmn_fll /* [fnc] Create and return completed dmn_sct */
  const int dmn_id, /* I [id] Dimension ID */
  const char * const dmn_nm); /* I [sng] Dimension name */
 
+dmn_sct * /* O [sct] Pointer to free'd dimension */
+nco_dmn_free /* [fnc] Free all memory associated with dimension structure */
+(dmn_sct *dmn); /* I/O [sct] Dimension to free */
+
 void
 nco_dmn_lmt_mrg /* [fnc] Merge limit structure information into dimension structures */
 (dmn_sct ** const dmn, /* I [sct] Dimension structures to modify */
  const int nbr_dmn, /* I [nbr] Number of dimension structures in structure list */
- const lmt_sct * const lmt, /* I [sct] Dimension limit information */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_sct,lmt), /* I [sct] Dimension limit information */
  const int lmt_nbr); /* I [nbr] Number of dimensions with user-specified limits */
 
+dmn_sct ** /* O [sct] Pointer to free'd structure list */
+nco_dmn_lst_free /* [fnc] Free memory associated with dimension structure list */
+(dmn_sct **dmn_lst, /* I/O [sct] Dimension structure list to free */
+ const int dmn_nbr); /* I [nbr] Number of dimension structures in list */
+
 nm_id_sct * /* O [sct] Dimension list */
 nco_dmn_lst_mk /* [fnc] Attach dimension IDs to dimension list */
 (const int nc_id, /* I [id] netCDF file ID */
