@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.22 2004/07/09 16:36:11 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_fl_utl.h,v 1.35 2007/02/23 21:59:29 zender Exp $ */
 
 /* Purpose: File manipulation */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_fl_utl.h" *//* File manipulation */
@@ -45,6 +45,16 @@
 extern "C" {
 #endif /* __cplusplus */
 
+int /* O [enm] Mode flag for nco_create() call */
+nco_create_mode_mrg /* [fnc] Merge clobber mode with user-specified file format */
+(const int clobber_mode, /* I [enm] Clobber mode (NC_CLOBBER or NC_NOCLOBBER) */
+ const int fl_out_fmt); /* I [enm] Output file format */
+
+int /* [rcd] Return code */
+nco_create_mode_prs /* [fnc] Parse user-specified file format */
+(const char * const fl_fmt_sng, /* [sng] User-specified file format string */
+ int * const fl_fmt_enm); /* I [enm] Output file format */
+
 void
 nco_fl_cmp_err_chk(void); /* [fnc] Perform error checking on file */
 
@@ -64,13 +74,13 @@ nco_fl_lst_mk /* [fnc] Create file list from command line positional arguments *
  int arg_crr, /* I [idx] Index of current argument */
  int * const fl_nbr, /* O [nbr] Number of files in input file list */
  char ** const fl_out, /* I/O [sng] Name of output file */
- bool *FL_LST_IN_FROM_STDIN); /* O [flg] fl_lst_in comes from stdin */
+ nco_bool *FL_LST_IN_FROM_STDIN); /* O [flg] fl_lst_in comes from stdin */
 
 char * /* O [sng] Filename of locally available file */
 nco_fl_mk_lcl /* [fnc] Retrieve input file and return local filename */
 (char *fl_nm, /* I/O [sng] Current filename, if any (destroyed) */
  const char * const fl_pth_lcl, /* I [sng] Local storage area for files retrieved from remote locations */
- int * const FILE_RETRIEVED_FROM_REMOTE_LOCATION); /* O [flg] File was retrieved from remote location */
+ nco_bool * const FILE_RETRIEVED_FROM_REMOTE_LOCATION); /* O [flg] File was retrieved from remote location */
 
 void
 nco_fl_mv /* [fnc] Move first file to second */
@@ -90,8 +100,9 @@ nco_fl_nm_prs /* [fnc] Construct file name from input arguments */
 char * /* O [sng] Name of temporary file actually opened */
 nco_fl_out_open /* [fnc] Open output file subject to availability and user input */
 (const char * const fl_out, /* I [sng] Name of file to open */
- const bool FORCE_APPEND, /* I [flg] Append to existing file, if any */
- const bool FORCE_OVERWRITE, /* I [flg] Overwrite existing file, if any */
+ const nco_bool FORCE_APPEND, /* I [flg] Append to existing file, if any */
+ const nco_bool FORCE_OVERWRITE, /* I [flg] Overwrite existing file, if any */
+ const int fl_out_fmt, /* I [enm] Output file format */
  int * const out_id); /* O [id] File ID */
 
 void
