@@ -1,54 +1,121 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.c,v 1.30 2004/08/20 22:24:32 rorik Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.c,v 1.57 2007/05/25 06:05:29 zender Exp $ */
 
 /* Purpose: Hyperslab limits */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_lmt.h" /* Hyperslab limits */
 
-lmt_sct /* [sct] Limit structure for dimension */
+lmt_sct * /* O [sct] Pointer to free'd structure */
+nco_lmt_free /* [fnc] Free memory associated with limit structure */
+(lmt_sct *lmt) /* I/O [sct] Limit structure to free */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated limit structure */
+  lmt->nm=(char *)nco_free(lmt->nm);
+  lmt->min_sng=(char *)nco_free(lmt->min_sng);
+  lmt->max_sng=(char *)nco_free(lmt->max_sng);
+  lmt->srd_sng=(char *)nco_free(lmt->srd_sng);
+
+  lmt=(lmt_sct *)nco_free(lmt);
+
+  return lmt;
+} /* end nco_lmt_free() */
+
+lmt_sct ** /* O [sct] Pointer to free'd structure list */
+nco_lmt_lst_free /* [fnc] Free memory associated with limit structure list */
+(lmt_sct **lmt_lst, /* I/O [sct] Limit structure list to free */
+ const int lmt_nbr) /* I [nbr] Number of limit structures in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated limit structure list */
+  int idx;
+
+  for(idx=0;idx<lmt_nbr;idx++) lmt_lst[idx]=nco_lmt_free(lmt_lst[idx]);
+
+  /* Free structure pointer last */
+  lmt_lst=(lmt_sct **)nco_free(lmt_lst);
+
+  return lmt_lst;
+} /* end nco_lmt_lst_free() */
+
+lmt_all_sct * /* O [sct] Pointer to free'd structure */
+nco_lmt_all_free /* [fnc] Free memory associated with limit structure */
+(lmt_all_sct *lmt_all) /* I/O [sct] Limit structure to free */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated lmt_all structure */
+  lmt_all->dmn_nm=(char *)nco_free(lmt_all->dmn_nm);
+  /* NB: lmt_dmn[idx] are free'd by nco_lmt_lst_free() in calling routine */
+  lmt_all->lmt_dmn=(lmt_sct **)nco_free(lmt_all->lmt_dmn);
+
+  lmt_all=(lmt_all_sct *)nco_free(lmt_all);
+
+  return lmt_all;
+} /* end nco_lmt_all_free() */
+
+lmt_all_sct ** /* O [sct] Pointer to free'd structure list */
+nco_lmt_all_lst_free /* [fnc] Free memory associated with lmt_all structure list */
+(lmt_all_sct **lmt_all_lst, /* I/O [sct] Limit structure list to free */
+ const int lmt_all_nbr) /* I [nbr] Number of limit strucgtures in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated limit structure list */
+  int idx;
+
+  for(idx=0;idx<lmt_all_nbr;idx++) lmt_all_lst[idx]=nco_lmt_all_free(lmt_all_lst[idx]);
+
+  /* Free structure pointer last */
+  lmt_all_lst=(lmt_all_sct **)nco_free(lmt_all_lst);
+
+  return lmt_all_lst;
+} /* end nco_lmt_all_lst_free() */
+
+lmt_sct * /* [sct] Limit structure for dimension */
 nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension */
 (const int nc_id, /* I [idx] netCDF file ID */
  const int dmn_id, /* I [idx] ID of dimension for this limit structure */
- const lmt_sct * const lmt, /* I [sct] Array of limit structures from nco_lmt_evl() */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_sct,lmt), /* I [sct] Array of limit structures from nco_lmt_evl() */ 
  int lmt_nbr, /* I [nbr] Number of limit structures */
- const bool FORTRAN_IDX_CNV) /* I [flg] Hyperslab indices obey Fortran convention */
+ const nco_bool FORTRAN_IDX_CNV) /* I [flg] Hyperslab indices obey Fortran convention */
 {
-  /* Purpose: Create stand-alone limit structure just for given dimension 
-     nco_lmt_sct_mk() is called by ncra() to generate limit structure for record dimension */
+  /* Purpose: Create stand-alone limit structure just for given dimension
+     ncra calls nco_lmt_sct_mk() to generate record dimension limit structure */
   
   int idx;
   int rcd; /* [rcd] Return code */
   
-  lmt_sct lmt_dim;
+  lmt_sct *lmt_dim;
   
+  lmt_dim=(lmt_sct *)nco_malloc(sizeof(lmt_sct));
+
   /* Initialize defaults to False, override later if warranted */
-  lmt_dim.is_usr_spc_lmt=False; /* True if any part of limit is user-specified, else False */
-  lmt_dim.is_usr_spc_max=False; /* True if user-specified, else False */
-  lmt_dim.is_usr_spc_min=False; /* True if user-specified, else False */
+  lmt_dim->is_usr_spc_lmt=False; /* True if any part of limit is user-specified, else False */
+  lmt_dim->is_usr_spc_max=False; /* True if user-specified, else False */
+  lmt_dim->is_usr_spc_min=False; /* True if user-specified, else False */
   /* rec_skp_nsh_spf is used for record dimension in multi-file operators */
-  lmt_dim.rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
+  lmt_dim->rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
   
   for(idx=0;idx<lmt_nbr;idx++){
     /* Copy user-specified limits, if any */
-    if(lmt[idx].id == dmn_id){
-      lmt_dim.is_usr_spc_lmt=True; /* True if any part of limit is user-specified, else False */
-      if(lmt[idx].max_sng == NULL){
-	lmt_dim.max_sng=NULL;
+    if(lmt[idx]->id == dmn_id){
+      lmt_dim->is_usr_spc_lmt=True; /* True if any part of limit is user-specified, else False */
+      if(lmt[idx]->max_sng == NULL){
+	lmt_dim->max_sng=NULL;
       }else{
-	lmt_dim.max_sng=(char *)strdup(lmt[idx].max_sng);
-	lmt_dim.is_usr_spc_max=True; /* True if user-specified, else False */
+	lmt_dim->max_sng=(char *)strdup(lmt[idx]->max_sng);
+	lmt_dim->is_usr_spc_max=True; /* True if user-specified, else False */
       } /* end if */
-      if(lmt[idx].min_sng == NULL){
-	lmt_dim.min_sng=NULL;
+      if(lmt[idx]->min_sng == NULL){
+	lmt_dim->min_sng=NULL;
       }else{
-	lmt_dim.min_sng=(char *)strdup(lmt[idx].min_sng);
-	lmt_dim.is_usr_spc_min=True; /* True if user-specified, else False */
+	lmt_dim->min_sng=(char *)strdup(lmt[idx]->min_sng);
+	lmt_dim->is_usr_spc_min=True; /* True if user-specified, else False */
       } /* end if */
-      if(lmt[idx].srd_sng != NULL) lmt_dim.srd_sng=(char *)strdup(lmt[idx].srd_sng); else lmt_dim.srd_sng=NULL;
-      lmt_dim.nm=(char *)strdup(lmt[idx].nm);
+      if(lmt[idx]->srd_sng != NULL) lmt_dim->srd_sng=(char *)strdup(lmt[idx]->srd_sng); else lmt_dim->srd_sng=NULL;
+      lmt_dim->nm=(char *)strdup(lmt[idx]->nm);
       break;
     } /* end if */
   } /* end loop over idx */
@@ -68,8 +135,8 @@ nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension *
       nco_exit(EXIT_FAILURE);
     } /* end if */
     
-    lmt_dim.nm=(char *)strdup(dmn_nm);
-    lmt_dim.srd_sng=NULL;
+    lmt_dim->nm=(char *)strdup(dmn_nm);
+    lmt_dim->srd_sng=NULL;
     /* Generate min and max strings to look as if user had specified them
        Adjust accordingly if FORTRAN_IDX_CNV was requested for other dimensions
        These sizes will later be decremented in nco_lmt_evl() where all information
@@ -81,13 +148,15 @@ nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension *
        dimension.
        Then, when nco_lmt_sct_mk() creates the record dimension structure, it must
        be created consistently with the FORTRAN_IDX_CNV flag for the other dimensions.
-       In order to do that, I must fill in the max_sng, min_sng, and srd_sng
-       arguments with strings as if they had been read from the keyboard.
-       Another solution would be to add a flag to lmt_sct indicating whether this
+       In order to do that, fill in max_sng, min_sng, and srd_sng
+       arguments with strings as if they had been read from keyboard.
+       An alternate solution is to add flag to lmt_sct indicating whether this
        limit struct had been automatically generated and then act accordingly. */
     /* Decrement cnt to C index value if necessary */
     if(!FORTRAN_IDX_CNV) cnt--; 
     if(cnt < 0L){
+      if(cnt == -1L) (void)fprintf(stdout,"%s: ERROR nco_lmt_sct_mk() reports record variable exists and is size zero, i.e., has no records yet.\n",prg_nm_get());
+      (void)fprintf(stdout,"%s: HINT: Perform record-oriented operations only after file has valid records.\n",prg_nm_get());
       (void)fprintf(stdout,"%s: cnt < 0 in nco_lmt_sct_mk()\n",prg_nm_get());
       nco_exit(EXIT_FAILURE);
     } /* end if */
@@ -95,12 +164,12 @@ nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension *
        Adding 1 is required for cnt=10,100,1000... */
     if(cnt < 10L) max_sng_sz=1; else max_sng_sz=1+(int)ceil(log10((double)cnt));
     /* Add one for NUL terminator */
-    lmt_dim.max_sng=(char *)nco_malloc(sizeof(char)*(max_sng_sz+1));
-    (void)sprintf(lmt_dim.max_sng,"%ld",cnt);
+    lmt_dim->max_sng=(char *)nco_malloc(sizeof(char)*(max_sng_sz+1));
+    (void)sprintf(lmt_dim->max_sng,"%ld",cnt);
     if(FORTRAN_IDX_CNV){
-      lmt_dim.min_sng=(char *)strdup("1");
+      lmt_dim->min_sng=(char *)strdup("1");
     }else{
-      lmt_dim.min_sng=(char *)strdup("0");
+      lmt_dim->min_sng=(char *)strdup("0");
     } /* end else */
   } /* end if user did not explicity specify limits for this dimension */
   
@@ -113,15 +182,15 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
 (int nc_id, /* I [idx] netCDF file ID */
  lmt_sct *lmt_ptr, /* I/O [sct] Structure from nco_lmt_prs() or from nco_lmt_sct_mk() to hold dimension limit information */
  long cnt_crr, /* I [nbr] Number of valid records already processed (only used for record dimensions in multi-file operators) */
- bool FORTRAN_IDX_CNV) /* I [flg] Hyperslab indices obey Fortran convention */
+ nco_bool FORTRAN_IDX_CNV) /* I [flg] Hyperslab indices obey Fortran convention */
 {
-  /* Threads: Routine is thread-unsafe */
+  /* NB: nco_lmt_evl() with same nc_id contains OpenMP critical region */
   /* Purpose: Take parsed list of dimension names, minima, and
      maxima strings and find appropriate indices into dimensions 
      for formulation of dimension start and count vectors, or fail trying. */
   
-  bool flg_no_data=False; /* True if file contains no data for hyperslab */
-  bool rec_dmn_and_mlt_fl_opr=False; /* True if record dimension in multi-file operator */
+  nco_bool flg_no_data=False; /* True if file contains no data for hyperslab */
+  nco_bool rec_dmn_and_mlt_fl_opr=False; /* True if record dimension in multi-file operator */
   
   dmn_sct dim;
   
@@ -239,13 +308,20 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
     (void)nco_inq_vartype(nc_id,dim.cid,&dim.type);
     
     /* Warn when coordinate type is weird */
-    if(dim.type == NC_BYTE || dim.type == NC_CHAR) (void)fprintf(stderr,"\n%s: WARNING Coordinate %s is type %s. Dimension truncation is unpredictable.\n",prg_nm_get(),lmt.nm,nco_typ_sng(dim.type));
+    if(dim.type == NC_BYTE || dim.type == NC_UBYTE || dim.type == NC_CHAR || dim.type == NC_STRING) (void)fprintf(stderr,"\n%s: WARNING Coordinate %s is type %s. Dimension truncation is unpredictable.\n",prg_nm_get(),lmt.nm,nco_typ_sng(dim.type));
     
     /* Allocate enough space to hold coordinate */
     dim.val.vp=(void *)nco_malloc(dmn_sz*nco_typ_lng(dim.type));
     
-    /* Retrieve coordinate */
-    nco_get_vara(nc_id,dim.cid,&dmn_srt,&dmn_sz,dim.val.vp,dim.type);
+#ifdef _OPENMP
+#pragma omp critical
+#endif /* _OPENMP */
+    { /* begin OpenMP critical */
+      /* Block is critical for identical in_id's
+	 Block is thread-safe for distinct in_id's */
+      /* Retrieve this coordinate */
+      nco_get_vara(nc_id,dim.cid,&dmn_srt,&dmn_sz,dim.val.vp,dim.type);
+    } /* end OpenMP critical */
     
     /* Convert coordinate to double-precision if neccessary */
     if(dim.type != NC_DOUBLE){
@@ -263,8 +339,14 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
       case NC_DOUBLE: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.dp[idx];} break; 
       case NC_INT: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.lp[idx];} break;
       case NC_SHORT: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.sp[idx];} break;
-      case NC_CHAR: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.cp[idx];} break;
+      case NC_USHORT: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.usp[idx];} break;
+      case NC_UINT: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.uip[idx];} break;
+      case NC_INT64: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.i64p[idx];} break;
+      case NC_UINT64: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.ui64p[idx];} break;
       case NC_BYTE: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.bp[idx];} break;
+      case NC_UBYTE: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.ubp[idx];} break;
+      case NC_CHAR: for(idx=0L;idx<dmn_sz;idx++) {dmn_val_dp[idx]=old_val.cp[idx];} break;
+      case NC_STRING: break; /* Do nothing */
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */
       
@@ -752,15 +834,15 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
   
   if(lmt.srt > lmt.end && !flg_no_data){
     if(prg_id != ncks) (void)fprintf(stderr,"WARNING: Possible instance of Schweitzer data hole requiring better diagnostics TODO #148\n");
-    if(prg_id != ncks) (void)fprintf(stderr,"HINT: If operation fails, try hyperslabbing wrapped dimension using ncks first, and then apply %s to the resulting file\n",prg_nm_get());
+    if(prg_id != ncks) (void)fprintf(stderr,"HINT: If operation fails, try multislabbing (http://nco.sf.net/nco.html#msa) wrapped dimension using ncks first, and then apply %s to the resulting file\n",prg_nm_get());
   } /* end dbg */
   
 } /* end nco_lmt_evl() */
 
-lmt_sct * /* O [sct] Structure with user-specified strings for min and max limits */
+lmt_sct ** /* O [sct] Structure list with user-specified strings for min and max limits */
 nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng elements */
 (const int lmt_nbr, /* I [nbr] number of dimensions with limits */
- char * const * const lmt_arg) /* I [sng] list of user-specified dimension limits */
+ CST_X_PTR_CST_PTR_CST_Y(char,lmt_arg)) /* I [sng] List of user-specified dimension limits */
 {
   /* Purpose: Set name, min_sng, max_sng elements of 
      comma separated list of names and ranges. This routine
@@ -768,7 +850,7 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
      does not attempt to validate dimensions or their ranges
      against those present in input netCDF file. */
   
-  /* Valid syntax adheres to nm,[min_sng][,[max_sng]][,srd_sng] */
+  /* Valid syntax adheres to nm,[min_sng][,[max_sng][,[srd_sng]]] */
   
   void nco_usg_prn(void);
   
@@ -776,27 +858,27 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
   
   const char dlm_sng[]=",";
   
-  lmt_sct *lmt=NULL_CEWI;
+  lmt_sct **lmt=NULL_CEWI;
   
   int idx;
   int arg_nbr;
   
-  if(lmt_nbr > 0) lmt=(lmt_sct *)nco_malloc(lmt_nbr*sizeof(lmt_sct));
-  
+  if(lmt_nbr > 0) lmt=(lmt_sct **)nco_malloc(lmt_nbr*sizeof(lmt_sct *));
   for(idx=0;idx<lmt_nbr;idx++){
-    
     /* Process hyperslab specifications as normal text list */
-    arg_lst=lst_prs(lmt_arg[idx],dlm_sng,&arg_nbr);
+    /* fxm: probably need to free arg_lst sometime... */
+    arg_lst=nco_lst_prs_2D(lmt_arg[idx],dlm_sng,&arg_nbr);
     
     /* Check syntax */
     if(
        arg_nbr < 2 || /* Need more than just dimension name */
        arg_nbr > 4 || /* Too much information */
        arg_lst[0] == NULL || /* Dimension name not specified */
+       (arg_nbr == 2 && arg_lst[1] == NULL) || /* No min specified */
        (arg_nbr == 3 && arg_lst[1] == NULL && arg_lst[2] == NULL) || /* No min or max when stride not specified */
        (arg_nbr == 4 && arg_lst[3] == NULL) || /* Stride should be specified */
        False){
-      (void)fprintf(stdout,"%s: ERROR in hyperslab specification for dimension %s\n",prg_nm_get(),lmt_arg[idx]);
+      (void)fprintf(stdout,"%s: ERROR in hyperslab specification for dimension %s\n%s: HINT Conform request to hyperslab documentation at http://nco.sf.net#hyp\n",prg_nm_get(),lmt_arg[idx],prg_nm_get());
       nco_exit(EXIT_FAILURE);
     } /* end if */
     
@@ -806,23 +888,32 @@ nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng element
        Hopefully, in routines that follow, branch followed when dimension has
        all default settings specified (e.g.,"-d foo,,,,") yields same answer
        as branch for which no hyperslab along that dimension was set. */
-    lmt[idx].nm=NULL;
-    lmt[idx].is_usr_spc_lmt=True; /* True if any part of limit is user-specified, else False */
-    lmt[idx].min_sng=NULL;
-    lmt[idx].max_sng=NULL;
-    lmt[idx].srd_sng=NULL;
+    lmt[idx]=(lmt_sct *)nco_malloc(sizeof(lmt_sct));
+    lmt[idx]->nm=NULL;
+    lmt[idx]->is_usr_spc_lmt=True; /* True if any part of limit is user-specified, else False */
+    lmt[idx]->min_sng=NULL;
+    lmt[idx]->max_sng=NULL;
+    lmt[idx]->srd_sng=NULL;
     /* rec_skp_nsh_spf is used for record dimension in multi-file operators */
-    lmt[idx].rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
+    lmt[idx]->rec_skp_nsh_spf=0L; /* Number of records skipped in initial superfluous files */
     
     /* Fill in structure */
-    lmt[idx].nm=arg_lst[0];
-    lmt[idx].min_sng=lmt[idx].max_sng=arg_lst[1];
-    if(arg_nbr > 2) lmt[idx].max_sng=arg_lst[2];
-    if(arg_nbr > 3) lmt[idx].srd_sng=arg_lst[3];
+    lmt[idx]->nm=arg_lst[0];
+    lmt[idx]->min_sng=arg_lst[1];
+    /* Setting min_sng and max_sng to same pointer would lead to dangerous double-free() condition */
+    if(arg_nbr <= 2) lmt[idx]->max_sng=(char *)strdup(arg_lst[1]);
+    if(arg_nbr > 2) lmt[idx]->max_sng=arg_lst[2]; 
+    if(arg_nbr > 3) lmt[idx]->srd_sng=arg_lst[3];
     
-    if(lmt[idx].max_sng == NULL) lmt[idx].is_usr_spc_max=False; else lmt[idx].is_usr_spc_max=True;
-    if(lmt[idx].min_sng == NULL) lmt[idx].is_usr_spc_min=False; else lmt[idx].is_usr_spc_min=True;
-  } /* End loop over lmt */
+    if(lmt[idx]->max_sng == NULL) lmt[idx]->is_usr_spc_max=False; else lmt[idx]->is_usr_spc_max=True;
+    if(lmt[idx]->min_sng == NULL) lmt[idx]->is_usr_spc_min=False; else lmt[idx]->is_usr_spc_min=True;
+    
+    /* Free current pointer array to strings
+       Strings themselves are untouched and will be free()'d with limit structures 
+       in nco_lmt_lst_free() */
+    arg_lst=(char **)nco_free(arg_lst);
+
+  } /* End loop over lmt structure list */
   
   return lmt;
   
@@ -842,7 +933,8 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
   char *usr_unt_sng; /* [sng] User-specified unit string */
   long att_sz; /* [nbr] "units" attribute size */
   nc_type att_typ; /* [enm] Atttribute type, probably NC_CHAR */
-  utUnit udu_sct_in,udu_sct_out; /* Unidata units structure */
+  utUnit udu_sct_in; /* Unidata units structure */
+  utUnit udu_sct_out; /* Unidata units structure */
   
 #ifdef UDUNITS_PATH
   /* UDUNITS_PATH macro expands to where autoconf found database file */
@@ -882,39 +974,31 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
   if(utIsTime(&udu_sct_out) && utHasOrigin(&udu_sct_out)){
     /* Parse limits into years, months, day, etc. */
     char **arg_lst; /* Array of date/time strings after parsing */
-    int arg_nbr;  /* [nbr] Number of arguments parsed */
+    int arg_nbr; /* [nbr] Number of arguments parsed */
     int year,month,day,hour,min;
     double sec;
     double *crr_val=lmt_val; /* Pointer to set min/max value */
-    char *crr_sng; /* Save copy because lst_prs() alters stuff */
     
-    /* Allocate space for limit string */
-    crr_sng=(char *)nco_malloc(((strlen(lmt_sng))+1)*sizeof(char));
-    /* Make working copy */
-    strcpy(crr_sng,lmt_sng);
     /* Clear date and time */
     year=0L; month=0L; day=0L; hour=0L; min=0L; sec=(double)0;
     /* Break up date string copy delimited by dashes */
-    arg_lst=lst_prs(crr_sng,"-",&arg_nbr);
+    arg_lst=nco_lst_prs_2D(lmt_sng,"-",&arg_nbr);
     /* Fill in year, month, day */
-    if((arg_nbr > 0) && (arg_lst[0] != NULL))
-      year =strtol(arg_lst[0],(char **)NULL,10);
-    if((arg_nbr > 1)  && (arg_lst[1] != NULL))
-      month=strtol(arg_lst[1],(char **)NULL,10);
-    if((arg_nbr > 2)  && (arg_lst[2] != NULL))
-      day  =strtol(arg_lst[2],(char **)NULL,10);
+    if((arg_nbr > 0) && (arg_lst[0] != NULL)) year=strtol(arg_lst[0],(char **)NULL,10);
+    if((arg_nbr > 1) && (arg_lst[1] != NULL)) month=strtol(arg_lst[1],(char **)NULL,10);
+    if((arg_nbr > 2) && (arg_lst[2] != NULL)) day=strtol(arg_lst[2],(char **)NULL,10);
     /* Advance pointer past date/time delimiting space */
     if(arg_lst[2] != NULL) arg_lst[2]=strrchr(arg_lst[2],' ');
     /* Step past last space */
     if(arg_lst[2] != NULL) arg_lst[arg_nbr-1]++;
     /* Break up remaining time string delimited by colons */
     if(arg_lst[2] != NULL){
-      arg_lst=lst_prs(arg_lst[2],":",&arg_nbr);
+      arg_lst=nco_lst_prs_2D(arg_lst[2],":",&arg_nbr);
       /* Fill in hour, minute, second */
-      if((arg_nbr > 0) && strlen(arg_lst[0])>2){ /* HHMM spec */
+      if((arg_nbr > 0) && strlen(arg_lst[0]) > 2){ /* HHMM spec */
 	hour=strtol(arg_lst[0],(char **)NULL,10);
 	hour=(long)(hour/100);
-	min =strtol(arg_lst[0],(char **)NULL,10);
+	min=strtol(arg_lst[0],(char **)NULL,10);
 	min=(long)(min%100);
       }else{ /* HH:MM:S.S spec */
 	if((arg_nbr > 0) && (arg_lst[0] != NULL)) hour=strtol(arg_lst[0],(char **)NULL,10);
@@ -925,7 +1009,6 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
     
       /* Convert dates into "time since ..." value */
     rcd=utInvCalendar(year,month,day,hour,min,sec,&udu_sct_out,crr_val);
-    (void)nco_free(crr_sng);
     
   }else{ /* if utIsTime() && utHasOrigin */
     
@@ -968,16 +1051,16 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
   (void)utTerm(); /* Free memory taken by UDUnits library */
   return 0; /* Successful conversion */
 #else /* !ENABLE_UDUNITS */
-  rcd=1+0*nc_id+0*dmn_id+0*(*lmt_val); /* CEWI removes unused parameter warnings */
+  rcd=(int)(1+0*nc_id+0*dmn_id+0*(*lmt_val)); /* CEWI removes unused parameter warnings */
   (void)fprintf(stdout,"UDUnits limit detected in \"%s\" but UDUnits library is unavailable to perform conversion.\nHINT: Re-compile and re-install NCO with UDUnits enabled. http://nco.sf.net/nco.html#udunits",lmt_sng);
   nco_exit(EXIT_FAILURE);
   return 1; /* Conversion failed */
 #endif /* !ENABLE_UDUNITS */
 }  /* end nco_lmt_udu_cnv() */
 
-int
+int /* O [enm] Limit type */
 nco_lmt_typ /* [fnc] Determine limit type */
-(char* sng) /* I [ptr] pointer to limit string */
+(char *sng) /* I [ptr] Pointer to limit string */
 {
   /* Purpose: Determine type of user-specified limit */
   
@@ -994,7 +1077,7 @@ nco_lmt_typ /* [fnc] Determine limit type */
   if(strchr(sng,'-') && ((char*)strchr(sng,'-') != (char*)sng))
     /* Non-leading dash indicates dates  */
     return lmt_crd_val;
-     /* Default: Limit is dimension index */
+  /* Default: Limit is dimension index */
   return lmt_dmn_idx;
 
 } /* end nco_lmt_typ() */
