@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.18 2004/06/18 23:12:29 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lmt.h,v 1.28 2007/02/23 21:59:30 zender Exp $ */
 
 /* Purpose: Hyperslab limits */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_lmt.h" *//* Hyperslab limits */
@@ -51,20 +51,42 @@ nco_lmt_evl /* [fnc] Parse user-specified limits into hyperslab specifications *
 (int nc_id, /* I [idx] netCDF file ID */
  lmt_sct *lmt_ptr, /* I/O [sct] Structure from nco_lmt_prs() or from nco_lmt_sct_mk() to hold dimension limit information */
  long cnt_crr, /* I [nbr] Number of valid records already processed (only used for record dimensions in multi-file operators) */
- bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
+ nco_bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
 
-lmt_sct * /* O [sct] Structure with user-specified strings for min and max limits */
+lmt_sct * /* O [sct] Pointer to free'd structure */
+nco_lmt_free /* [fnc] Free memory associated with limit structure */
+(lmt_sct *lmt); /* I/O [sct] Limit structure to free */
+
+lmt_sct ** /* O [sct] Pointer to free'd structure list */
+nco_lmt_lst_free /* [fnc] Free memory associated with limit structure list */
+(lmt_sct **lmt_lst, /* I/O [sct] Limit structure list to free */
+ const int lmt_nbr); /* I [nbr] Number of limit structures in list */
+
+lmt_all_sct * /* O [sct] Pointer to free'd structure */
+nco_lmt_all_free /* [fnc] Free memory associated with lmt_all structure */
+(lmt_all_sct *lmt_all); /* I/O [sct] Limit structure to free */
+
+lmt_all_sct ** /* O [sct] Pointer to free'd structure list */
+nco_lmt_all_lst_free /* [fnc] Free memory associated with lmt_all structure list */
+(lmt_all_sct **lmt_all_lst, /* I/O [sct] Limit structure list to free */
+ const int lmt_all_nbr); /* I [nbr] Number of limit structures in list */
+
+lmt_sct ** /* O [sct] Structure list with user-specified strings for min and max limits */
 nco_lmt_prs /* [fnc] Create limit structures with name, min_sng, max_sng elements */
 (const int lmt_nbr, /* I [nbr] number of dimensions with limits */
- char * const * const lmt_arg); /* I [sng] list of user-specified dimension limits */
+ CST_X_PTR_CST_PTR_CST_Y(char,lmt_arg)); /* I [sng] List of user-specified dimension limits */
 
-lmt_sct /* [sct] Limit structure for dimension */
+lmt_sct * /* [sct] Limit structure for dimension */
 nco_lmt_sct_mk /* [fnc] Create stand-alone limit structure for given dimension */
 (const int nc_id, /* I [idx] netCDF file ID */
  const int dmn_id, /* I [idx] ID of dimension for this limit structure */
- const lmt_sct * const lmt, /* I [sct] Array of limit structures from nco_lmt_evl() */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_sct,lmt), /* I [sct] Array of limit structures from nco_lmt_evl() */ 
  int lmt_nbr, /* I [nbr] Number of limit structures */
- const bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
+ const nco_bool FORTRAN_IDX_CNV); /* I [flg] Hyperslab indices obey Fortran convention */
+
+int /* O [enm] Limit type */
+nco_lmt_typ /* [fnc] Determine limit type */
+(char *sng);/* I [ptr] Pointer to limit string */
 
 int /* [rcd] Successful conversion returns 0 */
 nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
@@ -73,10 +95,6 @@ nco_lmt_udu_cnv /* [fnc] Convert from Unidata units to coordinate value */
  char *lmt_sng, /* I [ptr] Limit string */
  double *lmt_val); /* O [val] Limit coordinate value */ 
 
-int /* 0 [enum] returns the limit type */
-nco_lmt_typ /* [fnc] determine limit type */
-(char* sng);/* I [ptr] pointer to the limit string */
-
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
