@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.22 2004/09/05 22:59:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.c,v 1.46 2007/05/19 00:11:35 zender Exp $ */
 
 /* Purpose: List utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_lst_utl.h" /* List utilities */
 
@@ -14,7 +14,7 @@
    See code in ${DATA}/tmp/testsort */
 
 void 
-indexx /* [fnc] Sort array of integers */
+nco_srt_ntg /* [fnc] Sort array of integers */
 (const int lmn_nbr, /* I [nbr] Number of elements */
  const int * const arr_in, /* I [idx] Array to sort */
  int * const idx) /* O [idx] Indices to sorted array */
@@ -22,12 +22,12 @@ indexx /* [fnc] Sort array of integers */
   /* Purpose: Stub for Numerical Recipes-compatible indexx() routine */
   long foo=sizeof(lmn_nbr)+sizeof(arr_in)+sizeof(idx); /* CEWI */
   foo++; /* CEWI */
-  (void)fprintf(stdout,"%s: ERROR indexx() routine should not be called\n",prg_nm_get());
+  (void)fprintf(stdout,"%s: ERROR nco_srt_ntg() routine should not be called\n",prg_nm_get());
   nco_exit(EXIT_FAILURE);
-} /* end indexx() */
+} /* end nco_srt_ntg() */
 
 void 
-indexx_alpha /* [fnc] Sort array of strings */
+nco_srt_lph /* [fnc] Sort array of strings */
 (const int lmn_nbr, /* I [nbr] Number of elements */
  char * const * const arr_in, /* I [sng] Strings to sort */
  int * const idx) /* O [idx] Indices to sorted array */
@@ -35,15 +35,15 @@ indexx_alpha /* [fnc] Sort array of strings */
   /* Purpose: Stub for Numerical Recipes-compatible indexx_alpha() routine */
   long foo=sizeof(lmn_nbr)+sizeof(arr_in)+sizeof(idx); /* CEWI */
   foo++; /* CEWI */
-  (void)fprintf(stdout,"%s: ERROR indexx_alpha() routine should not be called\n",prg_nm_get());
+  (void)fprintf(stdout,"%s: ERROR nco_srt_lph() routine should not be called\n",prg_nm_get());
   nco_exit(EXIT_FAILURE);
-} /* end indexx() */
+} /* end nco_srt_lph() */
 
 nm_id_sct * /* O [sct] Sorted output list */
 lst_heapsort /* [fnc] Heapsort input lists numerically or alphabetically */
 (nm_id_sct *lst, /* I/O [sct] Current list (destroyed) */
  const int nbr_lst, /* I [nbr] number of members in list */
- const bool ALPHABETIZE_OUTPUT) /* I [flg] Alphabetize extraction list */
+ const nco_bool ALPHABETIZE_OUTPUT) /* I [flg] Alphabetize extraction list */
 {
   /* Purpose: Sort extraction lists numerically or alphabetically
      Routine is deprecated in favor of nco_lst_srt_nm_id() which uses system qsort()
@@ -102,17 +102,30 @@ lst_heapsort /* [fnc] Heapsort input lists numerically or alphabetically */
 } /* end lst_heapsort() */
 
 char ** /* O [sng] Array of list elements */
-lst_prs /* [fnc] Create list of strings from given string and arbitrary delimiter */
+nco_lst_prs_1D /* [fnc] Create 1D array of strings from given string and delimiter */
 (char * const sng_in, /* I/O [sng] Delimited argument list (delimiters are changed to NULL on output */
- const char * const dlm_sng, /* I [sng] delimiter string */
+ const char * const dlm_sng, /* I [sng] Delimiter string */
  int * const nbr_lst) /* O [nbr] number of elements in list */
 {
   /* Purpose: Create list of strings from given string and arbitrary delimiter
      Routine is often called with system memory, e.g., with strings from
      command line arguments whose memory was allocated by shell or by getopt().
-     A conservative policy would be, therefore, to never modify input string
+     Conservative policy would be, therefore, to never modify input string
      However, we are safe if any modifications do not extend input string
-     Thus this routine is allowed to replace delimiter strings by NULs */
+     Thus this routine is allowed to replace delimiter strings by NULs
+
+     NB: Function takes single string as input and returns "list of strings" 
+     However, this list of strings was not obtained by malloc'ing each string 
+     It was obtained by inserting delimiters in a single string 
+     Hence do not try to separately free() each member of list of strings
+
+     Contrasting nco_lst_prs_1D() to successor nco_lst_prs_2D():
+     nco_lst_prs_2D() creates two-dimensional output string list, i.e., list of pointers to separately malloc()'d buffers
+     nco_lst_prs_2D() does not modify input
+     nco_lst_prs_2D() output list should be free'd by nco_sng_lst_free()
+     nco_lst_prs_1D() creates one-dimensional string list by inserting NULs into single buffer
+     nco_lst_prs_1D() modifies input (by inserting NULs)
+     nco_lst_prs_1D() output list may be free'd by nco_free() */
 
   /* Number of list members is always one more than number of delimiters, e.g.,
      foo,,3, has 4 arguments: "foo", "", "3" and "".
@@ -123,7 +136,7 @@ lst_prs /* [fnc] Create list of strings from given string and arbitrary delimite
      On the other hand, '\0', the empty string, can be printed but is not as useful as a flag
      Currently, NCO implements former convention, where default selections are set to NULL */
     
-  char **lst;
+  char **lst; /* O [sng] Array of list elements */
   char *sng_in_ptr;
 
   int dlm_lng;
@@ -144,12 +157,6 @@ lst_prs /* [fnc] Create list of strings from given string and arbitrary delimite
     (*nbr_lst)++;
   } /* end while */
 
-  /* valgrind fxm: 20040814 valgrind reports: 
-     ==32444== 4 bytes in 1 blocks are definitely lost in loss record 1 of 6
-     ==32444==    at 0x1B906EDD: malloc (vg_replace_malloc.c:131)
-     ==32444==    by 0x8055DE9: nco_malloc (nco_mmr.c:85)
-     ==32444==    by 0x8055A2C: lst_prs (nco_lst_utl.c:147)
-     ==32444==    by 0x8049D3B: main (ncpdq.c:272) */
   lst=(char **)nco_malloc(*nbr_lst*sizeof(char *));
 
   sng_in_ptr=sng_in; 
@@ -170,7 +177,7 @@ lst_prs /* [fnc] Create list of strings from given string and arbitrary delimite
     if(strlen(lst[idx]) == 0) lst[idx]=NULL;
 
   if(dbg_lvl_get() == 5){
-    (void)fprintf(stderr,"%d elements in list delimited by \"%s\"\n",*nbr_lst,dlm_sng);
+    (void)fprintf(stderr,"nco_lst_prs_1d() reports %d elements in list delimited by \"%s\"\n",*nbr_lst,dlm_sng);
     for(idx=0;idx<*nbr_lst;idx++) 
       (void)fprintf(stderr,"lst[%d] = %s\n",idx,(lst[idx] == NULL) ? "NULL" : lst[idx]);
     (void)fprintf(stderr,"\n");
@@ -178,18 +185,111 @@ lst_prs /* [fnc] Create list of strings from given string and arbitrary delimite
   } /* end debug */
 
   return lst;
-} /* end lst_prs() */
+} /* end nco_lst_prs_1D() */
+
+char ** /* O [sng] List of strings */
+nco_lst_prs_2D /* [fnc] Create list of strings from given string and delimiter */
+(const char * const sng_in, /* I [sng] Delimited argument list */
+ const char * const dlm_sng, /* I [sng] Delimiter string */
+ int * const nbr_lst) /* O [nbr] number of elements in list */
+{
+  /* Purpose: Create list of strings from given string and arbitrary delimiter
+     Algorithm recursively copies all text up to delimiter into new string and
+     then appends new string to output list
+     Output list has no delimiter strings
+     Contrasting nco_lst_prs_2D() to predecessor nco_lst_prs_1D():
+     nco_lst_prs_2D() creates two-dimensional output string list, i.e., list of pointers to separately malloc()'d buffers
+     nco_lst_prs_2D() does not modify input
+     nco_lst_prs_2D() output list should be free'd by nco_sng_lst_free()
+     nco_lst_prs_1D() creates one-dimensional string list by inserting NULs into single buffer
+     nco_lst_prs_1D() modifies input (by inserting NULs)
+     nco_lst_prs_1D() output list may be free'd by nco_free() */
+
+  /* Number of list members is always one more than number of delimiters, e.g.,
+     foo,,3, has 4 arguments: "foo", "", "3" and "".
+     A delimiter without an argument is valid syntax to indicate default argument
+     Therefore a storage convention is necessary to indicate default argument was selected
+     Either NULL or '\0' can be used without requiring additional flag
+     NULL is not printable, but is useful as a logical flag since its value is False
+     On the other hand, '\0', the empty string, can be printed but is not as useful as a flag
+     Currently, NCO implements former convention, where default selections are set to NULL */
+    
+  char **sng_lst_out; /* O [sng] Array of list elements */
+  char *sng_in_cpy;
+  char *dlm_ptr_crr;
+  char *sng_out_srt;
+
+  int dlm_lng;
+  int idx;
+
+  /* Delimiter must be NUL-terminated (a string) so we may find its length */
+  dlm_lng=strlen(dlm_sng);
+
+  /* Create duplicate to search, modify, copy, and free */
+  sng_in_cpy=(char *)strdup(sng_in); 
+
+  /* Increment temporary dummy pointer dlm_ptr_crr in strstr() search loops */
+  dlm_ptr_crr=sng_in_cpy;
+
+  /* First element does not require preceding delimiter */
+  *nbr_lst=1;
+
+  /* Count list members */
+  while((dlm_ptr_crr=strstr(dlm_ptr_crr,dlm_sng))){
+    dlm_ptr_crr+=dlm_lng;
+    (*nbr_lst)++;
+  } /* end while */
+
+  /* Calling routine has responsibility to free this memory */
+  sng_lst_out=(char **)nco_malloc(*nbr_lst*sizeof(char *));
+
+  dlm_ptr_crr=sng_in_cpy; 
+  sng_out_srt=sng_in_cpy;
+  idx=0;
+  while((dlm_ptr_crr=strstr(sng_out_srt,dlm_sng))){
+    /* This loop brackets and grabs "previous" arguments
+       Grab final argument after loop */
+    /* NUL-terminate previous arg */
+    *dlm_ptr_crr='\0';
+    /* Calling routine has responsibility to free this memory */
+    sng_lst_out[idx++]=(char *)strdup(sng_out_srt);
+    sng_out_srt=dlm_ptr_crr+dlm_lng;
+  } /* end while */
+  /* Grab final argument after last delimiter
+     This also handles case of string with no delimiters */
+  sng_lst_out[idx++]=(char *)strdup(sng_out_srt);
+
+  /* Assume default list member when two delimiters are adjacent to eachother, 
+     i.e., when length of string between delimiters is 0. 
+     If list ends with delimiter, last element of list is also assumed to be default list member */
+  /* This loop sets default list members to NULL */
+  for(idx=0;idx<*nbr_lst;idx++)
+    if(strlen(sng_lst_out[idx]) == 0) sng_lst_out[idx]=NULL;
+
+  if(dbg_lvl_get() == 5){
+    (void)fprintf(stderr,"nco_lst_prs_2D() reports %d elements in list delimited by \"%s\"\n",*nbr_lst,dlm_sng);
+    for(idx=0;idx<*nbr_lst;idx++) 
+      (void)fprintf(stderr,"sng_lst_out[%d] = %s\n",idx,(sng_lst_out[idx] == NULL) ? "NULL" : sng_lst_out[idx]);
+    (void)fprintf(stderr,"\n");
+    (void)fflush(stderr);
+  } /* end debug */
+
+  /* Free duplicate of sng_in */
+  sng_in_cpy=(char *)nco_free(sng_in_cpy); 
+
+  return sng_lst_out;
+} /* end nco_lst_prs_2D() */
 
 int /* O [enm] Comparison result [<,=,>] 0 iff val_1 [<,==,>] val_2 */
 nco_cmp_chr /* [fnc] Compare two characters */
-(const void *val_1, /* I [chr] Character to compare */
- const void *val_2) /* I [chr] Character to compare */
+(const void * val_1, /* I [chr] Character to compare */
+ const void * val_2) /* I [chr] Character to compare */
 {
   /* Purpose: Compare two characters
      Function is suitable for argument to ANSI C qsort() routine in stdlib.h
      Code based on responses to my comp.lang.c thread 20040101 */
-  const char * const val_1_cp=val_1;
-  const char * const val_2_cp=val_2;
+  const char * const val_1_cp=(const char *)val_1;
+  const char * const val_2_cp=(const char *)val_2;
   return *val_1_cp < *val_2_cp ? -1 : (*val_1_cp > *val_2_cp);
   /* Alternative one-liner:
      return (*val_1_cp > *val_2_cp) - (*val_1_cp < *val_2_cp); */
@@ -203,8 +303,8 @@ nco_cmp_int /* [fnc] Compare two integers */
   /* Purpose: Compare two integers
      Function is suitable for argument to ANSI C qsort() routine in stdlib.h
      Code based on responses to my comp.lang.c thread 20040101 */
-  const int * const val_1_ip=val_1;
-  const int * const val_2_ip=val_2;
+  const int * const val_1_ip=(const int *)val_1;
+  const int * const val_2_ip=(const int *)val_2;
   return *val_1_ip < *val_2_ip ? -1 : (*val_1_ip > *val_2_ip);
   /* Alternative one-liner:
      return (*val_1_ip > *val_2_ip) - (*val_1_ip < *val_2_ip); */
@@ -243,9 +343,9 @@ nco_cmp_nm_id_id /* [fnc] Compare two nm_id_sct's by ID member */
   /* Purpose: Compare two nm_id_sct's by ID structure member
      NB: This function uses a method which is, in general, unsafe
      By performing casts and then subracting, we are subject to overflow
-     conditions should the integer values by close to INT_MAX or INT_MIN.
+     conditions should integer values be close to INT_MAX or INT_MIN.
      However, we know that nm_id_sct.id values are always small
-     Thus we use this slightly unsafe method in order to show that a 
+     Thus we use this slightly unsafe method in order to show that
      comparison function may be written (albeit unsafely) in one line. */
   return (*(nm_id_sct const *)val_1).id-(*(nm_id_sct const *)val_2).id;
 } /* end nco_cmp_nm_id_nm() */
@@ -267,26 +367,61 @@ nco_cmp_ptr_unn /* Compare values of two pointer unions of same type */
     const float * const op2_fp=op2.fp;
     return *op1_fp < *op2_fp ? -1 : (*op1_fp > *op2_fp);}
     break;
-  case NC_DOUBLE: break;
+  case NC_DOUBLE:
     {const double * const op1_dp=op1.dp;
     const double * const op2_dp=op2.dp;
     return *op1_dp < *op2_dp ? -1 : (*op1_dp > *op2_dp);}
-  case NC_SHORT: break;
+    break;
+  case NC_INT:
+    {const nco_int * const op1_lp=op1.lp;
+    const nco_int * const op2_lp=op2.lp;
+    return *op1_lp < *op2_lp ? -1 : (*op1_lp > *op2_lp);}
+    break;
+  case NC_SHORT:
     {const short * const op1_sp=op1.sp;
     const short * const op2_sp=op2.sp;
     return *op1_sp < *op2_sp ? -1 : (*op1_sp > *op2_sp);}
-  case NC_INT: break;
-    {const nco_long * const op1_lp=op1.lp;
-    const nco_long * const op2_lp=op2.lp;
-    return *op1_lp < *op2_lp ? -1 : (*op1_lp > *op2_lp);}
-  case NC_BYTE: break;
-    {const signed char * const op1_bp=op1.bp;
-    const signed char * const op2_bp=op2.bp;
+    break;
+  case NC_USHORT:
+    {const nco_ushort * const op1_usp=op1.usp;
+    const nco_ushort * const op2_usp=op2.usp;
+    return *op1_usp < *op2_usp ? -1 : (*op1_usp > *op2_usp);}
+    break;
+  case NC_UINT:
+    {const nco_uint * const op1_uip=op1.uip;
+    const nco_uint * const op2_uip=op2.uip;
+    return *op1_uip < *op2_uip ? -1 : (*op1_uip > *op2_uip);}
+    break;
+  case NC_INT64:
+    {const nco_int64 * const op1_i64p=op1.i64p;
+    const nco_int64 * const op2_i64p=op2.i64p;
+    return *op1_i64p < *op2_i64p ? -1 : (*op1_i64p > *op2_i64p);}
+    break;
+  case NC_UINT64:
+    {const nco_uint64 * const op1_ui64p=op1.ui64p;
+    const nco_uint64 * const op2_ui64p=op2.ui64p;
+    return *op1_ui64p < *op2_ui64p ? -1 : (*op1_ui64p > *op2_ui64p);}
+    break;
+  case NC_BYTE:
+    {const nco_byte * const op1_bp=op1.bp;
+    const nco_byte * const op2_bp=op2.bp;
     return *op1_bp < *op2_bp ? -1 : (*op1_bp > *op2_bp);}
-  case NC_CHAR: break;
-    {const unsigned char * const op1_cp=op1.cp;
-    const unsigned char * const op2_cp=op2.cp;
+    break;
+  case NC_UBYTE:
+    {const nco_ubyte * const op1_ubp=op1.ubp;
+    const nco_ubyte * const op2_ubp=op2.ubp;
+    return *op1_ubp < *op2_ubp ? -1 : (*op1_ubp > *op2_ubp);}
+    break;
+  case NC_CHAR:
+    {const nco_char * const op1_cp=op1.cp;
+    const nco_char * const op2_cp=op2.cp;
     return *op1_cp < *op2_cp ? -1 : (*op1_cp > *op2_cp);}
+    break;
+  case NC_STRING:
+    {const nco_string * const op1_sngp=op1.sngp;
+    const nco_string * const op2_sngp=op2.sngp;
+    return *op1_sngp < *op2_sngp ? -1 : (*op1_sngp > *op2_sngp);}
+    break;
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
@@ -305,7 +440,7 @@ nco_lst_comma2hash /* [fnc] Replace commas with hashes when within braces */
      not to overwrite ends of strings */
   char *cp;
   char *cp_cnv=NULL; /* [ptr] Location of comma following open brace */
-  bool openbrace=False; /* [flg] Open brace has been found */
+  nco_bool openbrace=False; /* [flg] Open brace has been found */
   cp=rx_sng;
   /* Loop over each character in string until first NUL encountered */
   while(*cp){
@@ -330,7 +465,7 @@ nm_id_sct * /* O [sct] Sorted output list */
 nco_lst_srt_nm_id /* [fnc] Sort name/ID input list numerically or alphabetically */
 (nm_id_sct * const lst, /* I/O [sct] Current list (destroyed) */
  const int nbr_lst, /* I [nbr] number of members in list */
- const bool ALPHABETIZE_OUTPUT) /* I [flg] Alphabetize extraction list */
+ const nco_bool ALPHABETIZE_OUTPUT) /* I [flg] Alphabetize extraction list */
 {
   /* Purpose: Sort extraction lists numerically or alphabetically */
   if(ALPHABETIZE_OUTPUT){
@@ -345,13 +480,33 @@ nco_lst_srt_nm_id /* [fnc] Sort name/ID input list numerically or alphabetically
   return lst;
 } /* end nco_lst_srt_nm_id() */
 
+nm_id_sct * /* O [sct] Pointer to free'd structure list */
+nco_nm_id_lst_free /* [fnc] Free memory associated with name-ID structure list */
+(nm_id_sct *nm_id_lst, /* I/O [sct] Name-ID structure list to free */
+ const int nm_id_nbr) /* I [nbr] Number of name-ID structures in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated name-ID structure list */
+  int idx;
+
+  for(idx=0;idx<nm_id_nbr;idx++){
+    nm_id_lst[idx].nm=(char *)nco_free(nm_id_lst[idx].nm);
+  } /* end loop over idx */
+
+  /* Free structure pointer last */
+  nm_id_lst=(nm_id_sct *)nco_free(nm_id_lst);
+
+  return nm_id_lst;
+} /* end nco_nm_id_lst_free() */
+
 char * /* O [sng] Concatenated string formed by joining all input strings */
-sng_lst_prs /* [fnc] Join list of strings together into one string */
-(X_CST_PTR_CST_PTR_Y(char,sng_lst), /* I [sng] List of pointers to strings to join together */
- const long lmn_nbr, /* O [nbr] Number of strings in list */
- const char * const dlm_sng) /* I [sng] delimiter string to use as glue */
+sng_lst_cat /* [fnc] Join string list together into one string, delete originals */
+(char ** const sng_lst, /* I/O [sng] List of pointers to strings to join together */
+ const long lmn_nbr, /* I [nbr] Number of strings in list */
+ const char * const dlm_sng) /* I [sng] Delimiter string to use as glue */
 {
   /* Purpose: Join list of strings together into one string
+     Delete original string list
      Elements of input list should all be NUL-terminated strings
      Elements with value NUL are interpreted as strings of zero length */
 
@@ -361,11 +516,14 @@ sng_lst_prs /* [fnc] Join list of strings together into one string */
   long lmn;
   long sng_sz=0L; /* NB: sng_sz get incremented */
 
-  if(lmn_nbr == 1L) return sng_lst[0];
+  if(lmn_nbr == 1L){
+    sng=(char *)strdup(sng_lst[0]);
+    goto cln_and_xit;
+  } /* lmn_nbr != 1L */
 
   /* Delimiter must be NUL-terminated (a string) so strlen() works */
   if(dlm_sng == NULL){
-    (void)fprintf(stdout,"%s: ERROR sng_lst_prs() reports delimiter string is NULL\n",prg_nm_get());
+    (void)fprintf(stdout,"%s: ERROR sng_lst_cat() reports delimiter string is NULL\n",prg_nm_get());
     nco_exit(EXIT_FAILURE);
   } /* end if */
   dlm_lng=strlen(dlm_sng); 
@@ -373,7 +531,7 @@ sng_lst_prs /* [fnc] Join list of strings together into one string */
   /* List elements must be NUL-terminated (strings) so strlen() works */
   for(lmn=0L;lmn<lmn_nbr;lmn++) sng_sz+=(sng_lst[lmn] == NULL) ? 0L : strlen(sng_lst[lmn])+dlm_lng;
   /* Add one for NUL byte */
-  sng=(char *)nco_malloc(sizeof(char)*(sng_sz+1));
+  sng=(char *)nco_malloc(sizeof(char)*(sng_sz+1L));
   /* NUL-terminate string for safety */
   sng[0]='\0';
   for(lmn=0L;lmn<lmn_nbr;lmn++){
@@ -382,6 +540,31 @@ sng_lst_prs /* [fnc] Join list of strings together into one string */
     if(lmn != lmn_nbr-1L && dlm_lng != 0) sng=strcat(sng,dlm_sng);
   } /* end loop over lmn */
 
+  /* Jump here if only one string */
+ cln_and_xit:
+  for(lmn=0L;lmn<lmn_nbr;lmn++){
+    if(sng_lst[lmn] != NULL) sng_lst[lmn]=(char *)nco_free(sng_lst[lmn]);
+  } /* end loop over lmn */
+
   return sng;
-} /* end sng_lst_prs() */
+} /* end sng_lst_cat() */
+
+char ** /* O [sng] Pointer to free'd string list */
+nco_sng_lst_free /* [fnc] Free memory associated with string list */
+(char **sng_lst, /* I/O [sng] String list to free */
+ const int sng_nbr) /* I [nbr] Number of strings in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated string list */
+  int idx;
+
+  for(idx=0;idx<sng_nbr;idx++){
+    sng_lst[idx]=(char *)nco_free(sng_lst[idx]);
+  } /* end loop over idx */
+
+  /* Free structure pointer last */
+  sng_lst=(char **)nco_free(sng_lst);
+
+  return sng_lst;
+} /* end nco_sng_lst_free() */
 
