@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.h,v 1.16 2004/09/05 22:59:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_lst_utl.h,v 1.30 2007/04/18 17:06:15 zender Exp $ */
 
 /* Purpose: List utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_lst_utl.h" *//* List utilities */
@@ -31,28 +31,34 @@ extern "C" {
 #endif /* __cplusplus */
 
 void 
-indexx /* [fnc] Sort array of integers */
+nco_srt_ntg /* [fnc] Sort array of integers */
 (const int lmn_nbr, /* I [nbr] Number of elements */
  const int * const arr_in, /* I [idx] Array to sort */
  int * const idx); /* O [idx] Indices to sorted array */
 
 void 
-indexx_alpha /* [fnc] Sort array of strings */
+nco_srt_lph /* [fnc] Sort array of strings */
 (const int lmn_nbr, /* I [nbr] Number of elements */
  char * const * const arr_in, /* I [sng] Strings to sort */
  int * const idx); /* O [idx] Indices to sorted array */
 
 char ** /* O [sng] Array of list elements */
-lst_prs /* [fnc] Create list of strings from given string and arbitrary delimiter */
+nco_lst_prs_1D /* [fnc] Create 1D array of strings from given string and delimiter */
 (char * const sng_in, /* I/O [sng] Delimited argument list (delimiters are changed to NULL on output */
- const char * const dlm_sng, /* I [sng] delimiter string */
- int * const nbr_lst); /* O [nbr] number of elements in list */
+ const char * const dlm_sng, /* I [sng] Delimiter string */
+ int * const nbr_lst); /* O [nbr] Number of elements in list */
+
+char ** /* O [sng] List of strings */
+nco_lst_prs_2D /* [fnc] Create 2D list of strings from given string and delimiter */
+(const char * const sng_in, /* I [sng] Delimited argument list */
+ const char * const dlm_sng, /* I [sng] Delimiter string */
+ int * const nbr_lst); /* O [nbr] Number of elements in list */
 
 nm_id_sct * /* O [sct] Sorted output list */
 lst_heapsort /* [fnc] Heapsort input lists numerically or alphabetically */
 (nm_id_sct *lst, /* I/O [sct] Current list (destroyed) */
  const int nbr_lst, /* I [nbr] number of members in list */
- const bool ALPHABETIZE_OUTPUT); /* I [flg] Alphabetize extraction list */
+ const nco_bool ALPHABETIZE_OUTPUT); /* I [flg] Alphabetize extraction list */
 
 int /* O [enm] Comparison result [<,=,>] 0 iff val_1 [<,==,>] val_2 */
 nco_cmp_chr /* [fnc] Compare two characters */
@@ -85,6 +91,11 @@ nco_cmp_ptr_unn /* Compare values of two pointer unions of same type */
  const ptr_unn op1, /* I [sct] First operand to compare */
  const ptr_unn op2); /* I [sct] Second operand to compare */
 
+dmn_sct ** /* O [sct] Pointer to free'd structure list */
+nco_dmn_lst_free /* [fnc] Free memory associated with dimension structure list */
+(dmn_sct **dmn_lst, /* I/O [sct] Dimension structure list to free */
+ const int dmn_nbr); /* I [nbr] Number of dimension structures in list */
+
 void 
 nco_lst_comma2hash /* [fnc] Convert {...,...} to {...#...} in regular expressions */
 (char * const rx_sng); /* [sng] Regular expression */
@@ -93,13 +104,23 @@ nm_id_sct * /* O [sct] Sorted output list */
 nco_lst_srt_nm_id /* [fnc] Sort name/ID input list numerically or alphabetically */
 (nm_id_sct * const lst, /* I/O [sct] Current list (destroyed) */
  const int nbr_lst, /* I [nbr] number of members in list */
- const bool ALPHABETIZE_OUTPUT); /* I [flg] Alphabetize extraction list */
+ const nco_bool ALPHABETIZE_OUTPUT); /* I [flg] Alphabetize extraction list */
+
+nm_id_sct * /* O [sct] Pointer to free'd structure list */
+nco_nm_id_lst_free /* [fnc] Free memory associated with name-ID structure list */
+(nm_id_sct *nm_id_lst, /* I/O [sct] Name-ID structure list to free */
+ const int nm_id_nbr); /* I [nbr] Number of name-ID structures in list */
+
+char ** /* O [sng] Pointer to free'd string list */
+nco_sng_lst_free /* [fnc] Free memory associated with string list */
+(char **sng_lst, /* I/O [sng] String list to free */
+ const int sng_nbr); /* I [nbr] Number of strings in list */
 
 char * /* O [sng] Concatenated string formed by joining all input strings */
-sng_lst_prs /* [fnc] Join list of strings together into one string */
-(X_CST_PTR_CST_PTR_Y(char,sng_lst), /* I [sng] List of pointers to strings to join together */
- const long lmn_nbr, /* O [nbr] Number of strings in list */
- const char * const dlm_sng); /* I [sng] delimiter string to use as glue */
+sng_lst_cat /* [fnc] Join string list together into one string, delete originals */
+(char ** const sng_lst, /* I/O [sng] List of pointers to strings to join together */
+ const long lmn_nbr, /* I [nbr] Number of strings in list */
+ const char * const dlm_sng); /* I [sng] Delimiter string to use as glue */
 
 #ifdef __cplusplus
 } /* end extern "C" */
