@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_mmr.h,v 1.13 2004/01/12 18:11:07 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_mmr.h,v 1.21 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Memory management */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_mmr.h" *//* Memory management */
@@ -17,6 +17,10 @@
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
 #include <string.h> /* strcmp. . . */
+#ifdef MACOSX
+# include <sys/time.h> /* machine time (needed by Mac OS X for struct rusage) */ 
+#endif /* !MACOSX */
+#include <sys/resource.h> /* Resource usage and limits */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
@@ -30,13 +34,6 @@
 extern "C" {
 #endif /* __cplusplus */
 
-enum nco_mmr_typ{ /* [enm] Memory allocation type */
-  nco_mmr_calloc, /* [enm] nco_calloc() */
-  nco_mmr_free, /* [enm] nco_free() */
-  nco_mmr_malloc, /* [enm] nco_malloc() */
-  nco_mmr_realloc /* [enm] nco_realloc() */
-}; /* end nco_mmr_typ enum */
-
 void * /* O [ptr] Pointer to calloc'd memory */
 nco_calloc /* [fnc] Wrapper for calloc() */
 (const size_t lmn_nbr, /* I [nbr] Number of elements to allocate */
@@ -64,14 +61,18 @@ void
 nco_malloc_err_hnt_prn /* [fnc] Explain meaning and workarounds for malloc() failures */
 (void);
 
+long /* O [B] Maximum resident set size */
+nco_mmr_rusage_prn /* [fnc] Print rusage memory usage statistics */
+(const int rusage_who); /* [enm] RUSAGE_SELF, RUSAGE_CHILDREN, RUSAGE_LWP */
+
 long /* O [nbr] Net memory currently allocated */
 nco_mmr_stt /* [fnc] Track memory statistics */
-(const int nco_mmr_typ, /* I [enm] Memory allocation type */
+(const nco_mmr_typ_enm nco_mmr_typ, /* I [enm] Memory allocation type */
  const size_t sz); /* I [B] Bytes allocated, deallocated, or reallocated */
 
-const char * /* O [sng] String describing type */
-nco_mmr_typ_sng /* [fnc] Convert netCDF type enum to string */
-(nc_type type); /* I [enm] netCDF type */
+const char * /* O [sng] String describing memory type */
+nco_mmr_typ_sng /* [fnc] Convert NCO memory management type enum to string */
+(const nco_mmr_typ_enm nco_mmr_typ); /* I [enm] NCO memory management type */
 
 void * /* O [ptr] Pointer to re-allocated memory */
 nco_realloc /* [fnc] Wrapper for realloc() */
