@@ -1,16 +1,16 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.c,v 1.23 2004/09/18 05:22:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.c,v 1.42 2007/05/25 05:24:23 zender Exp $ */
 
 /* Purpose: Multi-slabbing algorithm */
 
-/* Copyright (C) 1995--2004 Charlie Zender and Henry Butowsky
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender and Henry Butowsky
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_msa.h" /* Multi-slabbing algorithm */
 
-/* fxm: strings statically allocated with MAX_LEN_FMT_SNG chars are susceptible to buffer overflow attacks */
+/* fxm: strings statically allocated with NCO_MAX_LEN_FMT_SNG chars are susceptible to buffer overflow attacks */
 /* Length should be computed at run time but is a pain */
-#define MAX_LEN_FMT_SNG 100
+#define NCO_MAX_LEN_FMT_SNG 100
 
 void *
 nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a single slab pointer */
@@ -20,6 +20,7 @@ nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
  lmt_all_sct **lmt_lst, /* [sct] List of limits in each dimension (this remains STATIC as we recurse) */
  var_sct *vara) /* [sct] Information for routine to read variable information and pass information between calls */
 {
+  /* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
   /* Purpose: Multi-slab algorithm (recursive routine, returns a single slab pointer */
   int idx;
   int nbr_slb;
@@ -31,6 +32,7 @@ nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
 
   if(nbr_slb == 1){
     lmt[dpt_crr]=lmt_lst[dpt_crr]->lmt_dmn[0];
+    /* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
     vp=nco_msa_rec_clc(dpt_crr+1,dpt_crr_max,lmt,lmt_lst,vara);
     return vp;
   } /* end if */
@@ -62,6 +64,7 @@ nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
     
     for(idx=0;idx<nbr_slb;idx++){
       lmt[dpt_crr]=lmt_lst[dpt_crr]->lmt_dmn[idx];
+      /* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
       cp_wrp[idx]=(char *)nco_msa_rec_clc(dpt_crr+1,dpt_crr_max,lmt,lmt_lst,vara);
       cp_sz[idx]=vara->sz;
     } /* end loop over idx */
@@ -114,9 +117,10 @@ nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
       } /* end while */
     } /* end else */  
  
-    (void)nco_free(indices);
-    (void)nco_free(cp_sz);
-    for(idx=0;idx<nbr_slb;idx++) (void)nco_free(cp_wrp[idx]);
+    indices=(long *)nco_free(indices);
+    cp_sz=(long *)nco_free(cp_sz);
+    for(idx=0;idx<nbr_slb;idx++) cp_wrp[idx]=(char *)nco_free(cp_wrp[idx]);
+    cp_wrp=(char **)nco_free(cp_wrp);
     
     vara->sz=var_sz;
     return vp;
@@ -142,16 +146,26 @@ nco_msa_rec_clc /* [fnc] Multi-slab algorithm (recursive routine, returns a sing
       mult_srd*=lmt[idx]->srd;
     } /* end loop over idx */
     
-    vp=(void*)nco_malloc(var_sz *nco_typ_lng(vara->type));
+    vp=(void *)nco_malloc(var_sz*nco_typ_lng(vara->type));
     
-    /* Check for stride */
-    if(mult_srd == 1L)
-      (void)nco_get_vara(vara->nc_id,vara->id,dmn_srt,dmn_cnt,vp,vara->type);
-    else
-      (void)nco_get_varm(vara->nc_id,vara->id,dmn_srt,dmn_cnt,dmn_srd,(long *)NULL,vp,vara->type);
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+    { /* begin potential OpenMP critical */
+      /* Check for stride */
+      if(var_sz > 0){ /* Allow for zero-size record variables TODO nco711 */
+	if(mult_srd == 1L)
+	  (void)nco_get_vara(vara->nc_id,vara->id,dmn_srt,dmn_cnt,vp,vara->type);
+	else
+	  (void)nco_get_varm(vara->nc_id,vara->id,dmn_srt,dmn_cnt,dmn_srd,(long *)NULL,vp,vara->type);
+      } /* end if var_sz */
+    } /* end potential OpenMP critical */
+    
+    dmn_srt=(long *)nco_free(dmn_srt);
+    dmn_cnt=(long *)nco_free(dmn_cnt);
+    dmn_srd=(long *)nco_free(dmn_srd);
     
     /* Put size into vara */
     vara->sz=var_sz;
+
     return vp;
   }/* end read_lbl */
   
@@ -176,9 +190,9 @@ nco_msa_prn_idx(lmt_all_sct *lmt_i)
     printf("slb_nbr=%d srt=%ld end=%ld cnt=%ld srd=%ld\n",slb_nbr,lmt.srt,lmt.end,lmt.cnt,lmt.srd);
 } /* end nco_msa_prn_idx() */
 
-bool /* [flg] There are more limits to process in the slab */
+nco_bool /* [flg] There are more limits to process in the slab */
 nco_msa_clc_idx
-(bool NORMALIZE,
+(nco_bool NORMALIZE,
  lmt_all_sct *lmt_a, /* I list of lmts for each dimension  */
  long *indices, /* I/O so routine can keep track of where its at */
  lmt_sct *lmt, /* O Output hyperslab */
@@ -190,7 +204,8 @@ nco_msa_clc_idx
      So the stride is ALWAYS 1 */
   int sz_idx;
   int size=lmt_a->lmt_dmn_nbr;
-  bool *min;
+  nco_bool *mnm;
+  nco_bool rcd;
   
   int prv_slb=0;
   int crr_slb=0;
@@ -198,25 +213,27 @@ nco_msa_clc_idx
   long prv_idx=long_CEWI;
   long cnt=0L;
   
-  min=(bool*)nco_malloc(size*sizeof(bool));
+  mnm=(nco_bool *)nco_malloc(size*sizeof(nco_bool));
   
   lmt->srt=-1L;
   lmt->cnt=0L;
   lmt->srd=0L;
   
   while(++cnt){
-    crr_idx=nco_msa_min_idx(indices,min,size);
+    crr_idx=nco_msa_min_idx(indices,mnm,size);
     
     crr_slb=-1;
     for(sz_idx=0;sz_idx<size;sz_idx++)
-      if(min[sz_idx]){crr_slb=sz_idx;break;}
+      if(mnm[sz_idx]){crr_slb=sz_idx;break;}
     
     if(crr_slb == -1){
-      if(lmt->srt == -1) return False;
-      else break;
+      if(lmt->srt == -1){
+	rcd=False;
+	goto cln_and_xit;
+      }else break;
     } /* endif */
     
-    if(min[prv_slb]) crr_slb=prv_slb;
+    if(mnm[prv_slb]) crr_slb=prv_slb;
     
     if(lmt->srt > -1 && crr_slb != prv_slb) break;
     
@@ -239,7 +256,7 @@ nco_msa_clc_idx
     } /* end if */
     
     for(sz_idx=0;sz_idx<size;sz_idx++){
-      if(min[sz_idx]){
+      if(mnm[sz_idx]){
 	indices[sz_idx]+=lmt_a->lmt_dmn[sz_idx]->srd;
 	if(indices[sz_idx] > lmt_a->lmt_dmn[sz_idx]->end) indices[sz_idx]=-1;
       }
@@ -256,26 +273,33 @@ nco_msa_clc_idx
     lmt->end=(lmt->end-lmt_a->lmt_dmn[*slb]->srt)/(lmt_a->lmt_dmn[*slb]->srd);
     lmt->srd=1L;
   } /* end if */
-  return True;
+
+  rcd=True;
+
+  /* Jump here if only one string */
+ cln_and_xit:
+  mnm=(nco_bool *)nco_free(mnm);
+
+  return rcd;
 } /* end nco_msa_clc_idx() */
 
 void
-nco_msa_ram_2_dsk(  /* convert hyperslab indices (in RAM) to hyperlsab indices relative */
-		  long *dmn_sbs_ram,   /* to disk. */
-lmt_all_sct **lmt_mult, 
-int nbr_dim,
-long *dmn_sbs_dsk,
-bool FREE){
-  /*  It doenst really convert RAM indices to disk indices , but given a set 
-      of RAM indices finds the next set of dsk incdices. So it only works if 
-      the indices fed to it are continuous */
+nco_msa_ram_2_dsk /* convert hyperslab indices (in RAM) to hyperlsab indices relative */
+(long *dmn_sbs_ram,   /* to disk. */
+ lmt_all_sct **lmt_mult, 
+ int nbr_dim,
+ long *dmn_sbs_dsk,
+ nco_bool FREE){
+  /*  It does not really convert RAM indices to disk indices, but given a set 
+      of RAM indices finds the next set of dsk incdices. 
+      So it only works if the indices fed to it are continuous */
   int idx;
   int jdx;
   int size;
   static int initialize;
   static long **dmn_indices;
   static long *dmn_sbs_prv;
-  static bool min[100];
+  static nco_bool mnm[100];
 
   if(!initialize){
     dmn_sbs_prv=(long *)nco_malloc(nbr_dim*sizeof(long));
@@ -314,10 +338,10 @@ bool FREE){
      continue;
      }
          
-    dmn_sbs_dsk[idx]=nco_msa_min_idx(dmn_indices[idx],min,size);
+    dmn_sbs_dsk[idx]=nco_msa_min_idx(dmn_indices[idx],mnm,size);
 
     for(jdx=0;jdx<size;jdx++){
-      if(min[jdx]){
+      if(mnm[jdx]){
 	dmn_indices[idx][jdx]+=lmt_mult[idx]->lmt_dmn[jdx]->srd;
 	if(dmn_indices[idx][jdx] > lmt_mult[idx]->lmt_dmn[jdx]->end) dmn_indices[idx][jdx]=-1;
       }
@@ -341,24 +365,26 @@ nco_msa_clc_cnt(lmt_all_sct *lmt_lst)
 {
   int idx;
   long cnt=0;
-  int  size=lmt_lst->lmt_dmn_nbr;
+  int size=lmt_lst->lmt_dmn_nbr;
   long *indices;
-  bool *min;
-  
-  indices=(long *)nco_malloc(size*sizeof(long));
-  min=(bool *)nco_malloc(size*sizeof(bool));
+  nco_bool *mnm;
   
+  /* Degenerate case */
   if(size == 1){
     lmt_lst->dmn_cnt=lmt_lst->lmt_dmn[0]->cnt;
     return;
   } /* end if */
-  /* initialise indices with srt from    */
-  for(idx=0;idx<size;idx++)
+
+  indices=(long *)nco_malloc(size*sizeof(long));
+  mnm=(nco_bool *)nco_malloc(size*sizeof(nco_bool));
+  
+  /* Initialize indices with srt */
+  for(idx=0;idx<size;idx++) 
     indices[idx]=lmt_lst->lmt_dmn[idx]->srt;
   
-  while(nco_msa_min_idx(indices,min,size) != LONG_MAX){
+  while(nco_msa_min_idx(indices,mnm,size) != LONG_MAX){
     for(idx=0;idx<size;idx++){
-      if(min[idx]){
+      if(mnm[idx]){
 	indices[idx]+=lmt_lst->lmt_dmn[idx]->srd;
 	if(indices[idx] > lmt_lst->lmt_dmn[idx]->end) indices[idx]=-1;
       } /* end if */
@@ -366,6 +392,11 @@ nco_msa_clc_cnt(lmt_all_sct *lmt_lst)
     cnt++;
   } /* end while */
   lmt_lst->dmn_cnt=cnt;
+
+  indices=(long *)nco_free(indices);
+  mnm=(nco_bool *)nco_free(mnm);
+
+  return; /* 20050109: fxm added return to void function to squelch erroneous gcc-3.4.2 warning */ 
 } /* end nco_msa_clc_cnt() */
 
 void
@@ -431,20 +462,20 @@ nco_msa_wrp_splt /* [fnc] Split wrapped dimensions */
   if(size==1 && lmt_lst->lmt_dmn_nbr==2) lmt_lst->WRP=True;
 } /* end nco_msa_wrp_splt() */
 
-long
-nco_msa_min_idx /* [fnc] Find minimum values in current */
-(long *current, /* [idx] Current indices */
- bool *min, /* [flg] Minimum */
- int size) /* [nbr] Size of current and min */
+long /* O [idx] Minimum value */
+nco_msa_min_idx /* [fnc] Find minimum values in current and return minimum value */
+(const long * const current, /* I [idx] Current indices */
+ nco_bool * const mnm, /* O [flg] Minimum */
+ const int size) /* I [nbr] Size of current and mnm */
 {
   int sz_idx;
   long min_val=LONG_MAX;
   
   for(sz_idx=0;sz_idx<size;sz_idx++)
-    if(current[sz_idx] != -1 && current[sz_idx]<min_val) min_val=current[sz_idx];
+    if(current[sz_idx] != -1 && current[sz_idx] < min_val) min_val=current[sz_idx];
   
   for(sz_idx=0;sz_idx<size;sz_idx++)
-    min[sz_idx]=((current[sz_idx] != -1 && current[sz_idx]== min_val) ? True : False);
+    mnm[sz_idx]=((current[sz_idx] != -1 && current[sz_idx] == min_val) ? True : False);
   
   return min_val;
 } /* end nco_msa_min_idx() */
@@ -454,11 +485,12 @@ nco_cpy_var_val_mlt_lmt /* [fnc] Copy variable data from input to output file */
 (const int in_id, /* I [id] netCDF input file ID */
  const int out_id, /* I [id] netCDF output file ID */
  FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
- const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+ const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
  char *var_nm, /* I [sng] Variable name */
- lmt_all_sct * const lmt_lst, /* I multi-hyperslab limits */
+ lmt_all_sct * const * lmt_lst, /* I multi-hyperslab limits */
  int nbr_dmn_fl) /* I [nbr] Number of multi-hyperslab limits */
 {
+  /* NB: nco_cpy_var_val_mlt_lmt() contains OpenMP critical region */
   /* Purpose: Copy variable data from input netCDF file to output netCDF file 
      Routine truncates dimensions in variable definition in output file according to user-specified limits.
      Routine copies variable-by-variable, old-style, used only by ncks */
@@ -504,8 +536,14 @@ nco_cpy_var_val_mlt_lmt /* [fnc] Copy variable data from input to output file */
   if(nbr_dim == 0){
     var_sz=1L;
     void_ptr=nco_malloc(nco_typ_lng(var_type));
-    (void)nco_get_var1(in_id,var_in_id,0L,void_ptr,var_type);
-    (void)nco_put_var1(out_id,var_out_id,0L,void_ptr,var_type);
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+    { /* begin potential OpenMP critical */
+      (void)nco_get_var1(in_id,var_in_id,0L,void_ptr,var_type);
+    } /* end potential OpenMP critical */
+    /* Block is always critical */
+    { /* begin OpenMP critical */
+      (void)nco_put_var1(out_id,var_out_id,0L,void_ptr,var_type);
+    } /* end OpenMP critical */
     if(NCO_BNR_WRT) nco_bnr_wrt(fp_bnr,var_nm,var_sz,var_type,void_ptr);
     (void)nco_free(void_ptr);
     return;
@@ -526,8 +564,8 @@ nco_cpy_var_val_mlt_lmt /* [fnc] Copy variable data from input to output file */
   /* Get dimension sizes from input file */
   for(idx=0;idx<nbr_dim;idx++){
     for(jdx=0;jdx<nbr_dmn_fl;jdx++){
-      if(dmn_id[idx] == lmt_lst[jdx].lmt_dmn[0]->id){
-	lmt_mult[idx]=&lmt_lst[jdx];
+      if(dmn_id[idx] == lmt_lst[jdx]->lmt_dmn[0]->id){
+	lmt_mult[idx]=lmt_lst[jdx];
         break;
       } /* end if */
     } /* end loop over jdx */
@@ -544,10 +582,14 @@ nco_cpy_var_val_mlt_lmt /* [fnc] Copy variable data from input to output file */
   vara.type=var_type;
   
   /* Call super-dooper recursive routine */
+  /* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
   void_ptr=nco_msa_rec_clc(0,nbr_dim,lmt,lmt_mult,&vara);
   var_sz=vara.sz;
   
-  (void)nco_put_vara(out_id,var_out_id,dmn_map_srt,dmn_map_out,void_ptr,var_type);
+  /* Block is always critical */
+  { /* begin OpenMP critical */
+    (void)nco_put_vara(out_id,var_out_id,dmn_map_srt,dmn_map_out,void_ptr,var_type);
+  } /* end OpenMP critical */
   
   if(NCO_BNR_WRT) nco_bnr_wrt(fp_bnr,var_nm,var_sz,var_type,void_ptr);
   
@@ -578,13 +620,14 @@ void
 nco_msa_prn_var_val   /* [fnc] Print variable data */
 (const int in_id, /* I [id] netCDF input file ID */
  const char * const var_nm, /* I [sng] Variable name */
- lmt_all_sct * const lmt_lst, /* I [sct] Dimension limits */
+ lmt_all_sct * const *lmt_lst, /* I [sct] Dimension limits */
  const int lmt_nbr, /* I [nbr] number of dimensions with user-specified limits */
  char * const dlm_sng, /* I [sng] User-specified delimiter string, if any */
- const bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
- const bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
- const bool PRN_DMN_IDX_CRD_VAL) /* I [flg] Print dimension/coordinate indices/values */
+ const nco_bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
+ const nco_bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
+ const nco_bool PRN_DMN_IDX_CRD_VAL) /* I [flg] Print dimension/coordinate indices/values */
 {
+  /* NB: nco_msa_prn_var_val() with same nc_id contains OpenMP critical region */
   /* Purpose:
      Get variable with limits from input file
      User supplied dlm_sng, print var (includes nbr_dmim == 0)
@@ -594,8 +637,9 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
      if PRN_DMN_IDX_CRD_VAL then read in co-ord dims
      if PRN.. = True print var taking account of FORTRAN (Use dims to calculate var indices */
   
+  nco_bool MALLOC_UNITS_SNG=False; /* [flg] Allocated memory for units string */
   char nul_chr='\0';
-  char var_sng[MAX_LEN_FMT_SNG];
+  char var_sng[NCO_MAX_LEN_FMT_SNG];
   char *unit_sng;
   
   /* Get variable with limits from input file */
@@ -629,7 +673,10 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
   if(var.nbr_dim == 0){
     var.sz=1L;
     var.val.vp=nco_malloc(nco_typ_lng(var.type));
-    (void)nco_get_var1(in_id,var.id ,0L,var.val.vp,var.type);
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+    { /* begin potential OpenMP critical */
+      (void)nco_get_var1(in_id,var.id,0L,var.val.vp,var.type);
+    } /* end potential OpenMP critical */
   } /* end if */
 
   dmn_id=(int *)nco_malloc(var.nbr_dim*sizeof(int));
@@ -643,13 +690,14 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
   /* Get dimension sizes from input file */
   for(idx=0;idx< var.nbr_dim;idx++)
     for(jdx=0;jdx< lmt_nbr;jdx++){
-      if(dmn_id[idx] == lmt_lst[jdx].lmt_dmn[0]->id){
-	lmt_mult[idx]=&lmt_lst[jdx];
+      if(dmn_id[idx] == lmt_lst[jdx]->lmt_dmn[0]->id){
+	lmt_mult[idx]=lmt_lst[jdx];
         break;
       } /* end if */
     } /* end loop over jdx */
 
   /* Call super-dooper recursive routine */
+  /* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
   var.val.vp=nco_msa_rec_clc(0,var.nbr_dim,lmt,lmt_mult,&var);
   /* Call also initializes var.sz with final size */
 
@@ -673,6 +721,12 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
       case NC_INT: (void)fprintf(stdout,dlm_sng,var.val.lp[lmn]); break;
       case NC_CHAR: (void)fprintf(stdout,dlm_sng,var.val.cp[lmn]); break;
       case NC_BYTE: (void)fprintf(stdout,dlm_sng,var.val.bp[lmn]); break;
+      case NC_UBYTE: (void)fprintf(stdout,dlm_sng,var.val.ubp[lmn]); break;
+      case NC_USHORT: (void)fprintf(stdout,dlm_sng,var.val.usp[lmn]); break;
+      case NC_UINT: (void)fprintf(stdout,dlm_sng,var.val.uip[lmn]); break;
+      case NC_INT64: (void)fprintf(stdout,dlm_sng,var.val.i64p[lmn]); break;
+      case NC_UINT64: (void)fprintf(stdout,dlm_sng,var.val.ui64p[lmn]); break;
+      case NC_STRING: (void)fprintf(stdout,dlm_sng,var.val.sngp[lmn]); break;
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */
     } /* end loop over element */
@@ -694,11 +748,11 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
 	unit_sng=(char *)nco_malloc((att_sz+1)*nco_typ_lng(att_typ));
 	(void)nco_get_att(in_id,var.id,units_nm,unit_sng,att_typ);
 	unit_sng[(att_sz+1)*nco_typ_lng(att_typ)-1]='\0';
+	MALLOC_UNITS_SNG=True; /* [flg] Allocated memory for units string */
       } /* end if */
     } /* end if */
   } /* end if PRN_DMN_UNITS */
 
-  /* if nbr_dim ==0 and dlm_sng==NULL print variable */
   if(var.nbr_dim == 0 && dlm_sng == NULL){
     /* Variable is scalar, byte, or character */
     lmn=0;
@@ -710,16 +764,22 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
     case NC_INT: (void)fprintf(stdout,var_sng,var_nm,var.val.lp[lmn],unit_sng); break;
     case NC_CHAR:
       if(var.val.cp[lmn] != '\0'){
-        (void)sprintf(var_sng,"%%s='%s' %%s\n",nco_typ_fmt_sng(var.type));
+        (void)sprintf(var_sng,"%%s = '%s' %%s\n",nco_typ_fmt_sng(var.type));
         (void)fprintf(stdout,var_sng,var_nm,var.val.cp[lmn],unit_sng);
       }else{ /* Deal with NUL character here */
-        (void)fprintf(stdout, "%s=\"\" %s\n",var_nm,unit_sng);
+        (void)fprintf(stdout, "%s = \"\" %s\n",var_nm,unit_sng);
       } /* end if */
       break;
     case NC_BYTE: (void)fprintf(stdout,var_sng,var_nm,(unsigned char)var.val.bp[lmn],unit_sng); break;
+    case NC_UBYTE: (void)fprintf(stdout,var_sng,var_nm,var.val.ubp[lmn],unit_sng); break;
+    case NC_USHORT: (void)fprintf(stdout,var_sng,var_nm,var.val.usp[lmn],unit_sng); break;
+    case NC_UINT: (void)fprintf(stdout,var_sng,var_nm,var.val.uip[lmn],unit_sng); break;
+    case NC_INT64: (void)fprintf(stdout,var_sng,var_nm,var.val.i64p[lmn],unit_sng); break;
+    case NC_UINT64: (void)fprintf(stdout,var_sng,var_nm,var.val.ui64p[lmn],unit_sng); break;
+    case NC_STRING: (void)fprintf(stdout,var_sng,var_nm,var.val.sngp[lmn],unit_sng); break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end switch */
-  } /* end if variable is a scalar, byte, or character */
+  } /* end if variable is scalar, byte, or character */
 
   if(var.nbr_dim > 0 && dlm_sng == NULL){
     long *mod_map_in;
@@ -768,6 +828,7 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
         var_crd.type=dim[idx].type;
         var_crd.id=dim[idx].cid;
         /* Read in coordinate var with limits applied */
+	/* NB: nco_msa_rec_clc() with same nc_id contains OpenMP critical region */
 	dim[idx].val.vp=nco_msa_rec_clc(0,1,lmt,lmt_mult+idx,&var_crd);
 
         /* typecast pointer before use */  
@@ -795,7 +856,7 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
         int dmn_idx;
         long dmn_sbs_prn;
 	long crd_idx_crr;
-	char dmn_sng[MAX_LEN_FMT_SNG];
+	char dmn_sng[NCO_MAX_LEN_FMT_SNG];
       
 	/* Loop over dimensions whose coordinates are to be printed */
 	for(idx=0;idx<var.nbr_dim;idx++){
@@ -827,8 +888,14 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
 	  case NC_INT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.lp[crd_idx_crr]); break;
 	  case NC_CHAR: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.cp[crd_idx_crr]); break;
 	  case NC_BYTE: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,(unsigned char)dim[dmn_idx].val.bp[crd_idx_crr]); break;
-	      default: nco_dfl_case_nc_type_err(); break;
-	      } /* end switch */
+	  case NC_UBYTE: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.ubp[crd_idx_crr]); break;
+	  case NC_USHORT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.usp[crd_idx_crr]); break;
+	  case NC_UINT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.uip[crd_idx_crr]); break;
+	  case NC_INT64: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.i64p[crd_idx_crr]); break;
+	  case NC_UINT64: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.ui64p[crd_idx_crr]); break;
+	  case NC_STRING: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.sngp[crd_idx_crr]); break;
+	  default: nco_dfl_case_nc_type_err(); break;
+	  } /* end switch */
 	} /* end loop over dimensions */
       } /* end if PRN_DMN_IDX_CRD_VAL */
       
@@ -836,7 +903,7 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
     lbl_chr_prn:
 
       if(var.type == NC_CHAR){
-        static bool NULL_IN_SLAB;
+        static nco_bool NULL_IN_SLAB;
 	static char *prn_sng;
         static int chr_cnt;
         static long dmn_sz;
@@ -896,6 +963,12 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
       case NC_INT: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.lp[lmn],unit_sng); break;
       case NC_CHAR: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.cp[lmn],unit_sng); break;
       case NC_BYTE: (void)fprintf(stdout,var_sng,var_nm,var_dsk,(unsigned char)var.val.bp[lmn],unit_sng); break;
+      case NC_UBYTE: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.ubp[lmn],unit_sng); break;
+      case NC_USHORT: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.usp[lmn],unit_sng); break;
+      case NC_UINT: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.uip[lmn],unit_sng); break;
+      case NC_INT64: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.i64p[lmn],unit_sng); break;
+      case NC_UINT64: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.ui64p[lmn],unit_sng); break;
+      case NC_STRING: (void)fprintf(stdout,var_sng,var_nm,var_dsk,var.val.sngp[lmn],unit_sng); break;
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */
     } /* end loop over elements */
@@ -912,6 +985,7 @@ nco_msa_prn_var_val   /* [fnc] Print variable data */
   var.val.vp=nco_free(var.val.vp);
   var.nm=(char *)nco_free(var.nm);
  
+  if(MALLOC_UNITS_SNG) unit_sng=(char *)nco_free(unit_sng);
   if(var.nbr_dim > 0){
     (void)nco_free(dmn_id);
     (void)nco_free(lmt_mult);
