@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.h,v 1.13 2004/09/18 05:22:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_msa.h,v 1.21 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Multi-slabbing algorithm */
 
-/* Copyright (C) 1995--2004 Charlie Zender and Henry Butowsky
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender and Henry Butowsky
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_msa.h" *//* Multi-slabbing algorithm */
@@ -41,18 +41,18 @@ extern "C" {
  (const int in_id, /* I [id] netCDF input file ID */
   const int out_id, /* I [id] netCDF output file ID */
   FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
-  const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+  const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
   char *var_nm, /* I [sng] Variable name */
-  lmt_all_sct * const lmt_lst, /* I multi-hyperslab limits */
+  lmt_all_sct * const * lmt_lst, /* I multi-hyperslab limits */
   int nbr_dmn_fl); /* I [nbr] Number of multi-hyperslab limits */
   
   void 
   nco_msa_c_2_f /* [fnc] Replace brackets with parentheses in a string */
   (char *sng); /* [sng] String to change from C to Fortran notation */
 
-  bool /* if false then we are at the end of the slab */
+  nco_bool /* if false then we are at the end of the slab */
   nco_msa_clc_idx
-  (bool NORMALIZE,         /* Return indices of slab within the slab */
+  (nco_bool NORMALIZE,         /* Return indices of slab within the slab */
    lmt_all_sct *lmt_a,         /* I list of lmts for each dimension  */
    long *indices,          /* I/O so routine can keep track of where its at */
    lmt_sct* lmt_out,      /* O  output hyperslab */
@@ -78,11 +78,11 @@ extern "C" {
    lmt_all_sct **lmt_lst, /* list of limits in each dimension (this remains STATIC as we recurse */
    var_sct *var1);    /* Infor for routine to read var (should not change */
   
-  long
-  nco_msa_min_idx /* find min values in current and return the min value*/
-  (long *current,   /* current indices */
-   bool *min,       /* element true if a minimum */
-   int size);       /* size of current and min */
+  long /* O [idx] Minimum value */
+  nco_msa_min_idx /* [fnc] Find minimum values in current and return minimum value */
+  (const long * const current, /* I [idx] Current indices */
+   nco_bool * const mnm, /* O [flg] Minimum */
+   const int size); /* I [nbr] Size of current and min */
 
   void             /* convert hyperlsab indices into indices relative to disk */ 
   nco_msa_ram_2_dsk( 
@@ -90,18 +90,18 @@ extern "C" {
   lmt_all_sct** lmt_mult,   /* input hyperlab limits     */
   int nbr_dmn,         /* number of dimensions */    
   long *dmn_sbs_dsk,  /* Output - indices relative to disk */
-  bool FREE);        /* Free static space on last call */
+  nco_bool FREE);        /* Free static space on last call */
 
   void
   nco_msa_prn_var_val   /* [fnc] Print variable data */
   (const int in_id, /* I [id] netCDF input file ID */
    const char * const var_nm, /* I [sng] Variable name */
-    lmt_all_sct *  const lmt_lst, /* I [sct] Dimension limits */
+    lmt_all_sct *  const * lmt_lst, /* I [sct] Dimension limits */
    const int lmt_nbr, /* I [nbr] number of dimensions with user-specified limits */
    char * const dlm_sng, /* I [sng] User-specified delimiter string, if any */
-   const bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
-   const bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
-   const bool PRN_DMN_IDX_CRD_VAL); /* I [flg] Print dimension/coordinate indices/values */
+   const nco_bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
+   const nco_bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
+   const nco_bool PRN_DMN_IDX_CRD_VAL); /* I [flg] Print dimension/coordinate indices/values */
   
 #ifdef __cplusplus
 } /* end extern "C" */
