@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_mss_val.h,v 1.12 2004/01/12 18:11:07 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_mss_val.h,v 1.19 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Missing value utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_mss_val.h" *//* Missing value utilities */
@@ -28,9 +28,10 @@
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_cnf_typ.h" /* Conform variable types */
 #include "nco_mmr.h" /* Memory management */
-#if (!defined HAVE_STRCASECMP) || (!defined HAVE_STRDUP)
+#include "nco_prn.h" /* Printing variables, attributes, metadata */
+#if (defined NEED_STRCASECMP) || (defined NEED_STRDUP)
 #include "nco_sng_utl.h" /* String utilities */
-#endif /* HAVE_STRCASECMP || HAVE_STRDUP */
+#endif /* NEED_STRCASECMP || NEED_STRDUP */
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,7 +41,7 @@ ptr_unn /* O [sct] Default missing value for type type */
 nco_mss_val_mk /* [fnc] Return default missing value for type type */
 (const nc_type type); /* I [enm] netCDF type of operand */
 
-bool /* O [flg] One or both operands have missing value */
+nco_bool /* O [flg] One or both operands have missing value */
 nco_mss_val_cnf /* [fnc] Change missing_value of var2 to missing_value of var1 */
 (var_sct * const var1, /* I [sct] Variable with template missing value to copy */
  var_sct * const var2); /* I/O [sct] Variable with missing value to fill in/overwrite */
