@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_omp.h,v 1.10 2004/07/02 23:01:13 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_omp.h,v 1.19 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: OpenMP utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_omp.h" *//* OpenMP utilities */
@@ -14,7 +14,7 @@
 
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
-#include <stdlib.h> /* strtod, strtol, malloc, getopt, exit */
+#include <stdlib.h> /* strtod, strtol, malloc, getopt, getenv, exit */
 #include <string.h> /* strcmp. . . */
 #include <unistd.h> /* POSIX stuff */
 
@@ -23,15 +23,32 @@
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 #ifdef _OPENMP
 #include <omp.h> /* OpenMP pragmas */
-#endif /* not _OPENMP */
+#endif /* !_OPENMP */
 
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
+#include "nco_ctl.h" /* Program flow control functions */
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
+#ifndef _OPENMP
+/* OpenMP is not available with this compiler
+   Prototype harmless stub routines for Uni-Processor (UP) code
+   These stubs reduce pre-processor proliferation */
+  int omp_get_dynamic(void);
+  int omp_get_max_threads(void);
+  int omp_get_nested(void);
+  int omp_get_num_procs(void);
+  int omp_get_num_threads(void);
+  int omp_get_thread_num(void);
+  int omp_in_parallel(void);
+  void omp_set_dynamic(int dynamic_threads);
+  void omp_set_nested(int nested);
+  void omp_set_num_threads(int num_threads);
+#endif /* _OPENMP */
+
 int /* O [nbr] Thread number */
 nco_openmp_ini /* [fnc] Initialize OpenMP threading environment */
 (int thr_nbr); /* I [nbr] Thread number */
