@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.c,v 1.53 2004/09/13 18:13:39 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.c,v 1.69 2007/05/18 22:37:45 zender Exp $ */
 
 /* Purpose: NCO utilities for packing and unpacking variables */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_pck.h" /* Packing and unpacking variables */
 
@@ -96,7 +96,7 @@ nco_dfl_case_pck_plc_err(void) /* [fnc] Print error and exit for illegal switch(
   nco_err_exit(0,fnc_nm);
 } /* end nco_dfl_case_pck_plc_err() */
 
-bool /* O [flg] NCO will attempt to pack variable */
+nco_bool /* O [flg] NCO will attempt to pack variable */
 nco_is_packable /* [fnc] Will NCO attempt to pack variable? */
 (const nc_type nc_typ_in) /* I [enm] Type of input variable */
 {
@@ -113,12 +113,18 @@ nco_is_packable /* [fnc] Will NCO attempt to pack variable? */
   switch(nc_typ_in){ 
   case NC_FLOAT: 
   case NC_DOUBLE: 
+  case NC_INT64: 
+  case NC_UINT64: 
   case NC_INT: 
+  case NC_UINT: 
     return True;
     break;
   case NC_SHORT: 
+  case NC_USHORT: 
   case NC_CHAR: 
   case NC_BYTE: 
+  case NC_UBYTE: 
+  case NC_STRING:
     return False;
     break;
   default: nco_dfl_case_nc_type_err(); break;
@@ -176,12 +182,12 @@ nco_pck_plc_get /* [fnc] Convert user-specified packing policy to key */
   prg_nm=prg_nm_get(); /* [sng] Program name */
 
   if(nco_pck_plc_sng == NULL){
-    if(!strcmp(prg_nm,"ncpdq")){
-      (void)fprintf(stdout,"%s: INFO %s reports %s invoked without explicit packing or dimension permutation options. Defaulting to packing policy \"all_new\".\n",prg_nm,fnc_nm,prg_nm);
+    if(strstr(prg_nm,"ncpdq")){
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: INFO %s reports %s invoked without explicit packing or dimension permutation options. Defaulting to packing policy \"all_new\".\n",prg_nm,fnc_nm,prg_nm);
       return nco_pck_plc_all_new_att;
     } /* endif */
-    if(!strcmp(prg_nm,"ncpack")) return nco_pck_plc_all_new_att;
-    if(!strcmp(prg_nm,"ncunpack")) return nco_pck_plc_upk;
+    if(strstr(prg_nm,"ncpack")) return nco_pck_plc_all_new_att;
+    if(strstr(prg_nm,"ncunpack")) return nco_pck_plc_upk;
     (void)fprintf(stderr,"%s: ERROR %s reports empty user-specified packing string in conjunction with unknown or ambiguous executable name %s\n",prg_nm,fnc_nm,prg_nm);
     nco_exit(EXIT_FAILURE);
   } /* endif */
@@ -201,7 +207,7 @@ nco_pck_plc_get /* [fnc] Convert user-specified packing policy to key */
   return nco_pck_plc_nil; /* Statement should not be reached */
 } /* end nco_pck_plc_get() */
 
-bool /* O [flg] Packing policy allows packing nc_typ_in */
+nco_bool /* O [flg] Packing policy allows packing nc_typ_in */
 nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
 (const int nco_pck_map,  /* I [enm] Packing map */
  const nc_type nc_typ_in, /* I [enm] Type of input variable */
@@ -218,7 +224,7 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
      In both cases, nc_typ_pck_out is only set if it is non-NULL */
 
   const char fnc_nm[]="nco_pck_plc_typ_get()"; /* [sng] Function name */
-  bool nco_pck_plc_alw; /* O [flg] Packing policy allows packing nc_typ_in */
+  nco_bool nco_pck_plc_alw; /* O [flg] Packing policy allows packing nc_typ_in */
   nc_type nc_typ_pck_out_tmp; /* O [enm] Type to pack variable to */
 
   /* Initialize output type to NAT and pack allow to False to help catch errors */
@@ -231,11 +237,17 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     switch(nc_typ_in){ 
     case NC_DOUBLE: 
     case NC_FLOAT: 
+    case NC_INT64: 
+    case NC_UINT64: 
     case NC_INT: 
+    case NC_UINT: 
       nc_typ_pck_out_tmp=NC_SHORT; nco_pck_plc_alw=True; break;
     case NC_SHORT: 
-    case NC_CHAR: 
+    case NC_USHORT: 
     case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -244,11 +256,17 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     switch(nc_typ_in){ 
     case NC_DOUBLE: 
     case NC_FLOAT: 
+    case NC_INT64: 
+    case NC_UINT64: 
     case NC_INT: 
+    case NC_UINT: 
     case NC_SHORT: 
+    case NC_USHORT: 
       nc_typ_pck_out_tmp=NC_CHAR; nco_pck_plc_alw=True; break;
-    case NC_CHAR: 
     case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -257,24 +275,38 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     switch(nc_typ_in){ 
     case NC_DOUBLE: 
     case NC_FLOAT: 
+    case NC_INT64: 
+    case NC_UINT64: 
     case NC_INT: 
+    case NC_UINT: 
     case NC_SHORT: 
+    case NC_USHORT: 
       nc_typ_pck_out_tmp=NC_BYTE; nco_pck_plc_alw=True; break;
-    case NC_CHAR: 
     case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
     break;
   case nco_pck_map_nxt_lsr:
     switch(nc_typ_in){ 
-    case NC_DOUBLE: nc_typ_pck_out_tmp=NC_INT; nco_pck_plc_alw=True; break; 
+    case NC_DOUBLE: 
+    case NC_INT64: 
+    case NC_UINT64: 
+      nc_typ_pck_out_tmp=NC_INT; nco_pck_plc_alw=True; break; 
     case NC_FLOAT: 
     case NC_INT: 
+    case NC_UINT: 
       nc_typ_pck_out_tmp=NC_SHORT; nco_pck_plc_alw=True; break;
-    case NC_SHORT: nc_typ_pck_out_tmp=NC_BYTE; nco_pck_plc_alw=True; break;
-    case NC_CHAR: 
+    case NC_SHORT: 
+    case NC_USHORT: 
+      nc_typ_pck_out_tmp=NC_BYTE; nco_pck_plc_alw=True; break;
     case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -284,10 +316,16 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     case NC_DOUBLE: 
     case NC_FLOAT: 
       nc_typ_pck_out_tmp=NC_SHORT; nco_pck_plc_alw=True; break;
-    case NC_INT:
-    case NC_SHORT:
-    case NC_CHAR:
-    case NC_BYTE:
+    case NC_INT64: 
+    case NC_UINT64: 
+    case NC_INT: 
+    case NC_UINT: 
+    case NC_SHORT: 
+    case NC_USHORT: 
+    case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -297,10 +335,16 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     case NC_DOUBLE: 
     case NC_FLOAT: 
       nc_typ_pck_out_tmp=NC_CHAR; nco_pck_plc_alw=True; break;
-    case NC_INT:
-    case NC_SHORT:
-    case NC_CHAR:
-    case NC_BYTE:
+    case NC_INT64: 
+    case NC_UINT64: 
+    case NC_INT: 
+    case NC_UINT: 
+    case NC_SHORT: 
+    case NC_USHORT: 
+    case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -310,10 +354,16 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
     case NC_DOUBLE: 
     case NC_FLOAT: 
       nc_typ_pck_out_tmp=NC_BYTE; nco_pck_plc_alw=True; break;
-    case NC_INT:
-    case NC_SHORT:
-    case NC_CHAR:
-    case NC_BYTE:
+    case NC_INT64: 
+    case NC_UINT64: 
+    case NC_INT: 
+    case NC_UINT: 
+    case NC_SHORT: 
+    case NC_USHORT: 
+    case NC_BYTE: 
+    case NC_UBYTE: 
+    case NC_CHAR: 
+    case NC_STRING: 
       nc_typ_pck_out_tmp=nc_typ_in; nco_pck_plc_alw=False; break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end nc_type switch */ 
@@ -326,11 +376,11 @@ nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
   
   /* Only fill in nc_typ_pck_out if it is non-NULL */
   if(nc_typ_pck_out != NULL) *nc_typ_pck_out=nc_typ_pck_out_tmp;
-
+  
   return nco_pck_plc_alw; /* O [flg] Packing policy allows packing nc_typ_in */
 } /* end nco_pck_plc_typ_get() */
 
-bool /* O [flg] Variable is packed on disk */
+nco_bool /* O [flg] Variable is packed on disk */
 nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
 (const int nc_id, /* I [idx] netCDF file ID */
  var_sct * const var) /* I/O [sct] Variable */
@@ -368,11 +418,11 @@ nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
   rcd=nco_inq_att_flg(nc_id,var->id,scl_fct_sng,&scl_fct_typ,&scl_fct_lng);
   if(rcd != NC_ENOTATT){
     if(scl_fct_typ == NC_BYTE || scl_fct_typ == NC_CHAR){
-      (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports scale_factor for %s is NC_BYTE or NC_CHAR. Will not attempt to unpack using scale_factor.\n",prg_nm_get(),var->nm); 
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports scale_factor for %s is NC_BYTE or NC_CHAR. Will not attempt to unpack using scale_factor.\n",prg_nm_get(),var->nm); 
       return False;
     } /* endif */
     if(scl_fct_lng != 1){
-      (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports %s has scale_factor of length %li. Will not attempt to unpack using scale_factor\n",prg_nm_get(),var->nm,scl_fct_lng); 
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports %s has scale_factor of length %li. Will not attempt to unpack using scale_factor\n",prg_nm_get(),var->nm,scl_fct_lng); 
       return False;
     } /* endif */
     var->has_scl_fct=True; /* [flg] Valid scale_factor attribute exists */
@@ -383,11 +433,11 @@ nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
   rcd=nco_inq_att_flg(nc_id,var->id,add_fst_sng,&add_fst_typ,&add_fst_lng);
   if(rcd != NC_ENOTATT){
     if(add_fst_typ == NC_BYTE || add_fst_typ == NC_CHAR){
-      (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports add_offset for %s is NC_BYTE or NC_CHAR. Will not attempt to unpack using add_offset.\n",prg_nm_get(),var->nm); 
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports add_offset for %s is NC_BYTE or NC_CHAR. Will not attempt to unpack using add_offset.\n",prg_nm_get(),var->nm); 
       return False;
     } /* endif */
     if(add_fst_lng != 1){
-      (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports %s has add_offset of length %li. Will not attempt to unpack.\n",prg_nm_get(),var->nm,add_fst_lng); 
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports %s has add_offset of length %li. Will not attempt to unpack.\n",prg_nm_get(),var->nm,add_fst_lng); 
       return False;
     } /* endif */
     var->has_add_fst=True; /* [flg] Valid add_offset attribute exists */
@@ -396,7 +446,7 @@ nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
 
   if(var->has_scl_fct && var->has_add_fst){
     if(scl_fct_typ != add_fst_typ){
-      (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports type of scale_factor does not equal type of add_offset. Will not attempt to unpack.\n",prg_nm_get());
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING nco_pck_dsk_inq() reports type of scale_factor does not equal type of add_offset. Will not attempt to unpack.\n",prg_nm_get());
       return False;
     } /* endif */
   } /* endif */
@@ -407,7 +457,7 @@ nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
     /* If variable is packed on disk and is in memory then variable is packed in memory */
     var->pck_ram=True; /* [flg] Variable is packed in memory */
     var->typ_upk=(var->has_scl_fct) ? scl_fct_typ : add_fst_typ; /* [enm] Type of variable when unpacked (expanded) (in memory) */
-    if(nco_is_rth_opr(prg_get()) && dbg_lvl_get() > 2){
+    if(nco_is_rth_opr(prg_get()) && dbg_lvl_get() >= nco_dbg_var){
       (void)fprintf(stdout,"%s: PACKING Variable %s is type %s packed into type %s\n",prg_nm_get(),var->nm,nco_typ_sng(var->typ_upk),nco_typ_sng(var->typ_dsk));
       (void)fprintf(stdout,"%s: DEBUG Packed variables processed by all arithmetic operators are unpacked automatically, and then stored unpacked in the output file. If you wish to repack them in the output file, use, e.g., ncap -O -s \"foo=pack(foo);\" out.nc out.nc. If you wish to pack all variables in a file, use, e.g., ncpdq -P all_new in.nc out.nc.\n",prg_nm_get());
     } /* endif print packing information */
@@ -432,13 +482,12 @@ nco_pck_mtd /* [fnc] Alter metadata according to packing specification */
   /* Purpose: Alter metadata according to packing specification */
   const char fnc_nm[]="nco_pck_mtd()"; /* [sng] Function name */
   nc_type nc_typ_pck_out; /* [enm] Type to pack to */
-  bool nco_pck_plc_alw; /* [flg] Packing policy allows packing nc_typ_in */
   
   switch(nco_pck_plc){
   case nco_pck_plc_all_xst_att:
     /* If variable is already packed do nothing otherwise pack to default type */
     if(var_in->pck_ram){
-      if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: DEBUG %s keeping existing packing parameters and type (%s) for %s\n",prg_nm_get(),fnc_nm,nco_typ_sng(var_in->type),var_in->nm);
+      if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: DEBUG %s keeping existing packing parameters and type (%s) for %s\n",prg_nm_get(),fnc_nm,nco_typ_sng(var_in->type),var_in->nm);
     }else{
       goto var_upk_try_to_pck;
     } /* endif */
@@ -449,7 +498,7 @@ nco_pck_mtd /* [fnc] Alter metadata according to packing specification */
       goto var_pck_try_to_rpk;
     }else{
       /* Variable is not packed so do nothing */
-      if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: INFO %s leaving variable %s of type %s as unpacked\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_upk));
+      if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: INFO %s leaving variable %s of type %s as unpacked\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_upk));
     } /* endelse */
     break;
   case nco_pck_plc_all_new_att:
@@ -461,7 +510,7 @@ nco_pck_mtd /* [fnc] Alter metadata according to packing specification */
     break;
   case nco_pck_plc_upk:
     var_out->type=var_in->typ_upk;
-    if(dbg_lvl_get() > 3){
+    if(dbg_lvl_get() >= nco_dbg_sbr){
       if(var_in->pck_ram) (void)fprintf(stdout,"%s: DEBUG %s will unpack variable %s from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->type),nco_typ_sng(var_out->type)); else (void)fprintf(stdout,"%s: DEBUG %s variable %s is already unpacked and of type %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->type)); 
     } /* endif dbg */
     break;
@@ -475,23 +524,23 @@ nco_pck_mtd /* [fnc] Alter metadata according to packing specification */
 
  var_upk_try_to_pck: /* end goto */
   /* Variable is not yet packed---try to pack it */
-  if((nco_pck_plc_alw=nco_pck_plc_typ_get(nco_pck_map,var_in->type,&nc_typ_pck_out))){
+  if(nco_pck_plc_typ_get(nco_pck_map,var_in->type,&nc_typ_pck_out)){
     var_out->type=nc_typ_pck_out;
-    if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: DEBUG %s will pack variable %s from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->type),nco_typ_sng(var_out->type));
+    if(dbg_lvl_get() >= nco_dbg_sbr) (void)fprintf(stdout,"%s: DEBUG %s will pack variable %s from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->type),nco_typ_sng(var_out->type));
   }else{
-    if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: INFO %s packing policy %s with packing map %s does not allow packing variable %s of type %s, skipping...\n",prg_nm_get(),fnc_nm,nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),var_in->nm,nco_typ_sng(var_in->type));
-  } /* endif nco_pck_plc_alw */
+    if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: INFO %s packing policy %s with packing map %s does not allow packing variable %s of type %s, skipping...\n",prg_nm_get(),fnc_nm,nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),var_in->nm,nco_typ_sng(var_in->type));
+  } /* !nco_pck_plc_alw */
   return;
   
  var_pck_try_to_rpk: /* end goto */
   /* Variable is already packed---try to re-pack it 
      Final packed variable type may differ from original */
-  if((nco_pck_plc_alw=nco_pck_plc_typ_get(nco_pck_map,var_in->typ_upk,&nc_typ_pck_out))){
+  if(nco_pck_plc_typ_get(nco_pck_map,var_in->typ_upk,&nc_typ_pck_out)){
     var_out->type=nc_typ_pck_out;
-    if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: DEBUG %s will re-pack variable %s of expanded type %s from current packing (type %s) into new packing of type %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->typ_upk),nco_typ_sng(var_in->type),nco_typ_sng(var_out->type));
+    if(dbg_lvl_get() >= nco_dbg_sbr) (void)fprintf(stdout,"%s: DEBUG %s will re-pack variable %s of expanded type %s from current packing (type %s) into new packing of type %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->typ_upk),nco_typ_sng(var_in->type),nco_typ_sng(var_out->type));
   }else{
-    if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: WARNING %s variable %s of expanded type %s is already packed into type %s and re-packing is requested but packing policy %s and packing map %s does not allow re-packing variables of type %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->typ_upk),nco_typ_sng(var_in->type),nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),nco_typ_sng(var_in->typ_upk));
-  } /* endif nco_pck_plc_alw */
+    if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: WARNING %s variable %s of expanded type %s is already packed into type %s and re-packing is requested but packing policy %s and packing map %s does not allow re-packing variables of type %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_in->typ_upk),nco_typ_sng(var_in->type),nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),nco_typ_sng(var_in->typ_upk));
+  } /* !nco_pck_plc_alw */
   return;
 
 } /* end nco_pck_mtd() */
@@ -507,9 +556,8 @@ nco_pck_val /* [fnc] Pack variable according to packing specification */
 {
   /* Purpose: Alter metadata according to packing specification */
   const char fnc_nm[]="nco_pck_val()"; /* [sng] Function name */
-  bool PCK_VAR_WITH_NEW_PCK_ATT=False; /* [flg] Insert new scale_factor and add_offset into lists */
+  nco_bool PCK_VAR_WITH_NEW_PCK_ATT=False; /* [flg] Insert new scale_factor and add_offset into lists */
   nc_type typ_out; /* [enm] Type in output file */
-  bool nco_pck_plc_alw; /* [flg] Packing policy allows packing nc_typ_in */
   
   /* typ_out contains type of variable defined in output file
      as defined by var_out->type which was set in var_pck_mtd() 
@@ -522,7 +570,7 @@ nco_pck_val /* [fnc] Pack variable according to packing specification */
     /* nco_var_pck() expects to alter var_out->type itself, if necessary */
     var_out->type=var_in->typ_dsk;
     if(var_in->pck_ram){
-      if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: INFO %s keeping existing packing attributes for variable %s\n",prg_nm_get(),fnc_nm,var_in->nm);
+      if(dbg_lvl_get() >= nco_dbg_sbr) (void)fprintf(stdout,"%s: INFO %s keeping existing packing attributes for variable %s\n",prg_nm_get(),fnc_nm,var_in->nm);
       /* Warn if packing attribute values are in memory for pre-packed variables */
       if(var_out->scl_fct.vp != NULL || var_out->add_fst.vp != NULL) (void)fprintf(stdout,"%s: WARNING %s reports variable %s has packing attribute values in memory. This is not supposed to happen through known code paths, but is not necessarily dangerous.\n",prg_nm_get(),fnc_nm,var_in->nm);
       /* Remove dangling pointer, see explanation below */
@@ -563,12 +611,12 @@ nco_pck_val /* [fnc] Pack variable according to packing specification */
 
  var_upk_try_to_pck: /* end goto */
   /* Variable is not yet packed---try to pack it */
-  if((nco_pck_plc_alw=nco_pck_plc_typ_get(nco_pck_map,var_out->type,(nc_type *)NULL))){
-    if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: INFO %s packing variable %s values from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_upk),nco_typ_sng(typ_out));
+  if(nco_pck_plc_typ_get(nco_pck_map,var_out->type,(nc_type *)NULL)){
+    if(dbg_lvl_get() >= nco_dbg_sbr) (void)fprintf(stdout,"%s: INFO %s packing variable %s values from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_upk),nco_typ_sng(typ_out));
     var_out=nco_var_pck(var_out,typ_out,&PCK_VAR_WITH_NEW_PCK_ATT);
   }else{
-    if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: INFO %s packing policy %s with packing map %s does not allow packing variable %s of type %s, skipping...\n",prg_nm_get(),fnc_nm,nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),var_in->nm,nco_typ_sng(var_out->typ_upk));
-  } /* endif nco_pck_plc_alw */ 
+    if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: INFO %s packing policy %s with packing map %s does not allow packing variable %s of type %s, skipping...\n",prg_nm_get(),fnc_nm,nco_pck_plc_sng_get(nco_pck_plc),nco_pck_map_sng_get(nco_pck_map),var_in->nm,nco_typ_sng(var_out->typ_upk));
+  } /* !nco_pck_plc_alw */ 
   /* Packing function nco_var_pck() usually free()'s var_out->val.vp 
      Hence var_in->val.vp is left with a dangling pointer
      In ncpdq, var_in->val.vp and var_out->val.vp point to same buffer 
@@ -609,7 +657,7 @@ nco_put_var_pck /* [fnc] Pack variable in memory and write packing attributes to
      ncpdq breaks up writing packed variables into multiple tasks, i.e.,
      ncpdq separates variable value writes from packing attribute value writes.
      This routine is intended to write a packed variable in one routine */
-  bool PCK_VAR_WITH_NEW_PCK_ATT=False; /* [flg] Insert new scale_factor and add_offset into lists */
+  nco_bool PCK_VAR_WITH_NEW_PCK_ATT=False; /* [flg] Insert new scale_factor and add_offset into lists */
   
   switch(nco_pck_plc){
   case nco_pck_plc_all_xst_att:
@@ -645,7 +693,7 @@ var_sct * /* O [sct] Packed variable */
 nco_var_pck /* [fnc] Pack variable in memory */
 (var_sct *var, /* I/O [sct] Variable to be packed */
  const nc_type nc_typ_pck, /* I [enm] Type of variable when packed (on disk). This should be same as typ_dsk except in cases where variable is packed in input file and unpacked in output file. */
- bool *PCK_VAR_WITH_NEW_PCK_ATT) /* O [flg] Routine generated new scale_factor/add_offset */
+ nco_bool *PCK_VAR_WITH_NEW_PCK_ATT) /* O [flg] Routine generated new scale_factor/add_offset */
 {
   /* Purpose: Pack variable 
      Routine is inverse of nco_var_upk(): nco_var_pck[nco_var_upk(var)]=var 
@@ -655,7 +703,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
      NB: Value buffer var->val.vp is usually free()'d here
      Variables in calling routine which point to var->val.vp will be left dangling */
 
-  bool PURE_MSS_VAL_FLD=False; /* [flg] Field is pure missing_value, i.e., no valid values */
+  nco_bool PURE_MSS_VAL_FLD=False; /* [flg] Field is pure missing_value, i.e., no valid values */
   const char fnc_nm[]="nco_var_pck()"; /* [sng] Function name */
   double scl_fct_dbl=double_CEWI; /* [sct] Double precision value of scale_factor */
   double add_fst_dbl=double_CEWI; /* [sct] Double precision value of add_offset */
@@ -670,7 +718,9 @@ nco_var_pck /* [fnc] Pack variable in memory */
   if(var->val.vp == NULL) (void)fprintf(stdout,"%s: ERROR %s called with empty var->val.vp\n",prg_nm_get(),fnc_nm);
   
   /* Packed type must be NC_BYTE, NC_CHAR, NC_SHORT, or NC_INT */
-  if(nc_typ_pck == NC_FLOAT || nc_typ_pck == NC_DOUBLE){
+  if(nc_typ_pck == NC_FLOAT || nc_typ_pck == NC_DOUBLE || 
+     nc_typ_pck == NC_UINT  || nc_typ_pck == NC_USHORT || 
+     nc_typ_pck == NC_UBYTE || nc_typ_pck == NC_STRING){
     (void)fprintf(stdout,"%s: ERROR %s called to pack variable %s with invalid packed type nc_typ_pck = %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(nc_typ_pck));
     nco_exit(EXIT_FAILURE);
   } /* endif */
@@ -679,7 +729,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
      Definition of "packable" determined by nco_pck_plc_typ_get()
      Prefer not to make nco_var_pck() rely directly on nco_pck_plc_typ_get()
      However, certain types are never packable */
-  if(var->type == NC_CHAR || var->type == NC_BYTE){
+  if(var->type == NC_BYTE || var->type == NC_UBYTE || var->type == NC_CHAR || var->type == NC_STRING){
     (void)fprintf(stdout,"%s: ERROR %s is asked to pack variable %s of type %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(var->type));
     nco_exit(EXIT_FAILURE);
   } /* endif */
@@ -738,8 +788,8 @@ nco_var_pck /* [fnc] Pack variable in memory */
     } /* endif has_mss_val */
 
     /* Find minimum and maximum values in data */
-    (void)nco_var_avg_reduce_max(var->type,var->sz,1L,var->has_mss_val,var->mss_val,var->val,ptr_unn_min);
-    (void)nco_var_avg_reduce_min(var->type,var->sz,1L,var->has_mss_val,var->mss_val,var->val,ptr_unn_max);
+    (void)nco_var_avg_rdc_max(var->type,var->sz,1L,var->has_mss_val,var->mss_val,var->val,ptr_unn_min);
+    (void)nco_var_avg_rdc_min(var->type,var->sz,1L,var->has_mss_val,var->mss_val,var->val,ptr_unn_max);
 
     /* Convert to NC_DOUBLE before 0.5*(min+max) operation */
     min_var=scl_ptr_mk_var(ptr_unn_min,var->type);
@@ -757,20 +807,26 @@ nco_var_pck /* [fnc] Pack variable in memory */
 
     /* Change value of missing value iff necessary to fit inside packed type */
     if(var->has_mss_val && !PURE_MSS_VAL_FLD){
-      double mss_val_dfl_dbl;
+      double mss_val_dfl_dbl=0.0; /* CEWI */
       switch(nc_typ_pck){ 
-      case NC_INT: mss_val_dfl_dbl=INT_MAX; break;
-      case NC_SHORT: mss_val_dfl_dbl=SHRT_MAX; break;
-      case NC_CHAR: mss_val_dfl_dbl=UCHAR_MAX; break;
-      case NC_BYTE: mss_val_dfl_dbl=CHAR_MAX; break;
-      case NC_FLOAT:
-      case NC_DOUBLE: 
+      case NC_FLOAT: mss_val_dfl_dbl=NC_FILL_FLOAT; break; 
+      case NC_DOUBLE: mss_val_dfl_dbl=NC_FILL_DOUBLE; break; 
+      case NC_INT: mss_val_dfl_dbl=NC_FILL_INT; break;
+      case NC_SHORT: mss_val_dfl_dbl=NC_FILL_SHORT; break;
+      case NC_USHORT: mss_val_dfl_dbl=NC_FILL_USHORT; break;
+      case NC_UINT: mss_val_dfl_dbl=NC_FILL_UINT; break;
+      case NC_INT64: mss_val_dfl_dbl=NC_FILL_INT64; break;
+      case NC_UINT64: mss_val_dfl_dbl=NC_FILL_UINT64; break;
+      case NC_BYTE: mss_val_dfl_dbl=NC_FILL_BYTE; break;
+      case NC_UBYTE: mss_val_dfl_dbl=NC_FILL_UBYTE; break;
+      case NC_CHAR: mss_val_dfl_dbl=NC_FILL_CHAR; break;
+      case NC_STRING: break; /* Do nothing */
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */ 
-      if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: %s mss_val_dfl = %g\n",prg_nm_get(),fnc_nm,mss_val_dfl_dbl);
+      if(dbg_lvl_get() >= nco_dbg_io) (void)fprintf(stdout,"%s: %s mss_val_dfl = %g\n",prg_nm_get(),fnc_nm,mss_val_dfl_dbl);
     } /* endif */
 
-    if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: %s: min_var = %g, max_var = %g\n",prg_nm_get(),var->nm,min_var->val.dp[0],max_var->val.dp[0]);
+    if(dbg_lvl_get() >= nco_dbg_io) (void)fprintf(stdout,"%s: %s: min_var = %g, max_var = %g\n",prg_nm_get(),var->nm,min_var->val.dp[0],max_var->val.dp[0]);
 
     /* add_offset is 0.5*(min+max) */
     /* max_var->val is overridden with add_offset answers, no longer valid as max_var */
@@ -819,8 +875,8 @@ nco_var_pck /* [fnc] Pack variable in memory */
     } /* end else */
 
     if(max_mns_min_dbl > max_mns_min_dbl_wrn){
-      (void)fprintf(stdout,"%s: WARNING %s reports data range of variable %s is = %g. The linear data packing technique defined by netCDF's packing convention and implemented by NCO result in significant precision loss over such a great range.\n",prg_nm_get(),fnc_nm,var->nm,max_mns_min_dbl);
-      if(var->has_mss_val) (void)fprintf(stdout,"%s: HINT variable %s has missing_value = %g. Consider specifying new missing_value to reduce range of data needing packing. See http://nco.sf.net/nco.html#ncatted for examples of how to change the missing_value attribute.\n",prg_nm_get(),fnc_nm,ptr_unn_mss_val_dbl.dp[0]);
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING %s reports data range of variable %s is = %g. The linear data packing technique defined by netCDF's packing convention and implemented by NCO result in significant precision loss over such a great range.\n",prg_nm_get(),fnc_nm,var->nm,max_mns_min_dbl);
+      if(dbg_lvl_get() >= nco_dbg_std) if(var->has_mss_val) (void)fprintf(stdout,"%s: HINT variable %s has NCO_MSS_VAL_SNG = %g. Consider specifying new NCO_MSS_VAL_SNG to reduce range of data needing packing. See http://nco.sf.net/nco.html#ncatted for examples of how to change the NCO_MSS_VAL_SNG attribute.\n",prg_nm_get(),fnc_nm,ptr_unn_mss_val_dbl.dp[0]);
     } /* endif large data range */
 
     /* Free minimum and maximum values */
@@ -854,7 +910,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
   /* Create double precision value of scale_factor for diagnostics */
   if(var->has_scl_fct){ /* [flg] Valid scale_factor attribute exists */
     scl_fct_dbl=ptr_unn_2_scl_dbl(var->scl_fct,var->type); 
-    if(scl_fct_dbl == 0.0) (void)fprintf(stdout,"%s: WARNING %s reports scl_fct_dbl = 0.0\n",prg_nm_get(),fnc_nm);
+    if(scl_fct_dbl == 0.0 && dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: WARNING %s reports scl_fct_dbl = 0.0\n",prg_nm_get(),fnc_nm);
   } /* endif */
   
   /* Create double precision value of add_offset for diagnostics */
@@ -862,7 +918,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
     add_fst_dbl=ptr_unn_2_scl_dbl(var->add_fst,var->type);
   } /* endif */
   
-  if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: %s reports variable %s has scl_fct_dbl = %g, add_fst_dbl = %g\n",prg_nm_get(),fnc_nm,var->nm,scl_fct_dbl,add_fst_dbl);
+  if(dbg_lvl_get() >= nco_dbg_io) (void)fprintf(stdout,"%s: %s reports variable %s has scl_fct_dbl = %g, add_fst_dbl = %g\n",prg_nm_get(),fnc_nm,var->nm,scl_fct_dbl,add_fst_dbl);
   
   /* Packing attributes now exist and are same type as variable in memory */
 
@@ -872,7 +928,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
      Using var_scv_[sub,multiply] instead of ncap_var_scv_[sub,multiply] avoids cost of deep copies
      Moreover, this keeps variable structure from changing (because ncap_var_scv_* functions all do deep copies before operations) and thus complicating memory management */
   if(var->has_add_fst){ /* [flg] Valid add_offset attribute exists */
-    bool has_mss_val_tmp; /* [flg] Temporary missing_value flag */
+    nco_bool has_mss_val_tmp; /* [flg] Temporary missing_value flag */
 
     /* Subtract add_offset from var */
     scv_sct add_fst_scv;
@@ -884,7 +940,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
     /* Dupe var_scv_sub() into subtracting missing values when all values are missing */
     if(PURE_MSS_VAL_FLD){
       has_mss_val_tmp=False;
-      (void)fprintf(stdout,"%s: INFO %s reports variable %s is completely missing_value = %g. Why do you store variables with no valid values?\n",prg_nm_get(),fnc_nm,var->nm,add_fst_dbl);
+      if(dbg_lvl_get() >= nco_dbg_std) (void)fprintf(stdout,"%s: INFO %s reports variable %s is filled completely with missing_value = %g. Why do you store variables with no valid values?\n",prg_nm_get(),fnc_nm,var->nm,add_fst_dbl);
     } /* !PURE_MSS_VAL_FLD */
     (void)var_scv_sub(var->type,var->sz,has_mss_val_tmp,var->mss_val,var->val,&add_fst_scv);
   } /* endif */
@@ -917,7 +973,7 @@ nco_var_pck /* [fnc] Pack variable in memory */
      This is where var->type is changed from original to packed type */
   var=nco_var_cnf_typ(nc_typ_pck,var);
 
-  if(dbg_lvl_get() >= 3) (void)fprintf(stdout,"%s: PACKING %s packed %s into %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(var->type));
+  if(dbg_lvl_get() >= nco_dbg_sbr) (void)fprintf(stdout,"%s: PACKING %s packed %s into %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(var->type));
 
   return var;
 } /* end nco_var_pck() */
@@ -926,7 +982,7 @@ var_sct * /* O [sct] Unpacked variable */
 nco_var_upk /* [fnc] Unpack variable in memory */
 (var_sct *var) /* I/O [sct] Variable to be unpacked */
 {
-  /* Threads: Routine is thread-unsafe */
+  /* Threads: Routine is thread-safe */
   /* Purpose: Unpack variable
      Routine is inverse of nco_var_pck(): nco_var_upk[nco_var_pck(var)]=var
      Routine handles missing_value's implicitly:
@@ -941,7 +997,10 @@ nco_var_upk /* [fnc] Unpack variable in memory */
   if(!var->pck_ram) return var;
 
   /* Routine should be called with variable already in memory */
-  if(var->val.vp == NULL) (void)fprintf(stdout,"%s: ERROR nco_var_upk() called with empty var->val.vp\n",prg_nm_get());
+  if(var->val.vp == NULL){
+    (void)fprintf(stdout,"%s: ERROR nco_var_upk() called with empty var->val.vp\n",prg_nm_get());
+    nco_exit(EXIT_FAILURE);
+  } /* endif */
 
   /* Packed variables are not guaranteed to have both scale_factor and add_offset
      scale_factor is guaranteed to be of type NC_FLOAT or NC_DOUBLE and of size 1 (a scalar) */
@@ -961,6 +1020,7 @@ nco_var_upk /* [fnc] Unpack variable in memory */
   if(var->has_add_fst){ /* [flg] Valid add_offset attribute exists */
     scv_sct add_fst_scv;
     var->add_fst.vp=(void *)nco_malloc(nco_typ_lng(var->typ_upk));
+    /* fxm TODO nco638 */
     (void)nco_get_att(var->nc_id,var->id,add_fst_sng,var->add_fst.vp,var->typ_upk);
     add_fst_scv=ptr_unn_2_scv(var->typ_upk,var->add_fst);
     /* Convert var to type of scale_factor for expansion */
@@ -980,7 +1040,7 @@ nco_var_upk /* [fnc] Unpack variable in memory */
   var->scl_fct.vp=nco_free(var->scl_fct.vp);
   var->add_fst.vp=nco_free(var->add_fst.vp);
 
-  if(dbg_lvl_get() > 2) (void)fprintf(stdout,"%s: PACKING %s unpacked %s into %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(var->type));
+  if(dbg_lvl_get() >= nco_dbg_var) (void)fprintf(stdout,"%s: PACKING %s unpacked %s into %s\n",prg_nm_get(),fnc_nm,var->nm,nco_typ_sng(var->type));
 
   return var;
 } /* end nco_var_upk() */
@@ -1007,7 +1067,7 @@ nco_var_upk_swp /* [fnc] Unpack var_in into var_out */
   var_sct *var_tmp; /* [sct] Temporary variable to be unpacked */
   
   if(var_in->pck_ram){
-    if(dbg_lvl_get() > 3) (void)fprintf(stdout,"%s: DEBUG %s unpacking variable %s values from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_pck),nco_typ_sng(var_out->typ_upk));
+    if(dbg_lvl_get() >= nco_dbg_io) (void)fprintf(stdout,"%s: DEBUG %s unpacking variable %s values from %s to %s\n",prg_nm_get(),fnc_nm,var_in->nm,nco_typ_sng(var_out->typ_pck),nco_typ_sng(var_out->typ_upk));
   }else{
     (void)fprintf(stderr,"%s: ERROR %s variable %s is already unpacked\n",prg_nm_get(),fnc_nm,var_in->nm);
     nco_exit(EXIT_FAILURE);
