@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.h,v 1.35 2004/09/08 00:42:30 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_pck.h,v 1.42 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Description (definition) of packing/unpacking functions */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_pck.h" *//* Packing and unpacking variables */
@@ -44,7 +44,7 @@ enum nco_pck_plc{ /* [enm] Packing policy */
 /* Packing type maps: */
 enum nco_pck_map{ /* [enm] Packing conversion map */
   /* NB: Packing to NC_CHAR is implemented but not advertised or supported because 
-     (char)0 == NUL, a special value with implicit conversion rules that are difficult to deal with.
+     (char)0 == NUL, a special value with difficult implicit conversion rules.
      We recommend always packing to type NC_BYTE rather than NC_CHAR */
   nco_pck_map_nil, /* 0 [enm] Do not convert anything, i.e., all types remain unchanged */
   nco_pck_map_hgh_sht, /* 1 [enm] Pack higher precision types to NC_SHORT, pack nothing else
@@ -68,14 +68,12 @@ extern "C" {
 #endif /* __cplusplus */
 
 void 
-nco_dfl_case_pck_map_err /* [fnc] Print error and exit for illegal switch(pck_map) case */
-(void);
+nco_dfl_case_pck_map_err(void); /* [fnc] Print error and exit for illegal switch(pck_map) case */
 
 void 
-nco_dfl_case_pck_plc_err /* [fnc] Print error and exit for illegal switch(nco_pck_plc) case */
-(void);
+nco_dfl_case_pck_plc_err(void); /* [fnc] Print error and exit for illegal switch(nco_pck_plc) case */
 
-bool /* O [flg] NCO will attempt to pack variable */
+nco_bool /* O [flg] NCO will attempt to pack variable */
 nco_is_packable /* [fnc] Will NCO attempt to pack variable? */
 (const nc_type nc_typ_in); /* I [enm] Type of input variable */
 
@@ -87,7 +85,7 @@ const char * /* O [sng] Packing policy string */
 nco_pck_plc_sng_get /* [fnc] Convert packing policy enum to string */
 (const int nco_pck_plc); /* I [enm] Packing policy */
 
-bool /* O [flg] Packing policy allows packing nc_typ_in */
+nco_bool /* O [flg] Packing policy allows packing nc_typ_in */
 nco_pck_plc_typ_get /* [fnc] Determine type, if any, to pack input type to */
 (const int nco_pck_map, /* I [enm] Packing map */
  const nc_type nc_typ_in, /* I [enm] Type of input variable */
@@ -117,7 +115,7 @@ int /* O [enm] Packing policy */
 nco_pck_plc_get /* [fnc] Convert user-specified packing policy to key */
 (const char *nco_pck_plc_sng); /* [sng] User-specified packing policy */
   
-bool /* O [flg] Variable is packed on disk */
+nco_bool /* O [flg] Variable is packed on disk */
 nco_pck_dsk_inq /* [fnc] Check whether variable is packed on disk */
 (const int nc_id, /* I [idx] netCDF file ID */
  var_sct *var); /* I/O [sct] Variable */
@@ -132,7 +130,7 @@ var_sct * /* O [sct] Packed variable */
 nco_var_pck /* [fnc] Pack variable in memory */
 (var_sct *var, /* I/O [sct] Variable to be packed */
  const nc_type typ_pck, /* I [enm] Type of variable when packed (on disk). This should be same as typ_dsk except in cases where variable is packed in input file and unpacked in output file. */
- bool *PCK_VAR_WITH_NEW_PCK_ATT); /* O [flg] Routine generated new scale_factor/add_offset */
+ nco_bool *PCK_VAR_WITH_NEW_PCK_ATT); /* O [flg] Routine generated new scale_factor/add_offset */
 
 var_sct * /* O [sct] Unpacked variable */
 nco_var_upk /* [fnc] Unpack variable in memory */
