@@ -1,17 +1,13 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.18 2004/09/24 00:54:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.c,v 1.41 2007/05/25 17:36:47 zender Exp $ */
 
 /* Purpose: Printing variables, attributes, metadata */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_prn.h" /* Printing variables, attributes, metadata */
 
-/* fxm: strings statically allocated with MAX_LEN_FMT_SNG chars are susceptible to buffer overflow attacks */
-/* Length should be computed at run time but is a pain */
-#define MAX_LEN_FMT_SNG 100
-
 void 
 nco_prn_att /* [fnc] Print all attributes of single variable */
 (const int in_id, /* I [id] netCDF input file ID */
@@ -26,7 +22,7 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
 
   char dlm_sng[3];
   char src_sng[NC_MAX_NAME];
-  char att_sng[MAX_LEN_FMT_SNG];
+  char att_sng[NCO_MAX_LEN_FMT_SNG];
 
   long att_lmn;
   long att_sz;
@@ -87,9 +83,28 @@ nco_prn_att /* [fnc] Print all attributes of single variable */
       } /* end loop over element */
       break;
     case NC_BYTE:
-      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,"%c",att[idx].val.bp[att_lmn]);
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.bp[att_lmn]);
+      break;
+    case NC_UBYTE:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.ubp[att_lmn]);
+      break;
+    case NC_USHORT:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.usp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      break;
+    case NC_UINT:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.uip[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      break;
+    case NC_INT64:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.i64p[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      break;
+    case NC_UINT64:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.ui64p[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      break;
+    case NC_STRING:
+      for(att_lmn=0;att_lmn<att_sz;att_lmn++) (void)fprintf(stdout,att_sng,att[idx].val.sngp[att_lmn],(att_lmn != att_sz-1) ? dlm_sng : "");
+      break;
+    default: nco_dfl_case_nc_type_err();
       break;
-    default: nco_dfl_case_nc_type_err(); break;
     } /* end switch */
     (void)fprintf(stdout,"\n");
     
@@ -113,25 +128,50 @@ nco_typ_fmt_sng /* [fnc] Provide sprintf() format string for specified type */
 (const nc_type typ) /* I [enm] netCDF type to provide format string for */
 {
   /* Purpose: Provide sprintf() format string for specified type */
-  switch (typ) {
+  static const char fmt_NC_FLOAT[]="%g"; /* %g defaults to 6 digits of precision */
+  static const char fmt_NC_DOUBLE[]="%.12g"; /* Specify 12 digits of precision for double precision */
+  static const char fmt_NC_INT[]="%li"; /* NCO currently stores NC_INT in native type long, but this will be deprecated when netCDF supports a 64 bit integer type */
+  static const char fmt_NC_SHORT[]="%hi";
+  static const char fmt_NC_CHAR[]="%c";
+  /* Formats useful in printing byte data as decimal notation */
+  static const char fmt_NC_BYTE[]="%i";
+  /* static const char fmt_NC_BYTE[]="%c"; */
+  /* NB: %hhi is GNU extension, not ANSI standard */
+  /* static const char fmt_NC_BYTE[]="%hhi";*/ /* Takes signed char as arg and prints 0..255 (unfortunately) */
+  /* static const char fmt_NC_BYTE[]="%hhu";*/ /* Takes unsigned char as arg and prints 0..255 */
+
+  static const char fmt_NC_UBYTE[]="%hhu"; /*  */
+  static const char fmt_NC_USHORT[]="%hu"; /*  */
+  static const char fmt_NC_UINT[]="%lu"; /*  */
+  static const char fmt_NC_INT64[]="%lli"; /*  */
+  static const char fmt_NC_UINT64[]="%llu"; /*  */
+  static const char fmt_NC_STRING[]="%s"; /*  */
+
+  switch (typ){
   case NC_FLOAT:
-    return "%g"; /* %g defaults to 6 digits of precision */
+    return fmt_NC_FLOAT;
   case NC_DOUBLE:
-    return "%.12g"; /* Specify 12 digits of precision for double precision */
+    return fmt_NC_DOUBLE;
   case NC_INT:
-    return "%li"; /* NCO currently stores NC_INT in native type long, but this will be deprecated when netCDF supports a 64 bit integer type */
-    /*  case NC_LONG:
-	return "%li"; */
+    return fmt_NC_INT;
   case NC_SHORT:
-    return "%hi";
+    return fmt_NC_SHORT;
   case NC_CHAR:
-    return "%c";
+    return fmt_NC_CHAR;
   case NC_BYTE:
-    /* return "%c"; */ /* Default */
-    /* Formats useful in printing byte data as decimal notation */
-    /*    return "%u";*/
-    /* NB: Uncertain wheter %hhi is ANSI standard or GNU extension */
-    return "%hhi"; /* Takes unsigned char as arg and prints 0..255 */
+    return fmt_NC_BYTE;
+  case NC_UBYTE:
+    return fmt_NC_UBYTE; 
+  case NC_USHORT:
+    return fmt_NC_USHORT; 
+  case NC_UINT:
+    return fmt_NC_UINT; 
+  case NC_INT64:
+    return fmt_NC_INT64; 
+  case NC_UINT64:
+    return fmt_NC_UINT64; 
+  case NC_STRING:
+    return fmt_NC_STRING; 
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
@@ -218,11 +258,11 @@ nco_prn_var_dfn /* [fnc] Print variable metadata */
   }else{
     long var_sz=1L;
 
-    (void)fprintf(stdout,"%s memory size is %ld*nco_typ_lng(%s) = %ld*%lu = %ld bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
-  } /* end if variable is a scalar */
+    (void)fprintf(stdout,"%s memory size is %ld*nco_typ_lng(%s) = %ld*%lu = %lu bytes\n",var_nm,var_sz,nco_typ_sng(var_typ),var_sz,(unsigned long)nco_typ_lng(var_typ),(unsigned long)(var_sz*nco_typ_lng(var_typ)));
+  } /* end if variable is scalar */
   (void)fflush(stdout);
   
-  /* Free the space allocated for dimension info. */
+  /* Free space allocated for dimension information */
   for(idx=0;idx<nbr_dim;idx++) dim[idx].nm=(char *)nco_free(dim[idx].nm);
   if(nbr_dim > 0){
     dim=(dmn_sct *)nco_free(dim);
@@ -238,19 +278,20 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
  const lmt_sct * const lmt, /* I [sct] Dimension limits */
  const int lmt_nbr, /* I [nbr] number of dimensions with user-specified limits */
  char * const dlm_sng, /* I [sng] User-specified delimiter string, if any */
- const bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
- const bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
- const bool PRN_DMN_IDX_CRD_VAL) /* I [flg] Print dimension/coordinate indices/values */
+ const nco_bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
+ const nco_bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
+ const nco_bool PRN_DMN_IDX_CRD_VAL) /* I [flg] Print dimension/coordinate indices/values */
 {
+  /* NB: nco_prn_var_val_lmt() with same nc_id contains OpenMP critical region */
   /* Purpose: Print variable data 
      Routine truncates dimensions of printed output variable in accord with user-specified limits
      fxm: routine does not correctly print hyperslabs which are wrapped, or which use non-unity stride */
 
-  bool SRD=False; /* Stride is non-unity */
-  bool WRP=False; /* Coordinate is wrapped */
+  nco_bool SRD=False; /* Stride is non-unity */
+  nco_bool WRP=False; /* Coordinate is wrapped */
 
   char nul_chr='\0';
-  char var_sng[MAX_LEN_FMT_SNG];
+  char var_sng[NCO_MAX_LEN_FMT_SNG];
   char *unit_sng=NULL;
   
   int rcd=NC_NOERR; /* [rcd] Return code */
@@ -285,7 +326,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   (void)nco_inq_var(in_id,var.id,(char *)NULL,&var.type,&var.nbr_dim,(int *)NULL,(int *)NULL);
 
   if(var.nbr_dim > 0){
-    /* Allocate space for dimension info */
+    /* Allocate space for dimension information */
     dim=(dmn_sct *)nco_malloc(var.nbr_dim*sizeof(dmn_sct));
     dmn_cnt=(long *)nco_malloc(var.nbr_dim*sizeof(long));
     dmn_id=(int *)nco_malloc(var.nbr_dim*sizeof(int));
@@ -348,12 +389,15 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       /* Allocate enough space to hold coordinate */
       dim[idx].val.vp=(void *)nco_malloc(dmn_cnt[idx]*nco_typ_lng(dim[idx].type));
       
-      /* Retrieve this coordinate */
-      if(dmn_srd[idx] == 1L) (void)nco_get_vara(in_id,dim[idx].cid,dmn_srt+idx,dmn_cnt+idx,dim[idx].val.vp,dim[idx].type); else nco_get_varm(in_id,dim[idx].cid,dmn_srt+idx,dmn_cnt+idx,dmn_srd+idx,(long *)NULL,dim[idx].val.vp,dim[idx].type);
-
+      /* Block is critical/thread-safe for identical/distinct in_id's */
+      { /* begin potential OpenMP critical */
+	/* Retrieve this coordinate */
+	if(dmn_srd[idx] == 1L) (void)nco_get_vara(in_id,dim[idx].cid,dmn_srt+idx,dmn_cnt+idx,dim[idx].val.vp,dim[idx].type); else nco_get_varm(in_id,dim[idx].cid,dmn_srt+idx,dmn_cnt+idx,dmn_srd+idx,(long *)NULL,dim[idx].val.vp,dim[idx].type);
+      } /* end potential OpenMP critical */
+      
       /* Typecast pointer to values before access */
       (void)cast_void_nctype(dim[idx].type,&dim[idx].val);
-
+      
     }else{ /* end if dimension is coordinate */   
       dim[idx].is_crd_dmn=False;
       dim[idx].cid=-1;
@@ -368,18 +412,21 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   /* Allocate enough space to hold variable */
   var.val.vp=(void *)nco_malloc(var.sz*nco_typ_lng(var.type));
   if(var.val.vp == NULL){
-    (void)fprintf(stderr,"%s: ERROR unable to malloc() %ld bytes for %s\n",prg_nm_get(),var.sz*nco_typ_lng(var.type),var.nm);
+    (void)fprintf(stderr,"%s: ERROR unable to malloc() %lu bytes for %s\n",prg_nm_get(),(unsigned long)var.sz*nco_typ_lng(var.type),var.nm);
     nco_exit(EXIT_FAILURE);
   } /* end if */
 
-  /* Get variable */
-  if(var.nbr_dim==0){
-    nco_get_var1(in_id,var.id,0L,var.val.vp,var.type); 
-  }else if(!SRD){
-    nco_get_vara(in_id,var.id,dmn_srt,dmn_cnt,var.val.vp,var.type);
-  }else if(SRD){
-    nco_get_varm(in_id,var.id,dmn_srt,dmn_cnt,dmn_srd,(long *)NULL,var.val.vp,var.type);
-  } /* end else */
+  /* Block is critical/thread-safe for identical/distinct in_id's */
+  { /* begin potential OpenMP critical */
+    /* Get variable */
+    if(var.nbr_dim==0){
+      nco_get_var1(in_id,var.id,0L,var.val.vp,var.type); 
+    }else if(!SRD){
+      nco_get_vara(in_id,var.id,dmn_srt,dmn_cnt,var.val.vp,var.type);
+    }else if(SRD){
+      nco_get_varm(in_id,var.id,dmn_srt,dmn_cnt,dmn_srd,(long *)NULL,var.val.vp,var.type);
+    } /* end else */
+  } /* end potential OpenMP critical */
 
   /* Typecast pointer to values before access */
   (void)cast_void_nctype(var.type,&var.val);
@@ -423,6 +470,12 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       case NC_INT: (void)fprintf(stdout,dlm_sng,var.val.lp[lmn]); break;
       case NC_CHAR: (void)fprintf(stdout,dlm_sng,var.val.cp[lmn]); break;
       case NC_BYTE: (void)fprintf(stdout,dlm_sng,var.val.bp[lmn]); break;
+      case NC_UBYTE: (void)fprintf(stdout,dlm_sng,var.val.ubp[lmn]); break;
+      case NC_USHORT: (void)fprintf(stdout,dlm_sng,var.val.usp[lmn]); break;
+      case NC_UINT: (void)fprintf(stdout,dlm_sng,var.val.uip[lmn]); break;
+      case NC_INT64: (void)fprintf(stdout,dlm_sng,var.val.i64p[lmn]); break;
+      case NC_UINT64: (void)fprintf(stdout,dlm_sng,var.val.ui64p[lmn]); break;
+      case NC_STRING: (void)fprintf(stdout,dlm_sng,var.val.sngp[lmn]); break;
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */
     } /* end loop over element */
@@ -430,7 +483,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
   } /* end if dlm_sng */
 
   if(var.nbr_dim == 0 && dlm_sng == NULL){ 
-    /* Variable is a scalar, byte, or character */
+    /* Variable is scalar, byte, or character */
     lmn=0;
     (void)sprintf(var_sng,"%%s = %s %%s\n",nco_typ_fmt_sng(var.type));
     switch(var.type){
@@ -443,16 +496,22 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       (void)fprintf(stdout,var_sng,var_nm,var.val.cp[lmn],unit_sng);
       break;
     case NC_BYTE: (void)fprintf(stdout,var_sng,var_nm,(unsigned char)var.val.bp[lmn],unit_sng); break;
+    case NC_UBYTE: (void)fprintf(stdout,var_sng,var_nm,var.val.ubp[lmn],unit_sng); break;
+    case NC_USHORT: (void)fprintf(stdout,var_sng,var_nm,var.val.usp[lmn],unit_sng); break;
+    case NC_UINT: (void)fprintf(stdout,var_sng,var_nm,var.val.uip[lmn],unit_sng); break;
+    case NC_INT64: (void)fprintf(stdout,var_sng,var_nm,var.val.i64p[lmn],unit_sng); break;
+    case NC_UINT64: (void)fprintf(stdout,var_sng,var_nm,var.val.ui64p[lmn],unit_sng); break;
+    case NC_STRING: (void)fprintf(stdout,var_sng,var_nm,var.val.sngp[lmn],unit_sng); break;
     default: nco_dfl_case_nc_type_err(); break;
     } /* end switch */
-  } /* end if variable is a scalar, byte, or character */
+  } /* end if variable is scalar, byte, or character */
 
   if(var.nbr_dim > 0 && dlm_sng == NULL){ 
     /* Generate nicely formatted output for multidimensional arrays */
 
     char arr_lft_dlm=char_CEWI;
     char arr_rgt_dlm=char_CEWI;
-    char dmn_sng[MAX_LEN_FMT_SNG];
+    char dmn_sng[NCO_MAX_LEN_FMT_SNG];
 
     int crd_idx_crr=int_CEWI; /* Current coordinate index */
     int dmn_idx=int_CEWI;
@@ -546,6 +605,12 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
 	      case NC_INT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.lp[crd_idx_crr]); break;
 	      case NC_CHAR: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.cp[crd_idx_crr]); break;
 	      case NC_BYTE: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,(unsigned char)dim[dmn_idx].val.bp[crd_idx_crr]); break;
+	      case NC_UBYTE: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.ubp[crd_idx_crr]); break;
+	      case NC_USHORT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.usp[crd_idx_crr]); break;
+	      case NC_UINT: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.uip[crd_idx_crr]); break;
+	      case NC_INT64: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.i64p[crd_idx_crr]); break;
+	      case NC_UINT64: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.ui64p[crd_idx_crr]); break;
+	      case NC_STRING: (void)fprintf(stdout,dmn_sng,dim[dmn_idx].nm,dmn_sbs_prn,dim[dmn_idx].val.sngp[crd_idx_crr]); break;
 	      default: nco_dfl_case_nc_type_err(); break;
 	      } /* end switch */
 	    }else{ /* if dimension is not a coordinate... */
@@ -568,7 +633,7 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
 	  /* Memory region is NUL-terminated, i.e., a valid string */
 	  /* Print strings inside double quotes */
 	  (void)sprintf(var_sng,"%%s%c%%ld--%%ld%c=\"%%s\" %%s",arr_lft_dlm,arr_rgt_dlm);
-	  /* var.val.cp is unsigned char * but strlen() requires const char * */
+	  /* var.val.cp is type nco_char and strlen() requires const char * */
 	  (void)fprintf(stdout,var_sng,var_nm,idx_crr,idx_crr+strlen((char *)var.val.cp+lmn),(char *)var.val.cp+lmn,unit_sng);
 	}else{
 	  /* Memory region is not NUL-terminated, print block of chars instead */
@@ -593,6 +658,12 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
       case NC_INT: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.lp[lmn],unit_sng); break;
       case NC_CHAR: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.cp[lmn],unit_sng); break;
       case NC_BYTE: (void)fprintf(stdout,var_sng,var_nm,idx_crr,(unsigned char)var.val.bp[lmn],unit_sng); break;
+      case NC_UBYTE: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.ubp[lmn],unit_sng); break;
+      case NC_USHORT: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.usp[lmn],unit_sng); break;
+      case NC_UINT: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.uip[lmn],unit_sng); break;
+      case NC_INT64: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.i64p[lmn],unit_sng); break;
+      case NC_UINT64: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.ui64p[lmn],unit_sng); break;
+      case NC_STRING: (void)fprintf(stdout,var_sng,var_nm,idx_crr,var.val.sngp[lmn],unit_sng); break;
       default: nco_dfl_case_nc_type_err(); break;
       } /* end switch */
     } /* end loop over element */
