@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.h,v 1.13 2004/09/18 05:22:03 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_prn.h,v 1.21 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Printing variables, attributes, metadata */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_prn.h" *//* Printing variables, attributes, metadata */
@@ -31,6 +31,10 @@
 #include "nco_mmr.h" /* Memory management */
 #include "nco_sng_utl.h" /* String utilities */
 
+/* fxm: strings statically allocated with NCO_MAX_LEN_FMT_SNG chars are susceptible to buffer overflow attacks */
+/* Length should be computed at run time but is a pain */
+#define NCO_MAX_LEN_FMT_SNG 100
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -56,9 +60,9 @@ nco_prn_var_val_lmt /* [fnc] Print variable data */
  const lmt_sct * const lmt, /* I [sct] Dimension limits */
  const int lmt_nbr, /* I [nbr] number of dimensions with user-specified limits */
  char * const dlm_sng, /* I [sng] User-specified delimiter string, if any */
- const bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
- const bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
- const bool PRN_DMN_IDX_CRD_VAL); /* I [flg] Print dimension/coordinate indices/values */
+ const nco_bool FORTRAN_IDX_CNV, /* I [flg] Hyperslab indices obey Fortran convention */
+ const nco_bool PRN_DMN_UNITS, /* I [flg] Print units attribute, if any */
+ const nco_bool PRN_DMN_IDX_CRD_VAL); /* I [flg] Print dimension/coordinate indices/values */
 
 #ifdef __cplusplus
 } /* end extern "C" */
