@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.12 2004/07/06 19:37:35 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.c,v 1.17 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Float-precision arithmetic */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_rth_flt.h" /* Float-precision arithmetic */
 
@@ -90,3 +90,23 @@ float tanf(float x){return (float)(tan((double)x));}
 #ifdef NEED_TANHF
 float tanhf(float x){return (float)(tanh((double)x));}
 #endif /* !NEED_TANHF */ 
+
+/* fxm TODO nco652 */
+double /* O [frc] Random fraction in [0,1] */
+rnd_nbr /* [fnc] Generate random fraction in [0,1] */
+(double x) /* I [frc] Immaterial */
+{
+  /* Purpose: Wrapper for system random number generator 
+     Output does not depend on input value of x */
+  long rnd_nbr_lng;
+  double rnd_nbr_dbl_frc;
+  x=x+0.0; /* CEWI */
+  rnd_nbr_lng=random();
+  rnd_nbr_dbl_frc=rnd_nbr_lng*1.0/RAND_MAX;
+  return rnd_nbr_dbl_frc;
+} /* end rnd_nbr() */
+
+#define NEED_RND_NBRF
+#ifdef NEED_RND_NBRF
+float rnd_nbrf(float x){return (float)(rnd_nbr((double)x));}
+#endif /* !NEED_RND_NBRF */ 
