@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.h,v 1.18 2004/07/09 16:36:11 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_flt.h,v 1.25 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Float-precision arithmetic */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_rth_flt.h" *//* Float-precision arithmetic */
@@ -21,6 +21,7 @@
 
 /* fxm stdio only needed for TODO ncap57 */
 #include <stdio.h> /* stderr, FILE, NULL, etc. */
+#include <stdlib.h> /* atof, atoi, malloc, getopt */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions and C library */
@@ -49,10 +50,11 @@ extern "C" {
   float sqrtf(float);
   float tanf(float);
 
-  /* Advanced math: erf, erfc, gamma */
+  /* Advanced math: erf, erfc, gamma, rnd_nbr */
   float erff(float);
   float erfcf(float);
   float gammaf(float);
+  float rnd_nbrf(float);
 
   /* Hyperbolic trigonometric: acosh, asinh, atanh, cosh, sinh, tanh */
   float acoshf(float);
@@ -74,6 +76,10 @@ extern "C" {
 
 #endif /* HPUX */
 
+double /* O [frc] Random fraction in [0,1] */
+rnd_nbr /* [fnc] Generate random fraction in [0,1] */
+(double x); /* I [frc] Immaterial */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
