@@ -1,13 +1,38 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.18 2004/09/03 06:28:10 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.c,v 1.30 2007/05/19 00:02:10 zender Exp $ */
 
 /* Purpose: Arithmetic controls and utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_rth_utl.h" /* Arithmetic controls and utilities */
 
+nco_rth_prc_rnk_enm /* [enm] Ranked precision of arithmetic type */
+nco_rth_prc_rnk /* [fnc] Rank precision of arithmetic type */
+(const nc_type nco_typ) /* I [enm] netCDF type of operand */
+{
+  /* Purpose: Ranked precision of arithmetic type */
+  switch(nco_typ){
+  case NC_FLOAT: return nco_rth_prc_rnk_float;
+  case NC_DOUBLE: return nco_rth_prc_rnk_double;
+  case NC_INT: return nco_rth_prc_rnk_int;
+  case NC_SHORT: return nco_rth_prc_rnk_short;
+  case NC_CHAR: return nco_rth_prc_rnk_char;
+  case NC_BYTE: return nco_rth_prc_rnk_byte;
+  case NC_UBYTE: return nco_rth_prc_rnk_ubyte;
+  case NC_USHORT: return nco_rth_prc_rnk_ushort;
+  case NC_UINT: return nco_rth_prc_rnk_uint;
+  case NC_INT64: return nco_rth_prc_rnk_int64;
+  case NC_UINT64: return nco_rth_prc_rnk_uint64;
+  case NC_STRING: return nco_rth_prc_rnk_string;
+  default: nco_dfl_case_nc_type_err(); break;
+  } /* end switch */
+
+  /* Some compilers, e.g., SGI cc, need return statement to end non-void functions */
+  return 0;
+} /* end nco_rth_prc_rnk() */
+
 void 
 nco_opr_drv /* [fnc] Intermediate control of arithmetic operations for ncra/ncea */
 (const long idx_rec, /* I [idx] Index of current record */
@@ -20,6 +45,16 @@ nco_opr_drv /* [fnc] Intermediate control of arithmetic operations for ncra/ncea
      These operations perform some, but not all, of necessary operations for each procedure
      Most arithmetic operations require additional procedures such as normalization be performed after all files/records have been processed */
   
+  /* NCO's paradigm is that coordinate variables represent grid axes
+     Reducing such grids to a single-value must be done
+     The most representative value of the grid is the average 
+     The total, min, max, rms, etc. of the grid usually makes no sense
+     Users are most interested in the mean grid coordinate */
+  if(var_prc->is_crd_var){
+    (void)nco_var_add_tll_ncra(var_prc->type,var_prc->sz,var_prc->has_mss_val,var_prc->mss_val,var_prc->tally,var_prc->val,var_prc_out->val);
+    return;
+  } /* !var_prc->is_crd_var */
+
   /* var_prc_out->type and var_prc->type should be equal and thus interchangeable
      var_prc_out->sz and var_prc->sz should be equal and thus interchangeable */
   switch (nco_op_typ){
@@ -67,6 +102,8 @@ nco_op_typ_get /* [fnc] Convert user-specified operation into operation key */
     /* If nco_op_typ_get() is called when user-specified option string is NULL, 
        then operation type may be implied by program name itself */
     if(!strcmp(prg_nm,"ncadd")) return nco_op_add;
+    if(!strcmp(prg_nm,"mpncbo")) return nco_op_sbt;
+    if(!strcmp(prg_nm,"mpncdiff")) return nco_op_sbt;
     if(!strcmp(prg_nm,"ncbo")) return nco_op_sbt;
     if(!strcmp(prg_nm,"ncdiff")) return nco_op_sbt;
     if(!strcmp(prg_nm,"ncsub")) return nco_op_sbt;
@@ -148,17 +185,27 @@ vec_set /* [fnc] Fill every value of first operand with value of second operand
     for(idx=0;idx<sz;idx++) op1.dp[idx]=op2;
     break;
   case NC_INT:
-    for(idx=0;idx<sz;idx++) op1.lp[idx]=(long)op2; /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.lp[idx]=(nco_int)op2; /* Coerce to avoid C++ compiler assignment warning */
     break;
   case NC_SHORT:
-    for(idx=0;idx<sz;idx++) op1.sp[idx]=(short)op2; /* Coerce to avoid C++ compiler assignment warning */
+    for(idx=0;idx<sz;idx++) op1.sp[idx]=(nco_short)op2; /* Coerce to avoid C++ compiler assignment warning */
     break;
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:
+    for(idx=0;idx<sz;idx++) op1.usp[idx]=(nco_ushort)op2; /* Coerce to avoid C++ compiler assignment warning */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  case NC_UINT:
+    for(idx=0;idx<sz;idx++) op1.uip[idx]=(nco_uint)op2; /* Coerce to avoid C++ compiler assignment warning */
     break;
+  case NC_INT64:
+    for(idx=0;idx<sz;idx++) op1.i64p[idx]=(nco_int64)op2; /* Coerce to avoid C++ compiler assignment warning */
+    break;
+  case NC_UINT64:
+    for(idx=0;idx<sz;idx++) op1.ui64p[idx]=(nco_uint64)op2; /* Coerce to avoid C++ compiler assignment warning */
+    break;
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
     default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
@@ -174,12 +221,19 @@ nco_zero_long /* [fnc] Zero all values of long array */
 {
   /* Purpose: Zero all values of long array */
 
+  size_t sz_byt; /* [B] Number of bytes in variable buffer */
+  sz_byt=(size_t)sz*sizeof(long);
+  (void)memset((void *)op1,(int)0,sz_byt);
+
+#if 0
+  /* Presumably this old method used until 20050321 is slower because of pointer de-referencing */
   long idx;
   if(op1 == NULL){
     (void)fprintf(stdout,"%s: ERROR nco_zero_long() asked to zero NULL pointer\n",prg_nm_get());
     nco_exit(EXIT_FAILURE);
   } /* endif */
   for(idx=0;idx<sz;idx++) op1[idx]=0L;
+#endif /* !0 */
 
 } /* end nco_zero_long() */
 
