@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.h,v 1.11 2004/07/19 22:16:20 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_rth_utl.h,v 1.16 2007/05/19 00:02:10 zender Exp $ */
 
 /* Purpose: Arithmetic controls and utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_rth_utl.h" *//* Arithmetic controls and utilities */
@@ -49,6 +49,10 @@ int /* O [enm] Relational operation */
 nco_op_prs_rlt /* [fnc] Convert Fortran abbreviation for relational operator into NCO operation key */
 (const char * const op_sng); /* I [sng] Fortran representation of relational operator */
 
+nco_rth_prc_rnk_enm /* [enm] Ranked precision of arithmetic type */
+nco_rth_prc_rnk /* [fnc] Rank precision of arithmetic type */
+(const nc_type type); /* I [enm] netCDF type of operand */
+
 void
 vec_set /* [fnc] Fill every value of first operand with value of second operand */
 (const nc_type type, /* I [enm] netCDF type of operand */
