@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.c,v 1.20 2004/05/13 18:18:59 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_scm.c,v 1.32 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Software configuration management */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_scm.h" /* Software configuration management */
 
@@ -12,7 +12,7 @@ char * /* O [sng] CVS version string */
 cvs_vrs_prs(void) /* [fnc] Return CVS version string */
 {
   /* Purpose: Return CVS version string */
-  bool dly_snp;
+  nco_bool dly_snp;
 
   char *cvs_mjr_vrs_sng=NULL;
   char *cvs_mnr_vrs_sng=NULL;
@@ -26,8 +26,13 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
   char *usc_1_ptr=NULL;
   char *usc_2_ptr=NULL;
 
-  const char cvs_Name[]="$Name:  $";
+  /* Unexpanded cvs keywords in cvs_Name trigger GCC 4.0+ warning
+     "./src/nco/nco_scm.c:81: warning: offset outside bounds of constant string"
+     because routine looks for cvs_Name+7 later on */
+  char cvs_Name[]="$Name: nco-3_9_0-1 $";
+  const char dlr_nm_cln_spc[]="$Name: "; /* [sng] Dollar name colon space */
   const char nco_sng[]="nco"; 
+  const char spc_dlr[]=" $"; /* [sng] Space dollar */
 
   int cvs_nm_sng_len;
   int cvs_vrs_sng_len;
@@ -41,15 +46,15 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
   long cvs_pch_vrs=-1L;
 
   /* Is cvs_Name keyword expanded? */
-  dlr_ptr=strstr(cvs_Name," $");
+  dlr_ptr=(char *)strstr(cvs_Name,spc_dlr);
   if(dlr_ptr == NULL && dbg_lvl_get() > 3)(void)fprintf(stderr,"%s: WARNING cvs_vrs_prs() reports dlr_ptr == NULL\n%s: HINT Make sure CVS export uses -kkv\n",prg_nm_get(),prg_nm_get());
-  cvs_nm_ptr=strstr(cvs_Name,"$Name: ");
+  cvs_nm_ptr=(char *)strstr(cvs_Name,dlr_nm_cln_spc);
   if(cvs_nm_ptr == NULL && dbg_lvl_get() > 3)(void)fprintf(stderr,"%s: WARNING cvs_vrs_prs() reports cvs_nm_ptr == NULL\n%s: HINT Make sure CVS export uses -kkv\n",prg_nm_get(),prg_nm_get());
-  cvs_nm_sng_len=(int)(dlr_ptr-cvs_nm_ptr-7); /* 7 is strlen("$Name: ") */
+  cvs_nm_sng_len=(int)(dlr_ptr-cvs_nm_ptr-strlen(dlr_nm_cln_spc)); /* 7 is strlen("$Name: ") */
   if(cvs_nm_sng_len > 0) dly_snp=False; else dly_snp=True;
 
-  /* If not, this is daily snapshot so use YYYYMMDD date for version string */
   if(dly_snp){
+    /* Presume this a daily snapshot, use YYYYMMDD date for version string */
     int mth;
     int day;
     int yr;
@@ -58,7 +63,7 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
 
     time_crr_time_t=time((time_t *)NULL);
     gmt_tm=gmtime(&time_crr_time_t); 
-    /* localtime() gives YYYYMMDD in MDT, but this conflicts with CVS, which uses GMT */
+    /* localtime() gives YYYYMMDD in e.g., PDT/MDT, but this conflicts with CVS, which uses GMT */
     /*    gmt_tm=localtime(&time_crr_time_t); */
 
     mth=gmt_tm->tm_mon+1;
@@ -73,7 +78,7 @@ cvs_vrs_prs(void) /* [fnc] Return CVS version string */
 
   /* cvs_nm_sng is, e.g., "nco1_1" */
   cvs_nm_sng=(char *)nco_malloc((size_t)cvs_nm_sng_len+1);
-  strncpy(cvs_nm_sng,cvs_Name+7,(size_t)cvs_nm_sng_len); /* 7 is strlen("$Name: ") */
+  cvs_nm_sng=strncpy(cvs_nm_sng,cvs_Name+strlen(dlr_nm_cln_spc),(size_t)cvs_nm_sng_len); /* strlen("$Name: ") = 7 */
   cvs_nm_sng[cvs_nm_sng_len]='\0';
 
   /* cvs_vrs_sng is, e.g., "1.1" */
@@ -147,9 +152,6 @@ copyright_prn /* [fnc] Print copyright notice */
   int date_cvs_lng;
   int vrs_cvs_lng;
   
-  /* C pre-processor macros for instantiating variable values with string tokens */
-#define XTKN2SNG(x) #x
-#define TKN2SNG(x) XTKN2SNG(x)
   const char date_cpp[]=__DATE__; /* [sng] Date from C pre-processor */
   /*  const char time_cpp[]=__TIME__; *//* [sng] Time from C pre-processor */
   const char vrs_cpp[]=TKN2SNG(VERSION); /* [sng] Version from C pre-processor */
@@ -185,13 +187,13 @@ copyright_prn /* [fnc] Print copyright notice */
   }else{
     (void)fprintf(stderr,"NCO netCDF Operators version %s built %s on %s by %s\n",vrs_cpp,date_cpp,hst_cpp,usr_cpp);
   } /* endif */
-  (void)fprintf(stderr,"Copyright (C) 1995--2004 Charlie Zender\n");
+  (void)fprintf(stderr,"Copyright (C) 1995--2007 Charlie Zender\n");
   if(strlen(CVS_Id) > strlen("*Id*")){
     (void)fprintf(stderr,"%s version %s\n",prg_nm_get(),vrs_cvs);
   }else{
     (void)fprintf(stderr,"%s version %s\n",prg_nm_get(),vrs_cpp);
   } /* endif */
-  (void)fprintf(stdout,"NCO is free software and comes with ABSOLUTELY NO WARRANTY\nNCO is distributed under the terms of the GNU General Public License\n");
+  (void)fprintf(stdout,"NCO is free software and comes with ABSOLUTELY NO WARRANTY\nYou may copy, distribute, and/or modify NCO under the terms of the GNU General Public License (GPL) Version 2\n");
 
   date_cvs=(char *)nco_free(date_cvs);
   vrs_rcs=(char *)nco_free(vrs_rcs);
