@@ -1,13 +1,27 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.c,v 1.15 2004/06/20 20:00:11 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.c,v 1.22 2007/05/21 05:58:41 zender Exp $ */
 
 /* Purpose: String utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_sng_utl.h" /* String utilities */
 
+#ifdef NEED_STRTOLL
+long long int /* O [nbr] String as long long integer */
+strtoll /* [fnc] Convert string to a long long integer */
+(const char *nptr, 
+ char **endptr, 
+ int base)
+{
+  /* Purpose: Compatibility function for strtoll()
+     Needed by some C++ compilers, e.g., AIX xlC */
+  long long nbr_out=0LL;
+  return nbr_out;
+} /* end strtoll() */
+#endif /* !NEED_STRTOLL */
+
 #ifdef NEED_STRCASECMP
 int /* O [enm] [-1,0,1] sng_1 [<,=,>] sng_2 */
 strcasecmp /* [fnc] Lexicographical case-insensitive string comparison */
@@ -54,13 +68,14 @@ nco_cmd_ln_sng /* [fnc] Re-construct command line from arguments */
   int idx;
 
   for(idx=0;idx<argc;idx++){
+    /* Add one to size of each argument for a space character */
     cmd_ln_sz+=(int)strlen(argv[idx])+1;
   } /* end loop over args */
-  cmd_ln=(char *)nco_malloc(cmd_ln_sz*sizeof(char));
   if(argc <= 0){
     cmd_ln=(char *)nco_malloc(sizeof(char));
     cmd_ln[0]='\0';
   }else{
+    cmd_ln=(char *)nco_malloc(cmd_ln_sz*sizeof(char));
     (void)strcpy(cmd_ln,argv[0]);
     for(idx=1;idx<argc;idx++){
       (void)strcat(cmd_ln," ");
@@ -91,7 +106,7 @@ sng_ascii_trn /* [fnc] Replace C language '\X' escape codes in string with ASCII
      when characters are "moved to the left" if C language escape sequences are embedded.
      Thus string length may shrink */
 
-  bool trn_flg; /* Translate this escape sequence */
+  nco_bool trn_flg; /* Translate this escape sequence */
 
   char *backslash_ptr; /* [ptr] Pointer to backslash character */
   char backslash_chr='\\'; /* [chr] Backslash character */
