@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.h,v 1.11 2004/06/15 22:37:48 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_sng_utl.h,v 1.17 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: String utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_sng_utl.h" *//* String utilities */
@@ -19,6 +19,9 @@
 /* Standard header files */
 #include <stdio.h> /* stderr, FILE, NULL, printf */
 #include <string.h> /* strcmp. . . */
+#ifdef NEED_STRCASECMP
+#include <ctype.h> /* tolower. . . */
+#endif /* !NEED_STRCASECMP */
 
 /* 3rd party vendors */
 
@@ -39,7 +42,7 @@ strcasecmp /* [fnc] Lexicographical case-insensitive string comparison */
 #ifdef NEED_STRDUP
 char * /* O [sng] Copy of input string */
 strdup /* [fnc] Duplicate string */
-(char *sng_in); /* I [sng] String to duplicate */
+(const char *sng_in); /* I [sng] String to duplicate */
 #endif /* !NEED_STRDUP */
 
 char * /* O [sng] Parsed command line */
