@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_avg.h,v 1.9 2004/01/12 18:11:07 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_avg.h,v 1.17 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Average variables */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_var_avg.h" *//* Average variables */
@@ -32,14 +32,16 @@ extern "C" {
 #endif /* __cplusplus */
 
 var_sct * /* O [sct] Partially (non-normalized) reduced variable */
-nco_var_avg /* [fnc] reduce given variable over specified dimensions */
+nco_var_avg /* [fnc] Reduce given variable over specified dimensions */
 (var_sct *var, /* I/O [sct] Variable to reduce (e.g., average) (destroyed) */
  dmn_sct * const * const dim, /* I [sct] Dimensions over which to reduce variable */
  const int nbr_dim, /* I [sct] Number of dimensions to reduce variable over */
- const int nco_op_typ); /* I [enm] Operation type, default is average */
+ const int nco_op_typ, /* I [enm] Operation type, default is average */
+ const nco_bool flg_rdd, /* I [flg] Retain degenerate dimensions */
+ ddra_info_sct * const ddra_info); /* O [sct] DDRA information */
 
 void
-nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
+nco_var_avg_rdc_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
 (const nc_type type, /* I [enm] netCDF type of operands */
  const long sz_op1, /* I [nbr] Size (in elements) of op1 */
  const long sz_op2, /* I [nbr] Size (in elements) of op2 */
@@ -50,7 +52,7 @@ nco_var_avg_reduce_ttl /* [fnc] Sum blocks of op1 into each element of op2 */
  ptr_unn op2); /* O [sct] Sum of each block of op1 */
 
 void
-nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of op2 */
+nco_var_avg_rdc_min /* [fnc] Place minimum of op1 blocks into each element of op2 */
 (const nc_type type, /* I [enm] netCDF type of operands */
  const long sz_op1, /* I [nbr] Size (in elements) of op1 */
  const long sz_op2, /* I [nbr] Size (in elements) of op2 */
@@ -60,7 +62,7 @@ nco_var_avg_reduce_min /* [fnc] Place minimum of op1 blocks into each element of
  ptr_unn op2); /* O [sct] Minimum of each block of op1 */
 
 void
-nco_var_avg_reduce_max /* [fnc] Place maximum of op1 blocks into each element of op2 */
+nco_var_avg_rdc_max /* [fnc] Place maximum of op1 blocks into each element of op2 */
 (const nc_type type, /* I [enm] netCDF type of operands */
  const long sz_op1, /* I [nbr] Size (in elements) of op1 */
  const long sz_op2, /* I [nbr] Size (in elements) of op2 */
