@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.24 2004/09/07 04:31:19 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_lst.h,v 1.40 2007/04/18 17:06:15 zender Exp $ */
 
 /* Purpose: Variable list utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_var_lst.h" *//* Variable list utilities */
@@ -35,6 +35,7 @@
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
 #include "nco_ctl.h" /* Program flow control functions */
+#include "nco_lst_utl.h" /* List utilities */
 #include "nco_mmr.h" /* Memory management */
 #include "nco_var_utl.h" /* Variable utilities */
 
@@ -46,25 +47,11 @@ nm_id_sct * /* O [sct] Variable extraction list */
 nco_var_lst_mk /* [fnc] Create variable extraction list */
 (const int nc_id, /* I [enm] netCDF file ID */
  const int nbr_var, /* I [nbr] Number of variables in input file */
- char **var_lst_in, /* I [sng] User-specified list of variable names and re */
- const bool PROCESS_ALL_COORDINATES, /* I [flg] Process all coordinates */
+ char * const * const var_lst_in, /* I [sng] User-specified list of variable names and rx's */
+ const nco_bool EXCLUDE_INPUT_LIST, /* I [flg] Exclude rather than extract */
+ const nco_bool EXTRACT_ALL_COORDINATES, /* I [flg] Process all coordinates */
  int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
 
-nm_id_sct * /* O [sct] Variable extraction list */
-nco_var_lst_mk_old /* [fnc] Create variable extraction list */
-(const int nc_id, /* I [enm] netCDF file ID */
- const int nbr_var, /* I [nbr] Number of variables in input file */
- CST_X_PTR_CST_PTR_CST_Y(char,var_lst_in), /* I [sng] User-specified list of variable names */
- const bool PROCESS_ALL_COORDINATES, /* I [flg] Process all coordinates */
- int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
-
-int /* O [nbr] Number of matches found */
-nco_var_meta_search /* [fnc] Search for pattern matches in var string list */
-(int nbr_var, /* I [nbr] number of vars in srch_sng and size of in_bool */
- nm_id_sct *in_lst, /* I [sct] List of all variables in input file (with IDs) */
- char *rexp, /* I [sng] Regular expression pattern */
- bool *in_bool); /* O [flg] Matched vars holder */
-
 nm_id_sct * /* O [sct] Extraction list */
 nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
 (const int nc_id, /* I netCDF file ID */
@@ -73,14 +60,30 @@ nco_var_lst_xcl /* [fnc] Convert exclusion list to extraction list */
  int * const nbr_xtr); /* I/O [nbr] Number of variables in exclusion/extraction list */
 
 nm_id_sct * /* O [sct] Extraction list */
-nco_var_lst_add_crd /* [fnc] Add all coordinates to extraction list */
+nco_var_lst_crd_add /* [fnc] Add all coordinates to extraction list */
 (const int nc_id, /* I [id] netCDF file ID */
  const int nbr_dim, /* I [nbr] Number of dimensions in input file */
+ const int nbr_var, /* I [nbr] Number of variables in input file */
  nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
- int * const nbr_xtr); /* I/O [nbr] Number of variables in current extraction list */
+ int * const nbr_xtr, /* I/O [nbr] Number of variables in current extraction list */
+ const nco_bool CNV_CCM_CCSM_CF); /* I [flg] file obeys CCM/CCSM/CF conventions */
+
+nm_id_sct * /* O [sct] Extraction list */
+nco_var_lst_crd_ass_add /* [fnc] Add coordinates associated extracted variables to extraction list */
+(const int nc_id, /* I netCDF file ID */
+ nm_id_sct *xtr_lst, /* I/O current extraction list (destroyed) */
+ int * const nbr_xtr, /* I/O number of variables in current extraction list */
+ const nco_bool CNV_CCM_CCSM_CF); /* I [flg] file obeys CCM/CCSM/CF conventions */
+
+nm_id_sct * /* O [sct] List with coordinate excluded */
+nco_var_lst_crd_xcl /* [fnc] Exclude given coordinates from extraction list */
+(const int nc_id, /* I [id] netCDF file ID */
+ const int dmn_id, /* I [id] Dimension ID of coordinate to remove from extraction list */
+ nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
+ int * const nbr_xtr); /* I/O [nbr] Number of variables in extraction list */
 
 void
-var_lst_convert /* [fnc] Make variable structure list from variable name ID list */
+nco_var_lst_convert /* [fnc] Make variable structure list from variable name ID list */
 (const int nc_id, /* I [enm] netCDF file ID */
  nm_id_sct *xtr_lst, /* I [sct] Current extraction list (destroyed) */
  const int nbr_xtr, /* I [nbr] Number of variables in input file */
@@ -94,7 +97,7 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
 (var_sct * const * const var, /* I [sct] Variable list (input file) */
  var_sct * const * const var_out, /* I [sct] Variable list (output file) */
  const int nbr_var, /* I [nbr] Number of variables */
- const bool NCAR_CCSM_FORMAT, /* I [flg] File adheres to NCAR CCSM conventions */
+ const nco_bool CNV_CCM_CCSM_CF, /* I [flg] File adheres to NCAR CCM/CCSM/CF conventions */
  const int nco_pck_map, /* I [enm] Packing map */
  const int nco_pck_plc, /* I [enm] Packing policy */
  CST_X_PTR_CST_PTR_CST_Y(dmn_sct,dmn_xcl), /* I [sct] Dimensions not allowed in fixed variables */
@@ -106,6 +109,20 @@ nco_var_lst_dvd /* [fnc] Divide input lists into output lists */
  var_sct *** const var_prc_out_ptr, /* O [sct] Processed-variables (output file) */
  int * const nbr_var_prc); /* O [nbr] Number of processed variables */
 
+int /* O [enm] Return code */
+nco_var_lst_mrg /* [fnc] Merge two variable lists into same order */
+(var_sct *** var_1_ptr, /* I/O [sct] Variable list 1 */
+ var_sct *** var_2_ptr, /* I/O [sct] Variable list 2 */
+ int * const var_nbr_1, /* I/O [nbr] Number of variables in list 1 */
+ int * const var_nbr_2); /* I/O [nbr] Number of variables in list 2 */
+
+int /* O [nbr] Number of matches found */
+nco_var_meta_search /* [fnc] Search for pattern matches in var string list */
+(int nbr_var, /* I [nbr] number of vars in srch_sng and size of var_xtr_rqs */
+ nm_id_sct *fl_in_var_lst, /* I [sct] All variables in input file (with IDs) */
+ char *rexp, /* I [sng] Regular expression pattern */
+ nco_bool *var_xtr_rqs); /* O [flg] Matched vars holder */
+
 #ifdef __cplusplus
 } /* end extern "C" */
 #endif /* __cplusplus */
