@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_scv.c,v 1.19 2004/07/19 21:01:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_scv.c,v 1.28 2007/05/18 21:20:25 zender Exp $ */
 
 /* Purpose: Arithmetic between variables and scalar values */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_var_scv.h" /* Arithmetic between variables and scalar values */
 
@@ -55,11 +55,11 @@ var_scv_add /* [fnc] Add scalar to variable */
     break;
   } /* endif NC_DOUBLE */
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op1.lp[idx]+=scv_lng;
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op1.lp[idx] != mss_val_lng) op1.lp[idx]+=scv_lng; 
       } /* end for */
@@ -67,23 +67,69 @@ var_scv_add /* [fnc] Add scalar to variable */
     break;
   } /* endif NC_INT */
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.sp[idx]+=scv_sht;
+      for(idx=0;idx<sz;idx++) op1.sp[idx]+=scv_short;
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op1.sp[idx] != mss_val_sht) op1.sp[idx]+=scv_sht;
+	if(op1.sp[idx] != mss_val_short) op1.sp[idx]+=scv_short;
       } /* end for */
     } /* end else */
     break;
   } /* endif NC_SHORT */
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.usp[idx]+=scv_ushort;
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.usp[idx] != mss_val_ushort) op1.usp[idx]+=scv_ushort;
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.uip[idx]+=scv_uint;
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.uip[idx] != mss_val_uint) op1.uip[idx]+=scv_uint;
+      } /* end for */
+    } /* end else */
     break;
+  } /* endif NC_UINT */
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.i64p[idx]+=scv_int64;
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.i64p[idx] != mss_val_int64) op1.i64p[idx]+=scv_int64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_INT64 */
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.ui64p[idx]+=scv_uint64;
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.ui64p[idx] != mss_val_uint64) op1.ui64p[idx]+=scv_uint64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_UINT64 */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -139,11 +185,11 @@ var_scv_sub /* [fnc] Subtract scalar from variable */
     break;
   }
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op1.lp[idx]-=scv_lng;
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op1.lp[idx] != mss_val_lng) op1.lp[idx]-=scv_lng; 
       } /* end for */
@@ -151,23 +197,69 @@ var_scv_sub /* [fnc] Subtract scalar from variable */
     break;
   }
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.sp[idx]-=scv_sht;
+      for(idx=0;idx<sz;idx++) op1.sp[idx]-=scv_short;
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op1.sp[idx] != mss_val_sht) op1.sp[idx]-=scv_sht;
+	if(op1.sp[idx] != mss_val_short) op1.sp[idx]-=scv_short;
       } /* end for */
     } /* end else */
     break;
   }
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.usp[idx]-=scv_ushort;
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.usp[idx] != mss_val_ushort) op1.usp[idx]-=scv_ushort;
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  }
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.uip[idx]-=scv_uint;
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.uip[idx] != mss_val_uint) op1.uip[idx]-=scv_uint;
+      } /* end for */
+    } /* end else */
     break;
+  }
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.i64p[idx]-=scv_int64;
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.i64p[idx] != mss_val_int64) op1.i64p[idx]-=scv_int64;
+      } /* end for */
+    } /* end else */
+    break;
+  }
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.ui64p[idx]-=scv_uint64;
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.ui64p[idx] != mss_val_uint64) op1.ui64p[idx]-=scv_uint64;
+      } /* end for */
+    } /* end else */
+    break;
+  }
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -223,11 +315,11 @@ var_scv_mlt /* [fnc] Multiply variable by scalar */
     break;
   }
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op1.lp[idx]*=scv_lng;
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op1.lp[idx] != mss_val_lng) op1.lp[idx]*=scv_lng; 
       } /* end for */
@@ -235,23 +327,69 @@ var_scv_mlt /* [fnc] Multiply variable by scalar */
     break;
   }
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.sp[idx]*=scv_sht;
+      for(idx=0;idx<sz;idx++) op1.sp[idx]*=scv_short;
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op1.sp[idx] != mss_val_sht) op1.sp[idx]*=scv_sht;
+	if(op1.sp[idx] != mss_val_short) op1.sp[idx]*=scv_short;
       } /* end for */
     } /* end else */
     break;
   }
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.usp[idx]*=scv_ushort;
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.usp[idx] != mss_val_ushort) op1.usp[idx]*=scv_ushort;
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  }
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.uip[idx]*=scv_uint;
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.uip[idx] != mss_val_uint) op1.uip[idx]*=scv_uint;
+      } /* end for */
+    } /* end else */
     break;
+  }
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.i64p[idx]*=scv_int64;
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.i64p[idx] != mss_val_int64) op1.i64p[idx]*=scv_int64;
+      } /* end for */
+    } /* end else */
+    break;
+  }
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.ui64p[idx]*=scv_uint64;
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.ui64p[idx] != mss_val_uint64) op1.ui64p[idx]*=scv_uint64;
+      } /* end for */
+    } /* end else */
+    break;
+  }
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -307,11 +445,11 @@ var_scv_dvd /* [fnc] Divide variable by scalar */
     break;
   } /* endif NC_DOUBLE */
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op1.lp[idx]/=scv_lng;
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op1.lp[idx] != mss_val_lng) op1.lp[idx]/=scv_lng; 
       } /* end for */
@@ -319,23 +457,69 @@ var_scv_dvd /* [fnc] Divide variable by scalar */
     break;
   } /* endif NC_INT */
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.sp[idx]/=scv_sht;
+      for(idx=0;idx<sz;idx++) op1.sp[idx]/=scv_short;
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op1.sp[idx] != mss_val_sht) op1.sp[idx]/=scv_sht;
+	if(op1.sp[idx] != mss_val_short) op1.sp[idx]/=scv_short;
       } /* end for */
     } /* end else */
     break;
-  } /* endif NC_SHORT */
-  case NC_CHAR:
-    /* Do nothing */
+  } /* endif NC_UINT64 */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.usp[idx]/=scv_ushort;
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.usp[idx] != mss_val_ushort) op1.usp[idx]/=scv_ushort;
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.uip[idx]/=scv_uint;
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.uip[idx] != mss_val_uint) op1.uip[idx]/=scv_uint;
+      } /* end for */
+    } /* end else */
     break;
+  } /* endif NC_UINT */
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.i64p[idx]/=scv_int64;
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.i64p[idx] != mss_val_int64) op1.i64p[idx]/=scv_int64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_INT64 */
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.ui64p[idx]/=scv_uint64;
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.ui64p[idx] != mss_val_uint64) op1.ui64p[idx]/=scv_uint64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_UINT64 */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -391,11 +575,11 @@ scv_var_dvd /* [fnc] Divide scalar by variable */
     break;
   } /* endif NC_DOUBLE */
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op2.lp[idx]=scv_lng/op2.lp[idx];
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op2.lp[idx] != mss_val_lng) op2.lp[idx]=scv_lng/op2.lp[idx];
       } /* end for */
@@ -403,23 +587,69 @@ scv_var_dvd /* [fnc] Divide scalar by variable */
     break;
   } /* endif NC_INT */
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op2.sp[idx]=scv_sht/op2.sp[idx];
+      for(idx=0;idx<sz;idx++) op2.sp[idx]=scv_short/op2.sp[idx];
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op2.sp[idx] != mss_val_sht) op2.sp[idx]=scv_sht/op2.sp[idx];
+	if(op2.sp[idx] != mss_val_short) op2.sp[idx]=scv_short/op2.sp[idx];
       } /* end for */
     } /* end else */
     break;
-  } /* endif NC_SHORT */
-  case NC_CHAR:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.usp[idx]=scv_ushort/op2.usp[idx];
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.usp[idx] != mss_val_ushort) op2.usp[idx]=scv_ushort/op2.usp[idx];
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.uip[idx]=scv_uint/op2.uip[idx];
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.uip[idx] != mss_val_uint) op2.uip[idx]=scv_uint/op2.uip[idx];
+      } /* end for */
+    } /* end else */
     break;
+  } /* endif NC_UINT */
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.i64p[idx]=scv_int64/op2.i64p[idx];
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.i64p[idx] != mss_val_int64) op2.i64p[idx]=scv_int64/op2.i64p[idx];
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_INT64 */
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.ui64p[idx]=scv_uint64/op2.ui64p[idx];
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.ui64p[idx] != mss_val_uint64) op2.ui64p[idx]=scv_uint64/op2.ui64p[idx];
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_UINT64 */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -489,11 +719,11 @@ var_scv_mod /* [fnc] Modulo variable by scalar */
     break;
   } /* endif NC_DOUBLE */
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op1.lp[idx]%=scv_lng;
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op1.lp[idx] != mss_val_lng) op1.lp[idx]%=scv_lng; 
       } /* end for */
@@ -501,23 +731,69 @@ var_scv_mod /* [fnc] Modulo variable by scalar */
     break;
   } /* endif NC_INT */
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op1.sp[idx]%=scv_sht;
+      for(idx=0;idx<sz;idx++) op1.sp[idx]%=scv_short;
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op1.sp[idx] != mss_val_sht) op1.sp[idx]%=scv_sht;
+	if(op1.sp[idx] != mss_val_short) op1.sp[idx]%=scv_short;
       } /* end for */
     } /* end else */
     break;
   } /* endif NC_SHORT */
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.usp[idx]%=scv_ushort;
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.usp[idx] != mss_val_ushort) op1.usp[idx]%=scv_ushort;
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.uip[idx]%=scv_uint;
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.uip[idx] != mss_val_uint) op1.uip[idx]%=scv_uint;
+      } /* end for */
+    } /* end else */
     break;
+  } /* endif NC_UINT */
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.i64p[idx]%=scv_int64;
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.i64p[idx] != mss_val_int64) op1.i64p[idx]%=scv_int64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_INT64 */
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op1.ui64p[idx]%=scv_uint64;
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op1.ui64p[idx] != mss_val_uint64) op1.ui64p[idx]%=scv_uint64;
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_UINT64 */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -584,11 +860,11 @@ scv_var_mod /* [fnc] Modulo scalar by variable */
     break;
   } /* endif NC_DOUBLE */
   case NC_INT:{
-    const nco_long scv_lng=scv->val.l;
+    const nco_int scv_lng=scv->val.l;
     if(!has_mss_val){
       for(idx=0;idx<sz;idx++) op2.lp[idx]=scv_lng%op2.lp[idx];
     }else{
-      const long mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
+      const nco_int mss_val_lng=*mss_val.lp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
 	if(op2.lp[idx] != mss_val_lng) op2.lp[idx]=scv_lng%op2.lp[idx];
       } /* end for */
@@ -596,23 +872,69 @@ scv_var_mod /* [fnc] Modulo scalar by variable */
     break;
   } /* endif NC_INT */
   case NC_SHORT:{
-    const short scv_sht=scv->val.s; 
+    const nco_short scv_short=scv->val.s; 
     if(!has_mss_val){
-      for(idx=0;idx<sz;idx++) op2.sp[idx]=scv_sht%op2.sp[idx];
+      for(idx=0;idx<sz;idx++) op2.sp[idx]=scv_short%op2.sp[idx];
     }else{
-      const short mss_val_sht=*mss_val.sp; /* Temporary variable reduces de-referencing */
+      const nco_short mss_val_short=*mss_val.sp; /* Temporary variable reduces de-referencing */
       for(idx=0;idx<sz;idx++){
-	if(op2.sp[idx] != mss_val_sht) op2.sp[idx]=scv_sht%op2.sp[idx];
+	if(op2.sp[idx] != mss_val_short) op2.sp[idx]=scv_short%op2.sp[idx];
       } /* end for */
     } /* end else */
     break;
   } /* endif NC_SHORT */
-  case NC_CHAR:
-    /* Do nothing */
+  case NC_USHORT:{
+    const nco_ushort scv_ushort=scv->val.us; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.usp[idx]=scv_ushort%op2.usp[idx];
+    }else{
+      const nco_ushort mss_val_ushort=*mss_val.usp; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.usp[idx] != mss_val_ushort) op2.usp[idx]=scv_ushort%op2.usp[idx];
+      } /* end for */
+    } /* end else */
     break;
-  case NC_BYTE:
-    /* Do nothing */
+  } /* endif NC_USHORT */
+  case NC_UINT:{
+    const nco_uint scv_uint=scv->val.ui; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.uip[idx]=scv_uint%op2.uip[idx];
+    }else{
+      const nco_uint mss_val_uint=*mss_val.uip; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.uip[idx] != mss_val_uint) op2.uip[idx]=scv_uint%op2.uip[idx];
+      } /* end for */
+    } /* end else */
     break;
+  } /* endif NC_UINT */
+  case NC_INT64:{
+    const nco_int64 scv_int64=scv->val.i64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.i64p[idx]=scv_int64%op2.i64p[idx];
+    }else{
+      const nco_int64 mss_val_int64=*mss_val.i64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.i64p[idx] != mss_val_int64) op2.i64p[idx]=scv_int64%op2.i64p[idx];
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_INT64 */
+  case NC_UINT64:{
+    const nco_uint64 scv_uint64=scv->val.ui64; 
+    if(!has_mss_val){
+      for(idx=0;idx<sz;idx++) op2.ui64p[idx]=scv_uint64%op2.ui64p[idx];
+    }else{
+      const nco_uint64 mss_val_uint64=*mss_val.ui64p; /* Temporary variable reduces de-referencing */
+      for(idx=0;idx<sz;idx++){
+	if(op2.ui64p[idx] != mss_val_uint64) op2.ui64p[idx]=scv_uint64%op2.ui64p[idx];
+      } /* end for */
+    } /* end else */
+    break;
+  } /* endif NC_UINT64 */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   
@@ -662,19 +984,17 @@ var_scv_pwr /* [fnc] Empower variable by scalar */
     } /* end else */
     break;
   } /* end NC_DOUBLE */
-  case NC_INT:
-    /* Do nothing */
     /* fxm: nco322 Implement integer empowerment? GSL? */
-    break;
-  case NC_SHORT:
-    /* Do nothing */
-    break;
-  case NC_CHAR:
-    /* Do nothing */
-    break;
-  case NC_BYTE:
-    /* Do nothing */
-    break;
+  case NC_INT: break; /* Do nothing */
+  case NC_SHORT: break; /* Do nothing */
+  case NC_USHORT: break; /* Do nothing */
+  case NC_UINT: break; /* Do nothing */
+  case NC_INT64: break; /* Do nothing */
+  case NC_UINT64: break; /* Do nothing */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   }/* end switch */
   
@@ -727,19 +1047,17 @@ scv_var_pwr /* [fnc] Empower scalar by variable */
     } /* end else */
     break;
   } /* end NC_DOUBLE */
-  case NC_INT:
-    /* Do nothing */
     /* fxm: nco322 Implement integer empowerment? GSL? */
-    break;
-  case NC_SHORT:
-    /* Do nothing */
-    break;
-  case NC_CHAR:
-    /* Do nothing */
-    break;
-  case NC_BYTE:
-    /* Do nothing */
-    break;
+  case NC_INT: break; /* Do nothing */
+  case NC_SHORT: break; /* Do nothing */
+  case NC_USHORT: break; /* Do nothing */
+  case NC_UINT: break; /* Do nothing */
+  case NC_INT64: break; /* Do nothing */
+  case NC_UINT64: break; /* Do nothing */
+  case NC_BYTE: break; /* Do nothing */
+  case NC_UBYTE: break; /* Do nothing */
+  case NC_CHAR: break; /* Do nothing */
+  case NC_STRING: break; /* Do nothing */
   default: nco_dfl_case_nc_type_err(); break;
   }/* end switch */
   
