@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.c,v 1.66 2004/09/07 06:09:46 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.c,v 1.124 2007/05/25 05:24:23 zender Exp $ */
 
 /* Purpose: Variable utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 #include "nco_var_utl.h" /* Variable utilities */
 
@@ -88,8 +88,8 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
  const int out_id, /* I [id] netCDF output file ID */
  const int rec_dmn_id, /* I [id] Input file record dimension ID  */
  const char * const var_nm, /* I [sng] Input variable name */
- const lmt_all_sct * const lmt_lst, /* I [sct] Hyperslab limits */
- const int lmt_lst_nbr) /* I [nbr] Number of hyperslab limits */
+ CST_X_PTR_CST_PTR_CST_Y(lmt_all_sct,lmt_all_lst), /* I [sct] Hyperslab limits */
+ const int lmt_all_lst_nbr) /* I [nbr] Number of hyperslab limits */
 {
   /* Purpose: Copy variable metadata from input netCDF file to output netCDF file
      This routine truncates dimensions in variable definition in output file according to user-specified limits.
@@ -140,18 +140,18 @@ nco_cpy_var_dfn_lmt /* Copy variable metadata from input to output file */
     /* If dimension has not been defined, copy it */
     if(rcd_lcl != NC_NOERR){
       if(dmn_in_id[idx] != rec_dmn_id){
-	int lmt_idx;
+	int lmt_all_idx;
 
 	/* Decide whether this dimension has any user-specified limits */
-	for(lmt_idx=0;lmt_idx<lmt_lst_nbr;lmt_idx++){
-	  if(lmt_lst[lmt_idx].lmt_dmn[0]->id == dmn_in_id[idx]){
-	    dmn_sz=lmt_lst[lmt_idx].dmn_cnt;
+	for(lmt_all_idx=0;lmt_all_idx<lmt_all_lst_nbr;lmt_all_idx++){
+	  if(lmt_all_lst[lmt_all_idx]->lmt_dmn[0]->id == dmn_in_id[idx]){
+	    dmn_sz=lmt_all_lst[lmt_all_idx]->dmn_cnt;
 	    break;
 	  } /* end if */
-	} /* end loop over lmt_idx */
-	(void)nco_def_dim(out_id,dmn_nm,dmn_sz,dmn_out_id+idx );
+	} /* end loop over lmt_all_idx */
+	(void)nco_def_dim(out_id,dmn_nm,dmn_sz,dmn_out_id+idx);
       }else{
-	(void)nco_def_dim(out_id,dmn_nm,NC_UNLIMITED,dmn_out_id+idx );
+	(void)nco_def_dim(out_id,dmn_nm,NC_UNLIMITED,dmn_out_id+idx);
       } /* end else */
     } /* end if */
   } /* end loop over dim */
@@ -171,11 +171,12 @@ nco_cpy_var_val /* [fnc] Copy variable from input to output file, no limits */
 (int in_id, /* I [id] netCDF input file ID */
  int out_id, /* I [id] netCDF output file ID */
  FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
- const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+ const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
  char *var_nm) /* I [sng] Variable name */
 {
-  /* Purpose: Copy variable data from input netCDF file to output netCDF file
-     Routine does not account for user-specified limits, it just copies what it findsa
+  /* NB: nco_cpy_var_val() contains OpenMP critical region */
+  /* Purpose: Copy single variable from input netCDF file to output netCDF file
+     Routine does not account for user-specified limits, it just copies what it finds
      Routine copies variable-by-variable, old-style, called only by ncks */
 
   const char fnc_nm[]="nco_cpy_var_val()"; /* [sng] Function name */
@@ -205,7 +206,7 @@ nco_cpy_var_val /* [fnc] Copy variable from input to output file, no limits */
   (void)nco_inq_var(out_id,var_out_id,(char *)NULL,&var_type,&nbr_dmn_out,(int *)NULL,(int *)NULL);
   (void)nco_inq_var(in_id,var_in_id,(char *)NULL,&var_type,&nbr_dmn_in,(int *)NULL,(int *)NULL);
   if(nbr_dmn_out != nbr_dmn_in){
-    (void)fprintf(stdout,"%s: ERROR attempt to write %d dimensional input variable %s to %d dimensional space in output file. \nHINT: When using -A (append) option, all appended variables must be the same rank in the input file as in the output file. ncwa operator is useful at ridding variables of extraneous (size = 1) dimensions. Read the manual to see how.\n",prg_nm_get(),nbr_dmn_in,var_nm,nbr_dmn_out);
+    (void)fprintf(stdout,"%s: ERROR attempt to write %d dimensional input variable %s to %d dimensional space in output file. \nHINT: When using -A (append) option, all appended variables must be the same rank in the input file as in the output file. The ncwa operator is useful at ridding variables of extraneous (size = 1) dimensions. See how at http://nco.sf.net/nco.html#ncwa\n",prg_nm_get(),nbr_dmn_in,var_nm,nbr_dmn_out);
     nco_exit(EXIT_FAILURE);
   } /* endif */
   nbr_dim=nbr_dmn_out;
@@ -238,9 +239,11 @@ nco_cpy_var_val /* [fnc] Copy variable from input to output file, no limits */
   if(nbr_dim==0){
     nco_get_var1(in_id,var_in_id,0L,void_ptr,var_type);
     nco_put_var1(out_id,var_out_id,0L,void_ptr,var_type);
-  }else{ /* end if variable is a scalar */
-    nco_get_vara(in_id,var_in_id,dmn_srt,dmn_cnt,void_ptr,var_type);
-    nco_put_vara(out_id,var_out_id,dmn_srt,dmn_cnt,void_ptr,var_type);
+  }else{ /* end if variable is scalar */
+    if(var_sz > 0){ /* Allow for zero-size record variables */
+      nco_get_vara(in_id,var_in_id,dmn_srt,dmn_cnt,void_ptr,var_type);
+      nco_put_vara(out_id,var_out_id,dmn_srt,dmn_cnt,void_ptr,var_type);
+    } /* end if var_sz */
   } /* end if variable is an array */
   /* Write unformatted binary data */
   if(NCO_BNR_WRT) nco_bnr_wrt(fp_bnr,var_nm,var_sz,var_type,void_ptr);
@@ -261,17 +264,18 @@ nco_cpy_var_val_lmt /* [fnc] Copy variable data from input to output file, simpl
 (const int in_id, /* I [id] netCDF input file ID */
  const int out_id, /* I [id] netCDF output file ID */
  FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
- const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+ const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
  char *var_nm, /* I [sng] Variable name */
  const lmt_sct * const lmt, /* I [sct] Hyperslab limits */
  const int lmt_nbr) /* I [nbr] Number of hyperslab limits */
 {
+  /* NB: nco_cpy_var_val_lmt() contains OpenMP critical region */
   /* Purpose: Copy variable data from input netCDF file to output netCDF file 
      Truncate dimensions in variable definition in output file according to user-specified limits
      Copy variable-by-variable, old-style, used only by ncks */
 
-  bool SRD=False;
-  bool WRP=False;
+  nco_bool SRD=False;
+  nco_bool WRP=False;
 
   const char fnc_nm[]="nco_cpy_var_val_lmt()"; /* [sng] Function name */
 
@@ -359,7 +363,7 @@ nco_cpy_var_val_lmt /* [fnc] Copy variable data from input to output file, simpl
   void_ptr=(void *)nco_malloc_dbg(var_sz*nco_typ_lng(var_type),"Unable to malloc() value buffer when copying hypserslab from input to output file",fnc_nm);
 
   /* Copy variable */
-  if(nbr_dim==0){ /* Copy scalar */
+  if(nbr_dim == 0){ /* Copy scalar */
     nco_get_var1(in_id,var_in_id,0L,void_ptr,var_type);
     nco_put_var1(out_id,var_out_id,0L,void_ptr,var_type);
     if(NCO_BNR_WRT) nco_bnr_wrt(fp_bnr,var_nm,var_sz,var_type,void_ptr);
@@ -441,8 +445,8 @@ nco_cpy_var_val_lmt /* [fnc] Copy variable data from input to output file, simpl
 
     if(False){
       /* If coordinate variable, perform monotonicity check */
-      bool CRD=False;
-      bool MNT=False;
+      nco_bool CRD=False;
+      nco_bool MNT=False;
 
       double val_dbl;
       double wrp_spn;
@@ -469,10 +473,13 @@ nco_cpy_var_val_lmt /* [fnc] Copy variable data from input to output file, simpl
 	  case NC_SHORT:
 	  case NC_CHAR:
 	  case NC_BYTE:
-	  default:
-	    (void)fprintf(stdout,"%s: ERROR Unknown nc_type %d in %s\n",prg_nm_get(),var_type,fnc_nm);
-	    nco_exit(EXIT_FAILURE);
-	    break;
+	  case NC_UBYTE: break;
+	  case NC_USHORT: break;
+	  case NC_UINT: break;
+	  case NC_INT64: break;
+	  case NC_UINT64: break;
+	  case NC_STRING: break;
+	  default: nco_dfl_case_nc_type_err(); break;
 	  } /* end switch */
 	  
 	  /* Ensure val_dbl is between specified bounds */
@@ -539,9 +546,8 @@ nco_var_dpl /* [fnc] Duplicate input variable */
   /* Shallow copy structure */
   (void)memcpy((void *)var_cpy,(const void *)var,sizeof(var_sct));
 
-  /* fxm: Should copy name as well, but var_free() does not free it, and 
-     var_lists do not strdup() user input so must make all changes at once 
-     in order to avoid adding memory leak */
+  /* Make sure var_free() frees names when variable is destructed */
+  if(var->nm != NULL) var_cpy->nm=(char *)strdup(var->nm);
 
   /* Deep copy dyamically allocated arrays currently defined in original */
   if(var->val.vp != NULL){
@@ -598,25 +604,42 @@ nco_var_get /* [fnc] Allocate, retrieve variable hyperslab from disk to memory *
 (const int nc_id, /* I [id] netCDF file ID */
  var_sct *var) /* I [sct] Variable to get */
 {
-  /* Threads: Routine contains thread-unsafe calls protected by critical regions */
+  /* NB: nco_var_get() with same nc_id contains OpenMP critical region */
   /* Purpose: Allocate and retrieve given variable hyperslab from disk into memory
      If variable is packed on disk then inquire about scale_factor and add_offset */
+  int idx;
+  long srd_prd=1L; /* [nbr] Product of strides */
   const char fnc_nm[]="nco_var_get()"; /* [sng] Function name */
 
   var->val.vp=(void *)nco_malloc_dbg(var->sz*nco_typ_lng(var->typ_dsk),"Unable to malloc() value buffer when retrieving variable from disk",fnc_nm);
 
-  if(False) (void)fprintf(stdout,"%s: DEBUG: fxm TODO nco354. Calling nco_get_vara() for variable %s with nc_id=%d, var_id=%d, var_srt=%li, var_cnt = %li, var_val = %g, var_typ = %s\n",prg_nm_get(),var->nm,nc_id,var->id,var->srt[0],var->cnt[0],var->val.dp[0],nco_typ_sng(var->typ_dsk));
-#ifdef _OPENMP
-#pragma omp critical
-#endif /* _OPENMP */
-  { /* begin OpenMP critical */
-    if(var->sz > 1){
-      (void)nco_get_vara(nc_id,var->id,var->srt,var->cnt,var->val.vp,var->typ_dsk);
-    }else{
-      (void)nco_get_var1(nc_id,var->id,var->srt,var->val.vp,var->typ_dsk);
-    } /* end else */
-  } /* end OpenMP critical */
+  if(False) (void)fprintf(stdout,"%s: DEBUG: fxm TODO nco354. Calling nco_get_vara() for variable %s with nc_id=%d, var_id=%d, var_srt=%li, var_cnt = %li, var_val = %g, var_typ = %s\n",prg_nm_get(),var->nm,nc_id,var->id,var->srt[0],var->cnt[0],var->val.fp[0],nco_typ_sng(var->typ_dsk));
 
+  /* 20051021: Removed this potentially critical region by parallelizing 
+     over in_id's in calling code */
+  /* 20051019: nco_get_var*() routines are potentially SMP-critical
+     netCDF library allows parallel reads by different processes, not threads
+     Parallel reads to the same nc_id by different threads are critical because
+     the underlying UNIX file open has limited stdin caching
+     Parallel reads to different nc_id's for same underlying file work because
+     each UNIX file open (for same file) creates own stdin caching */
+  /* 20050629: Removing this critical region and calling with identical nc_id's causes multiple ncwa/ncra regressions */
+  { /* begin potential OpenMP critical */
+    /* Block is critical/thread-safe for identical/distinct in_id's */
+
+    /* Is stride > 1? */
+    for(idx=0;idx<var->nbr_dim;idx++) srd_prd*=var->srd[idx];
+
+    if(srd_prd == 1L){ 
+      if(var->sz > 1L)
+	(void)nco_get_vara(nc_id,var->id,var->srt,var->cnt,var->val.vp,var->typ_dsk);
+      else
+	(void)nco_get_var1(nc_id,var->id,var->srt,var->val.vp,var->typ_dsk);
+    }else{ 
+      (void)nco_get_varm(nc_id,var->id,var->srt,var->cnt,var->srd,(long *)NULL,var->val.vp,var->typ_dsk);
+    } /* endif non-unity stride  */
+  } /* end potential OpenMP critical */
+  
   /* Packing properties initially obtained by nco_pck_dsk_inq() in nco_var_fll()
      Multi-file operators (MFOs) call nco_var_get() multiple times for each variable
      In between subsequent calls to nco_var_get(), variable may be unpacked 
@@ -631,8 +654,8 @@ nco_var_get /* [fnc] Allocate, retrieve variable hyperslab from disk to memory *
      fxm nco457: Why not always convert missing_value to variable type, 
      even when variable is not packed? Answer: because doing this appears
      to break some ncra tests */
-    if(var->pck_dsk) var=nco_cnv_mss_val_typ(var,var->typ_dsk);
-    /*    var=nco_cnv_mss_val_typ(var,var->typ_dsk);*/
+  if(var->pck_dsk) var=nco_cnv_mss_val_typ(var,var->typ_dsk);
+  /*    var=nco_cnv_mss_val_typ(var,var->typ_dsk);*/
 
   /* Type of variable and missing value in memory are now same as type on disk */
   var->type=var->typ_dsk; /* [enm] Type of variable in RAM */
@@ -645,26 +668,22 @@ nco_var_get /* [fnc] Allocate, retrieve variable hyperslab from disk to memory *
   if(nco_is_rth_opr(prg_get())){
     /* Arithmetic operators must unpack variables before performing arithmetic
        Otherwise arithmetic will produce garbage results */
-#ifdef _OPENMP
-#pragma omp critical
-#endif /* _OPENMP */
+    /* 20050519: Not sure why I originally made nco_var_upk() call SMP-critical
+       20050629: Making this region multi-threaded causes no problems */
     if(var->pck_dsk) var=nco_var_upk(var);
   } /* endif arithmetic operator */
-
+  
 } /* end nco_var_get() */
 
 void
 nco_xrf_dmn /* [fnc] Switch pointers to dimension structures so var->dim points to var->dim->xrf */
 (var_sct * const var) /* I [sct] Variable to manipulate */
 {
-  /* Purpose: Switch pointers to dimension structures so var->dim points to var->dim->xrf.
+  /* Purpose: Switch pointers to dimension structures so var->dim points to var->dim->xrf
      Routine makes dim element of variable structure from nco_var_dpl() refer to counterparts
      of dimensions directly associated with variable it was duplicated from */
-  
   int idx;
-  
   for(idx=0;idx<var->nbr_dim;idx++) var->dim[idx]=var->dim[idx]->xrf;
-  
 } /* end nco_xrf_dmn() */
 
 void
@@ -673,22 +692,18 @@ nco_xrf_var /* [fnc] Make xrf elements of variable structures point to eachother
  var_sct * const var_2) /* I/O [sct] Related variable */
 {
   /* Purpose: Make xrf elements of variable structures point to eachother */
-
   var_1->xrf=var_2;
   var_2->xrf=var_1;
-
 } /* end nco_xrf_var() */
 
 var_sct * /* O [sct] Pointer to free'd variable */
 nco_var_free /* [fnc] Free all memory associated with variable structure */
-(var_sct *var) /* I [sct] Variable to free */
+(var_sct *var) /* I/O [sct] Variable to free */
 {
   /* Threads: Routine is thread safe and calls no unsafe routines */
   /* Purpose: Free all memory associated with a dynamically allocated variable structure */
-  
-  /* fxm: var->nm is not free()'d because names may be owned by optarg list (system)
-     This assumption needs to be changed before freeing name pointer */
-  /*  var->nm=nco_free(var->nm);*/
+
+  var->nm=(char *)nco_free(var->nm);
   var->val.vp=nco_free(var->val.vp);
   var->mss_val.vp=nco_free(var->mss_val.vp);
   var->tally=(long *)nco_free(var->tally);
@@ -705,9 +720,25 @@ nco_var_free /* [fnc] Free all memory associated with variable structure */
   var=(var_sct *)nco_free(var);
 
   return NULL;
-
 } /* end nco_var_free() */
 
+var_sct ** /* O [sct] Pointer to free'd structure list */
+nco_var_lst_free /* [fnc] Free memory associated with variable structure list */
+(var_sct **var_lst, /* I/O [sct] Variable structure list to free */
+ const int var_nbr) /* I [nbr] Number of variable structures in list */
+{
+  /* Threads: Routine is thread safe and calls no unsafe routines */
+  /* Purpose: Free all memory associated with dynamically allocated variable structure list */
+  int idx;
+
+  for(idx=0;idx<var_nbr;idx++) var_lst[idx]=nco_var_free(var_lst[idx]);
+
+  /* Free structure pointer last */
+  var_lst=(var_sct **)nco_free(var_lst);
+
+  return var_lst;
+} /* end nco_var_lst_free() */
+
 int /* O [enm] Return code */
 var_dfl_set /* [fnc] Set defaults for each member of variable structure */
 (var_sct * const var) /* I [sct] Variable strucutre to initialize to defaults */
@@ -727,7 +758,7 @@ var_dfl_set /* [fnc] Set defaults for each member of variable structure */
   var->typ_upk=NC_NAT; /* Type of variable when unpacked (expanded) (in memory) */
   var->is_rec_var=False;
   var->is_crd_var=False;
-  /* Size of 1 is assumed in nco_var_fll() */
+  /* nco_var_fll() assumes size of 1 */
   var->sz=1L;
   var->sz_rec=1L;
   var->cid=-1;
@@ -745,6 +776,8 @@ var_dfl_set /* [fnc] Set defaults for each member of variable structure */
   var->srt=(long *)NULL;
   var->end=(long *)NULL;
   var->srd=(long *)NULL;
+  var->undefined=False;
+  var->is_fix_var=True; /* Is this a fixed (non-processed) variable? */
 
   /* Members related to packing */
   var->has_scl_fct=False; /* [flg] Valid scale_factor attribute exists */
@@ -795,11 +828,9 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
      So local variable var usually refers to var_prc_out in calling function 
      Hence names may look reversed in this function, and xrf is frequently used
 
-     fxm TODO nco402:
-     Have ncap,ncra,ncbo call with FIXED_KEEP_PACKED to keep fixed variables unaltered
-     We do not want to un-necessarily unpack variables that are fixed, not processed */
+     20060217: Packed fixed (non-processed) variables are now passed through unaltered */
 
-  bool PCK_ATT_CPY=True; /* [flg] Copy attributes "scale_factor", "add_offset" */
+  nco_bool PCK_ATT_CPY=True; /* [flg] Copy attributes "scale_factor", "add_offset" */
 
   const char fnc_nm[]="nco_var_dfn()"; /* [sng] Function name */
 
@@ -819,30 +850,30 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
     /* Checking only nco_is_rth_opr() is too simplistic
        1. All variables handled by arithmetic operators are currently unpacked on reading
        2. However "fixed variables" appear in many arithemetic operators
-          ncra, for instance, treats non-record variables as fixed (does not average them)
-	  ncbo treats coordinate variables as fixed  (does not subtract them)
-	  It would be best not to alter [un-]packing of arithmetic fixed variables
+	  ncbo treats coordinate variables as fixed (does not subtract them)
+          ncra treats non-record variables as fixed (does not average them)
+	  ncwa treats variables without averaging dimensions as fixed (does not average them)
+	  It is best not to alter [un-]pack fixed (non-processed) variables
        3. ncap, an arithmetic operator, also has "fixed variables", i.e., 
           pre-existing non-LHS variables copied directly to output.
 	  These "fixed" ncap variables should remain unaltered
 	  However, this is not presently done
 	  nco_var_dfn() needs more information to handle "fixed" variables correctly because
 	  Some ncap "fixed" variables appear on RHS in definitions of LHS variables
-          These RHS fixed variable must be separately unpacked during RHS algebra
+          These RHS fixed variables must be separately unpacked during RHS algebra
 	  Currently, ncap only calls nco_var_dfn() for fixed variables
 	  ncap uses its own routine, ncap_var_write(), for RHS variable definitions
-	  Tentative solution 20030119: fxm TODO nco402 implement this:
-	  Change ncap-specific condition to more generic FIXED_KEEP_PACKED flag 
-	  Pass this flag into nco_var_dfn()
        4. All variables in non-arithmetic operators (except ncpdq) should remain un-altered
-       5. ncpdq is non-arithemetic operator
+       5. ncpdq is non-arithmetic operator
           However, ncpdq specially handles fine-grained control [un-]packing options */
     if(nco_is_rth_opr(prg_id)){
       /* Arithmetic operators store values as unpacked... */
       typ_out=var[idx]->typ_upk; 
-      /* ...with one exception...
+      /* ...with two exceptions...
 	 ncap [un-]packing precedes nco_var_dfn() call, sets var->type appropriately */
       if(prg_id == ncap) typ_out=var[idx]->type;
+      /* ...and pass through fixed (non-processed) variables untouched... */
+      if(var[idx]->is_fix_var) typ_out=var[idx]->type;
     }else{
       /* Non-arithmetic operators leave things alone by default
 	 ncpdq first modifies var_out->type, then calls nco_var_dfn(), then [un-]packs */
@@ -882,7 +913,9 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
 
       if(dbg_lvl_get() > 3 && prg_id != ncwa){
 	/* fxm TODO nco374 diagnostic information fails for ncwa since var[idx]->dim[dmn_idx]->nm
-	   contains _wrong name_ when variables will be averaged. */
+	   contains _wrong name_ when variables will be averaged.
+	   ncwa does contain write name information now if retain_degenerate_dimensions 
+	   option is in effect, but this is the exception rather than the rule. */
 	(void)fprintf(stdout,"%s: DEBUG %s about to define variable %s with %d dimension%s%s",prg_nm_get(),fnc_nm,var[idx]->nm,dmn_nbr,(dmn_nbr == 1) ? "" : "s",(dmn_nbr > 0) ? " (ordinal,output ID): " : "");
 	for(dmn_idx=0;dmn_idx<dmn_nbr;dmn_idx++){
 	  (void)fprintf(stdout,"%s (%d,%s)%s",var[idx]->dim[dmn_idx]->nm,dmn_idx,"unknown",(dmn_idx < dmn_nbr-1) ? ", " : "");
@@ -895,7 +928,9 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
       
       if(dbg_lvl_get() > 3 && prg_id != ncwa){
 	/* fxm TODO nco374 diagnostic information fails for ncwa since var[idx]->dim[dmn_idx]->nm
-	   contains _wrong name_ when variables will be averaged. */
+	   contains _wrong name_ when variables will be averaged.
+	   ncwa does contain write name information now if retain_degenerate_dimensions 
+	   option is in effect, but this is the exception rather than the rule. */
 	(void)fprintf(stdout,"%s: DEBUG %s defined variable %s with %d dimension%s%s",prg_nm_get(),fnc_nm,var[idx]->nm,dmn_nbr,(dmn_nbr == 1) ? "" : "s",(dmn_nbr > 0) ? " (ordinal,output ID): " : "");
 	for(dmn_idx=0;dmn_idx<dmn_nbr;dmn_idx++){
 	  (void)fprintf(stdout,"%s (%d,%d)%s",var[idx]->dim[dmn_idx]->nm,dmn_idx,dmn_id_vec[dmn_idx],(dmn_idx < dmn_nbr-1) ? ", " : "");
@@ -930,17 +965,18 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
     /* Copy exising packing attributes, if any, unless... */
     if(nco_is_rth_opr(prg_id) && /* ...operator is arithmetic... */
        prg_id != ncap && /* ...and is not ncap (hence it must be, e.g., ncra, ncbo)... */
+       !var[idx]->is_fix_var && /* ...and variable is processed (not fixed)... */
        var[idx]->xrf->pck_dsk) /* ...and variable is packed in input file... */
       PCK_ATT_CPY=False;
 
-    /* Do not copy packing attributres when unpacking variables 
+    /* Do not copy packing attributes when unpacking variables 
        ncpdq is currently only operator that passes values other than nco_pck_plc_nil */
     if(nco_pck_plc == nco_pck_plc_upk) /* ...and variable will be _unpacked_ ... */
       PCK_ATT_CPY=False;
     
     /* Recall that:
        var      refers to output variable structure
-       var->xrf refers to input variable structure 
+       var->xrf refers to input  variable structure 
        ncpdq may pre-define packing attributes below regardless of PCK_ATT_CPY */ 
     (void)nco_att_cpy(in_id,out_id,var[idx]->xrf->id,var[idx]->id,PCK_ATT_CPY);
     
@@ -949,9 +985,8 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
        Recall ncap calls ncap_var_write() to define newly packed LHS variables 
        If operator will attempt to pack some variables... */
     if(nco_pck_plc != nco_pck_plc_nil && nco_pck_plc != nco_pck_plc_upk){ 
-      bool nco_pck_plc_alw; /* O [flg] Packing policy allows packing nc_typ_in */
       /* ...and expanded variable is pack-able... */
-      if((nco_pck_plc_alw=nco_pck_plc_typ_get(nco_pck_map,var[idx]->typ_upk,(nc_type *)NULL))){
+      if(nco_pck_plc_typ_get(nco_pck_map,var[idx]->typ_upk,(nc_type *)NULL)){
 	/* ...and operator will pack this particular variable... */
 	if(
 	   /* ...either because operator newly packs all variables... */
@@ -979,7 +1014,7 @@ nco_var_dfn /* [fnc] Define variables and write their attributes to output file
 	  (void)nco_put_att(out_id,var[idx]->id,add_fst_sng,typ_out,1,zero_var->val.vp);
 	  zero_var=(var_sct *)nco_var_free(zero_var);
 	} /* endif this variable will be packed or re-packed */
-      } /* endif nco_pck_plc_alw */
+      } /* !nco_pck_plc_alw */
     } /* endif nco_pck_plc involves packing */
   } /* end loop over idx variables to define */
 } /* end nco_var_dfn() */
@@ -991,130 +1026,39 @@ nco_var_val_cpy /* [fnc] Copy variables data from input to output file */
  var_sct ** const var, /* I/O [sct] Variables to copy to output file */
  const int nbr_var) /* I [nbr] Number of variables */
 {
-  /* Purpose: Copy variable data for every variable in input variable structure list
-     from input file to output file */
+  /* NB: nco_var_val_cpy() contains OpenMP critical region */
+  /* Purpose: Copy every variable in input variable structure list 
+     from input file to output file. Only data (not metadata) are copied. */
   
   int idx;
+  int dmn_idx;
+  long srd_prd=1L; /* [nbr] Product of strides */
   
   for(idx=0;idx<nbr_var;idx++){
     var[idx]->xrf->val.vp=var[idx]->val.vp=(void *)nco_malloc(var[idx]->sz*nco_typ_lng(var[idx]->type));
     if(var[idx]->nbr_dim == 0){
       nco_get_var1(in_id,var[idx]->id,var[idx]->srt,var[idx]->val.vp,var[idx]->type);
       nco_put_var1(out_id,var[idx]->xrf->id,var[idx]->xrf->srt,var[idx]->xrf->val.vp,var[idx]->type);
-    }else{ /* end if variable is a scalar */
-      nco_get_vara(in_id,var[idx]->id,var[idx]->srt,var[idx]->cnt,var[idx]->val.vp,var[idx]->type);
-      nco_put_vara(out_id,var[idx]->xrf->id,var[idx]->xrf->srt,var[idx]->xrf->cnt,var[idx]->xrf->val.vp,var[idx]->type);
+    }else{ /* end if variable is scalar */
+      if(var[idx]->sz > 0){ /* Do nothing for zero-size record variables */
+
+	/* Is stride > 1? */
+	for(dmn_idx=0;dmn_idx<var[idx]->nbr_dim;dmn_idx++) srd_prd*=var[idx]->srd[dmn_idx];
+
+	if(srd_prd == 1L){ 
+	  nco_get_vara(in_id,var[idx]->id,var[idx]->srt,var[idx]->cnt,var[idx]->val.vp,var[idx]->type);
+	  nco_put_vara(out_id,var[idx]->xrf->id,var[idx]->xrf->srt,var[idx]->xrf->cnt,var[idx]->xrf->val.vp,var[idx]->type);
+	}else{
+	  (void)nco_get_varm(in_id,var[idx]->id,var[idx]->srt,var[idx]->cnt,var[idx]->srd,(long *)NULL,var[idx]->val.vp,var[idx]->type);
+	  (void)nco_put_varm(out_id,var[idx]->xrf->id,var[idx]->xrf->srt,var[idx]->xrf->cnt,var[idx]->xrf->srd,(long *)NULL,var[idx]->xrf->val.vp,var[idx]->type);
+	} /* endif variable has non-unity stride */
+      } /* end if var_sz */
     } /* end if variable is an array */
     var[idx]->val.vp=var[idx]->xrf->val.vp=nco_free(var[idx]->val.vp);
   } /* end loop over idx */
-
+    
 } /* end nco_var_val_cpy() */
 
-nm_id_sct * /* O [sct] List with coordinate excluded */
-nco_var_lst_crd_xcl /* [fnc] Exclude given coordinates from extraction list */
-(const int nc_id, /* I [id] netCDF file ID */
- const int dmn_id, /* I [id] Dimension ID of coordinate to remove from extraction list */
- nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
- int * const nbr_xtr) /* I/O [nbr] Number of variables in extraction list */
-{
-  /* Purpose: Modify extraction list to exclude coordinate, if any, associated with given dimension ID */
-  
-  char crd_nm[NC_MAX_NAME];
-
-  int idx;
-  int crd_id=-1;
-  int rcd=NC_NOERR; /* [rcd] Return code */
-  
-  /* What is variable ID of record coordinate, if any? */
-  (void)nco_inq_dimname(nc_id,dmn_id,crd_nm);
-   
-  rcd=nco_inq_varid_flg(nc_id,crd_nm,&crd_id);
-  if(rcd == NC_NOERR){
-    /* Is coordinate on extraction list? */
-    for(idx=0;idx<*nbr_xtr;idx++){
-      if(xtr_lst[idx].id == crd_id) break;
-    } /* end loop over idx */
-    if(idx != *nbr_xtr){
-      nm_id_sct *var_lst_tmp;
-      
-      var_lst_tmp=(nm_id_sct *)nco_malloc(*nbr_xtr*sizeof(nm_id_sct));
-      /* Copy the extract list to the temporary extract list and reallocate the extract list */
-      (void)memcpy((void *)var_lst_tmp,(void *)xtr_lst,*nbr_xtr*sizeof(nm_id_sct));
-      (*nbr_xtr)--;
-      xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,*nbr_xtr*sizeof(nm_id_sct));
-      /* Collapse the temporary extract list into the permanent list by copying 
-	 all but the coordinate. NB: the ordering of the list is conserved. */
-      (void)memcpy((void *)xtr_lst,(void *)var_lst_tmp,idx*sizeof(nm_id_sct));
-      (void)memcpy((void *)(xtr_lst+idx),(void *)(var_lst_tmp+idx+1),(*nbr_xtr-idx)*sizeof(nm_id_sct));
-      /* Free the memory for coordinate name in the extract list before losing the pointer */
-      var_lst_tmp[idx].nm=(char *)nco_free(var_lst_tmp[idx].nm);
-      var_lst_tmp=(nm_id_sct *)nco_free(var_lst_tmp);
-    } /* end if */
-  } /* end if */
-  
-  return xtr_lst;
-  
-} /* end nco_var_lst_crd_xcl() */
-
-nm_id_sct * /* O [sct] Extraction list */
-nco_var_lst_ass_crd_add /* [fnc] Add coordinates associated extracted variables to extraction list */
-(const int nc_id, /* I netCDF file ID */
- nm_id_sct *xtr_lst, /* I/O current extraction list (destroyed) */
- int * const nbr_xtr) /* I/O number of variables in current extraction list */
-{
-  /* Purpose: Add coordinates associated extracted variables to extraction list
-     This helps with making concise nco_malloc() calls down road */
-
-  char dmn_nm[NC_MAX_NAME];
-
-  int crd_id;
-  int dmn_id[NC_MAX_DIMS];
-  int idx_dmn;
-  int idx_var_dim;
-  int idx_var;
-  int nbr_dim;
-  int nbr_var_dim;
-  int rcd=NC_NOERR; /* [rcd] Return code */
-
-  /* Get number of dimensions */
-  (void)nco_inq(nc_id,&nbr_dim,(int *)NULL,(int *)NULL,(int *)NULL);
-
-  /* ...for each dimension in input file... */
-  for(idx_dmn=0;idx_dmn<nbr_dim;idx_dmn++){
-    /* ...see if it is a coordinate dimension... */
-    (void)nco_inq_dimname(nc_id,idx_dmn,dmn_nm);
-     
-    rcd=nco_inq_varid_flg(nc_id,dmn_nm,&crd_id);
-    if(rcd == NC_NOERR){
-      /* Is this coordinate already on extraction list? */
-      for(idx_var=0;idx_var<*nbr_xtr;idx_var++){
-	if(crd_id == xtr_lst[idx_var].id) break;
-      } /* end loop over idx_var */
-      if(idx_var == *nbr_xtr){
-	/* ...the coordinate is not on the list, is it associated with any variables?... */
-	for(idx_var=0;idx_var<*nbr_xtr;idx_var++){
-	  /* Get number of dimensions and dimension IDs for variable. */
-	  (void)nco_inq_var(nc_id,xtr_lst[idx_var].id,(char *)NULL,(nc_type *)NULL,&nbr_var_dim,dmn_id,(int *)NULL);
-	  for(idx_var_dim=0;idx_var_dim<nbr_var_dim;idx_var_dim++){
-	    if(idx_dmn == dmn_id[idx_var_dim]) break;
-	  } /* end loop over idx_var_dim */
-	  if(idx_var_dim != nbr_var_dim){
-	    /* Add the coordinate to the list */
-	    xtr_lst=(nm_id_sct *)nco_realloc((void *)xtr_lst,(*nbr_xtr+1)*sizeof(nm_id_sct));
-	    xtr_lst[*nbr_xtr].nm=(char *)strdup(dmn_nm);
-	    xtr_lst[*nbr_xtr].id=crd_id;
-	    (*nbr_xtr)++;
-	    break;
-	  } /* end if */
-	} /* end loop over idx_var */
-      } /* end if coordinate was not already on the list */
-    } /* end if dimension is a coordinate */
-  } /* end loop over idx_dmn */
-  
-  return xtr_lst;
-  
-} /* end nco_var_lst_ass_crd_add() */
-
 var_sct * /* O [sct] Variable structure */
 nco_var_fll /* [fnc] Allocate variable structure and fill with metadata */
 (const int nc_id, /* I [id] netCDF file ID */
@@ -1140,6 +1084,7 @@ nco_var_fll /* [fnc] Allocate variable structure and fill with metadata */
   (void)var_dfl_set(var); /* [fnc] Set defaults for each member of variable structure */
 
   /* Fill in known fields */
+  /* Make sure var_free() frees names when variable is destructed */
   var->nm=(char *)strdup(var_nm);
   var->id=var_id;
   var->nc_id=nc_id;
@@ -1220,20 +1165,21 @@ nco_var_fll /* [fnc] Allocate variable structure and fill with metadata */
 
   /* Portions of variable structure depend on packing properties, e.g., typ_upk
      nco_pck_dsk_inq() fills in these portions harmlessly */
-  (void)nco_pck_dsk_inq(nc_id,var);
-
+    (void)nco_pck_dsk_inq(nc_id,var);
+  
+  /* used when parsing script in ncap */
+    var->undefined=False;
   return var;
 } /* end nco_var_fll() */
 
 void
-nco_var_refresh /* [fnc] Update variable metadata (var ID, dmn_nbr, mss_val) */
+nco_var_mtd_refresh /* [fnc] Update variable metadata (dmn_nbr, ID, mss_val, type) */
 (const int nc_id, /* I [id] netCDF input-file ID */
  var_sct * const var) /* I/O [sct] Variable to update */
 {
-  /* Threads: Routine contains thread-unsafe calls protected by critical regions */
   /* Purpose: Update variable ID, number of dimensions, and missing_value attribute for given variable
-     nco_var_refresh() is called in file loop in multi-file operators because each new file may have 
-     different variable ID and missing_value for same variable.
+     nco_var_mtd_refresh() is called in file loop in multi-file operators because each new file may have 
+     different variable ID, type, and missing_value for same variable.
      This is necessary, for example, if computer model runs on one machine, e.g., SGI,
      and then run is restarted on another, e.g., Cray. 
      If internal floating point representations differ between these architecture, 
@@ -1241,49 +1187,56 @@ nco_var_refresh /* [fnc] Update variable metadata (var ID, dmn_nbr, mss_val) */
      Variable IDs may change when some but not all model output files are 
      manipulated (i.e., variables added or deleted), followed by processing
      all files are processed in a batch.
-     NCO is the only tool I know of which makes this all user-transparent
-     Thus this capability is very important to maintain */
+     NCO is only known tool that makes this all user-transparent
+     Thus this capability is very important to maintain
+     fxm: why isn't variable type set here? */
 
   /* Refresh variable ID */
   var->nc_id=nc_id;
 
-#ifdef _OPENMP
-#pragma omp critical
-#endif /* _OPENMP */
-  { /* begin OpenMP critical */
-    (void)nco_inq_varid(var->nc_id,var->nm,&var->id);
+  /* 20050519: Not sure why I originally made next four lines SMP-critical
+     20050629: Making next four lines multi-threaded causes no problems */
+  (void)nco_inq_varid(var->nc_id,var->nm,&var->id);
     
-    /* fxm: Not sure if/why necessary to refresh number of dimensions...but it should not hurt */
-    /* Refresh number of dimensions in variable */
-    (void)nco_inq_varndims(var->nc_id,var->id,&var->nbr_dim);
+  /* fxm: Not sure if/why necessary to refresh number of dimensions...though it should not hurt */
+  /* Refresh number of dimensions in variable */
+  (void)nco_inq_varndims(var->nc_id,var->id,&var->nbr_dim);
     
-    /* Refresh number of attributes and missing value attribute, if any */
-    var->has_mss_val=nco_mss_val_get(var->nc_id,var);
-  } /* end OpenMP critical */
+  /* Set variable type so following nco_mss_val_get() casts missing_value to correct type */
+  (void)nco_inq_vartype(var->nc_id,var->id,&var->type);
 
-  /* PJR requested warning to be added when multiple file operators worked on 
-     variables with missing_value since so many things could go wrong */
-  /* fxm: This should be un-necessary since multi-file packing ostensibly works */
+  /* Refresh number of attributes and missing value attribute, if any */
+  var->has_mss_val=nco_mss_val_get(var->nc_id,var);
+  
 #if 0
+  /* PJR requested warning to be added when multiple file operators worked on 
+     variables with missing_value since so many things could go wrong
+     Now un-necessary since multi-file packing ostensibly works
+     Leave code here in case we find it does not work */
   if(nco_is_rth_opr(prg_get()) && var->pck_dsk){
-    if(var->has_mss_val) (void)fprintf(stdout,"%s: WARNING Variable \"%s\" is packed and has valid \"missing_value\" attribute in multi-file arithmetic operator. Arithmetic on this variable will only be correct if...  \n",prg_nm_get(),var_nm);
+    if(var->has_mss_val) (void)fprintf(stdout,"%s: WARNING Variable \"%s\" is packed and has valid \"NCO_MSS_VAL_SNG\" attribute in multi-file arithmetic operator. Arithmetic on this variable will only be correct if...\n",prg_nm_get(),var_nm);
   } /* endif variable is packed */
 #endif /* endif False */
 
-} /* end nco_var_refresh() */
+} /* end nco_var_mtd_refresh() */
 
 void
-nco_var_srt_zero /* [fnc] Zero srt array of variable structure */
-(var_sct ** const var, /* I [sct] Variables whose srt arrays will be zeroed */
+nco_var_srd_srt_set /* [fnc] Assign zero-start and unity-stride vectors to variables */
+(var_sct ** const var, /* I [sct] Variables whose start and stride arrays to set */
  const int nbr_var) /* I [nbr] Number of structures in variable structure list */
 {
-  /* Purpose: Zero srt array of variable structure */
+  /* Purpose: Zero start (srt) and stride (srd) arrays of variable 
+     This is useful for setting variables to the default output state, in
+     which data is written in a block with no offset */
 
   int idx;
   int idx_dmn;
 
-  for(idx=0;idx<nbr_var;idx++)
-    for(idx_dmn=0;idx_dmn<var[idx]->nbr_dim;idx_dmn++)
+  for(idx=0;idx<nbr_var;idx++){
+    for(idx_dmn=0;idx_dmn<var[idx]->nbr_dim;idx_dmn++){
       var[idx]->srt[idx_dmn]=0L;
-
-} /* end nco_var_srt_zero() */
+      var[idx]->srd[idx_dmn]=1L;
+    } /* end loop over dimensions */
+  } /* end loop over variables */
+  
+} /* end nco_var_srd_srt_set() */
