@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.h,v 1.26 2004/09/07 04:31:19 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_var_utl.h,v 1.44 2007/02/23 21:59:31 zender Exp $ */
 
 /* Purpose: Variable utilities */
 
-/* Copyright (C) 1995--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 1995--2007 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Usage:
    #include "nco_var_utl.h" *//* Variable utilities */
@@ -27,7 +27,7 @@
 #include "nco_netcdf.h" /* NCO wrappers for netCDF C library */
 #ifdef _OPENMP
 #include <omp.h> /* OpenMP pragmas */
-#endif /* not _OPENMP */
+#endif /* !_OPENMP */
 
 /* Personal headers */
 #include "nco.h" /* netCDF Operator (NCO) definitions */
@@ -35,6 +35,7 @@
 #include "nco_bnr.h" /* Binary write utilities */
 #include "nco_ctl.h" /* Program flow control functions */
 #include "nco_mmr.h" /* Memory management */
+#include "nco_mss_val.h" /* Missing value utilities */
 #include "nco_pck.h" /* Packing and unpacking variables */
 
 #ifdef __cplusplus
@@ -54,15 +55,15 @@ extern "C" {
    const int out_id, /* I [id] netCDF output file ID */
    const int rec_dmn_id, /* I [id] Input file record dimension ID  */
    const char * const var_nm, /* I [sng] Input variable name */
-   const lmt_all_sct * const lmt_lst, /* I [sct] Hyperslab limits */
-   const int lmt_lst_nbr); /* I [nbr] Number of hyperslab limits */
+   CST_X_PTR_CST_PTR_CST_Y(lmt_all_sct,lmt_all_lst), /* I [sct] Hyperslab limits */
+   const int lmt_all_lst_nbr); /* I [nbr] Number of hyperslab limits */
 
   void
   nco_cpy_var_val /* [fnc] Copy variable data from input to output file, no limits */
   (int in_id, /* I [id] netCDF input file ID */
    int out_id, /* I [id] netCDF output file ID */
    FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
-   const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+   const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
    char *var_nm); /* I [sng] Variable name */
 
   void
@@ -70,7 +71,7 @@ extern "C" {
   (const int in_id, /* I [id] netCDF input file ID */
    const int out_id, /* I [id] netCDF output file ID */
    FILE * const fp_bnr, /* I [fl] Unformatted binary output file handle */
-   const bool NCO_BNR_WRT, /* I [flg] Write binary file */
+   const nco_bool NCO_BNR_WRT, /* I [flg] Write binary file */
    char *var_nm, /* I [sng] Variable name */
    const lmt_sct * const lmt, /* I [sct] Hyperslab limits */
    const int lmt_nbr); /* I [nbr] Number of hyperslab limits */
@@ -112,34 +113,26 @@ extern "C" {
   
   var_sct * /* O [sct] Pointer to free'd variable */
   nco_var_free /* [fnc] Free all memory associated with variable structure */
-  (var_sct *var); /* I [sct] Variable to free */
+  (var_sct *var); /* I/O [sct] Variable to free */
   
   void
   nco_var_get /* [fnc] Allocate, retrieve variable hyperslab from disk to memory */
   (const int nc_id, /* I [id] netCDF file ID */
    var_sct *var); /* I [sct] Variable to get */
   
-  nm_id_sct * /* O [sct] List with coordinate excluded */
-  nco_var_lst_crd_xcl /* [fnc] Exclude given coordinates from extraction list */
-  (const int nc_id, /* I [id] netCDF file ID */
-   const int dmn_id, /* I [id] Dimension ID of coordinate to remove from extraction list */
-   nm_id_sct *xtr_lst, /* I/O [sct] Current extraction list (destroyed) */
-   int * const nbr_xtr); /* I/O [nbr] Number of variables in extraction list */
-  
-  nm_id_sct * /* O [sct] Extraction list */
-  nco_var_lst_ass_crd_add /* [fnc] Add coordinates associated extracted variables to extraction list */
-  (const int nc_id, /* I netCDF file ID */
-   nm_id_sct *xtr_lst, /* I/O current extraction list (destroyed) */
-   int * const nbr_xtr); /* I/O number of variables in current extraction list */
-  
+  var_sct ** /* O [sct] Pointer to free'd structure list */
+  nco_var_lst_free /* [fnc] Free memory associated with variable structure list */
+  (var_sct **var_lst, /* I/O [sct] Variable structure list to free */
+   const int var_nbr); /* I [nbr] Number of variable structures in list */
+
   void
-  nco_var_refresh /* [fnc] Update variable metadata (var ID, dmn_nbr, mss_val) */
+  nco_var_mtd_refresh /* [fnc] Update variable metadata (dmn_nbr, ID, mss_val, type) */
   (const int nc_id, /* I [id] netCDF input-file ID */
    var_sct * const var); /* I/O [sct] Variable to update */
   
   void
-  nco_var_srt_zero /* [fnc] Zero srt array of variable structure */
-  (var_sct ** const var, /* I [sct] Variables whose srt arrays will be zeroed */
+  nco_var_srd_srt_set /* [fnc] Assign zero-start and unity-stride vectors to variables */
+  (var_sct ** const var, /* I [sct] Variables whose start and stride arrays to set */
    const int nbr_var); /* I [nbr] Number of structures in variable structure list */
   
   void
