@@ -1,17 +1,22 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $
+// $Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.24 2007/05/15 19:22:11 zender Exp $
 
 // Purpose: Test for C++ programs using libnco_c++ API
 
-/* Copyright (C) 2001--2004 Charlie Zender
-   This software may be modified and/or re-distributed under the terms of the GNU General Public License (GPL) Version 2
-   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
+/* Copyright (C) 2001--2005 Charlie Zender
+   You may copy, distribute, and/or modify this software under the terms of the GNU General Public License (GPL) Version 2
+   See http://www.gnu.org/copyleft/gpl.html for full license text */
 
 /* Compilation
    make -W tst.cc OPTS=D tst
    make OPTS=D tst
-   cd ${HOME}/nco/src/nco_c++;make -f Makefile.old -W OPTS=D tst.cc tst;cd -
+   scp ~/nco/src/nco_c++/tstcc dust.ess.uci.edu:nco/src/nco_c++
+   cd ${HOME}/nco/src/nco_c++;make -f Makefile.old OPTS=D -W tst.cc tst;cd -
+   cd ${HOME}/nco/src/nco_c++;make -f Makefile.old OPTS=D lib_cln cln;make -f Makefile.old OPTS=D all;cd -
    cd ${HOME}/nco/src/nco_c++;make OPTS=D tst;cd - */
 
+/* Usage:
+   tst */
+
 // etags ~/nco/src/nco_c++/*.cc ~/nco/src/nco_c++/*.hh ~/mie/*.cc ~/mie/*.hh ~/slr_spc/*.cc ~/slr_spc/*.hh ~/ck/htrn.c ~/c++/*.cc ~/c++/*.hh 
 
 // Autotools may enable/disable library features
@@ -50,10 +55,10 @@ int main(int argc,char **argv)
   long sz_nbr(1); // [nbr] Number of sizes
   register long idx; // [idx] Counting index
 
-  const std::string CVS_Date("$Date: 2004/06/03 05:52:43 $"); // [sng] CVS date string
-  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $"); // [sng] CVS header string
-  const std::string CVS_Id("$Id: tst.cc,v 1.15 2004/06/03 05:52:43 zender Exp $"); // [sng] CVS identification string
-  const std::string CVS_Revision("$Revision: 1.15 $"); // [sng] CVS revision string
+  const std::string CVS_Date("$Date: 2007/05/15 19:22:11 $"); // [sng] CVS date string
+  const std::string CVS_Header("$Header: /cvsroot/nco/nco/src/nco_c++/tst.cc,v 1.24 2007/05/15 19:22:11 zender Exp $"); // [sng] CVS header string
+  const std::string CVS_Id("$Id: tst.cc,v 1.24 2007/05/15 19:22:11 zender Exp $"); // [sng] CVS identification string
+  const std::string CVS_Revision("$Revision: 1.24 $"); // [sng] CVS revision string
   const std::string date_cvs(CVS_Date.length() > 7 ? CVS_Date.substr(7,19) : static_cast<std::string>("Unknown")); // [sng] Date from CVS
   const std::string sbr_nm("main"); // [sng] Subroutine name
   const std::string prg_nm("libnco_c++"); // [sng] Program name
@@ -93,7 +98,21 @@ int main(int argc,char **argv)
   } // end loop over idx
 
   // Open output file
-  const int nc_out(nco_create(fl_out,NC_CLOBBER)); 
+  int fl_out_fmt(NC_FORMAT_CLASSIC); // [enm] Output file format
+  int nccreate_mode(NC_CLOBBER); // [enm] Mode flag for nco_create() call
+#ifdef ENABLE_NETCDF4
+  fl_out_fmt=NC_FORMAT_NETCDF4; // [enm] Output file format
+  if(fl_out_fmt == NC_FORMAT_64BIT){
+    nccreate_mode|=NC_64BIT_OFFSET;
+  }else if(fl_out_fmt == NC_FORMAT_NETCDF4){
+    nccreate_mode|=NC_NETCDF4;
+  }else if(fl_out_fmt == NC_FORMAT_NETCDF4_CLASSIC){
+    nccreate_mode|=NC_NETCDF4|NC_CLASSIC_MODEL;
+  } /* end else fl_out_fmt */
+#else // !ENABLE_NETCDF4
+  if(fl_out_fmt == NC_FORMAT_CLASSIC) nccreate_mode+=0; // CEWI
+#endif // !ENABLE_NETCDF4
+  const int nc_out(nco_create(fl_out,nccreate_mode)); 
   const nc_type nco_xtyp(nco_get_xtype(static_cast<prc_cmp>(1.0))); // [enm] External netCDF type
   std::cout << "INFO External netCDF type of prc_cmp variables will be " << nco_typ_sng(nco_xtyp) << std::endl;
  
@@ -158,6 +177,15 @@ int main(int argc,char **argv)
   rcd=nco_put_var(nc_out,static_cast<std::string>("sz_ldb"),sz_ldb); delete []sz_ldb;
   rcd=nco_put_var(nc_out,static_cast<std::string>("sz_nbr"),sz_nbr);
 
+  int rcd_opt(NC_ENOTVAR); // [enm] Acceptable return code
+  int var_id; // [id] Variable ID
+  int var_ndims; // [nbr] Number of dimensions
+  const std::string var_nm_nonesuch("quark"); // [sng]
+  rcd=nco_inq_varid(nc_out,var_nm_nonesuch,var_id,rcd_opt);
+  std::cout << "tst: nco_inq_varid() for variable \"" << var_nm_nonesuch << "\" with rcd_opt = " << rcd_opt << " yields rcd = " << rcd << std::endl;
+  rcd=nco_inq_varndims(nc_out,var_nm_nonesuch,var_ndims,rcd_opt);
+  std::cout << "tst: nco_inq_varndims() for variable \"" << var_nm_nonesuch << "\" with rcd_opt = " << rcd_opt << " yields rcd = " << rcd << std::endl;
+
   // Close output file
   rcd=nco_close(nc_out); // [fnc] Close netCDF file
   std::cerr << "Wrote results to " << fl_out << std::endl;
